if not modules then modules = { } end modules ['mult-low'] = {
    version   = 1.001,
    comment   = "companion to mult-ini.mkiv",
    author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
    copyright = "PRAGMA ADE / ConTeXt Development Team",
    license   = "see context related readme files"
}

-- for syntax highlighters, only the ones that are for users (boring to collect them)

return {
    ["constants"] = {
        --
        "zerocount", "minusone", "minustwo",
        "plusone", "plustwo", "plusthree", "plusfour", "plusfive", "plussix", "plusseven", "pluseight",
        "plusnine", "plusten", "pluseleven", "plustwelve", "plusthirteen", "plusfourteen", "plusfifteen", "plussixteen",
        "plusfifty", "plussixty", "plushundred", "plusonehundred", "plustwohundred", "plusfivehundred",
        "plusonehundredtwentyfive", "plustwohundredfifty",
        "plusthousand", "plustenthousand", "plustwentythousand", "medcard", "maxcard", "maxcardminusone", "maxiterator",
        "plusonethousand", "plustwothousand", "plusthreethousand", "plusfourthousand", "plusfivethousand",
        "plussixthousand", "plusseventhousand", "pluseightthousand", "plusninethousand",
        "plusfifteenhundred", "plustwentyfivehundred", "plusfortyfivehundred", "plusseventyfivehundred",
        "plusninetynine", "plusthirtytwo",  "plusfourtytwo", "plustwentyfour", "plustwentyfive",
        "plusthousandfifty", "plustwelvehundredfifty",
        "zeropoint", "onepoint", "halfapoint", "onebasepoint", "maxcount", "maxdimen", "scaledpoint", "thousandpoint",
        "points", "halfpoint", "twopoints", "threepoints",
        "zeroskip", "centeringskip", "stretchingskip", "shrinkingskip",
        "centeringfillskip", "stretchingfillskip", "shrinkingfillskip",
        "centeringfilllskip", "stretchingfilllskip", "shrinkingfilllskip",
        "zeromuskip", "onemuskip",
        "pluscxxvii", "pluscxxviii", "pluscclv", "pluscclvi",
        "normalpagebox", "delayedpagebox",
        "binaryshiftedten", "binaryshiftedtwenty", "binaryshiftedthirty",
        "thickermuskip", "continuationmuskip", "fencemuskip", "mathinterwordmuskip",
        "zerofloat",
        --
        "directionlefttoright", "directionrighttoleft",
        --
        "endoflinetoken", "outputnewlinechar",
        --
        "emptytoks", "empty", "undefined",
        --
        "blankspace", "relaxedspace", -- old new
        --
        "prerollrun",
        --
        "voidbox", "emptybox", "emptyvbox", "emptyhbox",
        --
        "bigskipamount", "medskipamount", "smallskipamount",
        --
        "fmtname", "fmtversion", "texengine", "texenginename", "texengineversion", "texenginefunctionality",
        "luatexengine", "pdftexengine", "xetexengine", "unknownengine",
        "contextformat", "contextversion", "contextlmtxmode", "contextmark", "mksuffix",
        --
        "activecatcode",
        --
        "bgroup", "egroup",
        "endline",
        --
        "conditionaltrue", "conditionalfalse",
        --
        "attributeunsetvalue",
        --
        "statuswrite",
        --
        "uprotationangle", "rightrotationangle", "downrotationangle", "leftrotationangle",
        --
        "inicatcodes",
        "ctxcatcodes", "texcatcodes", "notcatcodes", "txtcatcodes", "vrbcatcodes",
        "prtcatcodes", "nilcatcodes", "luacatcodes", "tpacatcodes", "tpbcatcodes",
        "xmlcatcodes", "ctdcatcodes", "rlncatcodes", "hshcatcodes",
        --
        "escapecatcode", "begingroupcatcode", "endgroupcatcode", "mathshiftcatcode", "alignmentcatcode",
        "endoflinecatcode", "parametercatcode", "superscriptcatcode", "subscriptcatcode", "ignorecatcode",
        "spacecatcode", "lettercatcode", "othercatcode", "activecatcode", "commentcatcode", "invalidcatcode",
        --
        "tabasciicode", "newlineasciicode", "formfeedasciicode", "endoflineasciicode", "endoffileasciicode",
        "commaasciicode", "spaceasciicode", "periodasciicode",
        "hashasciicode", "dollarasciicode", "commentasciicode", "ampersandasciicode",
        "colonasciicode", "semicolonasciicode", "backslashasciicode", "circumflexasciicode", "underscoreasciicode",
        "leftbraceasciicode", "barasciicode", "rightbraceasciicode", "tildeasciicode", "delasciicode",
        "leftparentasciicode", "rightparentasciicode",
        "lessthanasciicode", "morethanasciicode", "doublecommentsignal",
        "atsignasciicode", "exclamationmarkasciicode", "questionmarkasciicode",
        "doublequoteasciicode", "singlequoteasciicode", "forwardslashasciicode",
        "primeasciicode", "hyphenasciicode", "percentasciicode", "leftbracketasciicode", "rightbracketasciicode",
        "zeroasciicode", "nineasciicode", "alowercaseasciicode", "zlowercaseasciicode",
        --
        "hsizefrozenparcode", "skipfrozenparcode", "hangfrozenparcode", "indentfrozenparcode", "parfillfrozenparcode",
        "adjustfrozenparcode", "protrudefrozenparcode", "tolerancefrozenparcode", "stretchfrozenparcode",
        "loosenessfrozenparcode", "lastlinefrozenparcode", "linepenaltyfrozenparcode", "clubpenaltyfrozenparcode",
        "widowpenaltyfrozenparcode", "displaypenaltyfrozenparcode", "brokenpenaltyfrozenparcode",
        "demeritsfrozenparcode", "shapefrozenparcode", "linefrozenparcode", "hyphenationfrozenparcode",
        "shapingpenaltyfrozenparcode", "orphanpenaltyfrozenparcode", "toddlerpenaltyfrozenparcode",
        "emergencyfrozenparcode", "parpassesfrozenparcode", "singlelinepenaltyfrozenparcode",
        "hyphenpenaltyfrozenparcode", "linesnappingfrozenparcode", "linebreakchecksfrozenparcode",
        "twindemeritsfrozenparcode", "fitnessclassesfrozenparcode", "allfrozenparcode",
        --
        "activemathcharcode",
        --
        "activetabtoken", "activeformfeedtoken", "activeendoflinetoken",
        --
        "batchmodecode", "nonstopmodecode", "scrollmodecode", "errorstopmodecode",
        --
        "bottomlevelgroupcode", "simplegroupcode", "hboxgroupcode", "adjustedhboxgroupcode", "vboxgroupcode",
        "vtopgroupcode", "aligngroupcode", "noaligngroupcode", "outputgroupcode", "mathgroupcode",
        "discretionarygroupcode", "insertgroupcode", "vadjustgroupcode", "vcentergroupcode", "mathabovegroupcode",
        "mathchoicegroupcode", "alsosimplegroupcode", "semisimplegroupcode", "mathshiftgroupcode", "mathleftgroupcode",
        "localboxgroupcode", "splitoffgroupcode", "splitkeepgroupcode", "preamblegroupcode",
        "alignsetgroupcode", "finrowgroupcode", "discretionarygroupcode",
        --
        "markautomigrationcode", "insertautomigrationcode", "adjustautomigrationcode", "preautomigrationcode", "postautomigrationcode",
        --
        "charnodecode", "hlistnodecode", "vlistnodecode", "rulenodecode", "insertnodecode", "marknodecode",
        "adjustnodecode", "ligaturenodecode", "discretionarynodecode", "whatsitnodecode", "mathnodecode",
        "gluenodecode", "kernnodecode", "penaltynodecode", "unsetnodecode", "mathsnodecode",
        --
     -- "charifcode", "catifcode", "numifcode", "dimifcode", "oddifcode", "vmodeifcode", "hmodeifcode",
     -- "mmodeifcode", "innerifcode", "voidifcode", "hboxifcode", "vboxifcode", "xifcode", "eofifcode",
     -- "trueifcode", "falseifcode", "caseifcode", "definedifcode", "csnameifcode", "fontcharifcode",
        --
        "overrulemathcontrolcode", "underrulemathcontrolcode", "radicalrulemathcontrolcode", "fractionrulemathcontrolcode",
        "accentskewhalfmathcontrolcode", "accentskewapplymathcontrolcode", "applyordinarykernpairmathcontrolcode",
        "applyverticalitalickernmathcontrolcode", "applyordinaryitalickernmathcontrolcode", "applycharitalickernmathcontrolcode",
        "reboxcharitalickernmathcontrolcode", "applyboxeditalickernmathcontrolcode", "staircasekernmathcontrolcode",
        "applytextitalickernmathcontrolcode", "applyscriptitalickernmathcontrolcode",
        "checkspaceitalickernmathcontrolcode", "checktextitalickernmathcontrolcode",
        "analyzescriptnucleuscharmathcontrolcode", "analyzescriptnucleuslistmathcontrolcode", "analyzescriptnucleusboxmathcontrolcode",
        "accenttopskewwithoffsetmathcontrolcode", "ignorekerndimensionsmathcontrolcode", "ignoreflataccentsmathcontrolcode",
        "extendaccentsmathcontrolcode", "extenddelimitersmathcontrolcode",
        --
        "normalparcontextcode", "vmodeparcontextcode", "vboxparcontextcode", "vtopparcontextcode", "vcenterparcontextcode",
        "vadjustparcontextcode", "insertparcontextcode", "outputparcontextcode", "alignparcontextcode",
        "noalignparcontextcode", "spanparcontextcode", "resetparcontextcode",
        --
        "normalparbegincode", "forceparbegincode", "indentparbegincode", "noindentparbegincode", "mathcharparbegincode",
        "charparbegincode", "boundaryparbegincode", "spaceparbegincode", "mathparbegincode",
        "kernparbegincode", "hskipparbegincode", "unhboxparbegincode", "valignparbegincode",
        "vruleparbegincode",
        --
        "fixedsuperorsubscriptsmodecode", "fixedsuperandsubscriptsmodecode", "ignoreemptyscriptsmodecode",
        "ignoreemptysuperscriptsmodecode", "ignoreemptysubscriptsmodecode", "ignoreemptyprimescriptsmodecode",
        --
        "inheritclassdoublescriptmodecode", "discardshapekerndoublescriptmodecode",
        "realignscriptsdoublescriptmodecode", "reorderprescriptsdoublescriptmodecode",
        --
        "leftoriginlistanchorcode", "leftheightlistanchorcode", "leftdepthlistanchorcode",
        "rightoriginlistanchorcode", "rightheightlistanchorcode", "rightdepthlistanchorcode",
        "centeroriginlistanchorcode", "centerheightlistanchorcode", "centerdepthlistanchorcode",
        "halfwaytotallistanchorcode", "halfwayheightlistanchorcode", "halfwaydepthlistanchorcode",
        "halfwayleftlistanchorcode", "halfwayrightlistanchorcode",
        --
        "underfullbadnessmodecode", "loosebadnessmodecode", "tightbadnessmodecode", "overfullbadnessmodecode",
        --
        "negatexlistsigncode", "negateylistsigncode", "negatelistsigncode",
        --
        "fontslantperpoint", "fontinterwordspace", "fontinterwordstretch", "fontinterwordshrink",
        "fontexheight", "fontemwidth", "fontextraspace", "slantperpoint",
        "mathexheight", "mathemwidth",
        "interwordspace", "interwordstretch", "interwordshrink", "exheight", "emwidth", "extraspace",
        "mathaxisheight",
        "muquad",
        --
        "smallcapsratio",
        "currentspaceskip", "currentspacevalue", "usedspacevalue",
        "mathstacktotal", "mathstackvgap",
        --
        -- maybe a different class
        --
        "startmode", "stopmode", "startnotmode", "stopnotmode", "startmodeset", "stopmodeset",
        "doifmode", "doifelsemode", "doifmodeelse", "doifnotmode",
        "startmodeset","stopmodeset",
        "startallmodes", "stopallmodes", "startnotallmodes", "stopnotallmodes",
        "doifallmodes", "doifelseallmodes", "doifallmodeselse", "doifnotallmodes",
        "startenvironment", "stopenvironment", "environment",
        "startcomponent", "stopcomponent", "component", "startlocalcomponent", "stoplocalcomponent",
        "startproduct", "stopproduct", "product",
        "startproject", "stopproject", "project",
        "starttext", "stoptext", "startnotext", "stopnotext",
        "startdocument", "stopdocument", "documentvariable", "unexpandeddocumentvariable", "setupdocument", "presetdocument",
        "doifelsedocumentvariable", "doifdocumentvariableelse", "doifdocumentvariable", "doifnotdocumentvariable",
        "startmodule", "stopmodule", "usemodule", "usetexmodule", "useluamodule","setupmodule","currentmoduleparameter","moduleparameter",
        "everystarttext", "everystoptext", "everyforgetall", "luaenvironment",
        --
        -- -- maybe do all the important every's
        --
     -- "everymathematics", "everyinsidemathematics",
        --
        "startTEXpage", "stopTEXpage",
     -- "startMPpage", "stopMPpage", -- already catched by nested lexer
        --
        "enablemode", "disablemode", "preventmode", "definemode",
        "globalenablemode", "globaldisablemode", "globalpreventmode",
        "pushmode", "popmode",
        --
        "typescriptone", "typescripttwo", "typescriptthree", "mathsizesuffix",
        --
        "mathordinarycode", "mathordcode", "mathoperatorcode", "mathopcode", "mathbinarycode", "mathbincode",
        "mathrelationcode", "mathrelcode", "mathopencode", "mathclosecode", "mathpunctuationcode",
        "mathpunctcode", "mathovercode", "mathundercode", "mathinnercode", "mathradicalcode",
        "mathfractioncode", "mathmiddlecode", "mathprimecode", "mathaccentcode", "mathfencedcode", "mathghostcode",
        "mathvariablecode",  "mathactivecode", "mathvcentercode", "mathconstructcode", "mathwrappedcode",
        "mathbegincode", "mathendcode", "mathexplicitcode", "mathdivisioncode", "mathfactorialcode",
        "mathdimensioncode", "mathexperimentalcode", "mathtextpunctuationcode", "mathcontinuationcode",
        "mathimaginarycode", "mathdifferentialcode", "mathexponentialcode", "mathintegralcode", "mathellipsiscode", "mathfunctioncode", "mathdigitcode",
     -- "mathtopaccentcode", "mathbottomaccentcode", "mathdelimitercode", "mathrootcode", "mathprintcode",        --
        "mathalphacode", "mathboxcode", "mathchoicecode", "mathnothingcode", "mathlimopcode", "mathnolopcode",
        "mathunsetcode", "mathunspacedcode", "mathallcode", "mathfakecode", "mathunarycode",
        "mathmaybeordinarycode", "mathmayberelationcode", "mathmaybebinarycode", "mathnumbergroupcode",
        "mathchemicalbondcode", "mathimplicationcode",
        --
        "mathnormalstylepreset", "mathcrampedstylepreset", "mathsubscriptstylepreset", "mathsuperscriptstylepreset", "mathsmallstylepreset",
        "mathsmallerstylepreset", "mathnumeratorstylepreset", "mathdenominatorstylepreset", "mathdoublesuperscriptstylepreset",
        --
        "constantnumber", "constantnumberargument", "constantdimen", "constantdimenargument", "constantemptyargument",
        --
        "periodicshrink",
        --
        "luastringsep", "!!bs", "!!es",
        --
        "lefttorightmark", "righttoleftmark", "lrm", "rlm",
        "bidilre", "bidirle", "bidipop", "bidilro", "bidirlo",
        --
        "breakablethinspace", "nobreakspace", "nonbreakablespace", "narrownobreakspace", "zerowidthnobreakspace",
        "ideographicspace", "ideographichalffillspace",
        "twoperemspace", "threeperemspace", "fourperemspace", "fiveperemspace", "sixperemspace",
        "figurespace", "punctuationspace", "hairspace", "enquad", "emquad",
        "zerowidthspace", "zerowidthnonjoiner", "zerowidthjoiner", "zwnj", "zwj",
        "optionalspace", "asciispacechar", "softhyphen", "autoinsertedspace",
        --
        "Ux", "eUx",
        --
        "mathstylevariantidentity", "mathstylevariantcramped", "mathstylevariantuncramped",
        --
     -- "parfillleftskip", "parfillrightskip",
        --
        "startlmtxmode", "stoplmtxmode", "startmkivmode", "stopmkivmode",
        --
        "wildcardsymbol", -- "plussymbol", "minussymbol",
        --
        "normalhyphenationcode", "automatichyphenationcode", "explicithyphenationcode", "syllablehyphenationcode", "uppercasehyphenationcode",
        "collapsehyphenationcode", "compoundhyphenationcode", "strictstarthyphenationcode", "strictendhyphenationcode",
        "automaticpenaltyhyphenationcode", "explicitpenaltyhyphenationcode", "permitgluehyphenationcode", "permitallhyphenationcode",
        "permitmathreplacehyphenationcode", "forcecheckhyphenationcode", "lazyligatureshyphenationcode", "forcehandlerhyphenationcode",
        "feedbackcompoundhyphenationcode", "ignoreboundshyphenationcode", "partialhyphenationcode", "completehyphenationcode",
        "replaceapostrophehyphenationcode",
        --
        "normalizelinenormalizecode", "parindentskipnormalizecode", "swaphangindentnormalizecode", "swapparsshapenormalizecode",
        "breakafterdirnormalizecode", "removemarginkernsnormalizecode", "clipwidthnormalizecode", "flattendiscretionariesnormalizecode",
        "discardzerotabskipsnormalizecode", "flattenhleadersnormalizecode", "balanceinlinemathnormalizecode",
        --
        "normalizeparnormalizeparcode", "flattenvleadersnormalizeparcode", "limitprevgrafnormalizeparcode",
        "keepinterlinepenaltiesnormalizeparcode", "removetrailingspacesnormalizeparcode",
        --
        "nopreslackclassoptioncode", "nopostslackclassoptioncode",
        "lefttopkernclassoptioncode", "righttopkernclassoptioncode", "leftbottomkernclassoptioncode", "rightbottomkernclassoptioncode",
        "lookaheadforendclassoptioncode", "noitaliccorrectionclassoptioncode",  "defaultmathclassoptions",
     -- "openfenceclassoptioncode", "closefenceclassoptioncode", "middlefenceclassoptioncode",
        "checkligatureclassoptioncode", "checkitaliccorrectionclassoptioncode", "checkkernpairclassoptioncode",
        "flattenclassoptioncode", "omitpenaltyclassoptioncode", "unpackclassoptioncode", "raiseprimeclassoptioncode",
        "carryoverlefttopkernclassoptioncode", "carryoverleftbottomkernclassoptioncode", "carryoverrighttopkernclassoptioncode", "carryoverrightbottomkernclassoptioncode",
        "preferdelimiterdimensionsclassoptioncode", "autoinjectclassoptioncode", "removeitaliccorrectionclassoptioncode",
        "operatoritaliccorrectionclassoptioncode", "shortinlineclassoptioncode",
        "pushnestingclassoptioncode", "popnestingclassoptioncode", "obeynestingclassoptioncode",
        --
        "checkheightinsertoptioncode",
        "pageinsertcategorycode",
        --
        "noitaliccorrectionglyphoptioncode", "nozeroitaliccorrectionglyphoptioncode",
        "noexpansionglyphoptioncode", "noprotrusionglyphoptioncode",
        "noleftkernglyphoptioncode",  "norightkernglyphoptioncode",
        "noleftligatureglyphoptioncode", "norightligatureglyphoptioncode",
        "textcheckitalicglyphoptioncode", "mathcheckitalicglyphoptioncode",
        "weightlessglyphoptioncode", "spacefactoroverloadglyphoptioncode",
        "checktoddlerglyphoptioncode", "checktwinglyphoptioncode",
        --
        "ignoretwincharactercontrolcode",
        --
        "repeatspecificationoptioncode", "doublespecificationoptioncode", "largestspecificationoptioncode",
        "presetsspecificationoptioncode", "integerspecificationoptioncode", "finalspecificationoptioncode",
        "defaultspecificationoptioncode", "ignorespecificationoptioncode", "rotatespecificationoptioncode",
        "constantspecificationoptioncode",
        --
        "shortmathoptioncode", "orphanedmathoptioncode", "displaymathoptioncode", "crampedmathoptioncode",
        "nosnappingmathoptioncode", "textmathoptioncode",
        --
     -- "noitaliccorrectionkerneloptioncode", "noleftpairkernkerneloptioncode", "norightpairkernkerneloptioncode",
     -- "autodiscretionarykerneloptioncode", "fulldiscretionarykerneloptioncode",
     -- "ignoredcharacterkerneloptioncode", "islargeoperatorkerneloptioncode", "hasitalicshapekerneloptioncode",
        --
        "nokerningcode", "noligaturingcode", "noitalicscode",
        --
        "allparpassclasses", "indecentparpassclasses", "looseparpassclasses", "tightparpassclasses", "almostdecentparpassclasses",
        --
        "verylooseparpassclass", "looseparpassclass", "almostlooseparpassclass", "barelylooseparpassclass",
        "decentparpassclass",
        "verytightparpassclass", "tightparpassclass", "almosttightparpassclass", "barelytightparpassclass",
        --
        "frozenflagcode", "tolerantflagcode", "protectedflagcode", "primitiveflagcode", "permanentflagcode", "noalignedflagcode", "immutableflagcode",
        "mutableflagcode", "globalflagcode", "overloadedflagcode", "immediateflagcode", "conditionalflagcode", "valueflagcode", "instanceflagcode",
        --
        "ordmathflattencode", "binmathflattencode", "relmathflattencode", "punctmathflattencode", "innermathflattencode",
        --
        "normalworddiscoptioncode", "preworddiscoptioncode", "postworddiscoptioncode",
        "preferbreakdiscoptioncode", "prefernobreakdiscoptioncode",
        "noitaliccorrectiondiscoptioncode", "nozeroitaliccorrectiondiscoptioncode",
        "textcheckitalicdiscoptioncode",
        --
        "ignoreprevdepthmvloptioncode", "noprevdepthmvloptioncode", "discardtopmvloptioncode", "discardbottommvloptioncode",
        --
        "continueifinputfile", "continuewhenlmtxmode", "continuewhenmkivmode",
        --
        "uunit",
        --
        "defaultdisplaywidowpenalty", "defaultwidowpenalty", "defaultclubpenalty", "defaultbrokenpenalty",
        "defaultgriddisplaywidowpenalty", "defaultgridwidowpenalty", "defaultgridclubpenalty", "defaultgridbrokenpenalty",
        --
        "luametatexverboseversion", "luametatexfunctionality",
        --
        "currentverbosemathstyle",
    },
    ["helpers"] = {
        --
        "pushglobalsetups", "popglobalsetups",
        "startsetups", "stopsetups",
        "startxmlsetups", "stopxmlsetups",
        "startluasetups", "stopluasetups",
        "starttexsetups", "stoptexsetups",
        "startrawsetups", "stoprawsetups",
        "startlocalsetups", "stoplocalsetups",
        "starttexdefinition", "stoptexdefinition",
        "starttexcode", "stoptexcode",
        "startcontextcode", "stopcontextcode",
        "startcontextdefinitioncode", "stopcontextdefinitioncode",
        "texdefinition",
        --
        "doifelsesetups", "doifsetupselse", "doifsetups", "doifnotsetups", "setup", "setups", "texsetup", "xmlsetup", "luasetup", "directsetup", "fastsetup",
        "copysetups", "resetsetups",
        "doifelsecommandhandler", "doifcommandhandlerelse", "doifnotcommandhandler", "doifcommandhandler",
        --
        "newmode", "setmode", "resetmode",
        "newsystemmode", "setsystemmode", "resetsystemmode", "pushsystemmode", "popsystemmode",
        "globalsetmode", "globalresetmode", "globalsetsystemmode", "globalresetsystemmode",
        "booleanmodevalue",
        --
        "newcount", "newdimen", "newskip", "newmuskip", "newbox", "newtoks", "newread", "newwrite", "newmarks", "newinsert", "newattribute", "newif", "newfloat", "newmvl",
        "newlanguage", "newfamily", "newfam", "newhelp", -- not used
        "newuserunit", "userunitcode",
        --
        "newinteger", "newdimension", "newgluespec", "newmugluespec", "newposit",
        "aliasinteger", "aliasdimension", "aliasposit",
        --
        "then",
        "begcsname",
        --
        "autorule",
        --
        "tobit", "tobits", "tohexa",
        --
        "strippedcsname","checkedstrippedcsname",
        --
        "nofarguments",
        "firstargumentfalse", "firstargumenttrue",
        "secondargumentfalse", "secondargumenttrue",
        "thirdargumentfalse", "thirdargumenttrue",
        "fourthargumentfalse", "fourthargumenttrue",
        "fifthargumentfalse", "fifthargumenttrue",
        "sixthargumentfalse", "sixthargumenttrue",
        "seventhargumentfalse", "seventhargumenttrue",
        --
     -- "vkern", "hkern", "vpenalty", "hpenalty", -- native in mkxl
        --
        "doglobal", "dodoglobal", "redoglobal", "resetglobal",
        --
        "donothing", "untraceddonothing", "dontcomplain", "moreboxtracing", "lessboxtracing", "noboxtracing", "forgetall",
        --
        "donetrue", "donefalse", "foundtrue", "foundfalse",
        "globaldonetrue", "globaldonefalse", "globalfoundtrue", "globalfoundfalse",
        --
        "inlineordisplaymath", "indisplaymath", "forcedisplaymath", "startforceddisplaymath", "stopforceddisplaymath",
        "startpickupmath", "stoppickupmath", "reqno", "forceinlinemath",
        --
        "mathortext",
        --
        "thebox",
        "htdp",
        "unvoidbox",
        "hfilll", "vfilll",
        --
        "mathbox", "mathlimop", "mathnolop", "mathnothing", "mathalpha",
        --
        "currentcatcodetable", "defaultcatcodetable", "catcodetablename",
        "newcatcodetable", "startcatcodetable", "stopcatcodetable", "startextendcatcodetable", "stopextendcatcodetable",
        "pushcatcodetable", "popcatcodetable", "restorecatcodes",
        "setcatcodetable", "letcatcodecommand", "defcatcodecommand", "uedcatcodecommand",
        --
        "hglue", "vglue", "hfillneg", "vfillneg", "hfilllneg", "vfilllneg",
        --
        "hsplit",
        --
        "ruledhss", "ruledhfil", "ruledhfill", "ruledhfilll", "ruledhfilneg", "ruledhfillneg", "normalhfillneg",  "normalhfilllneg",
        "ruledvss", "ruledvfil", "ruledvfill", "ruledvfilll", "ruledvfilneg", "ruledvfillneg", "normalvfillneg",  "normalvfilllneg",
        "ruledhbox", "ruledvbox", "ruledvtop", "ruleddbox", "ruledvcenter", "ruledmbox",
        "ruledhpack", "ruledvpack", "ruledtpack", "ruleddpack",
        "ruledvsplit", "ruledtsplit", "ruleddsplit",
        "ruledhskip", "ruledvskip", "ruledkern", "ruledmskip", "ruledmkern",
        "ruledhglue", "ruledvglue", "normalhglue", "normalvglue",
        "ruledpenalty",
        --
        "filledhboxb", "filledhboxr", "filledhboxg", "filledhboxc", "filledhboxm", "filledhboxy", "filledhboxk",
        --
        "scratchstring", "scratchstringone", "scratchstringtwo", "tempstring",
        "scratchcounter", "globalscratchcounter", "privatescratchcounter",
        "scratchfloat", "globalscratchfloat", "privatescratchfloat",
        "scratchdimen", "globalscratchdimen", "privatescratchdimen",
        "scratchskip", "globalscratchskip", "privatescratchskip",
        "scratchmuskip", "globalscratchmuskip", "privatescratchmuskip",
        "scratchtoks", "globalscratchtoks", "privatescratchtoks",
        "scratchbox", "globalscratchbox", "privatescratchbox",
        "scratchmacro", "scratchmacroone", "scratchmacrotwo",
        --
        "ignoredtoks",
        --
        "scratchconditiontrue", "scratchconditionfalse", "ifscratchcondition",
        "scratchconditiononetrue", "scratchconditiononefalse", "ifscratchconditionone",
        "scratchconditiontwotrue", "scratchconditiontwofalse", "ifscratchconditiontwo",
        --
        "globalscratchcounterone", "globalscratchcountertwo", "globalscratchcounterthree",
        --
        "groupedcommand", "groupedcommandcs",
        "triggergroupedcommand", "triggergroupedcommandcs",
        "simplegroupedcommand", "simplegroupedcommandcs",
        "pickupgroupedcommand", "pickupgroupedcommandcs",
        "mathgroupedcommandcs",
        --
        "usedbaselineskip", "usedlineskip", "usedlineskiplimit",
        --
        "availablehsize", "localhsize", "setlocalhsize", "distributedhsize", "hsizefraction",
        --
        "next", "nexttoken",
        --
        "nextbox", "dowithnextbox", "dowithnextboxcs", "dowithnextboxcontent", "dowithnextboxcontentcs", "flushnextbox",
        "boxisempty", "boxtostring", "contentostring", "prerolltostring",
        --
        "givenwidth", "givenheight", "givendepth", "scangivendimensions",
        --
        "scratchwidth", "scratchheight", "scratchdepth", "scratchoffset", "scratchdistance", "scratchtotal", "scratchitalic",
        "scratchhsize", "scratchvsize", "scratchclass",
        "scratchxoffset", "scratchyoffset", "scratchhoffset", "scratchvoffset",
        "scratchxposition", "scratchyposition",
        "scratchtopoffset", "scratchbottomoffset", "scratchleftoffset", "scratchrightoffset",
        --
        "scratchcounterone", "scratchcountertwo", "scratchcounterthree", "scratchcounterfour", "scratchcounterfive", "scratchcountersix",
        "scratchfloatone", "scratchfloattwo", "scratchfloatthree", "scratchfloatfour", "scratchfloatfive", "scratchfloatsix",
        "scratchdimenone", "scratchdimentwo", "scratchdimenthree", "scratchdimenfour", "scratchdimenfive", "scratchdimensix",
        "scratchskipone", "scratchskiptwo", "scratchskipthree", "scratchskipfour", "scratchskipfive", "scratchskipsix",
        "scratchmuskipone", "scratchmuskiptwo", "scratchmuskipthree", "scratchmuskipfour", "scratchmuskipfive", "scratchmuskipsix",
        "scratchtoksone", "scratchtokstwo", "scratchtoksthree", "scratchtoksfour", "scratchtoksfive", "scratchtokssix",
        "scratchboxone", "scratchboxtwo", "scratchboxthree", "scratchboxfour", "scratchboxfive", "scratchboxsix",
        "scratchnx", "scratchny", "scratchmx", "scratchmy", "scratchsx", "scratchsy",
        "scratchunicode", "scratchunicodeone", "scratchunicodetwo", "scratchunicodethree",
        "scratchmin", "scratchmax",
        "scratchread", "scratchwrite",
        --
        "pfsin", "pfcos", "pftan", "pfasin", "pfacos", "pfatan", "pfsinh", "pfcosh", "pftanh", "pfasinh", "pfacosh", "pfatanh",
        "pfsqrt", "pflog", "pfexp", "pfceil", "pffloor", "pfround", "pfabs", "pfrad", "pfdeg", "pfatantwo", "pfpow", "pfmod", "pfrem",
        --
        "scratchleftskip", "scratchrightskip", "scratchtopskip", "scratchbottomskip",
        --
        "startimage", "stopimage",
        --
        "doif", "doifnot", "doifelse",
        "firstinset",
        "doifinset", "doifnotinset",
        "doifelseinset", "doifinsetelse",
        "doifelsenextchar", "doifnextcharelse",
        "doifelsenextcharcs", "doifnextcharcselse",
        "doifelsenextoptional", "doifnextoptionalelse",
        "doifelsenextoptionalcs", "doifnextoptionalcselse",
        "doifelsefastoptionalcheck", "doiffastoptionalcheckelse",
        "doifelsefastoptionalcheckcs", "doiffastoptionalcheckcselse",
        "doifelsenextbgroup", "doifnextbgroupelse",
        "doifelsenextbgroupcs", "doifnextbgroupcselse",
        "doifelsenextparenthesis", "doifnextparenthesiselse",
        "doifelseundefined", "doifundefinedelse",
        "doifelsedefined", "doifdefinedelse",
        "doifundefined", "doifdefined",
        "doifelsevalue", "doifvalue", "doifnotvalue",
        "doifnothing", "doifsomething",
        "doifelsenothing", "doifnothingelse",
        "doifelsesomething", "doifsomethingelse",
        "doifvaluenothing", "doifvaluesomething",
        "doifelsevaluenothing", "doifvaluenothingelse",
        "doifelsedimension", "doifdimensionelse",
        "doifelsenumber", "doifnumberelse", "doifnumber", "doifnotnumber",
        "doifelsecommon", "doifcommonelse", "doifcommon", "doifnotcommon",
        "doifinstring", "doifnotinstring", "doifelseinstring", "doifinstringelse",
        "doifelseassignment", "doifassignmentelse", "docheckassignment", "doifelseassignmentcs", "doifassignmentelsecs",
        "validassignment", "novalidassignment",
        "doiftext", "doifelsetext", "doiftextelse", "doifnottext", "validtext",
        --
        "quitcondition", "truecondition", "falsecondition",
        --
        "tracingall", "tracingnone", "loggingall", "tracingcatcodes", "tracingprocess", "tracingtypesetting",

        "showluatokens",
        --
        "aliasmacro",
        --
        "removetoks", "appendtoks", "prependtoks", "appendtotoks", "prependtotoks", "to",
        --
        -- "everyendpar",
        --
        "endgraf", "endpar", "reseteverypar", "finishpar", "null", "space", "quad", "enspace", "emspace", "charspace", "nbsp", "crlf",
        "obeyspaces", "obeylines", "obeytabs", "obeypages", "obeyedspace", "obeyedline", "obeyedtab", "obeyedpage",
        "normalspace", "naturalspace", "controlspace", "normalspaces",
        "ignoretabs", "ignorelines", "ignorepages", "ignoreeofs", "setcontrolspaces",
        --
        "executeifdefined",
        --
        "singleexpandafter", "doubleexpandafter", "tripleexpandafter",
        --
        "dontleavehmode", "removelastspace", "removeunwantedspaces", "keepunwantedspaces",
        "removepunctuation", "ignoreparskip", "forcestrutdepth", "onlynonbreakablespace",
        --
        "wait", "writestatus", "writeline", "define",
        "defineexpandable", "redefine",
        --
        "setmeasure", "setemeasure", "setgmeasure", "setxmeasure", "definemeasure", "freezemeasure",
        "measure", "measured", "directmeasure",
        "setquantity", "setequantity", "setgquantity", "setxquantity", "definequantity", "freezequantity",
        "quantity", "quantitied", "directquantity",
     -- "quantified",
        --
        "installcorenamespace",
        --
        "getvalue", "getuvalue", "setvalue", "setevalue", "setgvalue", "setxvalue", "letvalue", "letgvalue",
        "resetvalue", "undefinevalue", "ignorevalue",
        "setuvalue", "setuevalue", "setugvalue", "setuxvalue",
        -- glet
        "globallet", "udef", "ugdef", "uedef", "uxdef", "checked", "unique",
        --
        "getparameters", "geteparameters", "getgparameters", "getxparameters", "forgetparameters", "copyparameters",
        --
        "getdummyparameters", "dummyparameter", "directdummyparameter", "setdummyparameter", "letdummyparameter",
        "setexpandeddummyparameter", "resetdummyparameter",
        "usedummystyleandcolor", "usedummystyleparameter", "usedummycolorparameter", "usedummylanguageparameter",
        --
        "processcommalist", "processcommacommand", "quitcommalist", "quitprevcommalist",
        "processaction", "processallactions", "processfirstactioninset", "processallactionsinset",
        --
        "unexpanded",
     -- "expanded",
        "startexpanded", "stopexpanded", "protect", "unprotect",
        --
        "firstofoneargument",
        "firstoftwoarguments", "secondoftwoarguments",
        "firstofthreearguments", "secondofthreearguments", "thirdofthreearguments",
        "firstoffourarguments", "secondoffourarguments", "thirdoffourarguments", "fourthoffourarguments",
        "firstoffivearguments", "secondoffivearguments", "thirdoffivearguments", "fourthoffivearguments", "fifthoffivearguments",
        "firstofsixarguments", "secondofsixarguments", "thirdofsixarguments", "fourthofsixarguments", "fifthofsixarguments", "sixthofsixarguments",
        --
        "firstofoneunexpanded",
        "firstoftwounexpanded", "secondoftwounexpanded",
        "firstofthreeunexpanded", "secondofthreeunexpanded", "thirdofthreeunexpanded",
        --
        "gobbleoneargument", "gobbletwoarguments", "gobblethreearguments", "gobblefourarguments", "gobblefivearguments", "gobblesixarguments", "gobblesevenarguments", "gobbleeightarguments", "gobbleninearguments", "gobbletenarguments",
        "gobbleoneoptional", "gobbletwooptionals", "gobblethreeoptionals", "gobblefouroptionals", "gobblefiveoptionals",
        --
        "dorecurse", "doloop", "exitloop", "dostepwiserecurse", "recurselevel", "recursedepth", "dofastloopcs", "fastloopindex", "fastloopfinal", "dowith",
        "doloopovermatch", "doloopovermatched", "doloopoverlist",
        --
        "newconstant", "setnewconstant", "setconstant", "setconstantvalue",
        "newconditional", "settrue", "setfalse", "settruevalue", "setfalsevalue", "setconditional",
        --
        "newmacro", "setnewmacro", "newfraction", "newsignal", "newboundary",
        --
        "dosingleempty", "dodoubleempty", "dotripleempty", "doquadrupleempty", "doquintupleempty", "dosixtupleempty", "doseventupleempty",
        "dosingleargument", "dodoubleargument", "dotripleargument", "doquadrupleargument", "doquintupleargument", "dosixtupleargument", "doseventupleargument",
        "dosinglegroupempty", "dodoublegroupempty", "dotriplegroupempty", "doquadruplegroupempty", "doquintuplegroupempty",
        "permitspacesbetweengroups", "dontpermitspacesbetweengroups",
        --
        "nopdfcompression", "maximumpdfcompression", "normalpdfcompression", "onlypdfobjectcompression", "nopdfobjectcompression",
        --
        "modulonumber", "dividenumber",
        --
        "getfirstcharacter", "doifelsefirstchar", "doiffirstcharelse",
        --
        "mathclassvalue",
        --
        "startnointerference", "stopnointerference",
        --
        "twodigits","threedigits",
        --
        "jobposx", "jobposy", "jobposw", "jobposh", "jobposd",
        --
        "leftorright",
        --
        "offinterlineskip", "oninterlineskip", "nointerlineskip",
        --
        "strut", "halfstrut", "quarterstrut", "depthstrut", "halflinestrut", "noheightstrut", "setstrut", "strutbox", "strutht", "strutdp", "strutwd", "struthtdp", "strutgap", "begstrut", "endstrut", "lineheight",
        "leftboundary", "rightboundary", "signalcharacter", "signalglyph",
        "ascender", "descender", "capheight",
        --
        "aligncontentleft", "aligncontentmiddle", "aligncontentright",
        --
        "shiftbox", "vpackbox", "hpackbox", "vpackedbox", "hpackedbox",
        --
        "vreflected",
        --
     -- "ordordspacing", "ordopspacing", "ordbinspacing", "ordrelspacing",
     -- "ordopenspacing", "ordclosespacing", "ordpunctspacing", "ordinnerspacing",
     -- "ordfracspacing", "ordradspacing", "ordmiddlespacing", "ordaccentspacing",
     -- --
     -- "opordspacing", "opopspacing", "opbinspacing", "oprelspacing",
     -- "opopenspacing", "opclosespacing", "oppunctspacing", "opinnerspacing",
     -- "opfracspacing", "opradspacing", "opmiddlespacing", "opaccentspacing",
     -- --
     -- "binordspacing", "binopspacing", "binbinspacing", "binrelspacing",
     -- "binopenspacing", "binclosespacing", "binpunctspacing", "bininnerspacing",
     -- "binfracspacing", "binradspacing", "binmiddlespacing", "binaccentspacing",
     -- --
     -- "relordspacing", "relopspacing", "relbinspacing", "relrelspacing",
     -- "relopenspacing", "relclosespacing", "relpunctspacing", "relinnerspacing",
     -- "relfracspacing", "relradspacing", "relmiddlespacing", "relaccentspacing",
     -- --
     -- "openordspacing", "openopspacing", "openbinspacing", "openrelspacing",
     -- "openopenspacing", "openclosespacing", "openpunctspacing", "openinnerspacing",
     -- "openfracspacing", "openradspacing", "openmiddlespacing", "openaccentspacing",
     -- --
     -- "closeordspacing", "closeopspacing", "closebinspacing", "closerelspacing",
     -- "closeopenspacing", "closeclosespacing", "closepunctspacing", "closeinnerspacing",
     -- "closefracspacing", "closeradspacing", "closemiddlespacing", "closeaccentspacing",
     -- --
     -- "punctordspacing", "punctopspacing", "punctbinspacing", "punctrelspacing",
     -- "punctopenspacing", "punctclosespacing", "punctpunctspacing", "punctinnerspacing",
     -- "punctfracspacing", "punctradspacing", "punctmiddlespacing", "punctaccentspacing",
     -- --
     -- "innerordspacing", "inneropspacing", "innerbinspacing", "innerrelspacing",
     -- "inneropenspacing", "innerclosespacing", "innerpunctspacing", "innerinnerspacing",
     -- "innerfracspacing", "innerradspacing", "innermiddlespacing", "inneraccentspacing",
     -- --
     -- "fracordspacing", "fracopspacing", "fracbinspacing", "fracrelspacing",
     -- "fracopenspacing", "fracclosespacing", "fracpunctspacing", "fracinnerspacing",
     -- "fracfracspacing", "fracradspacing", "fracmiddlespacing", "fracaccentspacing",
     -- --
     -- "radordspacing", "radopspacing", "radbinspacing", "radrelspacing",
     -- "radopenspacing", "radclosespacing", "radpunctspacing", "radinnerspacing",
     -- "radfracspacing", "radradspacing", "radmiddlespacing", "radaccentspacing",
     -- --
     -- "middleordspacing", "middleopspacing", "middlebinspacing", "middlerelspacing",
     -- "middleopenspacing", "middleclosespacing", "middlepunctspacing", "middleinnerspacing",
     -- "middlefracspacing", "middleradspacing", "middlemiddlespacing", "middleaccentspacing",
     -- --
     -- "accentordspacing", "accentopspacing", "accentbinspacing", "accentrelspacing",
     -- "accentopenspacing", "accentclosespacing", "accentpunctspacing", "accentinnerspacing",
     -- "accentfracspacing", "accentradspacing", "accentmiddlespacing", "accentaccentspacing",
        --
        "normalreqno",
        --
        "startimath", "stopimath", "normalstartimath", "normalstopimath",
        "startdmath", "stopdmath", "normalstartdmath", "normalstopdmath",
        --
        "uncramped", "cramped",
        "mathstyletrigger", "triggermathstyle", "triggeredmathstyle",
        "mathstylefont", "mathsmallstylefont", "mathstyleface", "mathsmallstyleface", "mathstylecommand", "mathpalette",
        "mathstylehbox", "mathstylevbox", "mathstylevcenter", "mathstylevcenteredhbox", "mathstylevcenteredvbox",
        "mathtext", "setmathsmalltextbox", "setmathtextbox",
        "pushmathstyle", "popmathstyle",
        --
        "triggerdisplaystyle", "triggertextstyle", "triggerscriptstyle", "triggerscriptscriptstyle",
        "triggeruncrampedstyle", "triggercrampedstyle",
        "triggersmallstyle", "triggeruncrampedsmallstyle", "triggercrampedsmallstyle",
        "triggerbigstyle", "triggeruncrampedbigstyle", "triggercrampedbigstyle",
        --
        "luaexpr",
        "expelsedoif", "expdoif", "expdoifnot",
        "expdoifelsecommon", "expdoifcommonelse",
        "expdoifelseinset", "expdoifinsetelse",
        --
        "glyphscaled", -- will change
        --
        "ctxdirectlua", "ctxlatelua", "ctxsprint", "ctxwrite", "ctxcommand", "ctxdirectcommand", "ctxlatecommand", "ctxreport",
        "ctxlua", "luacode", "lateluacode", "directluacode",
        "registerctxluafile", "ctxloadluafile",
        "luaversion", "luamajorversion", "luaminorversion",
        "ctxluacode", "luaconditional", "luaexpanded", "ctxluamatch", "ctxluamatchfile",
        "startluaparameterset", "stopluaparameterset", "luaparameterset",
        "definenamedlua",
        "obeylualines", "obeyluatokens",
        "startluacode", "stopluacode", "startlua", "stoplua",
        "startctxfunction","stopctxfunction","ctxfunction",
        "startctxfunctiondefinition","stopctxfunctiondefinition",
        "installctxfunction", "installprotectedctxfunction",  "installprotectedctxscanner", "installctxscanner", "resetctxscanner",
        "cldprocessfile", "cldloadfile", "cldloadviafile", "cldcontext", "cldcommand",
        --
        "carryoverpar",
        "freezeparagraphproperties", "defrostparagraphproperties",
        "setparagraphfreezing", "forgetparagraphfreezing",
        "updateparagraphproperties", "updateparagraphpenalties", "updateparagraphdemerits", "updateparagraphshapes", "updateparagraphlines",
        "updateparagraphpasses",
        --
        "assumelongusagecs",
        --
        "righttolefthbox", "lefttorighthbox", "righttoleftvbox", "lefttorightvbox", "righttoleftvtop", "lefttorightvtop",
        "rtlhbox", "ltrhbox", "rtlvbox", "ltrvbox", "rtlvtop", "ltrvtop",
        "autodirhbox", "autodirvbox", "autodirvtop",
        "leftorrighthbox", "leftorrightvbox", "leftorrightvtop",
        "lefttoright", "righttoleft", "checkedlefttoright", "checkedrighttoleft",
        "foolbidimode",
        "synchronizelayoutdirection","synchronizedisplaydirection","synchronizeinlinedirection",
        "dirlre", "dirrle", "dirlro", "dirrlo",
        "rtltext", "ltrtext",
        --
        "lesshyphens", "morehyphens", "nohyphens", "dohyphens", "dohyphencollapsing", "nohyphencollapsing",
        "compounddiscretionary",
        "doapostrophes", "noapostrophes",
        --
        "Ucheckedstartdisplaymath", "Ucheckedstopdisplaymath",
        --
        "break", "nobreak", "allowbreak", "goodbreak",
        --
        "nospace", "nospacing", "dospacing",
        --
        "naturalhbox", "naturalvbox", "naturalvtop", "naturalhpack", "naturalvpack", "naturaltpack",
        "reversehbox", "reversevbox", "reversevtop", "reversehpack", "reversevpack", "reversetpack",
        --
        "hcontainer", "vcontainer", "tcontainer",
        --
        "frule",
        --
        "compoundhyphenpenalty",
        --
        "start", "stop",
        --
        "unsupportedcs",
        --
        "openout", "closeout", "write", "openin", "closein", "read", "readline", "readlinedirect", "readfromterminal",
        --
        "boxlines", "boxline", "setboxline", "copyboxline",
        "boxlinewd","boxlineht", "boxlinedp",
        "boxlinenw","boxlinenh", "boxlinend",
        "boxlinels", "boxliners", "boxlinelh", "boxlinerh",
        "boxlinelp", "boxlinerp", "boxlinein",
        "boxrangewd", "boxrangeht", "boxrangedp",
        "boxlinemaxwd","boxlinemaxht", "boxlinemaxdp",
        --
        "bitwiseset", "bitwiseand", "bitwiseor", "bitwisexor", "bitwisenot", "bitwisenil",
        "ifbitwiseand", "bitwise", "bitwiseshift", "bitwiseflip",
        -- old ... very low level
        "textdir", "linedir", "pardir", "boxdir",
        --
        "prelistbox", "postlistbox", "prelistcopy", "postlistcopy", "setprelistbox", "setpostlistbox",
        --
        "noligaturing", "nokerning", "noexpansion", "noprotrusion",
        "noleftkerning", "noleftligaturing", "norightkerning", "norightligaturing", "noitaliccorrection",
        "overloadspacefactor",
         --
        "futureletnexttoken", "defbackslashbreak", "letbackslashbreak",
        --
        "pushoverloadmode", "popoverloadmode", "pushrunstate", "poprunstate",
        --
        "suggestedalias",
        --
        "showboxhere",
        --
        "discoptioncodestring", "flagcodestring", "frozenparcodestring", "glyphoptioncodestring", "groupcodestring",
        "hyphenationcodestring", "mathcontrolcodestring", "mathflattencodestring", "normalizecodestring",
        "parcontextcodestring",
        --
        "newlocalcount", "newlocaldimen", "newlocalskip", "newlocalmuskip", "newlocaltoks", "newlocalbox",
        "newlocalwrite", "newlocalread", "newlocalfloat",
        "setnewlocalcount", "setnewlocaldimen", "setnewlocalskip", "setnewlocalmuskip", "setnewlocaltoks", "setnewlocalbox",
        --
        "ifexpression",
        --
        "localcontrolledrepeating", "expandedrepeating", "unexpandedrepeating",
        --
        "lastchkinteger", "ifchkinteger",
        --
     -- "mathopen", "mathclose", "mathinner",
        "mathordinary", "mathoperator", "mathbinary", "mathrelation", "mathpunctuation", "mathfraction",
        "mathradical", "mathmiddle", "mathaccent", "mathfenced", "mathghost", "mathvariable", "mathactive",
        "mathvcenter", "mathimaginary", "mathdifferential", "mathexponential", "mathdigit", "mathdivision",
        "mathfactorial", "mathwrapped", "mathconstruct", "mathdimension", "mathunary", "mathchemicalbond",
        "mathimplication",
     -- "mathfunction", "mathexplicit", "mathbegin", "mathend",
        --
        "filebasename", "filenameonly", "filedirname", "filesuffix",
        --
        "setmathoption", "resetmathoption",
        --
        "Ustack", "Umathdict", "Umathclass",
        "Ustyle", "Uchar",
        "Usuperscript", "Usubscript", "Unosuperscript", "Unosubscript", "Uprimescript",
        "Usuperprescript", "Usubprescript", "Unosuperprescript", "Unosubprescript",
        --
        "ignorefile",
        --
        "boxwidth", "boxheight", "boxdepth",
        --
        "shiftparshape", "rotateparshape",
        --
        "granularfitnessclasses", "granularadjacentdemerits",
        "matchallfitnessclasses",
        --
        "defaultmathforwardpenalties", "defaultmathbackwardpenalties",
        "optimalmathforwardpenalties", "optimalmathbackwardpenalties",
        "lesswidowpenalties", "lessclubpenalties", "lessbrokenpenalties",
        "strictwidowpenalties", "strictwidowpenaltiestwo", "strictwidowpenaltiesthree", "strictwidowpenaltiesfour",
        "strictclubpenalties", "strictclubpenaltiestwo", "strictclubpenaltiesthree", "strictclubpenaltiesfour",
        "strictbrokenpenalties",
        "lessorphanpenalties", "lessorphanpenaltiestwo", "lessorphanpenaltiesthree","lessorphanpenaltiesfour",
        --
        "nohpenalties", "novpenalties",
        --
        "toddlerpenalty", "orphanpenalty",
        --
        "digitspace",
        --
        "inhibitprimitive",
        --
        "thespecification",
    }
}
