return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["n"]=0,
 },
 ["metadata"]={
  ["mnemonic"]="te",
  ["source"]="hyph-te",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="ఁంఃఅఆఇఈఉఊఋఌఎఏఐఒఓఔకఖగఘఙచఛజఝఞటఠడఢణతథదధనపఫబభమయరఱలళవశషసహాిీుూృౄెేైొోౌ్ౕౖౠౡ‌‍",
  ["data"]="2‍2 1‌1 అ1 ఆ1 ఇ1 ఈ1 ఉ1 ఊ1 ఋ1 ౠ1 ఌ1 ౡ1 ఎ1 ఏ1 ఐ1 ఒ1 ఓ1 ఔ1 ా1 ి1 ీ1 ు1 ూ1 ృ1 ౄ1 ె1 ే1 ై1 ొ1 ో1 ౌ1 1క 1ఖ 1గ 1ఘ 1ఙ 1చ 1ఛ 1జ 1ఝ 1ఞ 1ట 1ఠ 1డ 1ఢ 1ణ 1త 1థ 1ద 1ధ 1న 1ప 1ఫ 1బ 1భ 1మ 1య 1ర 1ఱ 1ల 1ళ 1వ 1శ 1ష 1స 1హ 2ఁ1 2ం1 2ః1 2ౕ1 2ౖ1 2్2",
  ["lefthyphenmin"]=1,
  ["length"]=367,
  ["n"]=72,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}