if not modules then modules = { } end modules ['node-bwc'] = {
    version   = 1.001,
    comment   = "companion to node-ini.mkiv",
    author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
    copyright = "PRAGMA ADE / ConTeXt Development Team",
    license   = "see context related readme files"
}

-- This is a backward compatibility list.

local nodes  = nodes
local nuts   = nodes.nuts

nuts .check_discretionaries   = nuts .checkdiscretionaries
nuts .copy_list               = nuts .copylist
nuts .copy_node               = nuts .copynode
nuts .copy_only               = nuts .copyonly
nuts .current_attr            = nuts .currentattributes
nuts .effective_glue          = nuts .effectiveglue
nuts .end_of_math             = nuts .endofmath
nuts .find_attribute          = nuts .findattribute
nuts .first_glyph             = nuts .firstglyph
nuts .flatten_discretionaries = nuts .flattendiscretionaries
nuts .flush_list              = nuts .flushlist
nuts .flush_node              = nuts .flushnode
nuts .get_attribute           = nuts .getattribute
nuts .get_synctex_fields      = nuts .getsynctexfields
nuts .has_attribute           = nuts .hasattribute
nuts .has_dimensions          = nuts .hasdimensions
nuts .has_field               = nuts .hasfield
nuts .has_glyph               = nuts .hasglyph
nuts .insert_after            = nuts .insertafter
nuts .insert_before           = nuts .insertbefore
nuts .is_direct               = nuts .isdirect
nuts .is_node                 = nuts .isnode
nuts .is_nut                  = nuts .isnut
nuts .is_zero_glue            = nuts .iszeroglue
nuts .mlist_to_hlist          = nuts .mlisttohlist
nuts .protect_glyph           = nuts .protectglyph
nuts .protect_glyphs          = nuts .protectglyphs
-----.protrusion_skippable    = nuts .protrusionskippable
nuts .set_attribute           = nuts .setattribute
-----.set_synctex_fields      = nuts .setsynctexfields
nuts .start_of_par            = nuts .startofpar
nuts .traverse_id             = nuts .traverseid
nuts .traverse_char           = nuts .traversechar
nuts .traverse_glyph          = nuts .traverseglyph
nuts .traverse_id             = nuts .traverseid
nuts .traverse_list           = nuts .traverselist
nuts .unprotect_glyph         = nuts .unprotectglyph
nuts .unprotect_glyphs        = nuts .unprotectglyphs
nuts .unset_attribute         = nuts .unsetattribute
nuts .uses_font               = nuts .usesfont
nuts .getattr                 = nuts .getattribute
nuts .getattrs                = nuts .getattributes
nuts .getattrlist             = nuts .getattributelist

-----.check_discretionaries   = nodes.checkdiscretionaries
nodes.copy_list               = nodes.copylist
nodes.copy_node               = nodes.copynode
nodes.current_attributes      = nodes.currentattributes
nodes.effective_glue          = nodes.effectiveglue
nodes.end_of_math             = nodes.endofmath
nodes.find_attribute          = nodes.findattribute
nodes.first_glyph             = nodes.firstglyph
-----.flatten_discretionaries = nodes.flattendiscretionaries
nodes.flush_list              = nodes.flushlist
nodes.flush_node              = nodes.flushnode
nodes.has_attribute           = nodes.hasattribute
nodes.has_field               = nodes.hasfield
nodes.has_glyph               = nodes.hasglyph
nodes.insert_after            = nodes.insertafter
nodes.insert_before           = nodes.insertbefore
nodes.is_node                 = nodes.isnode
nodes.is_direct               = nodes.isdirect
nodes.is_nut                  = nodes.isnut
nodes.is_zero_glue            = nodes.iszeroglue
nodes.last_node               = nodes.lastnode
nodes.mlist_to_hlist          = nodes.mlisttohlist
nodes.protect_glyph           = nodes.protectglyph
nodes.protect_glyphs          = nodes.protectglyphs
nodes.protrusion_skippable    = nodes.protrusionskippable
nodes.set_attribute           = nodes.setattribute
nodes.traverse_id             = nodes.traverseid
nodes.traverse_char           = nodes.traversechar
nodes.traverse_glyph          = nodes.traverseglyph
nodes.traverse_id             = nodes.traverseid
nodes.traverse_list           = nodes.traverselist
nodes.unprotect_glyph         = nodes.unprotectglyph
nodes.unprotect_glyphs        = nodes.unprotectglyphs
nodes.unset_attribute         = nodes.unsetattribute
nodes.uses_font               = nodes.usesfont
