return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["n"]=0,
 },
 ["metadata"]={
  ["mnemonic"]="lt",
  ["source"]="hyph-lt",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="abcdefghijklmnoprstuvwyząčėęįšūųž",
  ["data"]=".ap1 .api1 .ap3r .arbi1 .arti1 .as3p .at1 .ata1 .at3r .aš3v .dina1 .ek3r .i2š3 .iši2 .kirti1 .nu1 .nusi1 .pie2č .sam1 .sida1 .sk4 .st4 .su5kr .te3s2 .uk3 .u2š3 .už1 .įs4 .šven1 a1a2 a1b a1c a2ch a1d a4dra a1e ae2l a1f 2a1g a4gr ag3ra a5grio a5gro a1h a5inf ai4sk ai4tr a1j a1k ak2l ak2vi a2kėt a1l a3li a5lo1 a3ly a3lė a3lū a1m a1n an3k2l an4sk an4tr a2o a1p ap1a4k ap3ei ap3i2m ap3l a3pr ap3s2 a1r ar1eit a4rg ari4s a1s asi1 asi5s a4sk as3kl a3sl as2mi as2mu a5s4n a4stu a1t at3aug ate5ist at3i2m ato1 atp4 a4tru at1ė2 atū2ž a4u au4kl au4sk au4sl au4t3r a1v a1w a1y a1z a1ą 2a1č a1ę a1ė2 a1į a1š a3šn aš4tr aš2v a1ų a1ū2 a1ž až2l ba3c balta1 ba4sl 2b1b 2b1c 4b3d bep4 be3s2 besi1 be3t2 2b1f 2b1g 2b1h b1j 2b1k 2b1l b2la b2liz 3b2lo b2lu 2b1m 2b1n 2b1p 2b2r b3ri 2b1s 2b3t bu4k 2b1v 2b1w 2b1z 2b1č 2b1š 2b1ž car4 2c1b 2c1c 2c1d 2c1f 2c1g 3chi 2c1k 2c1l 2c1m 2c1n 2c1p 2c1r 2c1s 2c1t cu4 cuk5 2c1v 2c1w 2c1z 2c1č 2c1š 2c1ž da3b4 2d1b 2d1c 2d1d de4k dema1 de4pr de4sp 2d1f 2d1g 2d1h di4p di4s dis5k 2d1j 2d3k 4d3l 2d1m 2d1n do3ri 2d1p 2d1r dro1 dro2b d2rė 4drų 2d1s 2d1t du2a du4k du5ka du4sl 2d1v d3va dvi3a 2d1w d4z 2d1č 2d1š d4ž džio1 e1a2 e2a3l eapi1 eat1 e1b2 ebe1 e3bl eb3r e1c e2ch e3d2 ed3ri e4dro ed3rė e1e e1f e1g eg3ra e1h e1ie ei4k3l 1ein ei4sk ei4sl 2eis3t eist2ra ei2š1 e1j e1k e3kr e1l e1m ema5s4 2e1n en3k2l enk4la eno1 ens4 4enta enu1 e2o e3or e3o2š e3p ep4li ep4r epra1 epri1 e1r ere3a4 eri4s ero1 erė2 e1s esi5s es4k eska1 e5sko e3s2v e1t e1u4 euž3 e1v e3vi e1w e1y e1z e1ą e1č e1ę e1ė e1į4 eįp3 eįs3k eįt3r e1š e3šn eš2v ešė3 e1ų e1ū e1ž 2f1b 2f1c 2f1d 2f1f 2f1g 2f1h fi4s5 2f1k 2f1l 2f1m 2f1n 2f1p 2f1r fri1 2f1s 2f1t 2f1v 2f1w 2f1z 2f1č 2f1š 2f1ž gaš3 2g1b 2g1c 2g1d ge4o1 2g1f 2g1g 2g1h 2g1k 2g1l g2le g2lo 2g1m 2g1n 3g2nų 2g1p2 3g2r grai2 g3ran 5g3re g4rei g3ri 4g4rio g3ro. g4rą 5grį 4grų 2g1s 4g1t 3gu 2g1v 2g1w gyva1 2g1z 2g1č 2g1š 2g1ž 2h1b 2h1c 2h1d 2h1f 2h1g 2h1h hi4b 2h1k 2h2l 2h1m h2me 2h1n 2h1p 2h1r 2h1s 2h1t 2h1v 2h1w 2h1z 2h1č 2h1š 2h1ž i2a ia5g4 i3aiš ia3k i3antę i3antė iap4 i3ar i3b2 i1c ice1 i1d id2r id3rė i2dėm i2e ie4d3r ie3g i3ei ie3kl i3ent ie4p5r ie4sk i1f 2i1g i3g2l ig3ru 2i1h i1i2 i1j i1k i3k2n ik3r i1l ilo1 i1m 1imd 4i1n in4kl 5inv i2o io4g3r iok2 io4pl i3or i1p i2p1j ip3ru i1r ira3s2 i3ri 4i1s i5sa i5si is4k i3sl is4li i5s4n i4ste isto1 2i3t2 i2u i1v i1w i1y i1z i2ą i1č i1ę i1ė2 i1į i1š i3šn iš5t i4šv išė2 1išš i2ų i2ū i3ž 3ja. jauna1 2j1b 2j1c 2j1d 2j1f 2j1g 2j1h 2j1j 2j1k 2j1l 2j1m 2j1n jot3 jo4tv 2j1p 2j1r 2j1s 2j1t 1ju 2j1v 2j1w 2j1z 2j1č 2j1š 1jū 2j1ž 3ka. 5ka3d kak3 ka4kl ka4pr 3kar 3kas 3kati 5kav kavar1 ka3z2 2k1b 2k1c 2k1d 3ke ke4b3 keren1 2k1f 2k1g 2k1h k4i 5ki. 5kia 3kib 3kil 5kit 2k1k 2k1l 3k2la k4lan 4kle. k2lel 4klo. 4klu. k2ly 4klą 4klų 2k1m 2k1n 3ko 2k1p 2k2r k4rau kris2 kri5st k3ro k4rov 4k3ru 5krun k4ry k5rą 2k1s k4sk ks3l k4s3p 4k1t k3ta 3ku 4kub ku4k ku4pr 2k1v k2va k3vo k2vė 2k1w 3kyt 2k1z 5ką 2k1č 2k1š kš2ly kšė3 2k1ž 3la. lap4s3to 2l1b lb3r 2l1c 2l1d 3le. le3c le4gr 4leč 2l1f 2l1g lg3s2t 2l1h 3li. li4a 5li4o 3lių 2l1j 2l1k lko1 2l1l 2l1m 2l1n 3lo. log4 4lop 4l1p lpna1 2l1r 4l1s l4sk l4s3p 4l3t 3lu. 2lup 4l1v 2l1w 3lyč 2l1z 3lą 2l1č 3lęs 3lėm 3lės 2lėč 3lį4 4l1š 3lų 2l1ž m2a 2m3aid 3mas mas3ki 2m1b mb3r 2m1c 2m1d me3c meis1 2m1f 2m1g 2m1h mi4gl mi4gr min4s mi4tr 2m1k 2m1l 2m1m 2m1n m2o 2mod 2m1p m4pl m3pr 2m1r 4m1s 2m1t m3ta 3muo 2m1v 2m1w 2m1z 2m1č 2m1š 2m1ž na3s2 na3t2 2n1b 2n1c ncen1 2n1d n3drė ne1 neg4 nei2m ne4o3 neor2 nerė3 nesi1 ne3sl ne3s2t ne3t2 neįs3t 2n1f 4n1g ng3l ng4r 2n1h ni4s 2n1j 2n1k nk3la nk3r n3kry 2n1l 2n1m 2n1n 5no 2n1p 2n1r 4n1s ns4ku ns3l n4s3p nst3r 4n1t n3ta nt4pl n3tru nt2ruo nua4 nu3b2 nu3g nu1i nuk2 nu4o3 nuos2 nu5s4 2n1v 2n1w 2n1z 4n1č 2n1š 2n1ž o1a o1b o1c o1d o3dr o1e o2et o1f o1g o1h o1i o1j o1k o3kr o1l olen1 o1m om4pr o1n ono1 o1o2 o1p o1r o2rie ori4s or4tr o2rę o1s o3sl os3le os4lo o3s2v o3t2 oto1 o1v o1w o1y o1z o1ą o1č o1ę o1ė o1į o1š oš2v o1ų o1ū o1ž pa3 pad2 pai2l pai2m pai2r 3pan pap4 p2ar parsi1 pa4r1ė2 pas2 pa5sr p2at2 2p1b 4p1c 4p3d2 p2e per1 pe2re pe2r3im per3s pe2rė 2p1f 2p1g 2p1h p2i p3ieš 2p3k2 3p2l p3le p3li 4plio p4liu p3lo p4loj p4lu p4ly 2p1m 4p1n 3po po4g poli1 2p1p 2p2r p3rai prau2si p3raš p3ri pris2 p5ro. p3rom p3ry 4prą p3rė p3rū 4p1s psi1 p5s4k p4s3ty p3s2v 4p3t2 3p4u pusiau1 pu4sk pu4sl pu4tr 2p1v 2p1w p2y 2p1z 2p1č p2ė 4p3š2 4p3ž r2a 3ra. ra3b 3rac ra1im rai4tį ra3kr 3ral 3ram ras4l ra3sm rau4ka. 3rav 2r1b rbo1 r3b4r 2r1c 4r1d 2reb 4rein 4reit 3rel re4p5 5res res3l 3ret 2r1f 2r1g 2r1h r2i 3ri. ri3d 3rij ri3kr 2ril ri2ma. 2r3imt 3rio ri3p risi1 ri3st rivin1 3rią 3rių 2riū 4r1k rk3ly r3k4r 4r1l 2r1m 4r1n r2o 3rod2 ro4gr 3roj 3rok 3ron 3rop 2r3org 5ros 2r1p r3p4r 2r1r 4r1s rs4ko. rs4p 4r1t rti5k4 rti3s2 r3t2r r3t2v r2u 2ruk 3rul 4run 3ruos 2rup 3rus ru4sk 4ruto 4r1v 2r1w 2r1z 3rą 4rąs 4r1č 4r1š r3š2l r3š2m 5rų 2r1ž sala1 2s1amž san5t sarka1 2s3b 2s1c 2s3d s2e se4kr senat4 2s1f 2s1g 2s1h si3auk si3a2v si3a2š si3d si3k4 si3p4 si3s2 si5š2v 3s2k2 4sk. s5ka. 4ske 5s4ken 5ski 5s4kle 5sk4r 5skub sk3va sk3vi 5sky 4ską 5skę 2s2l 3sle s3li s3lo 4s3lu 4s3lū 2s1m 4sme. 4smę 4s3n so4dr 3s2p s3pe 4spn 4spu 4s1r s3ri 2s1s 2s1t s2tal s2ten 4stin s2tod s2toj 3s2tov st2rai st2v 4stę 4s3tė 4stų s2tū su1 su3bl su3d2 su3g2 suk2l su3s2 susi1 suž4 2s1v s2ve 3s2vy 2s1w 2s1z są3 4s3č 5sė 2s1š 2s1ž t2a 2tab ta3kr ta5s 2tat taura1 2tač 2t1b 2t1c 2t1d 2teb 3tem te4o te3t2 2t1f 4t3g 2t1h t2i ti4gr ti4k3l 3tin 2tip 4t3j 4t3k t4kl 4t3l 4t3m 2t1n t2o 3toje 2tolį to3s2 2tow 2t3p t4pj tp3lū tp4r 2t1r 4trio t2riš 4tro 4trą 4trų 4t3s4 tsi1 tskri1 4t1t tua4 3tur 2t1v t2vo 4tvėj 2t1w 3tyd 2t1z 2t1č 3tę t2ėm t3ėmu t3ėmę t3ėmė 2t3š2 2t3ž u1a2 u3ai u1b u2bj u1c u1d u1e2 u1f u1g ug4r ug5rio u1h u1i2m u5in ui2r u1j u1k u3kl uk2le u3kr u3kv u5kų u1l u1m u1n u2o u3or uo4sl u3p up4l up3ro u1r urk2 ur3kl u5ro1 ur3s2 u1s u2s1al u3sl us3la us3le usva1 us3ve u3t2 uto1 2utr u1u u1v u1w u1y u1z u1ą u1č u1ę u1ė2 u1į u1š u3š2l u3š2n uš2v u1ų u1ū2 u1ž uži2m u3žl už3v už1ė2 3va. vap4 va3t 2v1b 2v1c 2v1d v2e 2vep 3ves 2v1f 2v1g 2v1h viesia1 vi4s5k vi4t3r 2v1j 2v1k 2v1l 2v1m 2v1n 2v1p 2v1r 2v4s 2v1t 2v1v 2v1w 2vydau 2v1z 3vą 2v1č 3vė 4vėp 2v1š 2v1ž 2w1b 2w1c 2w1d 2w1f 2w1g 2w1h 2w1k 2w1l 2w1m 2w1n 2w1p 2w1r 2w1s 2w1t 2w1v 2w1w 2w1z 2w1č 2w1š 2w1ž y1a y1b y1c y1d y1e y1f y1g ygia1 y1h y1i y1j y1k y4k3l y1l y1m y1n y1o y1p y3r y1s y4sk y1t y1u y1v y1w y1y y1z y1ą y1č y1ę y1ė y1į y1š y1ų y1ū y1ž 2z1b 2z1c 2z1d 2z1f 2z1g 2z1h 2z1k 2z1l 2z1m 2z1n 2z1p 2z1r 2z1s 2z1t 2z1v 2z1w 2z1z 2z1č 2z1š 2z1ž ą1a ą1b ą1c ą1d ą1e ą1f ą1g ą1h ą1i ą1j ą1k ą1l ą1m ą1n ą1o ą1p ą1r ą1s ą1t ą1u ą1v ą1w ą1y ą1z ą1ą ą1č ą1ę ą1ė ą1į ą1š ą1ų ą1ū ą1ž 2č1b 2č1c 2č1d čeko1 2č1f 2č1g 2č1h čin1 2č1k 2č1l 2č1m 2č1n 2č1p 2č1r 2č1s 2č1t 2č1v 2č1w 2č1z 2č1č 2č1š 2č1ž ę1a ę1b ę1c ę1d ę1e ę1f ę1g ę1h ę1i ę1j ę1k ę1l ę1m ę1n ę1o ę1p ę1r ę1s ę1t ę1u ę1v ę1w ę1y ę1z ę1ą ę1č ę1ę ę1ė ę1į ę1š ę1ų ę1ū ę1ž ė1a ė1b ė1c ė1d ė1e ė1f ė1g ė1h ė1i ė1j ė1k ė4k3l 2ė3l ė1m ė3me ė1n ė1o ė1p ė1r 2ė1s ės3l 2ė3t ė4tr ė1u ė1v ė1w ė1y ė1z ė1ą ė1č ė1ę ė1ė ė1į ė1š ė1ų ė1ū ė1ž į1a į1b į1c į1d2 į2e į1f į4g į1h į1i į1j į2k4 į2l į2m į1n į1o į4p į2r į1s įsi1 įs3l įs2m įs2r įst2 į2t2 į1u į2v į1w į1y į1z į1ą į1č į1ę į1ė2 į1į į1š į1ų į1ū į1ž 2š1b2 2š1c 2š3d2 š1ei še2v 2š1f 2š1g4 2š1h 2š5ist šiuk1 2š1k2 2š1l š2lij š2lu 3šly 2š1m 2š1n š2ne šno1 šo2r 2š1p4 2š1r 2š3s4 šsi1 šsikap1 4š3t2 šu4š 3š1v š2vi š4vyd 2š1w 2š1z 4š3č šė2j 2š1š 2š1ž ų1a ų1b ų1c ų1d ų1e ų1f ų1g ų1h ų1i ų1j ų1k ų1l ų1m ų1n ų1o ų1p ų1r ų1s ų1t ų1u ų1v ų1w ų1y ų1z ų1ą ų1č ų1ę ų1ė ų1į ų1š ų1ų ų1ū ų1ž ū1a ū1b ū1c ū1d ū1e ū1f ū1g ū1h ū1i ū1j ū1k ū4k3l ū1l ū1m ū1n ū1o ū1p ū1r ū1s ū4s3k ūs3l ūs3t ū1t ū1u ū1v ū1w ū1y ū1z ū1ą ū1č ū1ę ū1ė ū1į ū1š ū1ų ū1ū ū1ž žant4 žants5 2ž3b2 2ž1c 2ž3d2 žen1 2ž3f4 2ž3g 2ž1h ži3mu žio3 ž1j 2ž3k2 2ž1l ž2lu 4ž1m 2ž1n 2ž3p 2ž1r 2ž1s žsi1 ž4sk ž4s5l žs3t 4ž3t2 ž2u žu3s2 3ž2v ž4vi ž3vo 2ž1w 2ž1z 2ž1č žįs3 2ž1š 2ž1ž",
  ["lefthyphenmin"]=1,
  ["length"]=8416,
  ["n"]=1546,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}