return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["n"]=0,
 },
 ["metadata"]={
  ["mnemonic"]="sq",
  ["source"]="hyph-sq",
  ["texcomment"]="% \\message{Albanian Hyphenation Patterns }",
 },
 ["patterns"]={
  ["characters"]="'abcdefghijklmnopqrstuvxyzçë",
  ["data"]="2'2 .a1jo. a1a 1b .b2 2b. b2l 2bsh 1c .c2 2c. 2cj 2cn 2ct 1ç .ç2 2ç. 2çs ç2k 1d d2h .d2 2d. d2j 2dn d2r 2drr 2dt d2shm 2dh. 2dhj2dht 2dhsh 2dh2j 2dht dh2r e1a e3ll e1u ë1a 1f .f2 2f. f2l f2r 2fs 2ft 3f2sh 2f2t. 1g .g2 2g. g2j 2gj. 2gjv 2gl 2gm 2gr 2gt 1h .h2 2h. 2hd 2hj 2hm 2hn 2ht 2hrr i1a i1e i1u .i2k3i .i2k3j 1j2 .j2 2j. 2j3c2 2j3d 2j3m 2j3p 2j3r 2j3t 2j3v 2j3s 2jf. j4tp 2jt. j3sh2m 1k .k2 2k. k2j 2kl 2km 2kth. k2r 2kt 2ks 2ksh 1l .l2 2l. 2lb 2lç 2lf 2lj 2lm 2ln l3n2g 2ls 2lt l2l2 4ll. 2ll3s 4ll3z 2ll3k 4ll3gj 2ll3n 2ll3t 1m .m2 2m. m2b mb2j mb2l mb2r m2j 2m3n2d 2mt 2mr 2m3sh2 2m4sh. 2m1v 1n .n2 2n. .ng2r 2nc 2nd n2dm n2dv n2d3sh 2ng 2nk 2nsp 2nsh n3sh2m 2nt 2nv 2nx 2nz n2j 2njt 2nj. 2njv o1i 1p .p2 2p. p2j 2pn 2pt p2je. 2ps p2r pa2s3her .pe2r3af .pë2r3af 1q .q2 2q. 2qj 2qk 2qm 2qn 2qt q2v 1r .r2 2r. 2rt 2rb2 2r2b3r 2rc 2rç 2rd 2rc2rd 2rf 2rg 2rh .ri3n2d 2rk 2rl 2rm 2rn r2n3d2 2rp 2rq 2rs 2r3sh2m 2rdh r2dht 2r3dr 2rj 2rv 2rz r2r .rr2 2rr. 2rrj 2rrk 2rrm 2rrn 2rrt 2rrs 1s .s2 2s. 2sh. 2sb 2sc 2sd 2sf 2sg 2sj 2sk 2sm 2sn sn2k 2sp 2ssh 2st st2r 3s2je2ll sk2ll s2ve. s2h .sh4 2shm 3sh2mj 2shj .sh2j sh2k 2shk. sh2n shn4d sh2p .sh2q 4sh3k2r 2shq 2sh3nj 3sh4pj 3sh4pr 2shr 2shs 2sht .sh2t 3sh2te. 1t .t2 4t. 2tk t2j 2tm 2tn 2tp 2t3sh2m t2r 2tv t2h .th2 2th. 2thç 2ths 2thf 2thm 2tht u1ar. u1a u1e 1v .v2 2v. 2vr v2j 1x x2h .x2 2x. y1 1z .zb2r .z2 2z. 2zhd 2zm 2zn 2zj 2zs 2zt 2zv z3sh2m z2h .zh2",
  ["lefthyphenmin"]=1,
  ["length"]=1418,
  ["n"]=303,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}