return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["n"]=0,
 },
 ["metadata"]={
  ["mnemonic"]="bn",
  ["source"]="hyph-bn",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="ঁংঃঅআইঈউঊঋঌএঐওঔকখগঘঙচছজঝঞটঠডঢণতথদধনপফবভমযরলশষসহ়ঽািীুূৃৄেৈোৌ্ৎৗড়ঢ়য়ৠৡৢৣ‌‍",
  ["data"]="2‍2 1‌1 অ1 আ1 ই1 ঈ1 উ1 ঊ1 ঋ1 ৠ1 ঌ1 ৡ1 এ1 ঐ1 ও1 ঔ1 া1 ি1 ী1 ু1 ূ1 ৃ1 ৄ1 ৢ1 ৣ1 ে1 ৈ1 ো1 ৌ1 2়2 ৗ1 1ক 1খ 1গ 1ঘ 1ঙ 1চ 1ছ 1জ 1ঝ 1ঞ 1ট 1ঠ 1ড 1ড় 1ঢ 1ঢ় 1ণ 1ত 1থ 1দ 1ধ 1ন 1প 1ফ 1ব 1ভ 1ম 1য 1য় 1র 1ল 1শ 1ষ 1স 1হ ৎ1 2ঃ1 2ং1 2ঁ1 2ঽ1 2্2",
  ["lefthyphenmin"]=1,
  ["length"]=367,
  ["n"]=72,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}