body {
	color: #FFFFFF ;
	background-color: <?lua pv('color-background-body')?> ;
	font-family: optima, verdana, futura, "lucida sans", arial, geneva, helvetica, sans ;
	font-size: 12px ;
	line-height: 18px ;
}
a:link, a:active, a:visited {
	color: #FFFFFF ;
}
a.dir-view:link, a.dir-view:active, a.dir-view:visited {
	color: #FFFFFF ;
	text-decoration: underline ;
}
.valid {
	color: #00FF00 ;
}
.invalid {
	color: #FF0000 ;
}
.invisible {
	visibility: hidden ;
}
button, .commonlink, .smallbutton {
	font-weight: bold ;
	font-size: 12px ;
	text-decoration: none ;
	color: #000000 ;
	border-color: #7F7F7F ;
	border-style: solid ;
	border-width: .125ex ;
	background-color: #FFFFFF ;
	padding: .5ex ;
}
.smallbutton {
	width: 1em ;
}
a.commonlink:link, a.commonlink:active, a.commonlink:visited, a.smalllink:link, a.smalllink:active, a.smalllink:visited {
	font-weight: bold ;
	font-size: 12px ;
	text-decoration: none ;
	color: #000000 ;
}
h1, .title {
	font-style: normal ;
	font-weight: normal ;
	font-size: 18px ;
	line-height: 18px ;
	margin-bottom: 20px ;
}
h2, .subtitle {
	font-style: normal ;
	font-weight: normal ;
	font-size: 12px ;
	margin-top: 18px ;
	margin-bottom: 18px ;
}
table {
	line-height: 18px ;
	font-size: 12px ;
	margin: 0 ;
}
th {
	font-weight: bold ;
	text-align: left ;
	padding-bottom: 6px ;
}
.tc {
	font-weight: bold ;
	text-align: left ;
}
p, li {
	max-width: 60em ;
}
.empty-line {
	margin-top: 4px ;
}
.more-room {
	margin-right: 1.5em ;
}
.much-more-room {
	margin-right: 3em ;
}
#main {
	position: absolute;
	left: 10% ;
	top: 10% ;
	right: 10% ;
	bottom: 10% ;
	z-index: 2 ;
	width: 80% ;
	height: 80% ;
	padding: 0% ;
	margin: 0% ;
	overflow: auto ;
	border-style: none ;
	border-width: 0 ;
	background-color: <?lua pv('color-background-main')?> ;
}
#main-settings {
	margin: 12px ;
	x_max-width: 60em ;
	line-height: 18px ;
	font-size: 12px ;
}
#left {
	position: absolute;
	top : 10% ;
	left: 0% ;
	bottom: 0%;
	right: 90% ;
	z-index: 1 ;
	width: 10% ;
	height: 90% ;
	padding: 0% ;
	margin: 0% ;
	font-size: 16px ;
	border-style: none ;
	border-width: 0 ;
	background-color: <?lua pv('color-background-one')?> ;
}
#right {
	position: absolute;
	top : 0% ;
	left: 90% ;
	bottom: 10% ;
	right: 0% ;
	z-index: 1 ;
	width: 10% ;
	height: 90% ;
	padding: 0% ;
	margin: 0% ;
	font-size: 16px ;
	border-style: none ;
	border-width: 0 ;
	background-color: <?lua pv('color-background-three')?> ;
	_margin-left: -15px ;
}
#bottom {
	position: absolute ;
	left: 10% ;
	right: 0% ;
	top: 90% ;
	bottom: 0% ;
	z-index: 1 ;
	width: 90% ;
	height: 10% ;
	padding: 0% ;
	margin: 0% ;
	font-size: 16px ;
	border-style: none ;
	border-width: 0 ;
	background-color: <?lua pv('color-background-two')?> ;
}
#top {
	position: absolute ;
	left: 0% ;
	right: 10% ;
	top: 0% ;
	bottom: 90% ;
	z-index: 1 ;
	width: 90% ;
	height: 10% ;
	padding: 0% ;
	margin: 0% ;
	font-size: 16px ;
	border-style: none ;
	border-width: 0 ;
	background-color: <?lua pv('color-background-four')?> ;
}
#top-one {
	position: absolute ;
	bottom: 50% ;
	width: 100% ;
	buggedheight: 100% ;
}
#top-two {
	position: relative ;
	margin-bottom: -9px ;
	margin-left: 12px ;
	margin-right: 12px ;
	line-height: 18px ;
	text-align: right ;
	vertical-align: middle ;
}
#bottom-one {
	position: absolute ;
	bottom: 50% ;
	width: 100% ;
	buggedheight: 100% ;
}
#bottom-two {
	position: relative ;
	margin-bottom: -9px ;
	margin-left: 12px ;
	margin-right: 12px ;
	line-height: 18px ;
	text-align: left ;
	vertical-align: middle ;
}
#left-one {
	position: absolute ;
	width: 100% ;
	buggedheight: 100% ;
}
#left-two {
	position: relative ;
	margin-top: 12px ;
	line-height: 18px ;
	text-align: center ;
	vertical-align: top ;
}
#right-one {
	display: table ;
	height: 100% ;
	width: 100% ;
}
#right-two {
	display: table-row ;
	height: 100% ;
	width: 100% ;
}
#right-three {
	display: table-cell ;
	width: 100% ;
	vertical-align: bottom ;
	_position: absolute ;
	_top: 100% ;
}
#right-four {
	text-align: center ;
	margin-bottom: 2ex ;
	_position: relative ;
	_top: -100% ;
}
#more-top {
	position: absolute;
	top: 0% ;
	left: 90% ;
	bottom: 90%;
	right: 0% ;
	z-index: 3 ;
	width: 10% ;
	height: 10% ;
	padding: 0% ;
	margin: 0% ;
	border-style: none ;
	border-width: 0 ;
}
#more-top-settings {
	text-align: center ;
}
#more-right-settings {
	margin-right: 12px ;
	margin-left: 12px ;
	line-height: 18px ;
	font-size: 10px ;
	text-align: center ;
}
#right-safari {
	_display: table ;
	width: 100% ;
	height: 100% ;
}
