return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["n"]=0,
 },
 ["metadata"]={
  ["mnemonic"]="gu",
  ["source"]="hyph-gu",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="ઁંઃઅઆઇઈઉઊઋએઐઓઔકખગઘઙચછજઝઞટઠડઢણતથદધનપફબભમયરલળવશષસહઽાિીુૂૃૄેૈોૌ્ૠૢૣ‌‍",
  ["data"]="2‍2 1‌1 અ1 આ1 ઇ1 ઈ1 ઉ1 ઊ1 ઋ1 ૠ1 એ1 ઐ1 ઓ1 ઔ1 ા1 િ1 ી1 ુ1 ૂ1 ૃ1 ૄ1 ૢ1 ૣ1 ે1 ૈ1 ો1 ૌ1 1ક 1ખ 1ગ 1ઘ 1ઙ 1ચ 1છ 1જ 1ઝ 1ઞ 1ટ 1ઠ 1ડ 1ઢ 1ણ 1ત 1થ 1દ 1ધ 1ન 1પ 1ફ 1બ 1ભ 1મ 1ય 1ર 1લ 1ળ 1વ 1શ 1ષ 1સ 1હ 2ઁ1 2ઃ1 2ઽ1 2્2 2ં2",
  ["lefthyphenmin"]=1,
  ["length"]=336,
  ["n"]=66,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}