-- This is just an example. We support both an xml format as well
-- as lua tables.

return {
    {
        filename        = "CoatedFOGRA39.icc",
        id              =  "FOGRA39",
        info            = "Coated FOGRA39 (ISO 12647-2:2004)",
        checksum        = "74ff62f330bf0dbe4495b5720542d511",
        url             = "http://www.color.org",
        outputcondition = "",
        version         = "21000000",
    }
}
