return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["characters"]="abdeghiklmnoprstuw",
  ["data"]="be-ra-be be-ra-hi be-rak be-ran-da be-ran-dal be-rang be-ra-ngas-an be-rang-sang be-ra-ngus be-ra-ni be-ran-tak-an be-ran-tam be-ran-tas be-ra-pa be-ras be-ren-deng be-re-ngut be-re-rot be-res be-re-wok be-ri be-ri-ngas be-ri-sik be-ri-ta be-rok be-ron-dong be-ron-tak be-ru-du be-ruk be-run-tun peng-eks-por peng-im-por te-ra te-rang te-ras te-ra-si te-ra-tai te-ra-wang te-ra-weh te-ri-ak te-ri-gu te-rik te-ri-ma te-ri-pang te-ro-bos te-ro-bos-an te-ro-mol te-rom-pah te-rom-pet te-ro-pong te-ro-wong-an te-ru-buk te-ru-na te-rus te-ru-si",
  ["length"]=541,
  ["n"]=55,
 },
 ["metadata"]={
  ["mnemonic"]="id",
  ["source"]="hyph-id",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="abcdefghijklmnopqrstuvwyz",
  ["data"]="a1 e1 i1 o1 u1 2b1d 2b1j 2b1k 2b1n 2b1s 2b1t 2c1k 2c1n 2d1k 2d1n 2d1p 2f1d 2f1k 2f1n 2f1t 2g1g 2g1k 2g1n 2h1k 2h1l 2h1m 2h1n 2h1w 2j1k 2j1n 2k1b 2k1k 2k1m 2k1n 2k1r 2k1s 2k1t 2l1b 2l1f 2l1g 2l1h 2l1k 2l1m 2l1n 2l1s 2l1t 2l1q 2m1b 2m1k 2m1l 2m1m 2m1n 2m1p 2m1r 2m1s 2n1c 2n1d 2n1f 2n1j 2n1k 2n1n 2n1p 2n1s 2n1t 2n1v 2p1k 2p1n 2p1p 2p1r 2p1t 2r1b 2r1c 2r1f 2r1g 2r1h 2r1j 2r1k 2r1l 2r1m 2r1n 2r1p 2r1r 2r1s 2r1t 2r1w 2r1y 2s1b 2s1k 2s1l 2s1m 2s1n 2s1p 2s1r 2s1s 2s1t 2s1w 2t1k 2t1l 2t1n 2t1t 2w1t 2ng1g 2ng1h 2ng1k 2ng1n 2ng1s 2n3s2t .be2r3 .te2r3 .me2ng3 .pe2r3 2ng. 2ny. i2o1n a2ir 1ba1ga2i 2b1an. 2c1an. 2d1an. 2f1an. 2g1an. 2h1an. 2j1an. 2k1an. 2l1an. 2m1an. 2ng1an. 2n1an. 2p1an. 2r1an. 2s1an. 2t1an. 2v1an. 2z1an. 3an. .a2ta2u .ta3ng4an. .le3ng4an. .ja3ng4an. .ma3ng4an. .pa3ng4an. .ri3ng4an. .de3ng4an.",
  ["lefthyphenmin"]=1,
  ["length"]=807,
  ["n"]=143,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}