return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["n"]=0,
 },
 ["metadata"]={
  ["mnemonic"]="kn",
  ["source"]="hyph-kn",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="ಂಃಅಆಇಈಉಊಋಌಎಏಐಒಓಔಕಖಗಘಙಚಛಜಝಞಟಠಡಢಣತಥದಧನಪಫಬಭಮಯರಱಲಳವಶಷಸಹಽಾಿೀುೂೃೄೆೇೈೊೋೌ್ೕೖೞೠೡ‌‍",
  ["data"]="2‍2 1‌1 ಅ1 ಆ1 ಇ1 ಈ1 ಉ1 ಊ1 ಋ1 ೠ1 ಌ1 ೡ1 ಎ1 ಏ1 ಐ1 ಒ1 ಓ1 ಔ1 ಾ1 ಿ1 ೀ1 ು1 ೂ1 ೃ1 ೄ1 ೆ1 ೇ1 ೈ1 ೊ1 ೋ1 ೌ1 1ಕ 1ಖ 1ಗ 1ಘ 1ಙ 1ಚ 1ಛ 1ಜ 1ಝ 1ಞ 1ಟ 1ಠ 1ಡ 1ಢ 1ಣ 1ತ 1ಥ 1ದ 1ಧ 1ನ 1ಪ 1ಫ 1ಬ 1ಭ 1ಮ 1ಯ 1ರ 1ಱ 1ಲ 1ಳ 1ೞ 1ವ 1ಶ 1ಷ 1ಸ 1ಹ 2ಂ1 2ಃ1 2ಽ1 2ೕ1 2ೖ1 2್2",
  ["lefthyphenmin"]=1,
  ["length"]=372,
  ["n"]=73,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}