if not modules then modules = { } end modules ['mult-def'] = {
    version   = 1.001,
    comment   = "companion to mult-ini.mkiv",
    author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
    copyright = "PRAGMA ADE / ConTeXt Development Team",
    license   = "see context related readme files",
    dataonly  = true,
}

-- We can remove some commands, like inleftmargin and so (see typo-mar.mkiv). Also,
-- some keywords are unique for mkiv or lmtx so at some point we might split this
-- file. On the other hand, it's a way to standardize.

-- StartMovie and StartSound etc can go.

return {
 ["commands"]={
--["CAPPED"]={
-- ["cs"]="KAP",
-- ["de"]="KAP",
-- ["en"]="CAP",
-- ["fr"]="CAP",
-- ["it"]="CAP",
-- ["nl"]="KAP",
-- ["pe"]="CAP",
-- ["ro"]="KAP",
--},
  ["Character"]={
   ["cs"]="Znak",
   ["de"]="Buchstabe",
   ["en"]="Character",
   ["fr"]="Caractere",
   ["it"]="Lettera",
   ["nl"]="Letter",
   ["pe"]="Character",
   ["ro"]="Litera",
  },
  ["Characters"]={
   ["cs"]="Znaky",
   ["de"]="Buchstaben",
   ["en"]="Characters",
   ["fr"]="Caracteres",
   ["it"]="Lettere",
   ["nl"]="Letters",
   ["pe"]="Characters",
   ["ro"]="Litere",
  },
  ["MONTH"]={
   ["cs"]="MESIC",
   ["de"]="MONAT",
   ["en"]="MONTH",
   ["fr"]="MOIS",
   ["it"]="MESE",
   ["nl"]="MAAND",
   ["pe"]="MONTH",
   ["ro"]="LUNA",
  },
  ["Numbers"]={
   ["cs"]="Cisla",
   ["de"]="Ziffern",
   ["en"]="Numbers",
   ["fr"]="Numeros",
   ["it"]="Numeri",
   ["nl"]="Cijfers",
   ["pe"]="Numbers",
   ["ro"]="Numere",
  },
  ["Romannumerals"]={
   ["cs"]="Rimskecislice",
   ["de"]="Roemischezahlen",
   ["en"]="Romannumerals",
   ["fr"]="Chiffresromains",
   ["it"]="Numeriromani",
   ["nl"]="Romeins",
   ["pe"]="Romannumerals",
   ["ro"]="Numereromane",
  },
  ["SmallCapped"]={
   ["cs"]="Kap",
   ["de"]="Kap",
   ["en"]="Cap",
   ["fr"]="PetitesCap",
   ["it"]="Cap",
   ["nl"]="Kap",
   ["pe"]="Cap",
   ["ro"]="Kap",
  },
  ["SmallCaps"]={
   ["cs"]="Kaps",
   ["de"]="Kaps",
   ["en"]="Caps",
   ["fr"]="PetitesCaps",
   ["it"]="Caps",
   ["nl"]="Kaps",
   ["pe"]="Caps",
   ["ro"]="Kaps",
  },
  ["WEEKDAY"]={
   ["cs"]="VSEDNIDEN",
   ["de"]="WOCHENTAG",
   ["en"]="WEEKDAY",
   ["fr"]="JOURSEMAINE",
   ["it"]="GIORNOSETTIMANA",
   ["nl"]="WEEKDAG",
   ["pe"]="WEEKDAY",
   ["ro"]="ZIDINSAPTAMANA",
  },
  ["WORD"]={
   ["cs"]="SLOVO",
   ["de"]="WORT",
   ["en"]="WORD",
   ["fr"]="MOT",
   ["it"]="PAROLA",
   ["nl"]="WOORD",
   ["pe"]="WORD",
   ["ro"]="CUVANT",
  },
  ["WORDS"]={
   ["cs"]="SLOVA",
   ["de"]="WOERTER",
   ["en"]="WORDS",
   ["fr"]="MOTS",
   ["it"]="PAROLE",
   ["nl"]="WOORDEN",
   ["pe"]="WORDS",
   ["ro"]="CUVINTE",
  },
  ["Word"]={
   ["cs"]="Slovo",
   ["de"]="Wort",
   ["en"]="Word",
   ["fr"]="Mot",
   ["it"]="Parola",
   ["nl"]="Woord",
   ["pe"]="Word",
   ["ro"]="Cuvant",
  },
  ["Words"]={
   ["cs"]="Slova",
   ["de"]="Woerter",
   ["en"]="Words",
   ["fr"]="Mots",
   ["it"]="Parole",
   ["nl"]="Woorden",
   ["pe"]="Words",
   ["ro"]="Cuvinte",
  },
  ["about"]={
   ["cs"]="oref",
   ["de"]="ueber",
   ["en"]="about",
   ["fr"]="apropos",
   ["it"]="intorno",
   ["nl"]="about",
   ["pe"]="درمورد",
   ["ro"]="despre",
  },
  ["adaptfontfeature"]={
   ["en"]="adaptfontfeature",
   ["fr"]="adapterfonctionnalitepolice",
  },
  ["adaptlayout"]={
   ["cs"]="prizpusobvzhled",
   ["de"]="passelayoutan",
   ["en"]="adaptlayout",
   ["fr"]="adaptermiseenpage",
   ["it"]="adattalayout",
   ["nl"]="paslayoutaan",
   ["pe"]="تنظیم‌طرح‌بندی",
   ["ro"]="adapteazaaspect",
  },
  ["addfeature"]={
   ["en"]="addfeature",
   ["fr"]="ajouterfonctionnalite",
  },
  ["alignbottom"]={
   ["en"]="alignbottom",
   ["fr"]="alignerbas",
  },
  ["aligned"]={
   ["en"]="aligned",
   ["fr"]="aligne",
  },
  ["alignedbox"]={
   ["en"]="alignedbox",
   ["fr"]="boitealignee",
  },
  ["alignedline"]={
   ["en"]="alignedline",
   ["fr"]="lignealignee",
  },
  ["alignhere"]={
   ["en"]="alignhere",
   ["fr"]="alignerici",
  },
  ["alignment"]={
   ["en"]="alignment",
   ["fr"]="alignement",
   ["nl"]="uitlijnen",
  },
  ["alignmentcharacter"]={
   ["en"]="alignmentcharacter",
   ["fr"]="caracterealignement",
  },
  ["applytocharacters"]={
   ["en"]="applytocharacters",
   ["fr"]="appliquerauxcaracteres",
  },
  ["applytofirstcharacter"]={
   ["en"]="applytofirstcharacter",
   ["fr"]="appliqueraupremiercaractere",
  },
  ["arg"]={
   ["cs"]="arg",
   ["de"]="arg",
   ["en"]="arg",
   ["fr"]="arg",
   ["it"]="arg",
   ["nl"]="arg",
   ["pe"]="افزودن",
   ["ro"]="arg",
  },
  ["at"]={
   ["cs"]="pref",
   ["de"]="bei",
   ["en"]="at",
   ["fr"]="en",
   ["it"]="al",
   ["nl"]="op",
   ["pe"]="در",
   ["ro"]="la",
  },
  ["atleftmargin"]={
   ["cs"]="atleftmargin",
   ["de"]="atleftmargin",
   ["en"]="atleftmargin",
   ["fr"]="enmargegauche",
   ["it"]="atleftmargin",
   ["nl"]="oplinkermarge",
 --["pe"]="درحاشیه‌چپ",  -- in/at clash
   ["ro"]="atleftmargin",
  },
  ["atpage"]={
   ["cs"]="nastrane",
   ["de"]="aufseite",
   ["en"]="atpage",
   ["fr"]="enpage",
   ["it"]="apagina",
   ["nl"]="oppagina",
   ["pe"]="درصفحه",
   ["ro"]="lapagina",
  },
  ["atrightmargin"]={
   ["cs"]="atrightmargin",
   ["de"]="atrightmargin",
   ["en"]="atrightmargin",
   ["fr"]="enmargedroite",
   ["it"]="atrightmargin",
   ["nl"]="oprechtermarge",
 --["pe"]="درحاشیه‌راست",  -- in/at clash
   ["ro"]="atrightmargin",
  },
  ["background"]={
   ["cs"]="pozadi",
   ["de"]="hintergrund",
   ["en"]="background",
   ["fr"]="arriereplan",
   ["it"]="sfondo",
   ["nl"]="achtergrond",
   ["pe"]="پس‌زمینه",
   ["ro"]="fundal",
  },
  ["backspace"]={
   ["cs"]="odsazenizleva",
   ["de"]="rumpfweite",
   ["en"]="backspace",
   ["fr"]="retourarrière",
   ["it"]="spaziodietro",
   ["nl"]="rugwit",
   ["pe"]="فاصله‌پشت",
   ["ro"]="spatiuspate",
  },
  ["blackrule"]={
   ["cs"]="cernalinka",
   ["de"]="rechteck",
   ["en"]="blackrule",
   ["fr"]="traitepais",
   ["it"]="lineanera",
   ["nl"]="blokje",
   ["pe"]="خط‌سیاه",
   ["ro"]="linieneagra",
  },
  ["blackrules"]={
   ["cs"]="cernelinky",
   ["de"]="rechtecke",
   ["en"]="blackrules",
   ["fr"]="traitsepais",
   ["it"]="lineenere",
   ["nl"]="blokjes",
   ["pe"]="خطهای‌سیاه",
   ["ro"]="liniinegre",
  },
  ["blank"]={
   ["cs"]="preskoc",
   ["de"]="blanko",
   ["en"]="blank",
   ["fr"]="vierge",
   ["it"]="rigovuoto",
   ["nl"]="blanko",
   ["pe"]="خالی",
   ["ro"]="blanc",
  },
  ["bookmark"]={
   ["cs"]="zalozka",
   ["de"]="bookmark",
   ["en"]="bookmark",
   ["fr"]="marquepage",
   ["it"]="segnalibro",
   ["nl"]="bookmark",
   ["pe"]="چوبخط",
   ["ro"]="semncarte",
  },
  ["bottombox"]={
   ["en"]="bottombox",
   ["fr"]="boitebas",
  },
  ["bottomdistance"]={
   ["cs"]="vzdalenostspodku",
   ["de"]="abstandunten",
   ["en"]="bottomdistance",
   ["fr"]="distancebas",
   ["it"]="distanzafondo",
   ["nl"]="onderafstand",
   ["pe"]="فاصله‌پایین",
   ["ro"]="distantajos",
  },
  ["bottomheight"]={
   ["cs"]="vyskaspodku",
   ["de"]="hoeheunten",
   ["en"]="bottomheight",
   ["fr"]="hauteurbas",
   ["it"]="altezzafondo",
   ["nl"]="onderhoogte",
   ["pe"]="ارتفاع‌پایین",
   ["ro"]="inaltimejos",
  },
  ["bottomleftbox"]={
   ["en"]="bottomleftbox",
   ["fr"]="basgaucheboite",
  },
  ["bottomrightbox"]={
   ["en"]="bottomrightbox",
   ["fr"]="boitebasdroite",
  },
  ["bottomspace"]={
   ["cs"]="bottomspace",
   ["de"]="bottomspace",
   ["en"]="bottomspace",
   ["fr"]="espacebas",
   ["it"]="spaziofondo",
   ["nl"]="bodemwit",
   ["pe"]="فضای‌پایین",
   ["ro"]="spatiujos",
  },
  ["buildtextbottomdot"]={
   ["en"]="buildtextbottomdot",
   ["fr"]="construirepointtextebas",
  },
  ["but"]={
   ["cs"]="spodek",
   ["de"]="but",
   ["en"]="but",
   ["fr"]="mais",
   ["it"]="fondo",
   ["nl"]="but",
   ["pe"]="اما",
   ["ro"]="but",
  },
  ["button"]={
   ["cs"]="tlacitko",
   ["de"]="knopf",
   ["en"]="button",
   ["fr"]="bouton",
   ["it"]="pulsante",
   ["nl"]="button",
   ["pe"]="دکمه",
   ["ro"]="buton",
  },
  ["bypassblocks"]={
   ["cs"]="bypassblocks",
   ["de"]="bypassblocks",
   ["en"]="bypassblocks",
   ["fr"]="eviteblocs",
   ["it"]="saltablocchi",
   ["nl"]="passeerblokken",
   ["pe"]="عبوربلوکها",
   ["ro"]="saripesteblocuri",
  },
  ["character"]={
   ["cs"]="znak",
   ["de"]="buchstabe",
   ["en"]="character",
   ["fr"]="caractere",
   ["it"]="lettera",
   ["nl"]="letter",
   ["pe"]="حرف",
   ["ro"]="litera",
  },
  ["characters"]={
   ["cs"]="znaky",
   ["de"]="buchstaben",
   ["en"]="characters",
   ["fr"]="caracteres",
   ["it"]="lettere",
   ["nl"]="letters",
   ["pe"]="حرفها",
   ["ro"]="litere",
  },
  ["chem"]={
   ["cs"]="chem",
   ["de"]="chem",
   ["en"]="chem",
   ["fr"]="chem",
   ["it"]="chim",
   ["nl"]="chem",
   ["pe"]="chem",
   ["ro"]="chem",
  },
  ["clip"]={
   ["cs"]="orez",
   ["de"]="ausschnitt",
   ["en"]="clip",
   ["fr"]="decouper",
   ["it"]="clip",
   ["nl"]="clip",
   ["pe"]="گیره",
   ["ro"]="clip",
  },
  ["clonefield"]={
   ["cs"]="klonujpole",
   ["de"]="klonierefeld",
   ["en"]="clonefield",
   ["fr"]="clonerchamp",
   ["it"]="clonacampo",
   ["nl"]="kloonveld",
   ["pe"]="میدان‌شبیه‌سازی",
   ["ro"]="cloneazacamp",
  },
  ["color"]={
   ["cs"]="barva",
   ["de"]="farbe",
   ["en"]="color",
   ["fr"]="couleur",
   ["it"]="colore",
   ["nl"]="kleur",
   ["pe"]="رنگ",
   ["ro"]="culoare",
  },
  ["colorbar"]={
   ["cs"]="barevnalista",
   ["de"]="farbbalken",
   ["en"]="colorbar",
   ["fr"]="barrecouleur",
   ["it"]="barracolori",
   ["nl"]="kleurenbalk",
   ["pe"]="میله‌رنگ",
   ["ro"]="baraculoare",
  },
  ["colorvalue"]={
   ["cs"]="hodnotabarvy",
   ["de"]="farbewert",
   ["en"]="colorvalue",
   ["fr"]="valeurcouleur",
   ["it"]="valorecolore",
   ["nl"]="kleurwaarde",
   ["pe"]="مقداررنگ",
   ["ro"]="valoareculoare",
  },
  ["column"]={
   ["cs"]="sloupec",
   ["de"]="spalte",
   ["en"]="column",
   ["fr"]="colonne",
   ["it"]="colonna",
   ["nl"]="kolom",
   ["pe"]="ستون",
   ["ro"]="coloana",
  },
  ["comparecolorgroup"]={
   ["cs"]="porovnejskupinubarev",
   ["de"]="vergleichefarbengruppe",
   ["en"]="comparecolorgroup",
   ["fr"]="comparergroupecouleur",
   ["it"]="confrontagruppocolori",
   ["nl"]="vergelijkkleurgroep",
   ["pe"]="مقایسه‌گروه‌رنگ",
   ["ro"]="comparagrupculoare",
  },
  ["comparepalet"]={
   ["cs"]="porovnejpaletu",
   ["de"]="vergleichepalette",
   ["en"]="comparepalet",
   ["fr"]="comparerpalette",
   ["it"]="confrontatavolozza",
   ["nl"]="vergelijkpalet",
   ["pe"]="لوح‌مقایسه",
   ["ro"]="comparapaleta",
  },
  ["completepagenumber"]={
   ["cs"]="completepagenumber",
   ["de"]="completepagenumber",
   ["en"]="completepagenumber",
   ["fr"]="numeropagecomplet",
   ["it"]="numeropaginacompleto",
   ["nl"]="volledigepaginanummer",
   ["pe"]="شماره‌صفحه‌کامل",
   ["ro"]="completeazanumarpagina",
  },
  ["completeregister"]={
   ["cs"]="completeregister",
   ["de"]="completeregister",
   ["en"]="completeregister",
   ["fr"]="registrecomplet",
   ["it"]="completeregister",
   ["nl"]="volledigregister",
   ["pe"]="ثبت‌کامل",
   ["ro"]="completeregister",
  },
  ["component"]={
   ["cs"]="komponenta",
   ["de"]="komponente",
   ["en"]="component",
   ["fr"]="composant",
   ["it"]="componenet",
   ["nl"]="onderdeel",
   ["pe"]="مولفه",
   ["ro"]="componenta",
  },
  ["convertnumber"]={
   ["cs"]="konvertujcislo",
   ["de"]="konvertierezahl",
   ["en"]="convertnumber",
   ["fr"]="convertirnumero",
   ["it"]="convertinumero",
   ["nl"]="converteernummer",
   ["pe"]="شماره‌مبدل",
   ["ro"]="convertestenumar",
  },
  ["copyfield"]={
   ["cs"]="kopirujpole",
   ["de"]="kopierefeld",
   ["en"]="copyfield",
   ["fr"]="copierchamp",
   ["it"]="copiacampo",
   ["nl"]="copieerveld",
   ["pe"]="میدان‌کپی",
   ["ro"]="copiazacamp",
  },
  ["correctwhitespace"]={
   ["cs"]="korekcebilehomista",
   ["de"]="korrigierezwischenraum",
   ["en"]="correctwhitespace",
   ["fr"]="corrigerespaceblanc",
   ["it"]="correggispaziobianco",
   ["nl"]="corrigeerwitruimte",
   ["pe"]="فضای‌سفیدصحیح",
   ["ro"]="corecteazaspatiualb",
  },
  ["coupledocument"]={
   ["cs"]="propojenydokument",
   ["de"]="verknuepfedokument",
   ["en"]="coupledocument",
   ["fr"]="coupledocument",
   ["it"]="accoppiadocumento",
   ["nl"]="koppeldocument",
   ["pe"]="نوشتارزوج",
   ["ro"]="cupleazadocument",
  },
  ["couplemarking"]={
   ["cs"]="propojeneznaceni",
   ["de"]="verknuepfebeschriftung",
   ["en"]="couplemarking",
   ["fr"]="couplemarquage",
   ["it"]="accoppiamarcatura",
   ["nl"]="koppelmarkering",
   ["pe"]="نشانه‌گذاری‌زوج",
   ["ro"]="cupleazamarcaje",
  },
  ["couplepage"]={
   ["cs"]="parovastrana",
   ["de"]="doppelseite",
   ["en"]="couplepage",
   ["fr"]="couplepage",
   ["it"]="accoppiapagina",
   ["nl"]="koppelpagina",
   ["pe"]="صفحه‌زوج",
   ["ro"]="paginadubla",
  },
  ["couplepaper"]={
   ["cs"]="dvoustrannypapir",
   ["de"]="doppelseitigespapier",
   ["en"]="couplepaper",
   ["fr"]="couplepapier",
   ["it"]="accoppiacarta",
   ["nl"]="koppelpapier",
   ["pe"]="کاغذزوج",
   ["ro"]="hartiedubla",
  },
  ["coupleregister"]={
   ["cs"]="propojenyrejstrik",
   ["de"]="verknuepfregister",
   ["en"]="coupleregister",
   ["fr"]="coupleregistre",
   ["it"]="accoppiaregistro",
   ["nl"]="koppelregister",
   ["pe"]="ثبت‌زوج",
   ["ro"]="cupleazaregistru",
  },
  ["crlf"]={
   ["cs"]="crlf",
   ["de"]="crlf",
   ["en"]="crlf",
   ["fr"]="crlf",
   ["it"]="crlf",
   ["nl"]="crlf",
   ["pe"]="crlf",
   ["ro"]="crlf",
  },
  ["currentdate"]={
   ["cs"]="aktualnidatum",
   ["de"]="heutigesdatum",
   ["en"]="currentdate",
   ["fr"]="datecourante",
   ["it"]="datadioggi",
   ["nl"]="huidigedatum",
   ["pe"]="تاریخ‌جاری",
   ["ro"]="datacurenta",
  },
  ["currentheadnumber"]={
   ["cs"]="aktualnicislonadpisu",
   ["de"]="heutigeskopfnummer",
   ["en"]="currentheadnumber",
   ["fr"]="numerotetecourante",
   ["it"]="numerotestacorrente",
   ["nl"]="huidigekopnummer",
   ["pe"]="شماره‌سرجاری",
   ["ro"]="numartitlucurent",
  },
  ["cutspace"]={
   ["cs"]="cutspace",
   ["de"]="cutspace",
   ["en"]="cutspace",
   ["fr"]="espacecoupe",
   ["it"]="cutspace",
   ["nl"]="snijwit",
   ["pe"]="فضای‌برش",
   ["ro"]="cutspace",
  },
  ["date"]={
   ["cs"]="datum",
   ["de"]="datum",
   ["en"]="date",
   ["fr"]="date",
   ["it"]="data",
   ["nl"]="datum",
   ["pe"]="تاریخ",
   ["ro"]="data",
  },
  ["decouplemarking"]={
   ["cs"]="rozpojeneznaceni",
   ["de"]="entknuepfebeschriftung",
   ["en"]="decouplemarking",
   ["fr"]="decouplemarquage",
   ["it"]="separamarcatura",
   ["nl"]="ontkoppelmarkering",
   ["pe"]="جداسازی‌نشانه‌گذاری",
   ["ro"]="decupleazamarcaje",
  },
  ["decrementnumber"]={
   ["cs"]="decrementnumber",
   ["de"]="decrementnumber",
   ["en"]="decrementnumber",
   ["fr"]="decrementenumero",
   ["it"]="decrementnumber",
   ["nl"]="verlaagnummer",
   ["pe"]="شماره‌کاهش",
   ["ro"]="decrementnumber",
  },
  ["define"]={
   ["cs"]="definuj",
   ["de"]="definieren",
   ["en"]="define",
   ["fr"]="définir",
   ["it"]="definisci",
   ["nl"]="definieer",
   ["pe"]="تعریف",
   ["ro"]="defineste",
  },
  ["defineaccent"]={
   ["cs"]="definujakcent",
   ["de"]="definiereakzent",
   ["en"]="defineaccent",
   ["fr"]="définiraccent",
   ["it"]="definisciaccento",
   ["nl"]="definieeraccent",
   ["pe"]="تعریف‌لهجه",
   ["ro"]="definesteaccent",
  },
  ["defineblank"]={
   ["cs"]="definujpreskok",
   ["de"]="definiereblanko",
   ["en"]="defineblank",
   ["fr"]="définirvierge",
   ["it"]="definiscirigovuoto",
   ["nl"]="definieerblanko",
   ["pe"]="تعریف‌خالی",
   ["ro"]="definesteblanc",
  },
  ["defineblock"]={
   ["cs"]="definujblok",
   ["de"]="definiereblock",
   ["en"]="defineblock",
   ["fr"]="définirbloc",
   ["it"]="definisciblocco",
   ["nl"]="definieerblok",
   ["pe"]="تعریف‌بلوک",
   ["ro"]="definestebloc",
  },
  ["definebodyfont"]={
   ["cs"]="definujzakladnifont",
   ["de"]="definierefliesstext",
   ["en"]="definebodyfont",
   ["fr"]="définirpolicedecorps",
   ["it"]="definiscifontdeltesto",
   ["nl"]="definieerkorps",
   ["pe"]="تعریف‌قلم‌متن",
   ["ro"]="definestefonttext",
  },
  ["definebodyfontenvironment"]={
   ["cs"]="definujprostredizakladnihofontu",
   ["de"]="definierefliesstextumgebung",
   ["en"]="definebodyfontenvironment",
   ["fr"]="définirenvironnementpolicedecorps",
   ["it"]="definisciambientefontdeltesto",
   ["nl"]="definieerkorpsomgeving",
   ["pe"]="تعریف‌محیط‌قلم‌بدنه",
   ["ro"]="definestemediulfonttext",
  },
  ["definebodyfontswitch"]={
   ["en"]="definebodyfontswitch",
   ["fr"]="définircommutateurpolicedecorps",
  },
  ["definebuffer"]={
   ["cs"]="definujbuffer",
   ["de"]="definierepuffer",
   ["en"]="definebuffer",
   ["fr"]="définirtampon",
   ["it"]="definiscibuffer",
   ["nl"]="definieerbuffer",
   ["pe"]="تعریف‌بافر",
   ["ro"]="definestebuffer",
  },
  ["definecharacter"]={
   ["cs"]="definujznak",
   ["de"]="definierezeichen",
   ["en"]="definecharacter",
   ["fr"]="définircaractere",
   ["it"]="definiscicarattere",
   ["nl"]="definieerkarakter",
   ["pe"]="تعریف‌حرف",
   ["ro"]="definestecaracter",
  },
  ["definecharacterkerning"]={
   ["en"]="definecharacterkerning",
   ["fr"]="définirapprochecaracteres",
  },
  ["definecharacterspacing"]={
   ["en"]="definecharacterspacing",
   ["fr"]="définirespacementcaracteres",
  },
  ["definecolor"]={
   ["cs"]="definujbarvu",
   ["de"]="definierefarbe",
   ["en"]="definecolor",
   ["fr"]="définircouleur",
   ["it"]="definiscicolore",
   ["nl"]="definieerkleur",
   ["pe"]="تعریف‌رنگ",
   ["ro"]="definesteculoare",
  },
  ["definecolorgroup"]={
   ["cs"]="definujskupinubarev",
   ["de"]="definierefarbengruppe",
   ["en"]="definecolorgroup",
   ["fr"]="définirgroupecouleur",
   ["it"]="definiscigruppocolori",
   ["nl"]="definieerkleurgroep",
   ["pe"]="تعریف‌گروه‌رنگ",
   ["ro"]="definestegrupculori",
  },
  ["definecolumnbreak"]={
   ["cs"]="definecolumnbreak",
   ["de"]="definecolumnbreak",
   ["en"]="definecolumnbreak",
   ["fr"]="définirsautdecolonne",
   ["it"]="definecolumnbreak",
   ["nl"]="definieerkolomovergang",
   ["pe"]="تعریف‌شکستن‌ستون",
   ["ro"]="definecolumnbreak",
  },
  ["definecolumnset"]={
   ["cs"]="definecolumnset",
   ["de"]="definecolumnset",
   ["en"]="definecolumnset",
   ["fr"]="définirjeucolonnes",
   ["it"]="definiscigruppocolonne",
   ["nl"]="definieerkolomgroep",
   ["pe"]="تعریف‌مجموعه‌ستون",
   ["ro"]="definecolumnset",
  },
  ["definecombination"]={
   ["cs"]="definecombination",
   ["de"]="definecombination",
   ["en"]="definecombination",
   ["fr"]="définircombinaison",
   ["it"]="definecombination",
   ["nl"]="definieercombinatie",
   ["pe"]="تعریف‌ترکیب",
   ["ro"]="definecombination",
  },
  ["definecombinedlist"]={
   ["cs"]="definujkombinovanyseznam",
   ["de"]="definierezusammengestellteliste",
   ["en"]="definecombinedlist",
   ["fr"]="définirlistecombinee",
   ["it"]="definiscielencocombinato",
   ["nl"]="definieersamengesteldelijst",
   ["pe"]="تعریف‌لیست‌ترکیبی",
   ["ro"]="definestelistacombinata",
  },
  ["definecommand"]={
   ["cs"]="definujprikaz",
   ["de"]="definierebefehl",
   ["en"]="definecommand",
   ["fr"]="définircommande",
   ["it"]="definiscicomando",
   ["nl"]="definieercommando",
   ["pe"]="تعریف‌فرمان",
   ["ro"]="definestecomanda",
  },
  ["defineconversion"]={
   ["cs"]="definujkonverzi",
   ["de"]="definierekonversion",
   ["en"]="defineconversion",
   ["fr"]="définirconversion",
   ["it"]="definisciconversione",
   ["nl"]="definieerconversie",
   ["pe"]="تعریف‌تبدیل",
   ["ro"]="definesteconversie",
  },
  ["definedescription"]={
   ["cs"]="definujpopis",
   ["de"]="definierebeschreibung",
   ["en"]="definedescription",
   ["fr"]="définirdescription",
   ["it"]="definiscidescrizione",
   ["nl"]="doordefinieren",
   ["pe"]="تعریف‌شرح",
   ["ro"]="definestedescriere",
  },
  ["definedfont"]={
   ["en"]="definedfont",
   ["fr"]="policedefinie",
  },
  ["defineenumeration"]={
   ["cs"]="definujvycet",
   ["de"]="definierenummerierung",
   ["en"]="defineenumeration",
   ["fr"]="définirénumération",
   ["it"]="definiscienumerazione",
   ["nl"]="doornummeren",
   ["pe"]="تعریف‌شماره‌بندی",
   ["ro"]="definesteenumerare",
  },
  ["definefallbackfamily"]={
   ["en"]="definefallbackfamily",
   ["fr"]="définirfamillederepli",
  },
  ["definefield"]={
   ["cs"]="definujpole",
   ["de"]="definierefeld",
   ["en"]="definefield",
   ["fr"]="définirchamp",
   ["it"]="definiscicampo",
   ["nl"]="definieerveld",
   ["pe"]="تعریف‌میدان",
   ["ro"]="definestecamp",
  },
  ["definefieldstack"]={
   ["cs"]="definujzasobnikpoli",
   ["de"]="definierefeldstapel",
   ["en"]="definefieldstack",
   ["fr"]="définirpilechamp",
   ["it"]="definiscistackcampi",
   ["nl"]="definieerveldstapel",
   ["pe"]="تعریف‌توده‌میدان",
   ["ro"]="definestestivacampuri",
  },
  ["definefiguresymbol"]={
   ["cs"]="definujobrazeksymbol",
   ["de"]="definiereabbsymbol",
   ["en"]="definefiguresymbol",
   ["fr"]="définirfiguresymbole",
   ["it"]="definiscifigurasimbolo",
   ["nl"]="definieerfiguursymbool",
   ["pe"]="تعریف‌نمادشکل",
   ["ro"]="definestesimbolfigura",
  },
  ["definefilefallback"]={
   ["en"]="definefilefallback",
   ["fr"]="définirfichierderepli",
  },
  ["definefirstline"]={
   ["en"]="definefirstline",
   ["fr"]="définirpremiereligne",
  },
  ["definefloat"]={
   ["cs"]="definujplvouciobjekt",
   ["de"]="definieregleitobjekt",
   ["en"]="definefloat",
   ["fr"]="définirflottant",
   ["it"]="definiscioggettomobile",
   ["nl"]="definieerplaatsblok",
   ["pe"]="تعریف‌شناور",
   ["ro"]="definestefloat",
  },
  ["definefont"]={
   ["cs"]="definujfont",
   ["de"]="definiereschrift",
   ["en"]="definefont",
   ["fr"]="définirpolice",
   ["it"]="definiscifont",
   ["nl"]="definieerfont",
   ["pe"]="تعریف‌قلم",
   ["ro"]="definestefont",
  },
  ["definefontalternative"]={
   ["en"]="definefontalternative",
   ["fr"]="définirvariantepolice",
  },
  ["definefontfallback"]={
   ["en"]="definefontfallback",
   ["fr"]="définirpolicederepli",
  },
  ["definefontfamily"]={
   ["en"]="definefontfamily",
   ["fr"]="définirfamillepolice",
  },
  ["definefontfamilypreset"]={
   ["en"]="definefontfamilypreset",
   ["fr"]="définirprereglagefamillepolice",
  },
  ["definefontfeature"]={
   ["en"]="definefontfeature",
   ["fr"]="définirfonctionnalitepolice",
  },
  ["definefontfile"]={
   ["en"]="definefontfile",
   ["fr"]="définirfichierpolice",
  },
  ["definefontsize"]={
   ["en"]="definefontsize",
   ["fr"]="définirtaillepolice",
  },
  ["definefontsolution"]={
   ["en"]="definefontsolution",
   ["fr"]="définirsolutionpolice",
  },
  ["definefontstyle"]={
   ["cs"]="definujstylfontu",
   ["de"]="definiereschriftstil",
   ["en"]="definefontstyle",
   ["fr"]="définirstylepolice",
   ["it"]="definiscistilefont",
   ["nl"]="definieerfontstijl",
   ["pe"]="تعریف‌سبک‌قلم",
   ["ro"]="definestestilfont",
  },
  ["definefontsynonym"]={
   ["cs"]="definujsynonumumfontu",
   ["de"]="defineschriftsynonym",
   ["en"]="definefontsynonym",
   ["fr"]="définirsynonymepolice",
   ["it"]="definiscisinonimofont",
   ["nl"]="definieerfontsynoniem",
   ["pe"]="تعریف‌مترادف‌قلم",
   ["ro"]="definestesinonimfont",
  },
  ["defineframed"]={
   ["cs"]="definujoramovani",
   ["de"]="definiereumrahmt",
   ["en"]="defineframed",
   ["fr"]="définirencadre",
   ["it"]="definisciincorniciato",
   ["nl"]="definieeromlijnd",
   ["pe"]="تعریف‌قالبی",
   ["ro"]="definesteinconjurare",
  },
  ["defineframedtext"]={
   ["cs"]="definujoramovanytext",
   ["de"]="definiereumrahmtertext",
   ["en"]="defineframedtext",
   ["fr"]="définirtexteencadre",
   ["it"]="definiscitestoincorniciato",
   ["nl"]="definieerkadertekst",
   ["pe"]="تعریف‌متن‌قالبی",
   ["ro"]="definestetextinconjurat",
  },
  ["definefrozenfont"]={
   ["en"]="definefrozenfont",
   ["fr"]="définirpolicegelee",
  },
  ["definehbox"]={
   ["cs"]="definujhbox",
   ["de"]="definierehbox",
   ["en"]="definehbox",
   ["fr"]="définirhbox",
   ["it"]="definiscihbox",
   ["nl"]="definieerhbox",
   ["pe"]="تعریف‌جعبه‌‌افقی",
   ["ro"]="definestehbox",
  },
  ["definehead"]={
   ["cs"]="definujnadpis",
   ["de"]="definiereueberschrift",
   ["en"]="definehead",
   ["fr"]="définirtete",
   ["it"]="definiscitesta",
   ["nl"]="definieerkop",
   ["pe"]="تعریف‌سر",
   ["ro"]="definesteantet",
  },
  ["definehyphenationfeatures"]={
   ["en"]="definehyphenationfeatures",
   ["fr"]="définirfonctionnalitescesure",
  },
  ["defineindentedtext"]={
   ["en"]="defineindentedtext",
   ["fr"]="définirtexteindente",
   ["nl"]="definieeringesprongentext",
  },
  ["defineinmargin"]={
   ["cs"]="defineinmargin",
   ["de"]="defineinmargin",
   ["en"]="defineinmargin",
   ["fr"]="définirdansmarge",
   ["it"]="defineinmargin",
   ["nl"]="definieerinmarge",
   ["pe"]="تعریف‌درون‌حاشیه",
   ["ro"]="defineinmargin",
  },
  ["defineinteractionmenu"]={
   ["cs"]="definujinterakcnimenu",
   ["de"]="definiereinteraktionsmenue",
   ["en"]="defineinteractionmenu",
   ["fr"]="définirmenuinteraction",
   ["it"]="definiscimenuinterazione",
   ["nl"]="definieerinteractiemenu",
   ["pe"]="تعریف‌منوی‌پانل",
   ["ro"]="definestemeniuinteractiune",
  },
  ["defineitemgroup"]={
   ["cs"]="defineitemgroup",
   ["de"]="defineitemgroup",
   ["en"]="defineitemgroup",
   ["fr"]="définirgroupedéléments",
   ["it"]="defineitemgroup",
   ["nl"]="definieeritemgroep",
   ["pe"]="تعریف‌گروه‌آیتم",
   ["ro"]="defineitemgroup",
  },
  ["defineitems"]={
   ["en"]="defineitems",
   ["fr"]="définiréléments",
  },
  ["definelabel"]={
   ["cs"]="definujpopisek",
   ["de"]="definierelabel",
   ["en"]="definelabel",
   ["fr"]="définiretiquette",
   ["it"]="definiscietichetta",
   ["nl"]="doorlabelen",
   ["pe"]="تعریف‌برچسب",
   ["ro"]="definesteeticheta",
  },
  ["definelayer"]={
   ["cs"]="definelayer",
   ["de"]="definelayer",
   ["en"]="definelayer",
   ["fr"]="définircalque",
   ["it"]="definelayer",
   ["nl"]="definieerlayer",
   ["pe"]="تعریف‌لایه",
   ["ro"]="definelayer",
  },
  ["definelayout"]={
   ["cs"]="definelayout",
   ["de"]="definelayout",
   ["en"]="definelayout",
   ["fr"]="définirmiseenpage",
   ["it"]="definiscilayout",
   ["nl"]="definieerlayout",
   ["pe"]="تعریف‌طرح‌بندی",
   ["ro"]="definelayout",
  },
  ["definelinenote"]={
   ["en"]="definelinenote",
   ["fr"]="définirnotedeligne",
  },
  ["definelist"]={
   ["cs"]="definujseznam",
   ["de"]="definiereliste",
   ["en"]="definelist",
   ["fr"]="définirliste",
   ["it"]="definiscielenco",
   ["nl"]="definieerlijst",
   ["pe"]="تعریف‌لیست",
   ["ro"]="definestelista",
  },
  ["definelogo"]={
   ["cs"]="definujlogo",
   ["de"]="definierelogo",
   ["en"]="definelogo",
   ["fr"]="définirlogo",
   ["it"]="definiscilogo",
   ["nl"]="definieerbeeldmerk",
   ["pe"]="تعریف‌آرم",
   ["ro"]="definestelogo",
  },
  ["definemainfield"]={
   ["cs"]="definujhlavnipole",
   ["de"]="definierehauptfeld",
   ["en"]="definemainfield",
   ["fr"]="définirchampprincipal",
   ["it"]="definiscicampoprincipale",
   ["nl"]="definieerhoofdveld",
   ["pe"]="تعریف‌میدان‌اصلی",
   ["ro"]="definestecampprincipal",
  },
  ["definemakeup"]={
   ["cs"]="definujupravu",
   ["de"]="definiereumbruch",
   ["en"]="definemakeup",
   ["fr"]="définircomposition",
   ["it"]="definiscimakeup",
   ["nl"]="definieeropmaak",
   ["pe"]="تعریف‌آرایش",
   ["ro"]="definestemakeup",
  },
  ["definemarking"]={
   ["cs"]="definujznaceni",
   ["de"]="definierebeschriftung",
   ["en"]="definemarking",
   ["fr"]="définirmarquage",
   ["it"]="definiscimarcatura",
   ["nl"]="definieermarkering",
   ["pe"]="تعریف‌نشانه‌گذاری",
   ["ro"]="definestemarcaje",
  },
  ["definemathalignment"]={
   ["cs"]="definemathalignment",
   ["de"]="definemathalignment",
   ["en"]="definemathalignment",
   ["fr"]="définiralignementmath",
   ["it"]="definemathalignment",
   ["nl"]="definieerwiskundeuitlijnen",
   ["pe"]="تعریف‌تنظیم‌ریاضی",
   ["ro"]="definemathalignment",
  },
  ["definemode"]={
   ["en"]="definemode",
   ["fr"]="définirmode",
  },
  ["definenamedcolor"]={
   ["en"]="definenamedcolor",
   ["fr"]="définircouleurnommee",
  },
  ["definenote"]={
   ["en"]="definenote",
   ["fr"]="définirnote",
  },
  ["defineoutput"]={
   ["cs"]="definujvystup",
   ["de"]="defineoutput",
   ["en"]="defineoutput",
   ["fr"]="définirsortie",
   ["it"]="definiscioutput",
   ["nl"]="definieeruitvoer",
   ["pe"]="تعریف‌خروجی",
   ["ro"]="definesteoutput",
  },
  ["defineoverlay"]={
   ["cs"]="definujprekryv",
   ["de"]="definiereoverlay",
   ["en"]="defineoverlay",
   ["fr"]="définirsuperposition",
   ["it"]="definiscisovrapposizione",
   ["nl"]="definieeroverlay",
   ["pe"]="تعریف‌پوشش",
   ["ro"]="definesteoverlay",
  },
  ["definepagebreak"]={
   ["cs"]="definepagebreak",
   ["de"]="definepagebreak",
   ["en"]="definepagebreak",
   ["fr"]="définirsautdepage",
   ["it"]="definepagebreak",
   ["nl"]="definieerpaginaovergang",
   ["pe"]="تعریف‌شکست‌صفحه",
   ["ro"]="definepagebreak",
  },
  ["definepalet"]={
   ["cs"]="definujpaletu",
   ["de"]="definierepalette",
   ["en"]="definepalet",
   ["fr"]="définirpalette",
   ["it"]="definiscitavolozza",
   ["nl"]="definieerpalet",
   ["pe"]="تعریف‌لوح",
   ["ro"]="definestepaleta",
  },
  ["definepapersize"]={
   ["cs"]="definujvelikostpapiru",
   ["de"]="definierepapierformat",
   ["en"]="definepapersize",
   ["fr"]="définirtaillepapier",
   ["it"]="definiscidimensionicarta",
   ["nl"]="definieerpapierformaat",
   ["pe"]="تعریف‌اندازه‌برگ",
   ["ro"]="definestedimensiunehartie",
  },
  ["defineparagraphs"]={
   ["cs"]="definujodstavce",
   ["de"]="definiereabsaetze",
   ["en"]="defineparagraphs",
   ["fr"]="définirparagraphes",
   ["it"]="definiscicapoversi",
   ["nl"]="definieeralineas",
   ["pe"]="تعریف‌پاراگرافها",
   ["ro"]="definesteparagraf",
  },
  ["defineperiodkerning"]={
   ["en"]="defineperiodkerning",
   ["fr"]="reglerapprochepoint",
  },
  ["defineplacement"]={
   ["cs"]="defineplacement",
   ["de"]="defineplacement",
   ["en"]="defineplacement",
   ["fr"]="définirplacement",
   ["it"]="defineplacement",
   ["nl"]="definieerplaats",
   ["pe"]="تعریف‌جانشانی",
   ["ro"]="defineplacement",
  },
  ["defineprofile"]={
   ["cs"]="definujprofil",
   ["de"]="definiereprofil",
   ["en"]="defineprofile",
   ["fr"]="définirprofil",
   ["it"]="definisciprofilo",
   ["nl"]="definieerprofiel",
   ["pe"]="تعریف‌پروفایل",
   ["ro"]="definesteprofil",
  },
  ["defineprogram"]={
   ["cs"]="definujprogram",
   ["de"]="definiereprogramme",
   ["en"]="defineprogram",
   ["fr"]="définirprogramme",
   ["it"]="definisciprogramma",
   ["nl"]="definieerprogramma",
   ["pe"]="تعریف‌برنامه",
   ["ro"]="definesteprogram",
  },
  ["definerawfont"]={
   ["cs"]="definerawfont",
   ["de"]="definerawfont",
   ["en"]="definerawfont",
   ["fr"]="définirpolicebrute",
   ["it"]="definiscifontgrezzo",
   ["nl"]="definerawfont",
   ["pe"]="تعریف‌قلم‌خام",
   ["ro"]="definestefontraw",
  },
  ["definereference"]={
   ["cs"]="definujodkaz",
   ["de"]="definierereferenz",
   ["en"]="definereference",
   ["fr"]="définirreference",
   ["it"]="definisciriferimento",
   ["nl"]="definieerreferentie",
   ["pe"]="تعریف‌مرجع",
   ["ro"]="definestereferinte",
  },
  ["definereferenceformat"]={
   ["cs"]="definujformatodkazu",
   ["de"]="definierereferenzformat",
   ["en"]="definereferenceformat",
   ["fr"]="définirformatrereference",
   ["it"]="definisciformatoriferimento",
   ["nl"]="definieerreferentieformaat",
   ["pe"]="تعریف‌شمایل‌مرجع",
   ["ro"]="definesteformatreferinte",
  },
  ["definereferencelist"]={
   ["cs"]="definujseznamodkazu",
   ["de"]="definierereferenzliste",
   ["en"]="definereferencelist",
   ["fr"]="définirlistereference",
   ["it"]="definiscilistariferimenti",
   ["nl"]="definieerreferentielijst",
   ["pe"]="تعریف‌لیست‌مرجع",
   ["ro"]="definestelistareferinte",
  },
  ["defineregister"]={
   ["cs"]="definujrejstrik",
   ["de"]="definiereregister",
   ["en"]="defineregister",
   ["fr"]="définirregistre",
   ["it"]="definisciregistro",
   ["nl"]="definieerregister",
   ["pe"]="تعریف‌ثبت",
   ["ro"]="definesteregistru",
  },
  ["definerule"]={
   ["cs"]="definerule",
   ["de"]="definerule",
   ["en"]="definerule",
   ["fr"]="définirtrait",
   ["it"]="definiscilinea",
   ["nl"]="definieerlijn",
   ["pe"]="تعریف‌خط‌حائل",
   ["ro"]="definesterigla",
  },
  ["definesection"]={
   ["cs"]="definujsekci",
   ["de"]="definiereabschnitt",
   ["en"]="definesection",
   ["fr"]="définirsection",
   ["it"]="definiscisezione",
   ["nl"]="definieersectie",
   ["pe"]="تعریف‌بخش",
   ["ro"]="definestesectiune",
  },
  ["definesectionblock"]={
   ["cs"]="definujbloksekce",
   ["de"]="definiereabschnittsblock",
   ["en"]="definesectionblock",
   ["fr"]="définirblocsection",
   ["it"]="definiscibloccosezione",
   ["nl"]="definieersectieblok",
   ["pe"]="تعریف‌بلوک‌بخش",
   ["ro"]="definesteblocsectiune",
  },
  ["definesorting"]={
   ["cs"]="definujtrideni",
   ["de"]="definieresortieren",
   ["en"]="definesorting",
   ["fr"]="définirtri",
   ["it"]="definisciordinamento",
   ["nl"]="definieersorteren",
   ["pe"]="تعریف‌ترتیب",
   ["ro"]="definestesortare",
  },
  ["definestartstop"]={
   ["cs"]="definujstartstop",
   ["de"]="definierestartstop",
   ["en"]="definestartstop",
   ["fr"]="définirdébutfin",
   ["it"]="definisciiniziatermina",
   ["nl"]="definieerstartstop",
   ["pe"]="تعریف‌شروع‌پایان",
   ["ro"]="definestestartstop",
  },
  ["definestyle"]={
   ["cs"]="definujstyl",
   ["de"]="definierestil",
   ["en"]="definestyle",
   ["fr"]="définirstyle",
   ["it"]="definiscistile",
   ["nl"]="definieerletter",
   ["pe"]="تعریف‌سبک",
   ["ro"]="definestestil",
  },
  ["definesubfield"]={
   ["cs"]="definujpodpole",
   ["de"]="definieresubfeld",
   ["en"]="definesubfield",
   ["fr"]="définirsouschamp",
   ["it"]="definiscisottocampo",
   ["nl"]="definieersubveld",
   ["pe"]="تعریف‌زیرمیدان",
   ["ro"]="definestesubcamp",
  },
  ["definesymbol"]={
   ["cs"]="definujsymbol",
   ["de"]="definieresymbol",
   ["en"]="definesymbol",
   ["fr"]="définirsymbole",
   ["it"]="definiscisimbolo",
   ["nl"]="definieersymbool",
   ["pe"]="تعریف‌نماد",
   ["ro"]="definestesimbol",
  },
  ["definesynonyms"]={
   ["cs"]="definujsynonyma",
   ["de"]="definieresynonyme",
   ["en"]="definesynonyms",
   ["fr"]="définirsynonymes",
   ["it"]="definiscisinonimi",
   ["nl"]="definieersynoniemen",
   ["pe"]="تعریف‌مترادفها",
   ["ro"]="definestesinonim",
  },
  ["definetabletemplate"]={
   ["cs"]="definujsablonutabulky",
   ["de"]="definieretabellenvorlage",
   ["en"]="definetabletemplate",
   ["fr"]="définirgabarittable",
   ["it"]="definiscimodellotabella",
   ["nl"]="definieertabelvorm",
   ["pe"]="تعریف‌الگوی‌جدول",
   ["ro"]="definestesablontabel",
  },
  ["definetabulate"]={
   ["cs"]="definujtabelaci",
   ["de"]="definieretabulator",
   ["en"]="definetabulate",
   ["fr"]="définirtabulation",
   ["it"]="definiscitabulato",
   ["nl"]="definieertabulatie",
   ["pe"]="تعریف‌جدول‌بندی",
   ["ro"]="definestetabulatori",
  },
  ["definetext"]={
   ["cs"]="definujtext",
   ["de"]="definieretext",
   ["en"]="definetext",
   ["fr"]="définirtexte",
   ["it"]="definiscitesto",
   ["nl"]="definieertekst",
   ["pe"]="تعریف‌متن",
   ["ro"]="definestetext",
  },
  ["definetextbackground"]={
   ["en"]="definetextbackground",
   ["fr"]="définirtextearriereplan",
   ["nl"]="definieertekstachtergrond",
  },
  ["definetextnote"]={
   ["en"]="definetextnote",
   ["fr"]="définirnotetexte",
  },
  ["definetextposition"]={
   ["cs"]="definetextposition",
   ["de"]="definetextposition",
   ["en"]="definetextposition",
   ["fr"]="définirpositiontexte",
   ["it"]="definisciposizionetesto",
   ["nl"]="definieertekstpositie",
   ["pe"]="تعریف‌مکان‌متن",
   ["ro"]="definestepozitietext",
  },
  ["definetextvariable"]={
   ["cs"]="definetextvariable",
   ["de"]="definetextvariable",
   ["en"]="definetextvariable",
   ["fr"]="définirvariabletexte",
   ["it"]="definiscivariabiletesto",
   ["nl"]="definieertekstvariabele",
   ["pe"]="تعریف‌متغیرمتن",
   ["ro"]="definestevariabilatext",
  },
  ["definetype"]={
   ["cs"]="definetype",
   ["de"]="definetype",
   ["en"]="definetype",
   ["fr"]="définirtranscrire",
   ["it"]="definiscitype",
   ["nl"]="definieertype",
   ["pe"]="تعریف‌تایپ",
   ["ro"]="definetype",
  },
  ["definetyping"]={
   ["cs"]="definujopis",
   ["de"]="definieretippen",
   ["en"]="definetyping",
   ["fr"]="définirtranscription",
   ["it"]="definiscityping",
   ["nl"]="definieertypen",
   ["pe"]="تعریف‌تایپ‌کردن",
   ["ro"]="definestetyping",
  },
  ["defineversion"]={
   ["cs"]="definujverzi",
   ["de"]="definiereversion",
   ["en"]="defineversion",
   ["fr"]="définirversion",
   ["it"]="definisciversion",
   ["nl"]="definieerversie",
   ["pe"]="تعریف‌نسخه",
   ["ro"]="definesteversiune",
  },
  ["depthspanningtext"]={
   ["en"]="depthspanningtext",
   ["fr"]="jambagetextecouvrant",
  },
  ["determineheadnumber"]={
   ["cs"]="stanovcislonadpisu",
   ["de"]="bestimmekopfnummer",
   ["en"]="determineheadnumber",
   ["fr"]="determinernumerotete",
   ["it"]="determinanumerotesta",
   ["nl"]="bepaalkopnummer",
   ["pe"]="تعیین‌شماره‌سر",
   ["ro"]="determinanumartitlu",
  },
  ["determinelistcharacteristics"]={
   ["cs"]="stanovcharakteristickuseznamu",
   ["de"]="bestimmelistencharakeristika",
   ["en"]="determinelistcharacteristics",
   ["fr"]="determinercaracteristiquesliste",
   ["it"]="determinacarattersticheelenco",
   ["nl"]="bepaallijstkenmerken",
   ["pe"]="تعیین‌مشخصات‌لیست",
   ["ro"]="determinacaracteristicilelistei",
  },
  ["determineregistercharacteristics"]={
   ["cs"]="urcicharakteristikurejstriku",
   ["de"]="bestimmeregistercharakteristika",
   ["en"]="determineregistercharacteristics",
   ["fr"]="determinercaracteristiquesregistre",
   ["it"]="determinacaratteristicheregistro",
   ["nl"]="bepaalregisterkenmerken",
   ["pe"]="تعیین‌مشخصات‌ثبت",
   ["ro"]="determinacaracteristiciregistru",
  },
  ["dimension"]={
   ["cs"]="rozmer",
   ["de"]="dimension",
   ["en"]="dimension",
   ["fr"]="dimension",
   ["it"]="dimensione",
   ["nl"]="dimensie",
   ["pe"]="بعد",
   ["ro"]="dimensiune",
  },
  ["disableinteractionmenu"]={
   ["cs"]="zablokujinterakcnimenu",
   ["de"]="inaktiviereinteraktionsmenue",
   ["en"]="disableinteractionmenu",
   ["fr"]="desactivermenuinteraction",
   ["it"]="disabilitamenuinterazione",
   ["nl"]="blokkeerinteractiemenu",
   ["pe"]="ازکارانداختن‌منوی‌پانل",
   ["ro"]="dezactiveazameniuinteractiune",
  },
  ["disablemode"]={
   ["en"]="disablemode",
   ["fr"]="desactivermode",
  },
  ["doaddfeature"]={
   ["en"]="doaddfeature",
   ["fr"]="faireajouterfonctionnalite",
  },
  ["domicile"]={
   ["cs"]="bydliste",
   ["de"]="wohnort",
   ["en"]="domicile",
   ["fr"]="domicile",
   ["it"]="domicilio",
   ["nl"]="woonplaats",
   ["pe"]="مسکن",
   ["ro"]="domiciliu",
  },
  ["dontconvertfont"]={
   ["en"]="dontconvertfont",
   ["fr"]="nepasconvertirpolice",
  },
  ["donttest"]={
   ["cs"]="zadnytest",
   ["de"]="keintest",
   ["en"]="donttest",
   ["fr"]="sanstester",
   ["it"]="notest",
   ["nl"]="geentest",
   ["pe"]="امتحان‌نکن",
   ["ro"]="nutesta",
  },
  ["edgedistance"]={
   ["cs"]="vzdalenostokraje",
   ["de"]="randabstand",
   ["en"]="edgedistance",
   ["fr"]="distancebord",
   ["it"]="distanzabordo",
   ["nl"]="randafstand",
   ["pe"]="فاصله‌لبه",
   ["ro"]="distantacolt",
  },
  ["edgewidth"]={
   ["cs"]="sirkaokraje",
   ["de"]="randbreite",
   ["en"]="edgewidth",
   ["fr"]="largeurbord",
   ["it"]="ampiezzabordo",
   ["nl"]="randbreedte",
   ["pe"]="عرض‌لبه",
   ["ro"]="latimecolturi",
  },
  ["emphasisboldface"]={
   ["en"]="emphasisboldface",
   ["fr"]="policegrasaccentue",
  },
  ["emptylines"]={
   ["cs"]="emptylines",
   ["de"]="emptylines",
   ["en"]="emptylines",
   ["fr"]="lignesvide",
   ["it"]="emptylines",
   ["nl"]="legeregels",
   ["pe"]="خطها‌خالی",
   ["ro"]="emptylines",
  },
  ["enablemode"]={
   ["en"]="enablemode",
   ["fr"]="activermode",
  },
  ["environment"]={
   ["cs"]="prostredi",
   ["de"]="umgebung",
   ["en"]="environment",
   ["fr"]="environnement",
   ["it"]="ambiente",
   ["nl"]="omgeving",
   ["pe"]="محیط",
   ["ro"]="mediu",
  },
  ["externalfigure"]={
   ["cs"]="externiobraz",
   ["de"]="externeabbildung",
   ["en"]="externalfigure",
   ["fr"]="figureexterne",
   ["it"]="figuraesterna",
   ["nl"]="externfiguur",
   ["pe"]="شکل‌خارجی",
   ["ro"]="figuraexterna",
  },
  ["fact"]={
   ["cs"]="fakt",
   ["de"]="geg",
   ["en"]="fact",
   ["fr"]="fait",
   ["it"]="fatto",
   ["nl"]="geg",
   ["pe"]="حقیقت",
   ["ro"]="fact",
  },
  ["fastswitchtobodyfont"]={
   ["en"]="fastswitchtobodyfont",
   ["fr"]="passerrapideapolicedecorps",
  },
  ["feature"]={
   ["en"]="feature",
   ["fr"]="fonctionnalite",
  },
  ["field"]={
   ["cs"]="pole",
   ["de"]="feld",
   ["en"]="field",
   ["fr"]="champ",
   ["it"]="campi",
   ["nl"]="veld",
   ["pe"]="میدان",
   ["ro"]="camp",
  },
  ["fieldstack"]={
   ["cs"]="zasobnikpoli",
   ["de"]="feldstapel",
   ["en"]="fieldstack",
   ["fr"]="pilechamp",
   ["it"]="stackcampi",
   ["nl"]="veldstapel",
   ["pe"]="میدان‌پشته",
   ["ro"]="stivacampuri",
  },
  ["fillinfield"]={
   ["cs"]="vyplnovepole",
   ["de"]="ausfuellfeld",
   ["en"]="fillinfield",
   ["fr"]="champarenseigner",
   ["it"]="camporiempimento",
   ["nl"]="invulveld",
   ["pe"]="درج‌درمیدان",
   ["ro"]="campumplere",
  },
  ["fillinline"]={
   ["cs"]="vyplnovyradek",
   ["de"]="gefuelltezeile",
   ["en"]="fillinline",
   ["fr"]="lignearenseigner",
   ["it"]="rigariempimento",
   ["nl"]="invulregel",
   ["pe"]="درج‌درخط",
   ["ro"]="linieumplere",
  },
  ["fillinrules"]={
   ["cs"]="vyplnovelinky",
   ["de"]="gefuelltesrechteck",
   ["en"]="fillinrules",
   ["fr"]="traitsarenseigner",
   ["it"]="lineeriempimento",
   ["nl"]="invullijnen",
   ["pe"]="درج‌درخطها",
   ["ro"]="rigleumplere",
  },
  ["fillintext"]={
   ["cs"]="vyplnenytext",
   ["de"]="ausfuelltext",
   ["en"]="fillintext",
   ["fr"]="textearenseigner",
   ["it"]="testoriempimento",
   ["nl"]="invultekst",
   ["pe"]="درج‌درمتن",
   ["ro"]="textumplere",
  },
  ["firstcharacter"]={
   ["en"]="firstcharacter",
   ["fr"]="premiercaractere",
  },
  ["fitfield"]={
   ["cs"]="prizpusobivepole",
   ["de"]="passendfeld",
   ["en"]="fitfield",
   ["fr"]="adapterchamp",
   ["it"]="adattacampo",
   ["nl"]="passendveld",
   ["pe"]="پرکردن‌میدان",
   ["ro"]="potrivestecamp",
  },
  ["fittopbaselinegrid"]={
   ["en"]="fittopbaselinegrid",
   ["fr"]="adaptergrillehautlignedebase",
  },
  ["fixedspace"]={
   ["cs"]="tvrdamezera",
   ["de"]="festesspatium",
   ["en"]="fixedspace",
   ["fr"]="espacefixe",
   ["it"]="spaziofisso",
   ["nl"]="vastespatie",
   ["pe"]="فضای‌ثابت",
   ["ro"]="spatiufixat",
  },
  ["fixedspaces"]={
   ["cs"]="tvrdemezery",
   ["de"]="festesspatium",
   ["en"]="fixedspaces",
   ["fr"]="espacesfixes",
   ["it"]="spazifissi",
   ["nl"]="vastespaties",
   ["pe"]="فضاهای‌ثابت",
   ["ro"]="spatiifixate",
  },
  ["flushnotes"]={
   ["en"]="flushnotes",
   ["fr"]="vidernotes",
  },
  ["followprofile"]={
   ["cs"]="dodrzujprofil",
   ["de"]="folgeprofil",
   ["en"]="followprofile",
   ["fr"]="suivreprofile",
   ["it"]="seguiprofilo",
   ["nl"]="volgprofiel",
   ["pe"]="پیروی‌پروفایل",
   ["ro"]="urmeazaprofil",
  },
  ["followprofileversion"]={
   ["cs"]="dodrzujverziprofilu",
   ["de"]="folgeprofilversion",
   ["en"]="followprofileversion",
   ["fr"]="suivreversionprofil",
   ["it"]="seguiversioneprofilo",
   ["nl"]="volgprofielversie",
   ["pe"]="پیروی‌نسخه‌پروفایل",
   ["ro"]="urmeazaversiuneprofil",
  },
  ["followversion"]={
   ["cs"]="dodrzujverzi",
   ["de"]="folgeversion",
   ["en"]="followversion",
   ["fr"]="suivreversion",
   ["it"]="seguiversione",
   ["nl"]="volgversie",
   ["pe"]="پیروی‌نسخه",
   ["ro"]="urmeazaversiune",
  },
  ["footerdistance"]={
   ["cs"]="vzdalenostupati",
   ["de"]="fusszeileabstand",
   ["en"]="footerdistance",
   ["fr"]="distancepdp",
   ["it"]="distanzapdp",
   ["nl"]="voetafstand",
   ["pe"]="فاصله‌ته‌برگ",
   ["ro"]="distantasubsol",
  },
  ["footerheight"]={
   ["cs"]="vyskaupati",
   ["de"]="fusszeilenhoehe",
   ["en"]="footerheight",
   ["fr"]="hauteurpdp",
   ["it"]="altezzapdp",
   ["nl"]="voethoogte",
   ["pe"]="ارتفاع‌ته‌برگ",
   ["ro"]="inaltimesubsol",
  },
  ["footnote"]={
   ["cs"]="poznamkapodcarou",
   ["de"]="fussnote",
   ["en"]="footnote",
   ["fr"]="notebdp",
   ["it"]="notapdp",
   ["nl"]="voetnoot",
   ["pe"]="پانوشت",
   ["ro"]="notasubsol",
  },
  ["footnotetext"]={
   ["cs"]="footnotetext",
   ["de"]="footnotetext",
   ["en"]="footnotetext",
   ["fr"]="textenotebdp",
   ["it"]="testonotapdp",
   ["nl"]="voetnoottekst",
   ["pe"]="متن‌پانوشت",
   ["ro"]="footnotetext",
  },
  ["forceblocks"]={
   ["cs"]="forceblocks",
   ["de"]="forceblocks",
   ["en"]="forceblocks",
   ["fr"]="forcerblocs",
   ["it"]="forzablocchi",
   ["nl"]="forceerblokken",
   ["pe"]="اجباربلوکها",
   ["ro"]="forteazablocuri",
  },
  ["forcecharacterstripping"]={
   ["en"]="forcecharacterstripping",
   ["fr"]="forcerdemontagecaracteres",
  },
  ["formulanumber"]={
   ["cs"]="cislorovnice",
   ["de"]="formelnummer",
   ["en"]="formulanumber",
   ["fr"]="numeroformule",
   ["it"]="numeroformula",
   ["nl"]="formulenummer",
   ["pe"]="شماره‌فرمول",
   ["ro"]="numarformula",
  },
  ["fraction"]={
   ["cs"]="zlomek",
   ["de"]="bruch",
   ["en"]="fraction",
   ["fr"]="fraction",
   ["it"]="frazione",
   ["nl"]="breuk",
   ["pe"]="کسر",
   ["ro"]="fractie",
  },
  ["framed"]={
   ["cs"]="oramovani",
   ["de"]="umrahmt",
   ["en"]="framed",
   ["fr"]="encadre",
   ["it"]="incorniciato",
   ["nl"]="omlijnd",
   ["pe"]="قالبی",
   ["ro"]="framed",
  },
  ["from"]={
   ["cs"]="zref",
   ["de"]="von",
   ["en"]="from",
   ["fr"]="de",
   ["it"]="da",
   ["nl"]="uit",
   ["pe"]="از",
   ["ro"]="din",
  },
  ["fromlinenote"]={
   ["en"]="fromlinenote",
   ["fr"]="denotedeligne",
  },
  ["getbuffer"]={
   ["cs"]="ziskejbuffer",
   ["de"]="holepuffer",
   ["en"]="getbuffer",
   ["fr"]="obtenirtampon",
   ["it"]="prendibuffer",
   ["nl"]="haalbuffer",
   ["pe"]="دریافت‌بافر",
   ["ro"]="adubuffer",
  },
  ["getfirstcharacter"]={
   ["en"]="getfirstcharacter",
   ["fr"]="obtenirpremiercaractere",
  },
  ["getmarking"]={
   ["cs"]="ziskejznaceni",
   ["de"]="holebeschriftung",
   ["en"]="getmarking",
   ["fr"]="obtenirmarquage",
   ["it"]="prendimarcatura",
   ["nl"]="haalmarkering",
   ["pe"]="دریافت‌نشانه",
   ["ro"]="adumarcaje",
  },
  ["getnumber"]={
   ["cs"]="getnumber",
   ["de"]="getnumber",
   ["en"]="getnumber",
   ["fr"]="obtenirnumero",
   ["it"]="getnumber",
   ["nl"]="haalnummer",
   ["pe"]="دریافت‌شماره",
   ["ro"]="getnumber",
  },
  ["globaldisablemode"]={
   ["en"]="globaldisablemode",
   ["fr"]="desactivermodeglobal",
  },
  ["globalenablemode"]={
   ["en"]="globalenablemode",
   ["fr"]="activermodeglobal",
  },
  ["globalpreventmode"]={
   ["en"]="globalpreventmode",
   ["fr"]="empechermodeglobal",
  },
  ["godown"]={
   ["cs"]="jdidolu",
   ["de"]="nachunten",
   ["en"]="godown",
   ["fr"]="allerenbas",
   ["it"]="vaigiu",
   ["nl"]="omlaag",
   ["pe"]="بروپایین",
   ["ro"]="injos",
  },
  ["goto"]={
   ["cs"]="jdina",
   ["de"]="zu",
   ["en"]="goto",
   ["fr"]="allera",
   ["it"]="vaia",
   ["nl"]="naar",
   ["pe"]="بروبه",
   ["ro"]="dute",
  },
  ["gotobox"]={
   ["cs"]="jdinabox",
   ["de"]="zurbox",
   ["en"]="gotobox",
   ["fr"]="alleraboite",
   ["it"]="vaiabox",
   ["nl"]="naarbox",
   ["pe"]="بروبه‌جعبه",
   ["ro"]="dutebox",
  },
  ["gotopage"]={
   ["cs"]="jdinastranu",
   ["de"]="zurseite",
   ["en"]="gotopage",
   ["fr"]="allerapage",
   ["it"]="vaiapagina",
   ["nl"]="naarpagina",
   ["pe"]="بروبه‌صفحه",
   ["ro"]="dutepagina",
  },
  ["graycolor"]={
   ["cs"]="sedabarva",
   ["de"]="graufarbe",
   ["en"]="graycolor",
   ["fr"]="couleurgrise",
   ["it"]="coloregrigio",
   ["nl"]="grijskleur",
   ["pe"]="رنگ‌خاکستری",
   ["ro"]="culoaregri",
  },
  ["greyvalue"]={
   ["cs"]="hodnotasedi",
   ["de"]="grauwert",
   ["en"]="greyvalue",
   ["fr"]="valeurgris",
   ["it"]="valoregrigio",
   ["nl"]="grijswaarde",
   ["pe"]="مقدارخاکستری",
   ["ro"]="valoaregri",
  },
  ["grid"]={
   ["cs"]="mrizka",
   ["de"]="gitter",
   ["en"]="grid",
   ["fr"]="grille",
   ["it"]="griglia",
   ["nl"]="rooster",
   ["pe"]="توری",
   ["ro"]="grid",
  },
  ["hairline"]={
   ["cs"]="vlasovalinka",
   ["de"]="haarlinie",
   ["en"]="hairline",
   ["fr"]="ligneavectrait",
   ["it"]="capello",
   ["nl"]="haarlijn",
   ["pe"]="خط‌مو",
   ["ro"]="firdepar",
  },
  ["head"]={
   ["cs"]="nadpis",
   ["de"]="kopf",
   ["en"]="head",
   ["fr"]="tête",
   ["it"]="testa",
   ["nl"]="kop",
   ["pe"]="سر",
   ["ro"]="titlu",
  },
  ["headerdistance"]={
   ["cs"]="vzdalenostzahlavi",
   ["de"]="kopfzeilenabstand",
   ["en"]="headerdistance",
   ["fr"]="distanceentête",
   ["it"]="distanzaintestazione",
   ["nl"]="hoofdafstand",
   ["pe"]="فاصله‌سربرگ",
   ["ro"]="distantaantet",
  },
  ["headerheight"]={
   ["cs"]="vyskazahlavi",
   ["de"]="kopfzeilenhoehe",
   ["en"]="headerheight",
   ["fr"]="hauteurentête",
   ["it"]="altezzaintestazione",
   ["nl"]="hoofdhoogte",
   ["pe"]="ارتفاع‌سربرگ",
   ["ro"]="inaltimeantet",
  },
  ["headlevel"]={
   ["cs"]="hlavniuroven",
   ["de"]="kopfniveau",
   ["en"]="headlevel",
   ["fr"]="niveautete",
   ["it"]="livellotesta",
   ["nl"]="kopniveau",
   ["pe"]="مرحله‌سر",
   ["ro"]="niveltitlu",
  },
  ["headnumber"]={
   ["cs"]="cislonadpisu",
   ["de"]="kopfnummer",
   ["en"]="headnumber",
   ["fr"]="numerotete",
   ["it"]="numerotesta",
   ["nl"]="kopnummer",
   ["pe"]="شماره‌سر",
   ["ro"]="numartitlu",
  },
  ["headsym"]={
   ["cs"]="headsym",
   ["de"]="headsym",
   ["en"]="headsym",
   ["fr"]="symtete",
   ["it"]="headsym",
   ["nl"]="kopsym",
   ["pe"]="نمادسر",
   ["ro"]="headsym",
  },
  ["headtext"]={
   ["cs"]="texthlavicky",
   ["de"]="ueberschrifttext",
   ["en"]="headtext",
   ["fr"]="textetete",
   ["it"]="testoinstestazioni",
   ["nl"]="koptekst",
   ["pe"]="متن‌سر",
   ["ro"]="texttitlu",
  },
  ["heightspanningtext"]={
   ["en"]="heightspanningtext",
   ["fr"]="hauteurtextecouvrant",
  },
  ["hideblocks"]={
   ["cs"]="schovejbloky",
   ["de"]="verbergebloecke",
   ["en"]="hideblocks",
   ["fr"]="cacherblocs",
   ["it"]="nascondiblocchi",
   ["nl"]="verbergblokken",
   ["pe"]="بلوکها‌پنهان",
   ["ro"]="ascundeblocuri",
  },
  ["high"]={
   ["cs"]="vysoky",
   ["de"]="hoch",
   ["en"]="high",
   ["fr"]="exposant",
   ["it"]="ap",
   ["nl"]="hoog",
   ["pe"]="بلند",
   ["ro"]="inalt",
  },
  ["hl"]={
   ["cs"]="hl",
   ["de"]="hl",
   ["en"]="hl",
   ["fr"]="lh",
   ["it"]="hl",
   ["nl"]="hl",
   ["pe"]="خ‌ا",
   ["ro"]="hl",
  },
  ["immediatebetweenlist"]={
   ["cs"]="immediatebetweenlist",
   ["de"]="immediatebetweenlist",
   ["en"]="immediatebetweenlist",
   ["fr"]="immediatemententreliste",
   ["it"]="immediatebetweenlist",
   ["nl"]="directtussenlijst",
   ["pe"]="فوری‌بین‌لیست",
   ["ro"]="immediatebetweenlist",
  },
  ["immediatetolist"]={
   ["cs"]="immediatetolist",
   ["de"]="immediatetolist",
   ["en"]="immediatetolist",
   ["fr"]="immediatementalister",
   ["it"]="immediatetolist",
   ["nl"]="directnaarlijst",
   ["pe"]="فوری‌به‌لیست",
   ["ro"]="immediatetolist",
  },
  ["in"]={
   ["cs"]="tref",
   ["de"]="in",
   ["en"]="in",
   ["fr"]="dans",
   ["it"]="in",
   ["nl"]="in",
   ["pe"]="درون",
   ["ro"]="in",
  },
  ["incrementnumber"]={
   ["cs"]="zvysujicicislo",
   ["de"]="nummererhoehen",
   ["en"]="incrementnumber",
   ["fr"]="incrementernumero",
   ["it"]="incrementanumero",
   ["nl"]="verhoognummer",
   ["pe"]="شماره‌افزایش",
   ["ro"]="numarincrement",
  },
  ["indenting"]={
   ["cs"]="odsazovani",
   ["de"]="einziehen",
   ["en"]="indenting",
   ["fr"]="indentation",
   ["it"]="rientro",
   ["nl"]="inspringen",
   ["pe"]="تورفتگی",
   ["ro"]="aliniat",
  },
  ["inframed"]={
   ["cs"]="zaramovani",
   ["de"]="imumriss",
   ["en"]="inframed",
   ["fr"]="dansencadre",
   ["it"]="circondato",
   ["nl"]="inlijnd",
   ["pe"]="درقالبی",
   ["ro"]="inframed",
  },
  ["infull"]={
   ["cs"]="plnezneni",
   ["de"]="volleswort",
   ["en"]="infull",
   ["fr"]="encomplet",
   ["it"]="perlungo",
   ["nl"]="voluit",
   ["pe"]="درپر",
   ["ro"]="pelung",
  },
  ["ininner"]={
   ["cs"]="ininner",
   ["de"]="ininner",
   ["en"]="ininner",
   ["fr"]="dansinterieur",
   ["it"]="ininner",
   ["nl"]="inbinnen",
   ["pe"]="درداخلی",
   ["ro"]="ininner",
  },
  ["inleft"]={
   ["cs"]="vlevo",
   ["de"]="imlinken",
   ["en"]="inleft",
   ["fr"]="dansgauche",
   ["it"]="insinistra",
   ["nl"]="inlinker",
   ["pe"]="درچپ",
   ["ro"]="instanga",
  },
  ["inleftedge"]={
   ["cs"]="nalevo",
   ["de"]="imlinkenrand",
   ["en"]="inleftedge",
   ["fr"]="dansbordgauche",
   ["it"]="inlatosinistro",
   ["nl"]="inlinkerrand",
   ["pe"]="درلبه‌چپ",
   ["ro"]="inparteastanga",
  },
  ["inleftmargin"]={
   ["cs"]="nalevyokraj",
   ["de"]="inlinkermarginale",
   ["en"]="inleftmargin",
   ["fr"]="dansmargegauche",
   ["it"]="inmarginesinistro",
   ["nl"]="inlinkermarge",
 --["pe"]="درحاشیه‌چپ",  -- in/at clash
   ["ro"]="inmargineastanga",
  },
  ["inline"]={
   ["cs"]="vradku",
   ["de"]="inzeile",
   ["en"]="inline",
   ["fr"]="enligne",
   ["it"]="inriga",
   ["nl"]="inregel",
   ["pe"]="درخط",
   ["ro"]="inlinie",
  },
  ["inmargin"]={
   ["cs"]="naokraj",
   ["de"]="inmarginalie",
   ["en"]="inmargin",
   ["fr"]="dansmarge",
   ["it"]="inmargine",
   ["nl"]="inmarge",
   ["pe"]="درحاشیه",
   ["ro"]="marginal",
  },
  ["inmframed"]={
   ["cs"]="mazaramovani",
   ["de"]="immaumrise",
   ["en"]="inmaframed",
   ["fr"]="dansencadremath",
   ["it"]="inmaframed",
   ["nl"]="inwilijnd",
   ["pe"]="inmframed",
   ["ro"]="inmaframed",
  },
  ["inneredgedistance"]={
   ["cs"]="inneredgedistance",
   ["de"]="inneredgedistance",
   ["en"]="inneredgedistance",
   ["fr"]="distancebordinterieur",
   ["it"]="inneredgedistance",
   ["nl"]="binnenrandafstand",
   ["pe"]="فاصله‌لبه‌داخلی",
   ["ro"]="inneredgedistance",
  },
  ["inneredgewidth"]={
   ["cs"]="inneredgewidth",
   ["de"]="inneredgewidth",
   ["en"]="inneredgewidth",
   ["fr"]="largeurbordinterieur",
   ["it"]="inneredgewidth",
   ["nl"]="binnenrandbreedte",
   ["pe"]="عرض‌لبه‌داخلی",
   ["ro"]="inneredgewidth",
  },
  ["innermargindistance"]={
   ["cs"]="innermargindistance",
   ["de"]="innermargindistance",
   ["en"]="innermargindistance",
   ["fr"]="distancemargeinterieure",
   ["it"]="innermargindistance",
   ["nl"]="binnenmargeafstand",
   ["pe"]="فاصله‌حاشیه‌داخلی",
   ["ro"]="innermargindistance",
  },
  ["innermarginwidth"]={
   ["cs"]="innermarginwidth",
   ["de"]="innermarginwidth",
   ["en"]="innermarginwidth",
   ["fr"]="largeurmargeinterieure",
   ["it"]="innermarginwidth",
   ["nl"]="binnenmargebreedte",
   ["pe"]="عرض‌حاشیه‌داخلی",
   ["ro"]="innermarginwidth",
  },
  ["inothermargin"]={
   ["cs"]="nadruhyokraj",
   ["de"]="inanderermarginale",
   ["en"]="inothermargin",
   ["fr"]="dansautremarge",
   ["it"]="inaltromargine",
   ["nl"]="inanderemarge",
   ["pe"]="درحاشیه‌دیگر",
   ["ro"]="inaltamargine",
  },
  ["inouter"]={
   ["cs"]="inouter",
   ["de"]="inouter",
   ["en"]="inouter",
   ["fr"]="dansexterieur",
   ["it"]="inouter",
   ["nl"]="inbuiten",
   ["pe"]="درخارجی",
   ["ro"]="inouter",
  },
  ["inright"]={
   ["cs"]="vpravo",
   ["de"]="imrechten",
   ["en"]="inright",
   ["fr"]="dansdroite",
   ["it"]="indestra",
   ["nl"]="inrechter",
   ["pe"]="درراست",
   ["ro"]="indreapta",
  },
  ["inrightedge"]={
   ["cs"]="napravo",
   ["de"]="imrechtenrand",
   ["en"]="inrightedge",
   ["fr"]="dansborddroit",
   ["it"]="inlatodestro",
   ["nl"]="inrechterrand",
   ["pe"]="درلبه‌راست",
   ["ro"]="inparteadreapta",
  },
  ["inrightmargin"]={
   ["cs"]="napravyokraj",
   ["de"]="inrechtermarginale",
   ["en"]="inrightmargin",
   ["fr"]="dansmargedroite",
   ["it"]="inmarginedestro",
   ["nl"]="inrechtermarge",
 --["pe"]="درحاشیه‌راست",  -- in/at clash
   ["ro"]="inmargineadreapta",
  },
  ["installlanguage"]={
   ["cs"]="instalacejazyka",
   ["de"]="installieresprache",
   ["en"]="installlanguage",
   ["fr"]="installerlangue",
   ["it"]="installalingua",
   ["nl"]="installeertaal",
   ["pe"]="نصب‌زبان",
   ["ro"]="instalarelimba",
  },
  ["interactionbar"]={
   ["cs"]="interakcnilista",
   ["de"]="interaktionsbalken",
   ["en"]="interactionbar",
   ["fr"]="barreinteraction",
   ["it"]="barrainterazione",
   ["nl"]="interactiebalk",
   ["pe"]="میله‌پانل",
   ["ro"]="barainteractiune",
  },
  ["interactionbuttons"]={
   ["cs"]="interakcnitlacitka",
   ["de"]="interaktionsknopfe",
   ["en"]="interactionbuttons",
   ["fr"]="boutonsinteraction",
   ["it"]="pulsantinterazione",
   ["nl"]="interactiebuttons",
   ["pe"]="دکمه‌پانل",
   ["ro"]="butoaneinteractiune",
  },
  ["interactionmenu"]={
   ["cs"]="interaktivnimenu",
   ["de"]="interaktionsmenue",
   ["en"]="interactionmenu",
   ["fr"]="menuinteraction",
   ["it"]="menuinterattivo",
   ["nl"]="interactiemenu",
   ["pe"]="منوی‌پانل",
   ["ro"]="meniuinteractiune",
  },
  ["item"]={
   ["cs"]="polozka",
   ["de"]="pos",
   ["en"]="item",
   ["fr"]="élément",
   ["it"]="elemento",
   ["nl"]="som",
   ["pe"]="آیتم",
   ["ro"]="element",
  },
  ["itemgroup"]={
   ["en"]="itemgroup",
   ["fr"]="groupedéléments",
  },
  ["items"]={
   ["cs"]="polozky",
   ["de"]="posten",
   ["en"]="items",
   ["fr"]="éléments",
   ["it"]="elementi",
   ["nl"]="items",
   ["pe"]="آیتمها",
   ["ro"]="element",
  },
  ["itemtag"]={
   ["en"]="itemtag",
   ["fr"]="étiquettedélément",
  },
  ["its"]={
   ["cs"]="pol",
   ["de"]="its",
   ["en"]="its",
   ["fr"]="els",
   ["it"]="el",
   ["nl"]="its",
   ["pe"]="its",
   ["ro"]="el",
  },
  ["keepblocks"]={
   ["cs"]="zachovejbloky",
   ["de"]="behaltebloecke",
   ["en"]="keepblocks",
   ["fr"]="conserverblocs",
   ["it"]="tieniblocchi",
   ["nl"]="handhaafblokken",
   ["pe"]="حفظ‌بلوکها",
   ["ro"]="pastreazablocuri",
  },
  ["kerncharacters"]={
   ["en"]="kerncharacters",
   ["fr"]="approchercaracteres",
  },
  ["label"]={
   ["cs"]="poppisek",
   ["de"]="label",
   ["en"]="label",
   ["fr"]="etiquette",
   ["it"]="etichetta",
   ["nl"]="label",
   ["pe"]="برچسب",
   ["ro"]="eticheta",
  },
  ["labels"]={
   ["cs"]="popisky",
   ["de"]="labels",
   ["en"]="labels",
   ["fr"]="etiquettes",
   ["it"]="etichette",
   ["nl"]="labels",
   ["pe"]="برچسبها",
   ["ro"]="etichete",
  },
  ["labeltext"]={
   ["cs"]="textpopisku",
   ["de"]="labeltext",
   ["en"]="labeltext",
   ["fr"]="texteetiquette",
   ["it"]="testoetichetta",
   ["nl"]="labeltekst",
   ["pe"]="متن‌برچسب",
   ["ro"]="texteticheta",
  },
  ["language"]={
   ["cs"]="jazyk",
   ["de"]="sprache",
   ["en"]="language",
   ["fr"]="langue",
   ["it"]="lingua",
   ["nl"]="taal",
   ["pe"]="زبان",
   ["ro"]="limba",
  },
  ["languageCharacters"]={
   ["en"]="languageCharacters",
   ["fr"]="Caractereslangue",
  },
  ["languagecharacters"]={
   ["en"]="languagecharacters",
   ["fr"]="caractereslangue",
  },
  ["leftaligned"]={
   ["cs"]="zarovnanovlevo",
   ["de"]="linksbuendig",
   ["en"]="leftaligned",
   ["fr"]="aligneagauche",
   ["it"]="allineasinistra",
   ["nl"]="regellinks",
   ["pe"]="چپ‌چین",
   ["ro"]="aliniatstanga",
  },
  ["leftbottombox"]={
   ["en"]="leftbottombox",
   ["fr"]="boitebasgauche",
  },
  ["leftedgedistance"]={
   ["cs"]="vzdalenostlevehookraje",
   ["de"]="abstandlinkerrand",
   ["en"]="leftedgedistance",
   ["fr"]="distancebordgauche",
   ["it"]="distanzabordosinistro",
   ["nl"]="linkerrandafstand",
   ["pe"]="فاصله‌لبه‌چپ",
   ["ro"]="distantacoltstanga",
  },
  ["leftedgewidth"]={
   ["cs"]="sirkalevehookraje",
   ["de"]="breitelinkerrand",
   ["en"]="leftedgewidth",
   ["fr"]="largeurbordgauche",
   ["it"]="ampiezzabordosinistro",
   ["nl"]="linkerrandbreedte",
   ["pe"]="عرض‌لبه‌چپ",
   ["ro"]="latimecoltstanga",
  },
  ["leftline"]={
   ["en"]="leftline",
   ["fr"]="lignegauche",
  },
  ["leftmargindistance"]={
   ["cs"]="vzdalenostlevemarginalie",
   ["de"]="linkemarginalafstand",
   ["en"]="leftmargindistance",
   ["fr"]="distancemargegauche",
   ["it"]="distanzamarginesinistro",
   ["nl"]="linkermargeafstand",
   ["pe"]="فاصله‌حاشیه‌چپ",
   ["ro"]="distantamarginestanga",
  },
  ["leftmarginwidth"]={
   ["cs"]="sirkalevemarginalie",
   ["de"]="linkemarginalbreite",
   ["en"]="leftmarginwidth",
   ["fr"]="largeurmargegauche",
   ["it"]="ampiezzamarginesinistro",
   ["nl"]="linkermargebreedte",
   ["pe"]="عرض‌حاشیه‌چپ",
   ["ro"]="latimemarginestanga",
  },
  ["leg"]={
   ["cs"]="leg",
   ["de"]="leg",
   ["en"]="leg",
   ["fr"]="leg",
   ["it"]="leg",
   ["nl"]="leg",
   ["pe"]="پا",
   ["ro"]="leg",
  },
  ["linecorrection"]={
   ["en"]="linecorrection",
   ["fr"]="correctionligne",
   ["nl"]="regelcorrectie",
  },
  ["linenote"]={
   ["en"]="linenote",
   ["fr"]="notedeligne",
  },
  ["linespanningtext"]={
   ["en"]="linespanningtext",
   ["fr"]="largeurtextecouvrant",
  },
  ["linewidth"]={
   ["cs"]="tloustkacary",
   ["de"]="liniendicke",
   ["en"]="linethickness",
   ["fr"]="largeurligne",
   ["it"]="spessoreriga",
   ["nl"]="lijndikte",
   ["pe"]="عرض‌خط",
   ["ro"]="grosimelinie",
  },
  ["listheight"]={
   ["cs"]="vyskaseznamu",
   ["de"]="listenhoehe",
   ["en"]="listheight",
   ["fr"]="hauteurliste",
   ["it"]="altezzaelenco",
   ["nl"]="lijsthoogte",
   ["pe"]="ارتفاع‌خط",
   ["ro"]="inaltimelista",
  },
  ["listlength"]={
   ["cs"]="delkaseznamu",
   ["de"]="listenlaenge",
   ["en"]="listlength",
   ["fr"]="longueurtexte",
   ["it"]="lunghezzaelenco",
   ["nl"]="lijstlengte",
   ["pe"]="طول‌لیست",
   ["ro"]="lungimelista",
  },
  ["listsymbol"]={
   ["cs"]="listsymbol",
   ["de"]="listsymbol",
   ["en"]="listsymbol",
   ["fr"]="symboleliste",
   ["it"]="listsymbol",
   ["nl"]="lijstsymbool",
   ["pe"]="نمادلیست",
   ["ro"]="listsymbol",
  },
  ["listwidth"]={
   ["cs"]="sirkaseznamu",
   ["de"]="listenbreite",
   ["en"]="listwidth",
   ["fr"]="largeurliste",
   ["it"]="ampiezzaelenco",
   ["nl"]="lijstbreedte",
   ["pe"]="عرض‌لیست",
   ["ro"]="latimelista",
  },
  ["loadfontgoodies"]={
   ["en"]="loadfontgoodies",
   ["fr"]="charerfriandisespolice",
  },
  ["localfootnotes"]={
   ["en"]="localfootnotes",
   ["fr"]="notesbdplocales",
   ["nl"]="lokalevoetnoten",
  },
  ["localnotes"]={
   ["en"]="localnotes",
   ["fr"]="noteslocales",
  },
  ["logfields"]={
   ["cs"]="zaznamovepole",
   ["de"]="registrierefelder",
   ["en"]="logfields",
   ["fr"]="logchamps",
   ["it"]="logcampi",
   ["nl"]="registreervelden",
   ["pe"]="میدانهای‌گزارش",
   ["ro"]="logcampuri",
  },
  ["lohi"]={
   ["cs"]="nivy",
   ["de"]="tiho",
   ["en"]="lohi",
   ["fr"]="inex",
   ["it"]="pedap",
   ["nl"]="laho",
   ["pe"]="پابا",
   ["ro"]="jossus",
  },
  ["low"]={
   ["cs"]="nizky",
   ["de"]="tief",
   ["en"]="low",
   ["fr"]="indice",
   ["it"]="ped",
   ["nl"]="laag",
   ["pe"]="پایین",
   ["ro"]="jos",
  },
  ["macroname"]={
   ["cs"]="jmeno",
   ["de"]="name",
   ["en"]="name",
   ["fr"]="nommacro",
   ["it"]="nome",
   ["nl"]="naam",
   ["pe"]="نام‌ماکرو",
   ["ro"]="nume",
  },
  ["mainlanguage"]={
   ["cs"]="hlavnijazyk",
   ["de"]="hauptsprache",
   ["en"]="mainlanguage",
   ["fr"]="langueprincipale",
   ["it"]="linguaprincipale",
   ["nl"]="hoofdtaal",
   ["pe"]="زبان‌اصلی",
   ["ro"]="limbaprincipala",
  },
  ["makeupheight"]={
   ["cs"]="vyskasazby",
   ["de"]="satzhoehe",
   ["en"]="makeupheight",
   ["fr"]="hauteurcomposition",
   ["it"]="altezzamakeup",
   ["nl"]="zethoogte",
   ["pe"]="ارتفاع‌آرایش",
   ["ro"]="inaltimemakeup",
  },
  ["makeupwidth"]={
   ["cs"]="sirkasazby",
   ["de"]="satzbreite",
   ["en"]="makeupwidth",
   ["fr"]="largeurcomposition",
   ["it"]="ampiezzamakeup",
   ["nl"]="zetbreedte",
   ["pe"]="عرض‌آرایش",
   ["ro"]="latimemakeup",
  },
  ["mar"]={
   ["cs"]="okr",
   ["de"]="mar",
   ["en"]="mar",
   ["fr"]="rem",
   ["it"]="mar",
   ["nl"]="mar",
   ["pe"]="حاش",
   ["ro"]="mar",
  },
  ["marginblock"]={
   ["en"]="marginblock",
   ["fr"]="blocmarginal",
   ["nl"]="margeblok",
  },
  ["margindistance"]={
   ["cs"]="vzdalenostmarginalie",
   ["de"]="marginalafstand",
   ["en"]="margindistance",
   ["fr"]="distancemarge",
   ["it"]="distanzamargine",
   ["nl"]="margeafstand",
   ["pe"]="فاصله‌حاشیه",
   ["ro"]="distantamargine",
  },
  ["marginrule"]={
   ["cs"]="marginalnilinka",
   ["de"]="marginallinie",
   ["en"]="marginrule",
   ["fr"]="traitmarge",
   ["it"]="lineamargine",
   ["nl"]="kantlijn",
   ["pe"]="خط‌حاشیه",
   ["ro"]="liniemargine",
  },
  ["margintext"]={
   ["cs"]="marginalnitext",
   ["de"]="marginaltext",
   ["en"]="margintext",
   ["fr"]="textemarge",
   ["it"]="testoinmargine",
   ["nl"]="margetekst",
   ["pe"]="متن‌حاشیه",
   ["ro"]="textmarginal",
  },
  ["margintitle"]={
   ["cs"]="marginalninadpis",
   ["de"]="marginaltitel",
   ["en"]="margintitle",
   ["fr"]="titremarge",
   ["it"]="titoloinmargine",
   ["nl"]="margetitel",
   ["pe"]="عنوان‌حاشیه",
   ["ro"]="titlumarginal",
  },
  ["marginwidth"]={
   ["cs"]="sirkamarginalie",
   ["de"]="marginalbreite",
   ["en"]="marginwidth",
   ["fr"]="largeurmarge",
   ["it"]="ampiezzamargine",
   ["nl"]="margebreedte",
   ["pe"]="عرض‌حاشیه",
   ["ro"]="latimemargine",
  },
  ["marginword"]={
   ["cs"]="marginalnislovo",
   ["de"]="marginalwort",
   ["en"]="marginword",
   ["fr"]="motmarge",
   ["it"]="parolainmargine",
   ["nl"]="margewoord",
   ["pe"]="کلمه‌حاشیه",
   ["ro"]="cuvantmarginal",
  },
  ["marking"]={
   ["cs"]="znaceni",
   ["de"]="beschriftung",
   ["en"]="marking",
   ["fr"]="marquage",
   ["it"]="marcatura",
   ["nl"]="markeer",
   ["pe"]="نشانه‌گذاری",
   ["ro"]="marcaje",
  },
  ["markversion"]={
   ["cs"]="oznacverzi",
   ["de"]="beschrifteversion",
   ["en"]="markversion",
   ["fr"]="marquerversion",
   ["it"]="marcaversione",
   ["nl"]="markeerversie",
   ["pe"]="نسخه‌نشانه",
   ["ro"]="marcheazaversiune",
  },
  ["mathematics"]={
   ["cs"]="matematika",
   ["de"]="mathematik",
   ["en"]="mathematics",
   ["fr"]="mathematics",
   ["it"]="matematica",
   ["nl"]="wiskunde",
   ["pe"]="ریاضی",
   ["ro"]="matematica",
  },
  ["menubutton"]={
   ["cs"]="tlacitkomenu",
   ["de"]="menueknopf",
   ["en"]="menubutton",
   ["fr"]="boutonmenu",
   ["it"]="pulsantemenu",
   ["nl"]="menubutton",
   ["pe"]="دکمه‌منو",
   ["ro"]="butonmeniu",
  },
  ["mframed"]={
   ["cs"]="maoramovani",
   ["de"]="maumrise",
   ["en"]="maframed",
   ["fr"]="encadremath",
   ["it"]="maframed",
   ["nl"]="wilijnd",
   ["pe"]="mframed",
   ["ro"]="maframed",
  },
  ["midaligned"]={
   ["cs"]="zarovnanonastred",
   ["de"]="zentriert",
   ["en"]="midaligned",
   ["fr"]="aligneaumilieu",
   ["it"]="allineacentro",
   ["nl"]="regelmidden",
   ["pe"]="تنظیم‌وسط",
   ["ro"]="aliniatcentru",
  },
  ["mirror"]={
   ["cs"]="zrcadlit",
   ["de"]="spiegeln",
   ["en"]="mirror",
   ["fr"]="refleter",
   ["it"]="riflessione",
   ["nl"]="spiegel",
   ["pe"]="آینه",
   ["ro"]="reflexie",
  },
  ["month"]={
   ["cs"]="mesic",
   ["de"]="monat",
   ["en"]="month",
   ["fr"]="mois",
   ["it"]="mese",
   ["nl"]="maand",
   ["pe"]="ماه",
   ["ro"]="luna",
  },
  ["moveformula"]={
   ["cs"]="moveformula",
   ["de"]="moveformula",
   ["en"]="moveformula",
   ["fr"]="deplacerformule",
   ["it"]="spostaformula",
   ["nl"]="verplaatsformule",
   ["pe"]="انتقال‌فرمول",
   ["ro"]="moveformula",
  },
  ["moveongrid"]={
   ["cs"]="premistinamrizku",
   ["de"]="amgitterneuausrichten",
   ["en"]="moveongrid",
   ["fr"]="deplacersurgrille",
   ["it"]="spostaagriglia",
   ["nl"]="verplaatsopgrid",
   ["pe"]="انتقال‌به‌توری",
   ["ro"]="mutapegrid",
  },
  ["movesidefloat"]={
   ["cs"]="movesidefloat",
   ["de"]="movesidefloat",
   ["en"]="movesidefloat",
   ["fr"]="deplacerflottantcote",
   ["it"]="movesidefloat",
   ["nl"]="verplaatszijblok",
   ["pe"]="انتقال‌کنار‌شناور",
   ["ro"]="movesidefloat",
  },
  ["navigating"]={
   ["cs"]="navigating",
   ["de"]="navigating",
   ["en"]="navigating",
   ["fr"]="navigation",
   ["it"]="navigating",
   ["nl"]="navigerend",
   ["pe"]="هدایت",
   ["ro"]="navigating",
  },
  ["nodimension"]={
   ["cs"]="zadnyrozmer",
   ["de"]="keindimension",
   ["en"]="nodimension",
   ["fr"]="sansdimension",
   ["it"]="nientedimensioni",
   ["nl"]="geendimensie",
   ["pe"]="بدون‌بعد",
   ["ro"]="faradimensiune",
  },
  ["noheaderandfooterlines"]={
   ["cs"]="zadnezahlaviaupati",
   ["de"]="keinekopfundfusszeilen",
   ["en"]="noheaderandfooterlines",
   ["fr"]="sansentêtenipdp",
   ["it"]="nientelineintestazionepdp",
   ["nl"]="geenhoofdenvoetregels",
   ["pe"]="بدون‌خط‌سروته‌برگ",
   ["ro"]="faraliniiantetsisubsol",
  },
  ["noindenting"]={
   ["cs"]="zadneodsazovani",
   ["de"]="nichteinziehen",
   ["en"]="noindenting",
   ["fr"]="sansindentation",
   ["it"]="nienterientro",
   ["nl"]="nietinspringen",
   ["pe"]="بدون‌تورفتگی",
   ["ro"]="faraaliniat",
  },
  ["noitem"]={
   ["en"]="noitem",
   ["fr"]="sansélément",
  },
  ["nolist"]={
   ["cs"]="zadnyseznam",
   ["de"]="keineliste",
   ["en"]="nolist",
   ["fr"]="sansliste",
   ["it"]="nienteelenco",
   ["nl"]="geenlijst",
   ["pe"]="بدون‌لیست",
   ["ro"]="faralista",
  },
  ["nomarking"]={
   ["cs"]="zadneznaceni",
   ["de"]="keinebeschriftung",
   ["en"]="nomarking",
   ["fr"]="sansmarquage",
   ["it"]="nientemarcatura",
   ["nl"]="geenmarkering",
   ["pe"]="بدون‌نشانه‌گذاری",
   ["ro"]="faramarcaje",
  },
  ["nomoreblocks"]={
   ["cs"]="zadnedalsibloky",
   ["de"]="keinebloeckemehr",
   ["en"]="nomoreblocks",
   ["fr"]="sansplusdeblocs",
   ["it"]="bastablocchi",
   ["nl"]="geenblokkenmeer",
   ["pe"]="بدون‌بلوکهای‌بیشتر",
   ["ro"]="gatablocuri",
  },
  ["nomorefiles"]={
   ["cs"]="zadnedalsisoubory",
   ["de"]="keinedateienmehr",
   ["en"]="nomorefiles",
   ["fr"]="sansplusdefichiers",
   ["it"]="bastafile",
   ["nl"]="geenfilesmeer",
   ["pe"]="بدون‌فایلهای‌بیشتر",
   ["ro"]="farafisiere",
  },
  ["nop"]={
   ["cs"]="nop",
   ["de"]="nop",
   ["en"]="nop",
   ["fr"]="san",
   ["it"]="nop",
   ["nl"]="nop",
   ["pe"]="منفی",
   ["ro"]="nop",
  },
  ["nospace"]={
   ["cs"]="zadnamezera",
   ["de"]="keinspatium",
   ["en"]="nospace",
   ["fr"]="sansespace",
   ["it"]="nientespazio",
   ["nl"]="geenspatie",
   ["pe"]="بدون‌فضا",
   ["ro"]="faraspatiu",
  },
  ["note"]={
   ["cs"]="poznamka",
   ["de"]="notiz",
   ["en"]="note",
   ["fr"]="note",
   ["it"]="nota",
   ["nl"]="noot",
   ["pe"]="یادداشت",
   ["ro"]="nota",
  },
  ["notesymbol"]={
   ["en"]="notesymbol",
   ["fr"]="symbolenote",
  },
  ["notopandbottomlines"]={
   ["cs"]="zadnehorniadolniradky",
   ["de"]="keinzeilenobenundunten",
   ["en"]="notopandbottomlines",
   ["fr"]="sansligneshautetbas",
   ["it"]="nientelineecimafondo",
   ["nl"]="geenbovenenonderregels",
   ["pe"]="بدون‌خط‌بالاوپایین",
   ["ro"]="faraliniisussijos",
  },
  ["notsmallcapped"]={
   ["cs"]="nokap",
   ["de"]="nokap",
   ["en"]="nocap",
   ["fr"]="paspetitescaps",
   ["it"]="nocap",
   ["nl"]="nokap",
   ["pe"]="سرپوش‌کوچک‌نه",
   ["ro"]="nokap",
  },
  ["nowhitespace"]={
   ["cs"]="zadnebilemisto",
   ["de"]="keinzwischenraum",
   ["en"]="nowhitespace",
   ["fr"]="sansespaceblanc",
   ["it"]="nientespaziobianco",
   ["nl"]="geenwitruimte",
   ["pe"]="بدون‌فضای‌سفید",
   ["ro"]="faraspatiualb",
  },
  ["numberofsubpages"]={
   ["cs"]="numberofsubpages",
   ["de"]="numberofsubpages",
   ["en"]="numberofsubpages",
   ["fr"]="nombredesouspages",
   ["it"]="numberofsubpages",
   ["nl"]="aantalsubpaginas",
   ["pe"]="شماره‌زیرصفحه",
   ["ro"]="numberofsubpages",
  },
  ["numbers"]={
   ["cs"]="cisla",
   ["de"]="ziffern",
   ["en"]="numbers",
   ["fr"]="numeros",
   ["it"]="numeri",
   ["nl"]="cijfers",
   ["pe"]="شماره‌ها",
   ["ro"]="numere",
  },
  ["outeredgedistance"]={
   ["cs"]="outeredgedistance",
   ["de"]="outeredgedistance",
   ["en"]="outeredgedistance",
   ["fr"]="distancebordexterieur",
   ["it"]="outeredgedistance",
   ["nl"]="buitenrandafstand",
   ["pe"]="فاصله‌لبه‌خارجی",
   ["ro"]="outeredgedistance",
  },
  ["outeredgewidth"]={
   ["cs"]="outeredgewidth",
   ["de"]="outeredgewidth",
   ["en"]="outeredgewidth",
   ["fr"]="largeurbordexterieur",
   ["it"]="outeredgewidth",
   ["nl"]="buitenrandbreedte",
   ["pe"]="عرض‌لبه‌خارجی",
   ["ro"]="outeredgewidth",
  },
  ["outermargindistance"]={
   ["cs"]="outermargindistance",
   ["de"]="outermargindistance",
   ["en"]="outermargindistance",
   ["fr"]="distancemargeexterieure",
   ["it"]="outermargindistance",
   ["nl"]="buitenmargeafstand",
   ["pe"]="فاصله‌حاشیه‌خارجی",
   ["ro"]="outermargindistance",
  },
  ["outermarginwidth"]={
   ["cs"]="outermarginwidth",
   ["de"]="outermarginwidth",
   ["en"]="outermarginwidth",
   ["fr"]="largeurmargeexterieure",
   ["it"]="outermarginwidth",
   ["nl"]="buitenmargebreedte",
   ["pe"]="عرض‌حاشیه‌خارجی",
   ["ro"]="outermarginwidth",
  },
  ["packed"]={
   ["cs"]="zhustene",
   ["de"]="kleinerdurchschuss",
   ["en"]="packed",
   ["fr"]="sansespaceentreitem",
   ["it"]="impaccato",
   ["nl"]="opelkaar",
   ["pe"]="فشرده",
   ["ro"]="impachetat",
  },
  ["page"]={
   ["cs"]="strana",
   ["de"]="seite",
   ["en"]="page",
   ["fr"]="page",
   ["it"]="pagina",
   ["nl"]="pagina",
   ["pe"]="صفحه",
   ["ro"]="pagina",
  },
  ["pagebreak"]={
   ["en"]="pagebreak",
   ["fr"]="sautdepage",
  },
  ["pagedepth"]={
   ["cs"]="pagedepth",
   ["de"]="pagedepth",
   ["en"]="pagedepth",
   ["fr"]="profondeurpage",
   ["it"]="pagedepth",
   ["nl"]="paginadiepte",
   ["pe"]="عمق‌صفحه",
   ["ro"]="pagedepth",
  },
  ["pagenumber"]={
   ["cs"]="cislostrany",
   ["de"]="seitenummer",
   ["en"]="pagenumber",
   ["fr"]="numeropage",
   ["it"]="numeropagina",
   ["nl"]="paginanummer",
   ["pe"]="شماره‌صفحه",
   ["ro"]="numarpagina",
  },
  ["pageoffset"]={
   ["cs"]="pageoffset",
   ["de"]="pageoffset",
   ["en"]="pageoffset",
   ["fr"]="decalagepage",
   ["it"]="pageoffset",
   ["nl"]="paginaoffset",
   ["pe"]="آفست‌صفحه",
   ["ro"]="pageoffset",
  },
  ["pagereference"]={
   ["cs"]="odkaznastranu",
   ["de"]="seitenreferenz",
   ["en"]="pagereference",
   ["fr"]="referencepage",
   ["it"]="riferimentopagina",
   ["nl"]="paginareferentie",
   ["pe"]="مرجع‌صفحه",
   ["ro"]="referintapagina",
  },
  ["paperheight"]={
   ["cs"]="vyskapapiru",
   ["de"]="papierhoehe",
   ["en"]="paperheight",
   ["fr"]="hauteurpapier",
   ["it"]="altezzacarta",
   ["nl"]="papierhoogte",
   ["pe"]="ارتفاع‌برگ",
   ["ro"]="inaltimehartie",
  },
  ["paperwidth"]={
   ["cs"]="sirkapapiru",
   ["de"]="papierbreite",
   ["en"]="paperwidth",
   ["fr"]="largeurpapier",
   ["it"]="ampiezzacarta",
   ["nl"]="papierbreedte",
   ["pe"]="عرض‌برگ",
   ["ro"]="latimehartie",
  },
  ["periods"]={
   ["cs"]="tecky",
   ["de"]="punkt",
   ["en"]="periods",
   ["fr"]="periodes",
   ["it"]="punti",
   ["nl"]="punten",
   ["pe"]="نقطه‌ها",
   ["ro"]="puncte",
  },
  ["placebookmarks"]={
   ["cs"]="umistizalozky",
   ["de"]="platzierebookmarks",
   ["en"]="placebookmarks",
   ["fr"]="placermarquepages",
   ["it"]="mettisegnalibro",
   ["nl"]="plaatsbookmarks",
   ["pe"]="درج‌چوب‌خط",
   ["ro"]="plaseazasemnecarte",
  },
  ["placecombinedlist"]={
   ["cs"]="umistikombinovanyseznam",
   ["de"]="platzierezusammengestellteliste",
   ["en"]="placecombinedlist",
   ["fr"]="placerlistecombinee",
   ["it"]="mettielencocombinato",
   ["nl"]="plaatssamengesteldelijst",
   ["pe"]="درج‌لیست‌مختلط",
   ["ro"]="punelistacombinata",
  },
  ["placefloat"]={
   ["cs"]="placefloat",
   ["de"]="placefloat",
   ["en"]="placefloat",
   ["fr"]="placerflottant",
   ["it"]="placefloat",
   ["nl"]="plaatsplaatsblok",
   ["pe"]="درج‌شناور",
   ["ro"]="placefloat",
  },
  ["placefootnotes"]={
   ["cs"]="umistipoznamkypodcarou",
   ["de"]="platzierefussnoten",
   ["en"]="placefootnotes",
   ["fr"]="placernotesbdp",
   ["it"]="mettinotepdp",
   ["nl"]="plaatsvoetnoten",
   ["pe"]="درج‌پانوشتها",
   ["ro"]="punenotesubsol",
  },
  ["placeformula"]={
   ["cs"]="umistirovnici",
   ["de"]="platziereformel",
   ["en"]="placeformula",
   ["fr"]="placerformule",
   ["it"]="mettiformula",
   ["nl"]="plaatsformule",
   ["pe"]="درج‌فرمول",
   ["ro"]="puneformula",
  },
  ["placeheadnumber"]={
   ["cs"]="placeheadnumber",
   ["de"]="placeheadnumber",
   ["en"]="placeheadnumber",
   ["fr"]="placernumerotete",
   ["it"]="posizionanumerotesta",
   ["nl"]="plaatskopnummer",
   ["pe"]="درج‌شماره‌سر",
   ["ro"]="placeheadnumber",
  },
  ["placeheadtext"]={
   ["cs"]="placeheadtext",
   ["de"]="placeheadtext",
   ["en"]="placeheadtext",
   ["fr"]="placertextetete",
   ["it"]="posizionatestotesta",
   ["nl"]="plaatskoptekst",
   ["pe"]="درج‌متن‌سر",
   ["ro"]="placeheadtext",
  },
  ["placelegend"]={
   ["cs"]="umistilegendu",
   ["de"]="platzierelegende",
   ["en"]="placelegend",
   ["fr"]="placerlegende",
   ["it"]="mettilegenda",
   ["nl"]="plaatslegenda",
   ["pe"]="درج‌راهنما",
   ["ro"]="punelegenda",
  },
  ["placelist"]={
   ["cs"]="umistiseznam",
   ["de"]="platziereliste",
   ["en"]="placelist",
   ["fr"]="placerliste",
   ["it"]="mettielenco",
   ["nl"]="plaatslijst",
   ["pe"]="درج‌لیست",
   ["ro"]="punelista",
  },
  ["placelistofsynonyms"]={
   ["en"]="placelistofsynonyms",
   ["fr"]="placerlistedesynonymes",
   ["nl"]="plaatslijstmetsynoniemen",
  },
  ["placelocalfootnotes"]={
   ["cs"]="umistilokalnipoznamkypodcarou",
   ["de"]="platzierelokalefussnoten",
   ["en"]="placelocalfootnotes",
   ["fr"]="placernotesbdplocales",
   ["it"]="mettinotepdplocali",
   ["nl"]="plaatslokalevoetnoten",
   ["pe"]="درج‌پانوشتهای‌موضعی",
   ["ro"]="punenotesubsollocale",
  },
  ["placelocalnotes"]={
   ["en"]="placelocalnotes",
   ["fr"]="placernoteslocales",
  },
  ["placelogos"]={
   ["cs"]="umistiloga",
   ["de"]="platzierelogo",
   ["en"]="placelogos",
   ["fr"]="placerlogo",
   ["it"]="mettiloghi",
   ["nl"]="plaatsbeeldmerken",
   ["pe"]="درج‌آرمها",
   ["ro"]="punelogouri",
  },
  ["placenamedfloat"]={
   ["en"]="placenamedfloat",
   ["fr"]="placerflottantnomme",
  },
  ["placenotes"]={
   ["en"]="placenotes",
   ["fr"]="placernotes",
  },
  ["placeongrid"]={
   ["cs"]="umistinamrizku",
   ["de"]="amgitterausrichten",
   ["en"]="placeongrid",
   ["fr"]="placersurgrille",
   ["it"]="mettiingriglia",
   ["nl"]="plaatsopgrid",
   ["pe"]="درج‌در‌توری",
   ["ro"]="plaseazapegrid",
  },
  ["placeontopofeachother"]={
   ["cs"]="umistinadsebe",
   ["de"]="platziereuntereinander",
   ["en"]="placeontopofeachother",
   ["fr"]="placerlunaudessusdelautre",
   ["it"]="mettiunosullaltro",
   ["nl"]="plaatsonderelkaar",
   ["pe"]="درج‌در‌بالای‌یکدیگر",
   ["ro"]="punedeasuprafiecareia",
  },
  ["placepagenumber"]={
   ["cs"]="placepagenumber",
   ["de"]="placepagenumber",
   ["en"]="placepagenumber",
   ["fr"]="placernumeropage",
   ["it"]="mettinumeropagina",
   ["nl"]="plaatspaginanummer",
   ["pe"]="درج‌شماره‌صفحه",
   ["ro"]="punenumarpagina",
  },
  ["placerawlist"]={
   ["cs"]="placerawlist",
   ["de"]="placerawlist",
   ["en"]="placerawlist",
   ["fr"]="placerlistebrute",
   ["it"]="placerawlist",
   ["nl"]="plaatsruwelijst",
   ["pe"]="درج‌لیست‌خام",
   ["ro"]="placerawlist",
  },
  ["placereferencelist"]={
   ["cs"]="placereferencelist",
   ["de"]="placereferencelist",
   ["en"]="placereferencelist",
   ["fr"]="placerlistereference",
   ["it"]="placereferencelist",
   ["nl"]="plaatsreferentielijst",
   ["pe"]="درج‌لیست‌مرجع",
   ["ro"]="placereferencelist",
  },
  ["placeregister"]={
   ["cs"]="umistirejstrik",
   ["de"]="platziereregister",
   ["en"]="placeregister",
   ["fr"]="placerregistre",
   ["it"]="mettiregistro",
   ["nl"]="plaatsregister",
   ["pe"]="درج‌ثبت",
   ["ro"]="puneregistru",
  },
  ["placerule"]={
   ["cs"]="placerule",
   ["de"]="placerule",
   ["en"]="placerule",
   ["fr"]="placertrait",
   ["it"]="mettilinea",
   ["nl"]="plaatslijn",
   ["pe"]="درج‌خط",
   ["ro"]="punerigla",
  },
  ["placesidebyside"]={
   ["cs"]="umistivedlesebe",
   ["de"]="platzierenebeneinander",
   ["en"]="placesidebyside",
   ["fr"]="placercoteacote",
   ["it"]="mettifiancoafianco",
   ["nl"]="plaatsnaastelkaar",
   ["pe"]="درج‌کنار‌به‌کنار",
   ["ro"]="punefatainfata",
  },
  ["placesubformula"]={
   ["cs"]="umistipodrovnici",
   ["de"]="platziereunterformel",
   ["en"]="placesubformula",
   ["fr"]="placersousformule",
   ["it"]="mettisottoformula",
   ["nl"]="plaatssubformule",
   ["pe"]="درج‌زیرفرمول",
   ["ro"]="punesubformula",
  },
  ["placetextvariable"]={
   ["cs"]="placetextvariable",
   ["de"]="placetextvariable",
   ["en"]="placetextvariable",
   ["fr"]="placervariabletexte",
   ["it"]="mettivariabiletesto",
   ["nl"]="plaatstekstvariabele",
   ["pe"]="درج‌متغیرمتن",
   ["ro"]="punevariabilatext",
  },
  ["position"]={
   ["cs"]="pozice",
   ["de"]="position",
   ["en"]="position",
   ["fr"]="position",
   ["it"]="posizione",
   ["nl"]="positioneer",
   ["pe"]="مکان",
   ["ro"]="pozitie",
  },
  ["positiontext"]={
   ["cs"]="positiontext",
   ["de"]="positiontext",
   ["en"]="positiontext",
   ["fr"]="texteposition",
   ["it"]="posizionatesto",
   ["nl"]="positioneertekst",
   ["pe"]="مکان‌متن",
   ["ro"]="pozitietext",
  },
  ["postponenotes"]={
   ["en"]="postponenotes",
   ["fr"]="reporternotes",
  },
  ["postponingnotes"]={
   ["en"]="postponingnotes",
   ["fr"]="reportnotes",
  },
  ["predefinedfont"]={
   ["en"]="predefinedfont",
   ["fr"]="policepredefinie",
  },
  ["predefinefont"]={
   ["en"]="predefinefont",
   ["fr"]="predéfinirpolice",
  },
  ["presetdocument"]={
   ["en"]="presetdocument",
   ["fr"]="prereglerdocument",
  },
  ["prettyprintbuffer"]={
   ["en"]="prettyprintbuffer",
   ["fr"]="imprimerjolimenttampon",
  },
  ["preventmode"]={
   ["en"]="preventmode",
   ["fr"]="empechermode",
  },
  ["printpaperheight"]={
   ["cs"]="vyskatiskpapiru",
   ["de"]="printpapierhoehe",
   ["en"]="printpaperheight",
   ["fr"]="hauteurpapierimpression",
   ["it"]="altezzacartastampa",
   ["nl"]="printpapierhoogte",
   ["pe"]="چاپ‌ارتفاع‌برگ",
   ["ro"]="inaltimehartieimprimanta",
  },
  ["printpaperwidth"]={
   ["cs"]="sirkatiskpapiru",
   ["de"]="printpapierbreite",
   ["en"]="printpaperwidth",
   ["fr"]="largeurpapierimpression",
   ["it"]="ampiezzacartastampa",
   ["nl"]="printpapierbreedte",
   ["pe"]="چاپ‌عرض‌برگ",
   ["ro"]="latimehartieimprimanta",
  },
  ["processblocks"]={
   ["cs"]="zpracujbloky",
   ["de"]="bearbeitebloecke",
   ["en"]="processblocks",
   ["fr"]="traiterblocs",
   ["it"]="elaborablocchi",
   ["nl"]="verwerkblokken",
   ["pe"]="بلوکهای‌پردازش",
   ["ro"]="proceseazabloc",
  },
  ["processbodyfontenvironmentlist"]={
   ["en"]="processbodyfontenvironmentlist",
   ["fr"]="traiterlisteenvironnementpolicedecorps",
  },
  ["processpage"]={
   ["cs"]="zpracujstranu",
   ["de"]="bearbeiteseite",
   ["en"]="processpage",
   ["fr"]="traiterpage",
   ["it"]="elaborapagina",
   ["nl"]="verwerkpagina",
   ["pe"]="صفحه‌پردازش",
   ["ro"]="proceseazapagina",
  },
  ["product"]={
   ["cs"]="produkt",
   ["de"]="produkt",
   ["en"]="product",
   ["fr"]="produit",
   ["it"]="prodotto",
   ["nl"]="produkt",
   ["pe"]="تولید",
   ["ro"]="produs",
  },
  ["program"]={
   ["cs"]="program",
   ["de"]="programm",
   ["en"]="program",
   ["fr"]="programme",
   ["it"]="programma",
   ["nl"]="programma",
   ["pe"]="برنامه",
   ["ro"]="program",
  },
  ["project"]={
   ["cs"]="projekt",
   ["de"]="projekt",
   ["en"]="project",
   ["fr"]="projet",
   ["it"]="progetto",
   ["nl"]="projekt",
   ["pe"]="پروژه",
   ["ro"]="proiect",
  },
  ["publication"]={
   ["cs"]="publikace",
   ["de"]="publikation",
   ["en"]="publication",
   ["fr"]="publication",
   ["it"]="pubblicazione",
   ["nl"]="publicatie",
   ["pe"]="نشر",
   ["ro"]="publicatie",
  },
  ["quotation"]={
   ["cs"]="citace",
   ["de"]="zitat",
   ["en"]="quotation",
   ["fr"]="citation",
   ["it"]="citazione",
   ["nl"]="citaat",
   ["pe"]="نقل‌قول",
   ["ro"]="citat",
  },
  ["quote"]={
   ["cs"]="citovat",
   ["de"]="zitieren",
   ["en"]="quote",
   ["fr"]="citer",
   ["it"]="menzione",
   ["nl"]="citeer",
   ["pe"]="نقل",
   ["ro"]="minicitat",
  },
  ["ran"]={
   ["cs"]="ran",
   ["de"]="ran",
   ["en"]="ran",
   ["fr"]="ran",
   ["it"]="ran",
   ["nl"]="ran",
   ["pe"]="ran",
   ["ro"]="ran",
  },
  ["redo"]={
   ["cs"]="opakovat",
   ["de"]="wiederholen",
   ["en"]="redo",
   ["fr"]="refaire",
   ["it"]="rifai",
   ["nl"]="herhaal",
   ["pe"]="انجام‌دوباره",
   ["ro"]="refa",
  },
  ["redoconvertfont"]={
   ["en"]="redoconvertfont",
   ["fr"]="refaireconvertirpolice",
  },
  ["ref"]={
   ["cs"]="ref",
   ["de"]="ref",
   ["en"]="ref",
   ["fr"]="ref",
   ["it"]="rif",
   ["nl"]="ref",
   ["pe"]="رج",
   ["ro"]="ref",
  },
  ["reference"]={
   ["cs"]="odkaz",
   ["de"]="referenz",
   ["en"]="reference",
   ["fr"]="reference",
   ["it"]="riferimento",
   ["nl"]="referentie",
   ["pe"]="مرجع",
   ["ro"]="referinta",
  },
  ["referral"]={
   ["cs"]="odkaz",
   ["de"]="verweis",
   ["en"]="referral",
   ["fr"]="redirection",
   ["it"]="referral",
   ["nl"]="kenmerk",
   ["pe"]="رجوع",
   ["ro"]="referit",
  },
  ["referraldate"]={
   ["cs"]="odkaznadatum",
   ["de"]="verweisdatum",
   ["en"]="referraldate",
   ["fr"]="dateredirection",
   ["it"]="datareferral",
   ["nl"]="kenmerkdatum",
   ["pe"]="تاریخ‌رجوع",
   ["ro"]="datareferit",
  },
  ["referring"]={
   ["cs"]="odkazujici",
   ["de"]="referieren",
   ["en"]="referring",
   ["fr"]="fairereference",
   ["it"]="referring",
   ["nl"]="refereer",
   ["pe"]="مراجعه",
   ["ro"]="referring",
  },
  ["registerfontclass"]={
   ["en"]="registerfontclass",
   ["fr"]="enregistrerclassepolice",
  },
  ["registerhyphenationexception"]={
   ["en"]="registerhyphenationexception",
   ["fr"]="enregistrerexceptioncesure",
  },
  ["registerhyphenationpattern"]={
   ["en"]="registerhyphenationpattern",
   ["fr"]="enregistrermotifcesure",
  },
  ["remark"]={
   ["cs"]="oznaceni",
   ["de"]="bemerkung",
   ["en"]="remark",
   ["fr"]="remarque",
   ["it"]="commento",
   ["nl"]="toelichting",
   ["pe"]="توجه",
   ["ro"]="remarca",
  },
  ["replacefeature"]={
   ["en"]="replacefeature",
   ["fr"]="remplacerfonctionnalite",
  },
  ["reset"]={
   ["cs"]="reset",
   ["de"]="ruecksetzten",
   ["en"]="reset",
   ["fr"]="raz",
   ["it"]="reimposta",
   ["nl"]="reset",
   ["pe"]="بازنشانی",
   ["ro"]="reset",
  },
  ["resetandaddfeature"]={
   ["en"]="resetandaddfeature",
   ["fr"]="razetajouterfonctionnalite",
  },
  ["resetcharacteralign"]={
   ["en"]="resetcharacteralign",
   ["fr"]="razalignercaractere",
  },
  ["resetcharacterkerning"]={
   ["en"]="resetcharacterkerning",
   ["fr"]="razapprochecaracteres",
  },
  ["resetcharacterspacing"]={
   ["en"]="resetcharacterspacing",
   ["fr"]="razespacementcaracteres",
  },
  ["resetcharacterstripping"]={
   ["en"]="resetcharacterstripping",
   ["fr"]="razdemontagecaracteres",
  },
  ["resetfeature"]={
   ["en"]="resetfeature",
   ["fr"]="razfonctionnalite",
  },
  ["resetfontcolorsheme"]={
   ["en"]="resetfontcolorsheme",
   ["fr"]="razthemecouleurpolice",
  },
  ["resetfontfallback"]={
   ["en"]="resetfontfallback",
   ["fr"]="razpolicederepli",
  },
  ["resetfontsolution"]={
   ["en"]="resetfontsolution",
   ["fr"]="zarsolutionpolice",
  },
  ["resethyphenationfeatures"]={
   ["en"]="resethyphenationfeatures",
   ["fr"]="razfonctionnalitescesure",
  },
  ["resetlocalfloats"]={
   ["en"]="resetlocalfloats",
   ["fr"]="razflottantslocaux",
  },
  ["resetmarking"]={
   ["cs"]="resetznaceni",
   ["de"]="ruecksetztenbeschriftung",
   ["en"]="resetmarking",
   ["fr"]="razmarquage",
   ["it"]="reimpostamarcatura",
   ["nl"]="resetmarkering",
   ["pe"]="بازنشانی‌نشانه‌گذاری",
   ["ro"]="reseteazamarcaje",
  },
  ["resetmode"]={
   ["en"]="resetmode",
   ["fr"]="razmode",
  },
  ["resetnumber"]={
   ["cs"]="resetnumber",
   ["de"]="resetnumber",
   ["en"]="resetnumber",
   ["fr"]="raznumero",
   ["it"]="resetnumber",
   ["nl"]="resetnummer",
   ["pe"]="بازنشانی‌شماره",
   ["ro"]="resetnumber",
  },
  ["resetpath"]={
   ["en"]="resetpath",
   ["fr"]="razchemin",
  },
  ["resetperiodkerning"]={
   ["en"]="resetperiodkerning",
   ["fr"]="razapprochepoint",
  },
  ["resetsystemmode"]={
   ["en"]="resetsystemmode",
   ["fr"]="razmodesysteme",
  },
  ["resettextcontent"]={
   ["cs"]="resettextcontent",
   ["de"]="resettextcontent",
   ["en"]="resettextcontent",
   ["fr"]="raztexte",
   ["it"]="resettextcontent",
   ["nl"]="resettekstinhoud",
   ["pe"]="بازنشانی‌متن",
   ["ro"]="resettextcontent",
  },
  ["resetvisualizers"]={
   ["en"]="resetvisualizers",
   ["fr"]="razvisualisateurs",
  },
  ["restoreglobalbodyfont"]={
   ["en"]="restoreglobalbodyfont",
   ["fr"]="restaurerpolicedecorpsglobale",
  },
  ["retestfeature"]={
   ["en"]="retestfeature",
   ["fr"]="retesterfonctionnalite",
  },
  ["revivefeature"]={
   ["en"]="revivefeature",
   ["fr"]="relancerfonctionnalite",
  },
  ["rightaligned"]={
   ["cs"]="zarovnanovpravo",
   ["de"]="rechtsbuendig",
   ["en"]="rightaligned",
   ["fr"]="aligneadroite",
   ["it"]="allineadestra",
   ["nl"]="regelrechts",
   ["pe"]="تنظیم‌راست",
   ["ro"]="aliniatdreapta",
  },
  ["rightbottombox"]={
   ["en"]="rightbottombox",
   ["fr"]="boitebasdroite",
  },
  ["rightedgedistance"]={
   ["cs"]="vzdalenostpravehookraje",
   ["de"]="abstandrechterrand",
   ["en"]="rightedgedistance",
   ["fr"]="distanceborddroit",
   ["it"]="distanzabordodestro",
   ["nl"]="rechterrandafstand",
   ["pe"]="فاصله‌لبه‌راست",
   ["ro"]="distantacoltdreapta",
  },
  ["rightedgewidth"]={
   ["cs"]="sirkapravehookraje",
   ["de"]="breiterechterrand",
   ["en"]="rightedgewidth",
   ["fr"]="largeurborddroit",
   ["it"]="ampiezzabordodestro",
   ["nl"]="rechterrandbreedte",
   ["pe"]="عرض‌لبه‌راست",
   ["ro"]="latimecoltdreapta",
  },
  ["rightline"]={
   ["en"]="rightline",
   ["fr"]="lignedroite",
  },
  ["rightmargindistance"]={
   ["cs"]="vzdalenostpravemarginalie",
   ["de"]="rechtemarginalafstand",
   ["en"]="rightmargindistance",
   ["fr"]="distancemargedroite",
   ["it"]="distanzamarginedestro",
   ["nl"]="rechtermargeafstand",
   ["pe"]="فاصله‌حاشیه‌راست",
   ["ro"]="distantamarginedreapta",
  },
  ["rightmarginwidth"]={
   ["cs"]="sirkapravemarginalie",
   ["de"]="rechtemarginalbreite",
   ["en"]="rightmarginwidth",
   ["fr"]="largeurmargedroite",
   ["it"]="ampiezzamarginedestro",
   ["nl"]="rechtermargebreedte",
   ["pe"]="عرض‌حاشیه‌راست",
   ["ro"]="latimemarginedreapta",
  },
  ["romannumerals"]={
   ["cs"]="rimskecislice",
   ["de"]="roemischezahlen",
   ["en"]="romannumerals",
   ["fr"]="chiffresromains",
   ["it"]="numeriromani",
   ["nl"]="romeins",
   ["pe"]="اعدادلاتین",
   ["ro"]="numereromane",
  },
  ["rotate"]={
   ["cs"]="otocit",
   ["de"]="drehen",
   ["en"]="rotate",
   ["fr"]="tourner",
   ["it"]="ruota",
   ["nl"]="roteer",
   ["pe"]="دوران",
   ["ro"]="roteste",
  },
  ["savebuffer"]={
   ["en"]="savebuffer",
   ["fr"]="enregistrertampon",
   ["nl"]="bewaarbuffer",
  },
  ["scale"]={
   ["cs"]="meritko",
   ["de"]="format",
   ["en"]="scale",
   ["fr"]="mettrealechelle",
   ["it"]="scala",
   ["nl"]="schaal",
   ["pe"]="مقیاس",
   ["ro"]="scala",
  },
  ["screen"]={
   ["cs"]="obrazovka",
   ["de"]="bildschirm",
   ["en"]="screen",
   ["fr"]="ecran",
   ["it"]="schermo",
   ["nl"]="scherm",
   ["pe"]="پرده",
   ["ro"]="ecran",
  },
  ["selectblocks"]={
   ["cs"]="vyberbloky",
   ["de"]="waehlebloeckeaus",
   ["en"]="selectblocks",
   ["fr"]="selectionnerblocs",
   ["it"]="selezionablocchi",
   ["nl"]="selecteerblokken",
   ["pe"]="انتخاب‌بلوکها",
   ["ro"]="selecteazablocuri",
  },
  ["selectpaper"]={
   ["cs"]="vyberpapir",
   ["de"]="waehlepapieraus",
   ["en"]="selectpaper",
   ["fr"]="selectionnerpapier",
   ["it"]="selezionacarta",
   ["nl"]="selecteerpapier",
   ["pe"]="انتخاب‌برگ",
   ["ro"]="selecteazahartie",
  },
  ["selectversion"]={
   ["cs"]="vyberverzi",
   ["de"]="waehleversionaus",
   ["en"]="selectversion",
   ["fr"]="selectionnerversion",
   ["it"]="selezionaversione",
   ["nl"]="selecteerversie",
   ["pe"]="انتخاب‌نسخه",
   ["ro"]="selecteazaversiune",
  },
  ["setbigbodyfont"]={
   ["en"]="setbigbodyfont",
   ["fr"]="reglerpolicedecorpsimportante",
  },
  ["setcharacteralign"]={
   ["en"]="setcharacteralign",
   ["fr"]="regleralignercaractere",
  },
  ["setcharacteraligndetail"]={
   ["en"]="setcharacteraligndetail",
   ["fr"]="reglerdetailalignercaractere",
  },
  ["setcharactercasing"]={
   ["en"]="setcharactercasing",
   ["fr"]="reglercassecaracteres",
  },
  ["setcharactercleaning"]={
   ["en"]="setcharactercleaning",
   ["fr"]="reglernettoyagecaracteres",
  },
  ["setcharacterkerning"]={
   ["en"]="setcharacterkerning",
   ["fr"]="reglerapprochecaracteres",
  },
  ["setcharacterspacing"]={
   ["en"]="setcharacterspacing",
   ["fr"]="reglerespacementcaracteres",
  },
  ["setcharacterstripping"]={
   ["en"]="setcharacterstripping",
   ["fr"]="reglerdemontagecaracteres",
  },
  ["setcurrentfontclass"]={
   ["en"]="setcurrentfontclass",
   ["fr"]="reglerclassepolicecourante",
  },
  ["setfirstline"]={
   ["en"]="setfirstline",
   ["fr"]="reglerpremiereligne",
  },
  ["setfirstpasscharacteralign"]={
   ["en"]="setfirstpasscharacteralign",
   ["fr"]="regleralignercaracterepremierepasse",
  },
  ["setfont"]={
   ["en"]="setfont",
   ["fr"]="reglerpolice",
  },
  ["setfontcolorsheme"]={
   ["en"]="setfontcolorsheme",
   ["fr"]="reglerthemecouleurpolice",
  },
  ["setfontfeature"]={
   ["en"]="setfontfeature",
   ["fr"]="reglerfonctionnalitepolice",
  },
  ["setfontsolution"]={
   ["en"]="setfontsolution",
   ["fr"]="reglersolutionpolice",
  },
  ["setfontstrut"]={
   ["en"]="setfontstrut",
   ["fr"]="reglerentretoisepolice",
  },
  ["sethyphenationfeatures"]={
   ["en"]="sethyphenationfeatures",
   ["fr"]="reglerfonctionnalitescesure",
  },
  ["setmainbodyfont"]={
   ["en"]="setmainbodyfont",
   ["fr"]="reglerpolicedecorpsprincipale",
  },
  ["setmode"]={
   ["en"]="setmode",
   ["fr"]="reglermode",
  },
  ["setnote"]={
   ["en"]="setnote",
   ["fr"]="reglernote",
  },
  ["setnotetext"]={
   ["en"]="setnotetext",
   ["fr"]="reglertextenote",
  },
  ["setnumber"]={
   ["cs"]="setnumber",
   ["de"]="setnumber",
   ["en"]="setnumber",
   ["fr"]="reglernumero",
   ["it"]="setnumber",
   ["nl"]="setnummer",
   ["pe"]="تعیین‌شماره",
   ["ro"]="setnumber",
  },
  ["setperiodkerning"]={
   ["en"]="setperiodkerning",
   ["fr"]="reglerapprochepoint",
  },
  ["setsecondpasscharacteralign"]={
   ["en"]="setsecondpasscharacteralign",
   ["fr"]="regleralignercaracteresecondepasse",
  },
  ["setsmallbodyfont"]={
   ["en"]="setsmallbodyfont",
   ["fr"]="reglerpolicedecorpspetite",
  },
  ["setsystemmode"]={
   ["en"]="setsystemmode",
   ["fr"]="reglermodesysteme",
  },
  ["settextcontent"]={
   ["cs"]="settextcontent",
   ["de"]="settext",
   ["en"]="settextcontent",
   ["fr"]="reglercontenutexte",
   ["it"]="settext",
   ["nl"]="steltekstinhoudin",
   ["pe"]="تعیین‌محتوای‌متن",
   ["ro"]="settextcontent",
  },
  ["settextvariable"]={
   ["cs"]="settextvariable",
   ["de"]="settextvariable",
   ["en"]="settextvariable",
   ["fr"]="reglervariabletexte",
   ["it"]="setvariabiletesto",
   ["nl"]="kentekstvariabeletoe",
   ["pe"]="تعیین‌متغیر‌متن",
   ["ro"]="setvariabilatext",
  },
  ["setupalign"]={
   ["cs"]="nastavzarovnani",
   ["de"]="stelleausrichtungein",
   ["en"]="setupalign",
   ["fr"]="configureraligner",
   ["it"]="impostaallineamento",
   ["nl"]="steluitlijnenin",
   ["pe"]="بارگذاری‌تنظیم",
   ["ro"]="seteazaalinierea",
  },
  ["setupanswerarea"]={
   ["cs"]="setupanswerarea",
   ["de"]="setupanswerarea",
   ["en"]="setupanswerarea",
   ["fr"]="configurerzonereponse",
   ["it"]="setupanswerarea",
   ["nl"]="stelantwoordgebiedin",
   ["pe"]="setupanswerarea",
   ["ro"]="setupanswerarea",
  },
  ["setuparranging"]={
   ["cs"]="nastavusporadani",
   ["de"]="stelleanordnenein",
   ["en"]="setuparranging",
   ["fr"]="configurerarrangement",
   ["it"]="impostaparranging",
   ["nl"]="stelarrangerenin",
   ["pe"]="بارگذاری‌ترتیب",
   ["ro"]="seteazaaranjareapag",
  },
  ["setupattachment"]={
   ["en"]="setupattachment",
   ["fr"]="configurepiecejointe",
  },
  ["setupbackground"]={
   ["cs"]="nastavpozadi",
   ["de"]="stellehintergrundein",
   ["en"]="setupbackground",
   ["fr"]="configurerarriereplan",
   ["it"]="impostasfondo",
   ["nl"]="stelachtergrondin",
   ["pe"]="بارگذاری‌پس‌زمینه",
   ["ro"]="seteazafundal",
  },
  ["setupbackgrounds"]={
   ["cs"]="nastavpozadi",
   ["de"]="stellehintergruendeein",
   ["en"]="setupbackgrounds",
   ["fr"]="configurerarriereplans",
   ["it"]="impostasfondi",
   ["nl"]="stelachtergrondenin",
   ["pe"]="بارگذاری‌پس‌زمینه‌ها",
   ["ro"]="seteazafundaluri",
  },
  ["setupblackrules"]={
   ["cs"]="nastavcernelinky",
   ["de"]="stellerechteckein",
   ["en"]="setupblackrules",
   ["fr"]="configurertraitsepais",
   ["it"]="impostalineenere",
   ["nl"]="stelblokjesin",
   ["pe"]="بارگذاری‌خطهای‌سیاه",
   ["ro"]="seteazaliniinegre",
  },
  ["setupblank"]={
   ["cs"]="nastavpreskok",
   ["de"]="stelleblankoein",
   ["en"]="setupblank",
   ["fr"]="configurervierge",
   ["it"]="impostarigovuoto",
   ["nl"]="stelblankoin",
   ["pe"]="بارگذاری‌خالی",
   ["ro"]="seteazablanc",
  },
  ["setupblock"]={
   ["cs"]="nastavblok",
   ["de"]="stelleblockein",
   ["en"]="setupblock",
   ["fr"]="configurerbloc",
   ["it"]="impostablocco",
   ["nl"]="stelblokin",
   ["pe"]="بارگذاری‌بلوک",
   ["ro"]="seteazabloc",
  },
  ["setupbodyfont"]={
   ["cs"]="nastavzakladnifont",
   ["de"]="stellefliesstextein",
   ["en"]="setupbodyfont",
   ["fr"]="configurerpolicedecorps",
   ["it"]="impostafontdeltesto",
   ["nl"]="stelkorpsin",
   ["pe"]="بارگذاری‌قلم‌متن",
   ["ro"]="seteazafonttext",
  },
  ["setupbodyfontenvironment"]={
   ["cs"]="nastavprostredizakladnihofontu",
   ["de"]="stellefliesstextumgebungein",
   ["en"]="setupbodyfontenvironment",
   ["fr"]="configurerenvironnementpolicedecorps",
   ["it"]="impostaambientefontdeltesto",
   ["nl"]="stelkorpsomgevingin",
   ["pe"]="بارگذاری‌محیط‌قلم‌متن",
   ["ro"]="seteazamediulfonttext",
  },
  ["setupbottom"]={
   ["cs"]="nastavspodek",
   ["de"]="stelleuntenein",
   ["en"]="setupbottom",
   ["fr"]="configurerbas",
   ["it"]="impostafondo",
   ["nl"]="stelonderin",
   ["pe"]="بارگذاری‌پایین",
   ["ro"]="seteazajos",
  },
  ["setupbottomtexts"]={
   ["cs"]="nastavdolnitexty",
   ["de"]="stelletextuntenein",
   ["en"]="setupbottomtexts",
   ["fr"]="configurertextesbas",
   ["it"]="impostatestifondo",
   ["nl"]="stelondertekstenin",
   ["pe"]="بارگذاری‌متن‌پایین",
   ["ro"]="seteazatextejos",
  },
  ["setupbuffer"]={
   ["cs"]="nastavbuffer",
   ["de"]="stellepufferein",
   ["en"]="setupbuffer",
   ["fr"]="configurertampon",
   ["it"]="impostabuffer",
   ["nl"]="stelbufferin",
   ["pe"]="بارگذاری‌بافر",
   ["ro"]="seteazabuffer",
  },
  ["setupbuttons"]={
   ["cs"]="nastavtlacitka",
   ["de"]="stelleknopfein",
   ["en"]="setupbuttons",
   ["fr"]="configurerboutons",
   ["it"]="impostapulsanti",
   ["nl"]="stelbuttonsin",
   ["pe"]="بارگذاری‌دکمه‌ها",
   ["ro"]="seteazabutoane",
  },
  ["setupcapitals"]={
   ["cs"]="nastavkapitalky",
   ["de"]="stelleversalienein",
   ["en"]="setupcapitals",
   ["fr"]="configurercapitales",
   ["it"]="impostamaiuscole",
   ["nl"]="stelkapitalenin",
   ["pe"]="setupcapitals",
   ["ro"]="seteazamajuscule",
  },
  ["setupcaption"]={
   ["cs"]="nastavpopisek",
   ["de"]="stellebildunterschriftein",
   ["en"]="setupcaption",
   ["fr"]="configurerintitule",
   ["it"]="impostacaption",
   ["nl"]="stelblokkopjein",
   ["pe"]="بارگذاری‌شرح",
   ["ro"]="seteazalegenda",
  },
  ["setupcaptions"]={
   ["cs"]="nastavpopisky",
   ["de"]="stellebilderunterschriftein",
   ["en"]="setupcaptions",
   ["fr"]="configurerintitules",
   ["it"]="impostacaptions",
   ["nl"]="stelblokkopjesin",
   ["pe"]="بارگذاری‌شرحها",
   ["ro"]="seteazalegendele",
  },
  ["setupcharacteralign"]={
   ["en"]="setupcharacteralign",
   ["fr"]="configureralignercaractere",
  },
  ["setupcharacterkerning"]={
   ["en"]="setupcharacterkerning",
   ["fr"]="configurerapprochecaracteres",
  },
  ["setupcharacterspacing"]={
   ["en"]="setupcharacterspacing",
   ["fr"]="configurerespacementcaracteres",
  },
  ["setupclipping"]={
   ["cs"]="nastavorez",
   ["de"]="stelleausschnittein",
   ["en"]="setupclipping",
   ["fr"]="configurerdecoupage",
   ["it"]="impostaclippling",
   ["nl"]="stelclipin",
   ["pe"]="بارگذاری‌چیدن",
   ["ro"]="seteazaclipping",
  },
  ["setupcolor"]={
   ["cs"]="nastavbarvu",
   ["de"]="stellefarbeein",
   ["en"]="setupcolor",
   ["fr"]="configurercouleur",
   ["it"]="impostacolore",
   ["nl"]="stelkleurin",
   ["pe"]="بارگذاری‌رنگ",
   ["ro"]="seteazaculoare",
  },
  ["setupcolors"]={
   ["cs"]="nastavbarvy",
   ["de"]="stellefarbenein",
   ["en"]="setupcolors",
   ["fr"]="configurercouleurs",
   ["it"]="impostacolori",
   ["nl"]="stelkleurenin",
   ["pe"]="بارگذاری‌رنگها",
   ["ro"]="seteazaculori",
  },
  ["setupcolumns"]={
   ["cs"]="nastavsloupce",
   ["de"]="stellespaltenein",
   ["en"]="setupcolumns",
   ["fr"]="configurercolonnes",
   ["it"]="impostacolonne",
   ["nl"]="stelkolommenin",
   ["pe"]="بارگذاری‌ستونها",
   ["ro"]="seteazacoloane",
  },
  ["setupcolumnset"]={
   ["cs"]="setupcolumnset",
   ["de"]="setupcolumnset",
   ["en"]="setupcolumnset",
   ["fr"]="configurerjeucolonnes",
   ["it"]="impostagruppocolonne",
   ["nl"]="stelkolomgroepin",
   ["pe"]="بارگذاری‌مجموعه‌ستون",
   ["ro"]="setupcolumnset",
  },
  ["setupcolumnsetlines"]={
   ["cs"]="setupcolumnsetlines",
   ["de"]="setupcolumnsetlines",
   ["en"]="setupcolumnsetlines",
   ["fr"]="configurerlignesjeucolonnes",
   ["it"]="setupcolumnsetlines",
   ["nl"]="stelkolomgroepregelsin",
   ["pe"]="بارگذاری‌خطهای‌مجموعه‌ستون",
   ["ro"]="setupcolumnsetlines",
  },
  ["setupcolumnsetstart"]={
   ["cs"]="setupcolumnsetstart",
   ["de"]="setupcolumnsetstart",
   ["en"]="setupcolumnsetstart",
   ["fr"]="configurerfinjeucolonnes",
   ["it"]="setupcolumnsetstart",
   ["nl"]="stelkolomgroepstartin",
   ["pe"]="بارگذاری‌شروع‌مجموعه‌ستون",
   ["ro"]="setupcolumnsetstart",
  },
  ["setupcombinations"]={
   ["cs"]="nastavspojeni",
   ["de"]="stellekombinationein",
   ["en"]="setupcombinations",
   ["fr"]="configurercombinaisons",
   ["it"]="impostacombinazioni",
   ["nl"]="stelcombinatiesin",
   ["pe"]="بارگذاری‌ترکیب‌ها",
   ["ro"]="seteazacombinari",
  },
  ["setupcombinedlist"]={
   ["cs"]="nastavkombinovanyseznam",
   ["de"]="stellezusammengestelltelisteein",
   ["en"]="setupcombinedlist",
   ["fr"]="configurerlistecombinee",
   ["it"]="impostaelencocombinato",
   ["nl"]="stelsamengesteldelijstin",
   ["pe"]="بارگذاری‌لیست‌ترکیبی",
   ["ro"]="seteazalistacombinata",
  },
  ["setupcomment"]={
   ["cs"]="nastavkomentar",
   ["de"]="stellekommentarein",
   ["en"]="setupcomment",
   ["fr"]="configurercommentaire",
   ["it"]="impostacommento",
   ["nl"]="stelcommentaarin",
   ["pe"]="بارگذاری‌توضیح",
   ["ro"]="seteazacomentariu",
  },
  ["setupcounter"]={
   ["en"]="setupcounter",
   ["fr"]="configurercompteur",
  },
  ["setupdescription"]={
   ["en"]="setupdescription",
   ["fr"]="configurerdescription",
   ["nl"]="steldoordefinierenin",
  },
  ["setupdescriptions"]={
   ["cs"]="nastavpopisy",
   ["de"]="definierebeschreibungen",
   ["en"]="setupdescriptions",
   ["fr"]="configurerdescriptions",
   ["it"]="impostadescrizioni",
   ["nl"]="steldoordefinierenin",
   ["pe"]="بارگذاری‌شرح",
   ["ro"]="seteazadescriere",
  },
  ["setupenumeration"]={
   ["en"]="setupenumeration",
   ["fr"]="configurerénumération",
   ["nl"]="steldoornummerenin",
  },
  ["setupenumerations"]={
   ["cs"]="nastavvycty",
   ["de"]="stellebeschreibungein",
   ["en"]="setupenumerations",
   ["fr"]="configurerénumérations",
   ["it"]="impostaenumerazioni",
   ["nl"]="steldoornummerenin",
   ["pe"]="بارگذاری‌شماره‌گذاریها",
   ["ro"]="seteazaenumerare",
  },
  ["setupexternalfigures"]={
   ["cs"]="nastavexterniobrazy",
   ["de"]="stelleexterneabbildungenein",
   ["en"]="setupexternalfigures",
   ["fr"]="configurerfiguresexterne",
   ["it"]="impostafigureesterne",
   ["nl"]="stelexternefigurenin",
   ["pe"]="بارگذاری‌شکلهای‌خارجی",
   ["ro"]="seteazafiguriexterne",
  },
  ["setupfield"]={
   ["cs"]="nastavpole",
   ["de"]="stellefeldein",
   ["en"]="setupfield",
   ["fr"]="configurerchamp",
   ["it"]="impostacampo",
   ["nl"]="stelveldin",
   ["pe"]="بارگذاری‌میدان",
   ["ro"]="seteazacamp",
  },
  ["setupfields"]={
   ["cs"]="nastavvsechnapole",
   ["de"]="stellefelderin",
   ["en"]="setupfields",
   ["fr"]="configurerchamps",
   ["it"]="impostacampi",
   ["nl"]="stelveldenin",
   ["pe"]="بارگذاری‌میدانها",
   ["ro"]="seteazacampuri",
  },
  ["setupfillinlines"]={
   ["cs"]="nastavvyplnoveradky",
   ["de"]="stellegefuelltezeileein",
   ["en"]="setupfillinlines",
   ["fr"]="configurerlignesarenseigner",
   ["it"]="impostarigheriempimento",
   ["nl"]="stelinvulregelsin",
   ["pe"]="بارگذاری‌پرکردن‌خطها",
   ["ro"]="seteazaliniiumplere",
  },
  ["setupfillinrules"]={
   ["cs"]="nastavvyplnovelinky",
   ["de"]="stellegefuelltesrechteckein",
   ["en"]="setupfillinrules",
   ["fr"]="configurertraitsarenseigner",
   ["it"]="impostalineeriempimento",
   ["nl"]="stelinvullijnenin",
   ["pe"]="بارگذاری‌درج‌درخطها",
   ["ro"]="seteazarigleumplere",
  },
  ["setupfirstline"]={
   ["en"]="setupfirstline",
   ["fr"]="configurerpremiereligne",
  },
  ["setupfloat"]={
   ["cs"]="nastavplvouciobjekt",
   ["de"]="stellegleitobjektein",
   ["en"]="setupfloat",
   ["fr"]="configurerflottant",
   ["it"]="impostaoggettomobile",
   ["nl"]="stelplaatsblokin",
   ["pe"]="بارگذاری‌شناور",
   ["ro"]="seteazafloat",
  },
  ["setupfloats"]={
   ["cs"]="nastavplvouciobjekty",
   ["de"]="stellegleitobjekteein",
   ["en"]="setupfloats",
   ["fr"]="configurerflottants",
   ["it"]="impostaoggettimobili",
   ["nl"]="stelplaatsblokkenin",
   ["pe"]="بارگذاری‌شناورها",
   ["ro"]="seteazafloats",
  },
  ["setupfloatsplitting"]={
   ["cs"]="nastavdeleniplvoucichobjektu",
   ["de"]="stelleplatziegeteiltegleitobjekt",
   ["en"]="setupfloatsplitting",
   ["fr"]="configurerpartageflottants",
   ["it"]="impostaspezzamentooggettomobile",
   ["nl"]="stelplaatsbloksplitsenin",
   ["pe"]="بارگذاری‌شکافتن‌شناورها",
   ["ro"]="seteazaimpartireafloat",
  },
  ["setupfontexpansion"]={
   ["en"]="setupfontexpansion",
   ["fr"]="configurerexpansionpolice",
  },
  ["setupfontprotrusion"]={
   ["en"]="setupfontprotrusion",
   ["fr"]="configurerprotrusionpolice",
  },
  ["setupfonts"]={
   ["en"]="setupfonts",
   ["fr"]="configurerpolices",
  },
  ["setupfontsolution"]={
   ["en"]="setupfontsolution",
   ["fr"]="configuresolutionpolice",
  },
  ["setupfooter"]={
   ["cs"]="nastavupati",
   ["de"]="stellefusszeileein",
   ["en"]="setupfooter",
   ["fr"]="configurerpdp",
   ["it"]="impostapdp",
   ["nl"]="stelvoetin",
   ["pe"]="بارگذاری‌ته‌برگ",
   ["ro"]="seteazasubsol",
  },
  ["setupfootertexts"]={
   ["cs"]="nastavtextyupati",
   ["de"]="stellefusszeilentextein",
   ["en"]="setupfootertexts",
   ["fr"]="configurertextespdp",
   ["it"]="impostatestipdp",
   ["nl"]="stelvoettekstenin",
   ["pe"]="بارگذاری‌متن‌پانوشت",
   ["ro"]="seteazatextesubsol",
  },
  ["setupfootnotedefinition"]={
   ["cs"]="nastavdefinicipoznamekpodcarou",
   ["de"]="stellefussnotendefinitionein",
   ["en"]="setupfootnotedefinition",
   ["fr"]="configurerdefinitionnotebdp",
   ["it"]="impostadefinizionenotepdp",
   ["nl"]="stelvoetnootdefinitiein",
   ["pe"]="بارگذاری‌تعریف‌پانوشت",
   ["ro"]="seteazadefinireanotasubsol",
  },
  ["setupfootnotes"]={
   ["cs"]="nastavpoznamkypodcarou",
   ["de"]="stellefussnotenein",
   ["en"]="setupfootnotes",
   ["fr"]="configurernotesbdp",
   ["it"]="impostanotepdp",
   ["nl"]="stelvoetnotenin",
   ["pe"]="بارگذاری‌پانوشتها",
   ["ro"]="seteazanotasubsol",
  },
  ["setupforms"]={
   ["cs"]="setupforms",
   ["de"]="setupforms",
   ["en"]="setupforms",
   ["fr"]="configurerformulaires",
   ["it"]="impostaforms",
   ["nl"]="stelformulierenin",
   ["pe"]="بارگذاری‌طرح",
   ["ro"]="seteazaformulare",
  },
  ["setupformula"]={
   ["en"]="setupformula",
   ["fr"]="configurerformule",
   ["nl"]="stelformulein",
  },
  ["setupformulas"]={
   ["cs"]="nastavrovnice",
   ["de"]="stelleformelnein",
   ["en"]="setupformulae",
   ["fr"]="configurerformules",
   ["it"]="impostaformule",
   ["nl"]="stelformulesin",
   ["pe"]="بارگذاری‌فرمولها",
   ["ro"]="seteazaformule",
  },
  ["setupframed"]={
   ["cs"]="nastavoramovani",
   ["de"]="stelleumrahmtein",
   ["en"]="setupframed",
   ["fr"]="configurerencadre",
   ["it"]="impostaincorniciato",
   ["nl"]="stelomlijndin",
   ["pe"]="بارگذاری‌قالبی",
   ["ro"]="seteazainconjurat",
  },
  ["setupframedtext"]={
   ["en"]="setupframedtext",
   ["fr"]="configurertexteencadre",
   ["nl"]="stelkadertekstin",
  },
  ["setupframedtexts"]={
   ["cs"]="nastavoramovanetexty",
   ["de"]="stelleumrahmtetexteein",
   ["en"]="setupframedtexts",
   ["fr"]="configurertextesencadres",
   ["it"]="impostatestiincorniciati",
   ["nl"]="stelkadertekstenin",
   ["pe"]="بارگذاری‌متن‌قالبی",
   ["ro"]="definestetexteinconjurate",
  },
  ["setuphead"]={
   ["cs"]="nastavnadpis",
   ["de"]="stelleueberschriftein",
   ["en"]="setuphead",
   ["fr"]="configurertête",
   ["it"]="impostatesta",
   ["nl"]="stelkopin",
   ["pe"]="بارگذاری‌سر",
   ["ro"]="seteazatitlu",
  },
  ["setupheader"]={
   ["cs"]="nastavzahlavi",
   ["de"]="stellekopfzeileein",
   ["en"]="setupheader",
   ["fr"]="configurerentête",
   ["it"]="impostainstestazione",
   ["nl"]="stelhoofdin",
   ["pe"]="بارگذاری‌سربرگ",
   ["ro"]="seteazaantet",
  },
  ["setupheadertexts"]={
   ["cs"]="nastavtextyzahlavi",
   ["de"]="stellekopfzeilentextein",
   ["en"]="setupheadertexts",
   ["fr"]="configurertextesentête",
   ["it"]="impostatestiintestazioni",
   ["nl"]="stelhoofdtekstenin",
   ["pe"]="بارگذاری‌متن‌سربرگ",
   ["ro"]="seteazatexteantet",
  },
  ["setupheadnumber"]={
   ["cs"]="nastavcislonadpisu",
   ["de"]="stellekopfzahlein",
   ["en"]="setupheadnumber",
   ["fr"]="configurernumerotête",
   ["it"]="impostanumerotesta",
   ["nl"]="stelkopnummerin",
   ["pe"]="بارگذاری‌شماره‌سر",
   ["ro"]="seteazanumartitlu",
  },
  ["setupheads"]={
   ["cs"]="nastavnadpisy",
   ["de"]="stelleueberschriftenein",
   ["en"]="setupheads",
   ["fr"]="configurertêtes",
   ["it"]="impostateste",
   ["nl"]="stelkoppenin",
   ["pe"]="بارگذاری‌سرها",
   ["ro"]="seteazatitluri",
  },
  ["setupheadtext"]={
   ["cs"]="nastavtexthlavicky",
   ["de"]="stelleueberschrifttextein",
   ["en"]="setupheadtext",
   ["fr"]="configurertextêtete",
   ["it"]="impostatestointestazioni",
   ["nl"]="stelkoptekstin",
   ["pe"]="بارگذاری‌متن‌سر",
   ["ro"]="seteazatexttitlu",
  },
  ["setuphyphenation"]={
   ["en"]="setuphyphenation",
   ["fr"]="configurercesure",
  },
  ["setuphyphenmark"]={
   ["cs"]="nastavdelitko",
   ["de"]="stellebindestrichein",
   ["en"]="setuphyphenmark",
   ["fr"]="configurermarquecesure",
   ["it"]="impostasegnosillabazione",
   ["nl"]="stelkoppeltekenin",
   ["pe"]="بارگذاری‌نشانه‌شکستن",
   ["ro"]="seteazaliniesilabe",
  },
  ["setupindentedtext"]={
   ["en"]="setupindentedtext",
   ["fr"]="configurertexteindente",
   ["nl"]="stelingesprongentextin",
  },
  ["setupindenting"]={
   ["cs"]="nastavodsazovani",
   ["de"]="stelleeinziehenein",
   ["en"]="setupindenting",
   ["fr"]="configurerindentation",
   ["it"]="impostarientro",
   ["nl"]="stelinspringenin",
   ["pe"]="بارگذاری‌تورفتگی",
   ["ro"]="seteazaaliniat",
  },
  ["setupinmargin"]={
   ["cs"]="nastavmarginalie",
   ["de"]="stelleinmarginalieein",
   ["en"]="setupinmargin",
   ["fr"]="configurerdansmarge",
   ["it"]="impostainmargine",
   ["nl"]="stelinmargein",
   ["pe"]="بارگذاری‌درون‌حاشیه",
   ["ro"]="seteazamarginal",
  },
  ["setupinteraction"]={
   ["cs"]="nastavinterakci",
   ["de"]="stelleinteraktionein",
   ["en"]="setupinteraction",
   ["fr"]="configurerinteraction",
   ["it"]="impostainterazione",
   ["nl"]="stelinteractiein",
   ["pe"]="بارگذاری‌پانل",
   ["ro"]="seteazainteractiunea",
  },
  ["setupinteractionbar"]={
   ["cs"]="nastavinterakcnilistu",
   ["de"]="stelleinteraktionsbalkenein",
   ["en"]="setupinteractionbar",
   ["fr"]="configurerbarreinteraction",
   ["it"]="impostabarrainterazione",
   ["nl"]="stelinteractiebalkin",
   ["pe"]="بارگذاری‌میله‌پانل",
   ["ro"]="seteazabarainteractiune",
  },
  ["setupinteractionmenu"]={
   ["cs"]="nastavinterakcnimenu",
   ["de"]="stelleinteraktionsmenueein",
   ["en"]="setupinteractionmenu",
   ["fr"]="configurermenuinteraction",
   ["it"]="impostamenuinterazione",
   ["nl"]="stelinteractiemenuin",
   ["pe"]="بارگذاری‌منوی‌پانل",
   ["ro"]="seteazameniuinteractiune",
  },
  ["setupinteractionscreen"]={
   ["cs"]="nastavinterakcniobrazovku",
   ["de"]="stelleinteraktionsbildschirmein",
   ["en"]="setupinteractionscreen",
   ["fr"]="configurerecraninteraction",
   ["it"]="impostaschermointerazione",
   ["nl"]="stelinteractieschermin",
   ["pe"]="بارگذاری‌پرده‌پانل",
   ["ro"]="seteazaecraninteractiune",
  },
  ["setupinterlinespace"]={
   ["cs"]="nastavmeziradkovoumezeru",
   ["de"]="stellezeilenabstandein",
   ["en"]="setupinterlinespace",
   ["fr"]="configurerespaceinterligne",
   ["it"]="impostainterlinea",
   ["nl"]="stelinterliniein",
   ["pe"]="بارگذاری‌فاصله‌بین‌خط",
   ["ro"]="seteazaspatiuinterliniar",
  },
  ["setupitemgroup"]={
   ["cs"]="setupitemgroup",
   ["de"]="setupitemgroup",
   ["en"]="setupitemgroup",
   ["fr"]="configurergroupedéléments",
   ["it"]="setupitemgroup",
   ["nl"]="stelitemgroepin",
   ["pe"]="بارگذاری‌گروه‌آیتم",
   ["ro"]="setupitemgroup",
  },
  ["setupitemgroups"]={
   ["cs"]="nastavvycty",
   ["de"]="stelleaufzaehlungenein",
   ["en"]="setupitemizations",
   ["fr"]="configurergroupesdéléments",
   ["it"]="impostaelencazioni",
   ["nl"]="stelopsommingenin",
   ["pe"]="بارگذاری‌گروههای‌آیتم",
   ["ro"]="setareitemization",
  },
  ["setupitems"]={
   ["cs"]="nastavpolozky",
   ["de"]="stellepostenein",
   ["en"]="setupitems",
   ["fr"]="configureréléments",
   ["it"]="impostaelementi",
   ["nl"]="stelitemsin",
   ["pe"]="بارگذاری‌آیتمها",
   ["ro"]="seteazaelemente",
  },
  ["setuplabeltext"]={
   ["cs"]="nastavtextpopisku",
   ["de"]="stellelabeltextein",
   ["en"]="setuplabeltext",
   ["fr"]="configureretiquettetexte",
   ["it"]="impostatestoetichette",
   ["nl"]="stellabeltekstin",
   ["pe"]="باگذاری‌متن‌برچسب",
   ["ro"]="seteazatexteticheta",
  },
  ["setuplanguage"]={
   ["cs"]="nastavjazyk",
   ["de"]="stellespracheein",
   ["en"]="setuplanguage",
   ["fr"]="configurerlangue",
   ["it"]="impostalingua",
   ["nl"]="steltaalin",
   ["pe"]="بارگذاری‌زبان",
   ["ro"]="setarelimba",
  },
  ["setuplayout"]={
   ["cs"]="nastavvzhled",
   ["de"]="stellelayoutein",
   ["en"]="setuplayout",
   ["fr"]="configurermiseenpage",
   ["it"]="impostalayout",
   ["nl"]="stellayoutin",
   ["pe"]="بارگذاری‌طرح‌بندی",
   ["ro"]="seteazaaspect",
  },
  ["setuplegend"]={
   ["cs"]="nastavlegendu",
   ["de"]="stellelegendeein",
   ["en"]="setuplegend",
   ["fr"]="configurerlegende",
   ["it"]="impostalegenda",
   ["nl"]="stellegendain",
   ["pe"]="بارگذاری‌راهنما",
   ["ro"]="seteazalegenda",
  },
  ["setuplinenote"]={
   ["en"]="setuplinenote",
   ["fr"]="configurernotedeligne",
  },
  ["setuplinenumbering"]={
   ["cs"]="nastavcislovaniradku",
   ["de"]="stellezeilennumerierungein",
   ["en"]="setuplinenumbering",
   ["fr"]="configurernumérotationligne",
   ["it"]="impostanumerazionerighe",
   ["nl"]="stelregelnummerenin",
   ["pe"]="بارگذاری‌شماره‌‌گذاری‌خط",
   ["ro"]="seteazanumerotarelinii",
  },
  ["setuplines"]={
   ["cs"]="nastavradky",
   ["de"]="stellezeilenein",
   ["en"]="setuplines",
   ["fr"]="configurerlignes",
   ["it"]="impostarighe",
   ["nl"]="stelregelsin",
   ["pe"]="بارگذاری‌خطها",
   ["ro"]="seteazalinii",
  },
  ["setuplinewidth"]={
   ["cs"]="nastavsirkucary",
   ["de"]="stellelinienbreiteein",
   ["en"]="setuplinewidth",
   ["fr"]="configurerepaisseurtrait",
   ["it"]="impostaampiezzariga",
   ["nl"]="stellijndiktein",
   ["pe"]="بارگذاری‌عرض‌خط",
   ["ro"]="seteazagrosimelinie",
  },
  ["setuplist"]={
   ["cs"]="nastavseznam",
   ["de"]="stellelisteein",
   ["en"]="setuplist",
   ["fr"]="configurerliste",
   ["it"]="impostaelenco",
   ["nl"]="stellijstin",
   ["pe"]="بارگذاری‌لیست",
   ["ro"]="seteazalista",
  },
  ["setupmakeup"]={
   ["cs"]="nastavupravu",
   ["de"]="stelleumbruchein",
   ["en"]="setupmakeup",
   ["fr"]="configurercomposition",
   ["it"]="impostamakeup",
   ["nl"]="stelopmaakin",
   ["pe"]="بارگذاری‌آرایش",
   ["ro"]="seteazamakeup",
  },
  ["setupmarginblock"]={
   ["en"]="setupmarginblock",
   ["fr"]="configurerblocmarginal",
   ["nl"]="stelmargeblokkenin",
  },
  ["setupmarginblocks"]={
   ["cs"]="nastavmarginalniblok",
   ["de"]="stellemarginalblockein",
   ["en"]="setupmarginblocks",
   ["fr"]="configurerblocmarge",
   ["it"]="impostablocchimargine",
   ["nl"]="stelmargeblokkenin",
   ["pe"]="بارگذاری‌بلوکهای‌حاشیه",
   ["ro"]="seteazablocurimarginale",
  },
  ["setupmargindata"]={
   ["en"]="setupmargindata",
   ["fr"]="configurerdonneesmarginales",
   ["nl"]="stelinmargein",
  },
  ["setupmarginrules"]={
   ["cs"]="nastavmarginalnilinky",
   ["de"]="stellemarginallinieein",
   ["en"]="setupmarginrules",
   ["fr"]="configurertraitsmarge",
   ["it"]="impostalineemargine",
   ["nl"]="stelkantlijnin",
   ["pe"]="بارگذاری‌خطهای‌حاشیه",
   ["ro"]="seteazaliniimargine",
  },
  ["setupmarking"]={
   ["cs"]="nastavznaceni",
   ["de"]="stellebeschriftungein",
   ["en"]="setupmarking",
   ["fr"]="configurermarquage",
   ["it"]="impostamarcatura",
   ["nl"]="stelmarkeringin",
   ["pe"]="بارگذاری‌نشانه‌گذاری",
   ["ro"]="seteazamarcaje",
  },
  ["setupmathalignment"]={
   ["cs"]="setupmathalignment",
   ["de"]="setupmathalignment",
   ["en"]="setupmathalignment",
   ["fr"]="configureralignementmath",
   ["it"]="setupmathalignment",
   ["nl"]="stelwiskundeuitlijnenin",
   ["pe"]="بارگذاری‌تنظیم‌ریاضی",
   ["ro"]="setupmathalignment",
  },
  ["setupnarrower"]={
   ["cs"]="nastavzuzeni",
   ["de"]="stelleengerein",
   ["en"]="setupnarrower",
   ["fr"]="configurerplusetroit",
   ["it"]="impostapiustretto",
   ["nl"]="stelsmallerin",
   ["pe"]="بارگذاری‌باریکتر",
   ["ro"]="seteazaingust",
  },
  ["setupnotation"]={
   ["en"]="setupnotation",
   ["fr"]="configurernotation",
  },
  ["setupnotations"]={
   ["en"]="setupnotations",
   ["fr"]="configurernotations",
  },
  ["setupnote"]={
   ["en"]="setupnote",
   ["fr"]="configurenote",
  },
  ["setupnotes"]={
   ["en"]="setupnotes",
   ["fr"]="configurenotes",
  },
  ["setupnumber"]={
   ["cs"]="setupnumber",
   ["de"]="setupnumber",
   ["en"]="setupnumber",
   ["fr"]="configurernumero",
   ["it"]="impostanumerazione",
   ["nl"]="stelnummerin",
   ["pe"]="بارگذاری‌شماره",
   ["ro"]="setupnumber",
  },
  ["setupnumbering"]={
   ["cs"]="nastavcislovani",
   ["de"]="stellenummerierungein",
   ["en"]="setupnumbering",
   ["fr"]="configurernumérotation",
   ["it"]="impostanumerazione",
   ["nl"]="stelnummerenin",
   ["pe"]="بارگذاری‌شماره‌گذاری",
   ["ro"]="seteazanumerotare",
  },
  ["setupoppositeplacing"]={
   ["cs"]="nastavumisteniprotejsku",
   ["de"]="stellegegenueberplatzierenein",
   ["en"]="setupoppositeplacing",
   ["fr"]="configurerplacementopposition",
   ["it"]="impostaposizionamentoopposti",
   ["nl"]="stelnaastplaatsenin",
   ["pe"]="بارگذاری‌درج‌مخالف",
   ["ro"]="seteazaplasareaopozita",
  },
  ["setupoutput"]={
   ["cs"]="nastavvystup",
   ["de"]="stelleausgabeein",
   ["en"]="setupoutput",
   ["fr"]="configurersortie",
   ["it"]="impostaoutput",
   ["nl"]="steluitvoerin",
   ["pe"]="بارگذاری‌خروجی",
   ["ro"]="setareoutput",
  },
  ["setuppagecomment"]={
   ["cs"]="nastavkomentarstrany",
   ["de"]="stelleseitenkommentarein",
   ["en"]="setuppagecomment",
   ["fr"]="configurercommentairepage",
   ["it"]="impostacommentopagina",
   ["nl"]="stelpaginacommentaarin",
   ["pe"]="بارگذاری‌توضیح‌صفحه",
   ["ro"]="seteazacomentariupagina",
  },
  ["setuppagenumber"]={
   ["cs"]="nastavcislostrany",
   ["de"]="stelleseitennummerein",
   ["en"]="setuppagenumber",
   ["fr"]="configurernumeropage",
   ["it"]="impostanumeropagina",
   ["nl"]="stelpaginanummerin",
   ["pe"]="بارگذاری‌شماره‌صفحه",
   ["ro"]="seteazanumarpagina",
  },
  ["setuppagenumbering"]={
   ["cs"]="nastavcislovanistran",
   ["de"]="stelleseitennummeriernungein",
   ["en"]="setuppagenumbering",
   ["fr"]="configurernumérotationpage",
   ["it"]="impostanumerazionepagina",
   ["nl"]="stelpaginanummeringin",
   ["pe"]="بارگذاری‌شماره‌گذاری‌صفحه",
   ["ro"]="seteazanumerotarepagina",
  },
  ["setuppagetransitions"]={
   ["cs"]="nastavprechodstrany",
   ["de"]="stelleseitenuebergangein",
   ["en"]="setuppagetransitions",
   ["fr"]="configurertransitionpage",
   ["it"]="impostatransizionepagina",
   ["nl"]="stelpaginaovergangenin",
   ["pe"]="بارگذاری‌گذارصفحه",
   ["ro"]="seteazatranzitiepagina",
  },
  ["setuppalet"]={
   ["cs"]="nastavpaletu",
   ["de"]="stellepaletteein",
   ["en"]="setuppalet",
   ["fr"]="configurerpalette",
   ["it"]="impostatavolozza",
   ["nl"]="stelpaletin",
   ["pe"]="بارگذاری‌لوح",
   ["ro"]="seteazapaleta",
  },
  ["setuppaper"]={
   ["cs"]="setuppaper",
   ["de"]="setuppaper",
   ["en"]="setuppaper",
   ["fr"]="configurerpapier",
   ["it"]="setuppaper",
   ["nl"]="stelpapierin",
   ["pe"]="بارگذاری‌برگ",
   ["ro"]="setuppaper",
  },
  ["setuppapersize"]={
   ["cs"]="nastavvelikostpapiru",
   ["de"]="stellepapierformatein",
   ["en"]="setuppapersize",
   ["fr"]="configurertaillepapier",
   ["it"]="impostadimensionicarta",
   ["nl"]="stelpapierformaatin",
   ["pe"]="بارگذاری‌اندازه‌برگ",
   ["ro"]="seteazadimensiunihartie",
  },
  ["setupparagraphnumbering"]={
   ["cs"]="nastavcislovaniodstavcu",
   ["de"]="stelleabsatznummerierungein",
   ["en"]="setupparagraphnumbering",
   ["fr"]="configurernumérotationparagraphe",
   ["it"]="impostanumerazionecapoversi",
   ["nl"]="stelparagraafnummerenin",
   ["pe"]="بارگذاری‌شماره‌گذاری‌پاراگراف",
   ["ro"]="seteazanumerotareparagrafe",
  },
  ["setupparagraphs"]={
   ["cs"]="nastavodstavce",
   ["de"]="stelleabsaetzeein",
   ["en"]="setupparagraphs",
   ["fr"]="configurerparagraphe",
   ["it"]="impostacapoversi",
   ["nl"]="stelalineasin",
   ["pe"]="بارگذاری‌پاراگرافها",
   ["ro"]="seteazaparagrafe",
  },
  ["setupperiodkerning"]={
   ["en"]="setupperiodkerning",
   ["fr"]="configurerapprochepoint",
  },
  ["setupplacement"]={
   ["cs"]="setupplacement",
   ["de"]="setupplacement",
   ["en"]="setupplacement",
   ["fr"]="configurerplacement",
   ["it"]="setupplacement",
   ["nl"]="stelplaatsin",
   ["pe"]="بارگذاری‌جانشانی",
   ["ro"]="setupplacement",
  },
  ["setuppositioning"]={
   ["cs"]="nastavumistovani",
   ["de"]="stellepositionierenein",
   ["en"]="setuppositioning",
   ["fr"]="configurerpositionnement",
   ["it"]="impostaposizionamento",
   ["nl"]="stelpositionerenin",
   ["pe"]="بارگذاری‌مکان‌گذاری",
   ["ro"]="setarepozitie",
  },
  ["setupprofiles"]={
   ["cs"]="nastavprofily",
   ["de"]="stelleprofilein",
   ["en"]="setupprofiles",
   ["fr"]="configurerprofils",
   ["it"]="impostaprofili",
   ["nl"]="stelprofielenin",
   ["pe"]="بارگذاری‌پروفایلها",
   ["ro"]="seteazaprofile",
  },
  ["setupprograms"]={
   ["cs"]="nastavprogramy",
   ["de"]="stelleprogrammein",
   ["en"]="setupprograms",
   ["fr"]="configurerprogrammes",
   ["it"]="impostaprogrammi",
   ["nl"]="stelprogrammasin",
   ["pe"]="بارگذاری‌برنامه‌ها",
   ["ro"]="seteazaprograme",
  },
  ["setuppublications"]={
   ["cs"]="nastavpublikace",
   ["de"]="stellepublikationein",
   ["en"]="setuppublications",
   ["fr"]="configurerpublications",
   ["it"]="impostapubblicazioni",
   ["nl"]="stelpublicatiesin",
   ["pe"]="بارگذاری‌نشرها",
   ["ro"]="seteazapublicatii",
  },
  ["setupquote"]={
   ["cs"]="nastavcitaci",
   ["de"]="stellezitierenein",
   ["en"]="setupquote",
   ["fr"]="configurerciter",
   ["it"]="impostamenzione",
   ["nl"]="stelciterenin",
   ["pe"]="بارگذاری‌نقل",
   ["ro"]="seteazaminicitat",
  },
  ["setupreferencelist"]={
   ["cs"]="nastavseznamodkazu",
   ["de"]="stellereferenzlisteein",
   ["en"]="setupreferencelist",
   ["fr"]="configurerlistereference",
   ["it"]="impostalistariferimenti",
   ["nl"]="stelreferentielijstin",
   ["pe"]="بارگذاری‌لیست‌مرجع",
   ["ro"]="seteazalistareferinte",
  },
  ["setupreferencing"]={
   ["cs"]="nastavodkazovani",
   ["de"]="stellereferenzierenein",
   ["en"]="setupreferencing",
   ["fr"]="configurerreferencement",
   ["it"]="impostariferimento",
   ["nl"]="stelrefererenin",
   ["pe"]="بارگذاری‌ارجاع",
   ["ro"]="seteazareferinte",
  },
  ["setupregister"]={
   ["cs"]="nastavrejstrik",
   ["de"]="stelleregisterein",
   ["en"]="setupregister",
   ["fr"]="configurerregistre",
   ["it"]="impostaregistro",
   ["nl"]="stelregisterin",
   ["pe"]="بارگذاری‌ثبت",
   ["ro"]="seteazaregistru",
  },
  ["setuprotate"]={
   ["cs"]="nastavotoceni",
   ["de"]="stelledrehenein",
   ["en"]="setuprotate",
   ["fr"]="configurerrotation",
   ["it"]="impostarotazione",
   ["nl"]="stelroterenin",
   ["pe"]="بارگذاری‌دوران",
   ["ro"]="seteazarotare",
  },
  ["setuprule"]={
   ["cs"]="setuprule",
   ["de"]="setuprule",
   ["en"]="setuprule",
   ["fr"]="configurertrait",
   ["it"]="impostalinea",
   ["nl"]="stellijnin",
   ["pe"]="بارگذاری‌خط",
   ["ro"]="seteazarigla",
  },
  ["setupscreens"]={
   ["cs"]="nastavrastr",
   ["de"]="stellerasterein",
   ["en"]="setupscreens",
   ["fr"]="configurerecran",
   ["it"]="impostaschermi",
   ["nl"]="stelrastersin",
   ["pe"]="بارگذاری‌پرده‌ها",
   ["ro"]="seteazaecrane",
  },
  ["setupsection"]={
   ["cs"]="nastavsekci",
   ["de"]="stelleabschnittein",
   ["en"]="setupsection",
   ["fr"]="configurersection",
   ["it"]="impostasezione",
   ["nl"]="stelsectiein",
   ["pe"]="بارگذاری‌بخش",
   ["ro"]="seteazasectiune",
  },
  ["setupsectionblock"]={
   ["cs"]="nastavbloksekce",
   ["de"]="stelleabschnittsblockein",
   ["en"]="setupsectionblock",
   ["fr"]="configurerblocsection",
   ["it"]="impostabloccosezione",
   ["nl"]="stelsectieblokin",
   ["pe"]="بارگذاری‌بلوک‌بخش",
   ["ro"]="seteazablocsectiune",
  },
  ["setupsorting"]={
   ["cs"]="nastavtrideni",
   ["de"]="stellesortierenein",
   ["en"]="setupsorting",
   ["fr"]="configurertri",
   ["it"]="impostaordinamento",
   ["nl"]="stelsorterenin",
   ["pe"]="بارگذاری‌ترتیب",
   ["ro"]="seteazasortare",
  },
  ["setupspacing"]={
   ["cs"]="nastavradkovani",
   ["de"]="stellespatiumein",
   ["en"]="setupspacing",
   ["fr"]="configurerespacement",
   ["it"]="impostaspaziatura",
   ["nl"]="stelspatieringin",
   ["pe"]="بارگذاری‌فضا‌گذاری",
   ["ro"]="seteazaspatiu",
  },
  ["setupstartstop"]={
   ["cs"]="setupstartstop",
   ["de"]="setupstartstop",
   ["en"]="setupstartstop",
   ["fr"]="configurerdébutfin",
   ["it"]="impostainiziatermina",
   ["nl"]="stelstartstopin",
   ["pe"]="بارگذاری‌شروع‌پایان",
   ["ro"]="setupstartstop",
  },
  ["setupstrut"]={
   ["cs"]="setupstrut",
   ["de"]="setupstrut",
   ["en"]="setupstrut",
   ["fr"]="configurerentretoise",
   ["it"]="impostastrut",
   ["nl"]="stelstrutin",
   ["pe"]="بارگذاری‌بست",
   ["ro"]="seteazastrut",
  },
  ["setupsubpagenumber"]={
   ["cs"]="nastavpodcislostrany",
   ["de"]="stelleunterseitennummerein",
   ["en"]="setupsubpagenumber",
   ["fr"]="configurernumerosouspage",
   ["it"]="impostanumerosottopagina",
   ["nl"]="stelsubpaginanummerin",
   ["pe"]="بارگذاری‌شماره‌زیرصفحه",
   ["ro"]="seteazanumarsubpagina",
  },
  ["setupsymbolset"]={
   ["cs"]="nastavsadusymbolu",
   ["de"]="stellesymbolsetein",
   ["en"]="setupsymbolset",
   ["fr"]="configurerjeusymboles",
   ["it"]="impostasetsimboli",
   ["nl"]="stelsymboolsetin",
   ["pe"]="بارگذاری‌مجموعه‌نماد",
   ["ro"]="seteazasimbol",
  },
  ["setupsynchronization"]={
   ["cs"]="nastavsynchronizaci",
   ["de"]="stellesynchronisationein",
   ["en"]="setupsynchronization",
   ["fr"]="configurersynchronisation",
   ["it"]="impostasincronizzazione",
   ["nl"]="stelsynchronisatiein",
   ["pe"]="بارگذاری‌تطابق",
   ["ro"]="setaresincronizare",
  },
  ["setupsynchronizationbar"]={
   ["cs"]="nastavsynchronizacnilistu",
   ["de"]="stellesynchronisationsbalkenein",
   ["en"]="setupsynchronizationbar",
   ["fr"]="configurerbarresynchronisation",
   ["it"]="impostabarrasincronizzazione",
   ["nl"]="stelsynchronisatiebalkin",
   ["pe"]="بارگذاری‌میله‌تطابق",
   ["ro"]="setarebarasincronizare",
  },
  ["setupsynonyms"]={
   ["cs"]="nastavsynonyma",
   ["de"]="stellesynonymein",
   ["en"]="setupsynonyms",
   ["fr"]="configurersynonymes",
   ["it"]="impostasinonimi",
   ["nl"]="stelsynoniemenin",
   ["pe"]="بارگذاری‌مترادفها",
   ["ro"]="seteazasinonime",
  },
  ["setupsystem"]={
   ["cs"]="nastavsystem",
   ["de"]="stellesystemein",
   ["en"]="setupsystem",
   ["fr"]="configurersysteme",
   ["it"]="impostasistema",
   ["nl"]="stelsysteemin",
   ["pe"]="بارگذاری‌سیستم",
   ["ro"]="seteazasistem",
  },
  ["setuptab"]={
   ["cs"]="nastavtab",
   ["de"]="stelletabein",
   ["en"]="setuptab",
   ["fr"]="configurertab",
   ["it"]="impostatab",
   ["nl"]="steltabin",
   ["pe"]="بارگذاری‌تب",
   ["ro"]="seteazatab",
  },
  ["setuptables"]={
   ["cs"]="nastavtabulky",
   ["de"]="stelletabellenein",
   ["en"]="setuptables",
   ["fr"]="configurertables",
   ["it"]="impostatabelle",
   ["nl"]="steltabellenin",
   ["pe"]="بارگذاری‌جدولها",
   ["ro"]="seteazatabele",
  },
  ["setuptabulate"]={
   ["cs"]="nastavtabelaci",
   ["de"]="stelletabulatorein",
   ["en"]="setuptabulate",
   ["fr"]="configurertabulation",
   ["it"]="impostatabulato",
   ["nl"]="steltabulatiein",
   ["pe"]="بارگذاری‌جدول‌بندی",
   ["ro"]="seteazatabulatori",
  },
  ["setuptext"]={
   ["cs"]="nastavtext",
   ["de"]="stelletextein",
   ["en"]="setuptext",
   ["fr"]="configurertexte",
   ["it"]="impostatesto",
   ["nl"]="steltekstin",
   ["pe"]="بارگذاری‌متن",
   ["ro"]="seteazatext",
  },
  ["setuptextbackground"]={
   ["en"]="setuptextbackground",
   ["fr"]="configurertextearriereplan",
   ["nl"]="steltekstachtergrondin",
  },
  ["setuptextnote"]={
   ["en"]="setuptextnote",
   ["fr"]="configurernotetexte",
  },
  ["setuptextposition"]={
   ["cs"]="setuptextposition",
   ["de"]="setuptextposition",
   ["en"]="setuptextposition",
   ["fr"]="configurerpositiontexte",
   ["it"]="impostaposizionetesto",
   ["nl"]="steltekstpositiein",
   ["pe"]="بارگذاری‌مکان‌متن",
   ["ro"]="seteazapozitietext",
  },
  ["setuptextrules"]={
   ["cs"]="nastavtextovelinky",
   ["de"]="stelletextumrissein",
   ["en"]="setuptextrules",
   ["fr"]="configurertextetraits",
   ["it"]="impostalineetesto",
   ["nl"]="steltekstlijnenin",
   ["pe"]="بارگذاری‌خطهای‌متن",
   ["ro"]="seteazarigletext",
  },
  ["setuptexttexts"]={
   ["cs"]="nastavtexttexty",
   ["de"]="stelletexttexteein",
   ["en"]="setuptexttexts",
   ["fr"]="configurertextestexte",
   ["it"]="impostatestotesti",
   ["nl"]="stelteksttekstenin",
   ["pe"]="بارگذاری‌متن‌متنها",
   ["ro"]="seteazatextetext",
  },
  ["setuptextvariable"]={
   ["cs"]="setuptextvariable",
   ["de"]="setuptextvariable",
   ["en"]="setuptextvariable",
   ["fr"]="configurervariabletexte",
   ["it"]="impostavariabiletesto",
   ["nl"]="steltekstvariabelein",
   ["pe"]="بارگذاری‌متغیر‌متن",
   ["ro"]="seteazavariabilatext",
  },
  ["setupthinrules"]={
   ["cs"]="nastavtenkelinky",
   ["de"]="stelleduennerumrissein",
   ["en"]="setupthinrules",
   ["fr"]="configurertraitsfins",
   ["it"]="impostalineesottili",
   ["nl"]="steldunnelijnenin",
   ["pe"]="بارگذاری‌خطها‌ی‌نازک",
   ["ro"]="seteazaliniesubtire",
  },
  ["setuptolerance"]={
   ["cs"]="nastavtoleranci",
   ["de"]="stelletoleranzein",
   ["en"]="setuptolerance",
   ["fr"]="configurertolerance",
   ["it"]="impostatolleranza",
   ["nl"]="steltolerantiein",
   ["pe"]="بارگذاری‌بردباری",
   ["ro"]="seteazatoleranta",
  },
  ["setuptop"]={
   ["cs"]="nastavhorejsek",
   ["de"]="stellenobenein",
   ["en"]="setuptop",
   ["fr"]="configurerhaut",
   ["it"]="impostacima",
   ["nl"]="stelbovenin",
   ["pe"]="بارگذاری‌بالا",
   ["ro"]="seteazasus",
  },
  ["setuptoptexts"]={
   ["cs"]="nastavhornitexty",
   ["de"]="stelletextobenein",
   ["en"]="setuptoptexts",
   ["fr"]="configurertexteshaut",
   ["it"]="impostatesticima",
   ["nl"]="stelboventekstenin",
   ["pe"]="بارگذاری‌متنهای‌بالا",
   ["ro"]="seteazatextesus",
  },
  ["setuptype"]={
   ["cs"]="nastavtype",
   ["de"]="stelletipein",
   ["en"]="setuptype",
   ["fr"]="configurertranscrire",
   ["it"]="impostatype",
   ["nl"]="steltypein",
   ["pe"]="بارگذاری‌تایپ",
   ["ro"]="seteazatype",
  },
  ["setuptyping"]={
   ["cs"]="nastavopis",
   ["de"]="stelletippenein",
   ["en"]="setuptyping",
   ["fr"]="configurertranscription",
   ["it"]="impostatyping",
   ["nl"]="steltypenin",
   ["pe"]="بارگذاری‌تایپ‌کردن",
   ["ro"]="seteazatyping",
  },
  ["setupunderbar"]={
   ["cs"]="nastavpodtrzeni",
   ["de"]="stelleunterstreichenein",
   ["en"]="setupunderbar",
   ["fr"]="configurersouligner",
   ["it"]="impostasottolinea",
   ["nl"]="stelonderstrepenin",
   ["pe"]="بارگذاری‌میله‌زیر",
   ["ro"]="seteazasublinie",
  },
  ["setupurl"]={
   ["cs"]="nastavurl",
   ["de"]="stelleurlein",
   ["en"]="setupurl",
   ["fr"]="configurerurl",
   ["it"]="impostaurl",
   ["nl"]="stelurlin",
   ["pe"]="setupurl",
   ["ro"]="seteazaurl",
  },
  ["setupversions"]={
   ["cs"]="nastavverze",
   ["de"]="stelleversionein",
   ["en"]="setupversions",
   ["fr"]="configurerversions",
   ["it"]="impostaversioni",
   ["nl"]="stelversiesin",
   ["pe"]="بارگذاری‌نسخه‌ها",
   ["ro"]="seteazaversiuni",
  },
  ["setupwhitespace"]={
   ["cs"]="nastavbilamista",
   ["de"]="stellezwischenraumein",
   ["en"]="setupwhitespace",
   ["fr"]="configurerespaceblanc",
   ["it"]="impostaspaziobianco",
   ["nl"]="stelwitruimtein",
   ["pe"]="بارگذاری‌فضای‌سفید",
   ["ro"]="seteazaspatiualb",
  },
  ["setvisualizerfont"]={
   ["en"]="setvisualizerfont",
   ["fr"]="reglervisualisateurpolice",
  },
  ["showbodyfont"]={
   ["cs"]="ukazzakladnifont",
   ["de"]="zeigefliesstext",
   ["en"]="showbodyfont",
   ["fr"]="montrerpolicedecorps",
   ["it"]="mostrafontdeltesto",
   ["nl"]="toonkorps",
   ["pe"]="نمایش‌قلم‌بدنه",
   ["ro"]="afiseazafonttext",
  },
  ["showbodyfontenvironment"]={
   ["cs"]="ukazpostredizakladnihofontu",
   ["de"]="zeigefliesstextumgebung",
   ["en"]="showbodyfontenvironment",
   ["fr"]="montrerenvironnementpolicedecorps",
   ["it"]="mostraambientefontdeltesto",
   ["nl"]="toonkorpsomgeving",
   ["pe"]="نمایش‌محیط‌قلم‌بدنه",
   ["ro"]="afiseazamediufonttext",
  },
  ["showcolor"]={
   ["cs"]="ukazbarvu",
   ["de"]="zeigefarbe",
   ["en"]="showcolor",
   ["fr"]="montrercouleur",
   ["it"]="mostracolore",
   ["nl"]="toonkleur",
   ["pe"]="نمایش‌رنگ",
   ["ro"]="afiseazaculoare",
  },
  ["showcolorgroup"]={
   ["cs"]="ukazskupinubarev",
   ["de"]="zeigefarbengruppe",
   ["en"]="showcolorgroup",
   ["fr"]="montrergroupecouleur",
   ["it"]="mostragruppocolori",
   ["nl"]="toonkleurgroep",
   ["pe"]="نمایش‌گروه‌رنگ",
   ["ro"]="afiseazagrupculoare",
  },
  ["showcounter"]={
   ["en"]="showcounter",
   ["fr"]="montrercompteur",
  },
  ["showexternalfigures"]={
   ["cs"]="ukazexterniobrazy",
   ["de"]="zeigeexterneabbildungen",
   ["en"]="showexternalfigures",
   ["fr"]="montrerfiguresexternes",
   ["it"]="mostrafiguresterne",
   ["nl"]="toonexternefiguren",
   ["pe"]="نمایش‌شکلهای‌خارجی",
   ["ro"]="afiseazafiguriexterne",
  },
  ["showfields"]={
   ["cs"]="ukazpole",
   ["de"]="zeigefelder",
   ["en"]="showfields",
   ["fr"]="montrerchamps",
   ["it"]="mostracampi",
   ["nl"]="toonvelden",
   ["pe"]="نمایش‌میدانها",
   ["ro"]="afiseazacampuri",
  },
  ["showfont"]={
   ["en"]="showfont",
   ["fr"]="montrerpolice",
  },
  ["showfontdata"]={
   ["en"]="showfontdata",
   ["fr"]="montrerdonneespolice",
  },
  ["showfontexpansion"]={
   ["en"]="showfontexpansion",
   ["fr"]="montrerexpansionpolice",
  },
  ["showfontitalics"]={
   ["en"]="showfontitalics",
   ["fr"]="montreritaliquespolice",
  },
  ["showfontkerns"]={
   ["en"]="showfontkerns",
   ["fr"]="montrerlesapproches",
  },
  ["showfontparameters"]={
   ["en"]="showfontparameters",
   ["fr"]="montrerparametrespolice",
  },
  ["showfontstrip"]={
   ["en"]="showfontstrip",
   ["fr"]="montrervisuellementpolice",
  },
  ["showfontstyle"]={
   ["en"]="showfontstyle",
   ["fr"]="montrerstylepolice",
  },
  ["showframe"]={
   ["cs"]="ukazramecek",
   ["de"]="zeigerahmen",
   ["en"]="showframe",
   ["fr"]="montrermiseenpage",
   ["it"]="mostracornice",
   ["nl"]="toonkader",
   ["pe"]="نمایش‌قالب",
   ["ro"]="afiseazarama",
  },
  ["showgrid"]={
   ["cs"]="ukazmrizku",
   ["de"]="zeigegitter",
   ["en"]="showgrid",
   ["fr"]="montrergrille",
   ["it"]="mostragriglia",
   ["nl"]="toongrid",
   ["pe"]="نمایش‌توری",
   ["ro"]="afiseazagrid",
  },
  ["showhyphenationtrace"]={
   ["en"]="showhyphenationtrace",
   ["fr"]="montrertracecesure",
  },
  ["showkerning"]={
   ["en"]="showkerning",
   ["fr"]="montrerapproche",
  },
  ["showlayout"]={
   ["cs"]="ukazvzhled",
   ["de"]="zeigelayout",
   ["en"]="showlayout",
   ["fr"]="montrermiseenpage",
   ["it"]="mostralyout",
   ["nl"]="toonlayout",
   ["pe"]="نمایش‌طرح‌بندی",
   ["ro"]="afiseazaaspect",
  },
  ["showmakeup"]={
   ["cs"]="ukazupravu",
   ["de"]="zeigeumbruch",
   ["en"]="showmakeup",
   ["fr"]="montrercomposition",
   ["it"]="mostramakeup",
   ["nl"]="toonopmaak",
   ["pe"]="نمایش‌آرایش",
   ["ro"]="afiseazamakeup",
  },
  ["showminimalbaseline"]={
   ["en"]="showminimalbaseline",
   ["fr"]="montrerlignedebase",
  },
  ["showpalet"]={
   ["cs"]="ukazpaletu",
   ["de"]="zeigepalette",
   ["en"]="showpalet",
   ["fr"]="montrerpalette",
   ["it"]="mostratavolozza",
   ["nl"]="toonpalet",
   ["pe"]="نمایش‌لوح",
   ["ro"]="afiseazapaleta",
  },
  ["showprint"]={
   ["cs"]="ukazvytisk",
   ["de"]="zeigedruck",
   ["en"]="showprint",
   ["fr"]="montrerimpression",
   ["it"]="mostrastampa",
   ["nl"]="toonprint",
   ["pe"]="نمایش‌چاپ",
   ["ro"]="afiseazatiparire",
  },
  ["showsetups"]={
   ["cs"]="ukaznastaveni",
   ["de"]="zeigeeinstellungen",
   ["en"]="showsetups",
   ["fr"]="montrerconfigurations",
   ["it"]="mostraimpostazioni",
   ["nl"]="tooninstellingen",
   ["pe"]="نمایش‌بارگذاریها",
   ["ro"]="afiseazasetari",
  },
  ["showstruts"]={
   ["cs"]="ukazpodpery",
   ["de"]="zeigestruts",
   ["en"]="showstruts",
   ["fr"]="montrerentretoises",
   ["it"]="mostrastruts",
   ["nl"]="toonstruts",
   ["pe"]="نمایش‌بستها",
   ["ro"]="afiseazastruts",
  },
  ["showsymbolset"]={
   ["cs"]="ukazsadusymbolu",
   ["de"]="showsymbolset",
   ["en"]="showsymbolset",
   ["fr"]="montrerjeusymboles",
   ["it"]="mostrasetsimboli",
   ["nl"]="toonsymboolset",
   ["pe"]="نمایش‌مجموعه‌علامت",
   ["ro"]="afiseazasetsimboluri",
  },
  ["smallcapped"]={
   ["cs"]="kap",
   ["de"]="kap",
   ["en"]="cap",
   ["fr"]="petitescap",
   ["it"]="cap",
   ["nl"]="kap",
   ["pe"]="cap",
   ["ro"]="kap",
  },
  ["someline"]={
   ["cs"]="nejakyradek",
   ["de"]="einezeile",
   ["en"]="someline",
   ["fr"]="uneligne",
   ["it"]="qualcheriga",
   ["nl"]="eenregel",
   ["pe"]="یک‌خط",
   ["ro"]="olinie",
  },
  ["somewhere"]={
   ["cs"]="nekde",
   ["de"]="irgendwo",
   ["en"]="somewhere",
   ["fr"]="qqpart",
   ["it"]="daqualcheparte",
   ["nl"]="ergens",
   ["pe"]="یک‌جا",
   ["ro"]="undeva",
  },
  ["space"]={
   ["cs"]="mezera",
   ["de"]="spatium",
   ["en"]="space",
   ["fr"]="espace",
   ["it"]="spazio",
   ["nl"]="spatie",
   ["pe"]="فضا",
   ["ro"]="spatiu",
  },
  ["specialitem"]={
   ["en"]="specialitem",
   ["fr"]="élémentspécial",
  },
  ["splitfloat"]={
   ["cs"]="rozdelplvouciobjekt",
   ["de"]="teilegleitobjekt",
   ["en"]="splitfloat",
   ["fr"]="flottantpartage",
   ["it"]="spezzaoggettomobile",
   ["nl"]="splitsplaatsblok",
   ["pe"]="شکافتن‌شناور",
   ["ro"]="impartefloat",
  },
  ["startalignment"]={
   ["cs"]="startzarovnavani",
   ["de"]="startausrichtung",
   ["en"]="startalignment",
   ["fr"]="débutalignement",
   ["it"]="iniziaallineamento",
   ["nl"]="startuitlijnen",
   ["pe"]="شروع‌تنظیم",
   ["ro"]="startaliniere",
  },
  ["startbackground"]={
   ["cs"]="startpozadi",
   ["de"]="starthintergrund",
   ["en"]="startbackground",
   ["fr"]="débutarriereplan",
   ["it"]="iniziasfondo",
   ["nl"]="startachtergrond",
   ["pe"]="شروع‌پس‌زمینه",
   ["ro"]="startfundal",
  },
  ["startcoding"]={
   ["cs"]="startkodovani",
   ["de"]="startkodierung",
   ["en"]="startcoding",
   ["fr"]="débutcodage",
   ["it"]="iniziacodifica",
   ["nl"]="startcodering",
   ["pe"]="شروع‌کد",
   ["ro"]="startcodificare",
  },
  ["startcolor"]={
   ["cs"]="startbarva",
   ["de"]="startfarbe",
   ["en"]="startcolor",
   ["fr"]="débutcouleur",
   ["it"]="iniziacolore",
   ["nl"]="startkleur",
   ["pe"]="شروع‌رنگ",
   ["ro"]="startculoare",
  },
  ["startcolumnmakeup"]={
   ["cs"]="startcolumnmakeup",
   ["de"]="startcolumnmakeup",
   ["en"]="startcolumnmakeup",
   ["fr"]="débutcompositioncolonne",
   ["it"]="startcolumnmakeup",
   ["nl"]="startkolomopmaak",
   ["pe"]="شروع‌آرایش‌ستون",
   ["ro"]="startcolumnmakeup",
  },
  ["startcolumns"]={
   ["cs"]="startsloupce",
   ["de"]="startspalten",
   ["en"]="startcolumns",
   ["fr"]="débutcolonnes",
   ["it"]="iniziacolonne",
   ["nl"]="startkolommen",
   ["pe"]="شروع‌ستونها",
   ["ro"]="startcoloane",
  },
  ["startcolumnset"]={
   ["cs"]="startcolumnset",
   ["de"]="startcolumnset",
   ["en"]="startcolumnset",
   ["fr"]="débutjeucolonnes",
   ["it"]="iniziagruppocolonne",
   ["nl"]="startkolomgroep",
   ["pe"]="شروع‌مجموعه‌ستون",
   ["ro"]="startcolumnset",
  },
  ["startcombination"]={
   ["cs"]="startspojeni",
   ["de"]="startkombination",
   ["en"]="startcombination",
   ["fr"]="débutcombinaison",
   ["it"]="iniziacombinazione",
   ["nl"]="startcombinatie",
   ["pe"]="شروع‌ترکیب",
   ["ro"]="startcombinare",
  },
  ["startcomponent"]={
   ["cs"]="startkomponenta",
   ["de"]="startkomponente",
   ["en"]="startcomponent",
   ["fr"]="débutcomposant",
   ["it"]="iniziacomponente",
   ["nl"]="startonderdeel",
   ["pe"]="شروع‌مولفه",
   ["ro"]="startcomponenta",
  },
  ["startdocument"]={
   ["cs"]="startdokument",
   ["de"]="startdokument",
   ["en"]="startdocument",
   ["fr"]="débutdocument",
   ["it"]="iniziadocumento",
   ["nl"]="startdocument",
   ["pe"]="شروع‌نوشتار",
   ["ro"]="startdocument",
  },
  ["startenvironment"]={
   ["cs"]="startprostredi",
   ["de"]="startumgebung",
   ["en"]="startenvironment",
   ["fr"]="débutenvironnement",
   ["it"]="iniziaambiente",
   ["nl"]="startomgeving",
   ["pe"]="شروع‌محیط",
   ["ro"]="startmediu",
  },
  ["startfigure"]={
   ["cs"]="startobraz",
   ["de"]="startabbildung",
   ["en"]="startfigure",
   ["fr"]="débutfigure",
   ["it"]="iniziafigura",
   ["nl"]="startfiguur",
   ["pe"]="شروع‌شکل",
   ["ro"]="startfigura",
  },
  ["startframed"]={
   ["en"]="startframed",
   ["fr"]="débutencadre",
   ["nl"]="startomlijnd",
  },
  ["startglobal"]={
   ["cs"]="startglobalni",
   ["de"]="startglobal",
   ["en"]="startglobal",
   ["fr"]="débutglobal",
   ["it"]="iniziaglobale",
   ["nl"]="startglobaal",
   ["pe"]="شروع‌سراسری",
   ["ro"]="startglobal",
  },
  ["startline"]={
   ["cs"]="startradek",
   ["de"]="startzeile",
   ["en"]="startline",
   ["fr"]="débutligneref",
   ["it"]="iniziariga",
   ["nl"]="startregel",
   ["pe"]="شروع‌خط",
   ["ro"]="startlinie",
  },
  ["startlinecorrection"]={
   ["cs"]="startkorekceradku",
   ["de"]="startzeilenkorrektur",
   ["en"]="startlinecorrection",
   ["fr"]="débutcorrectionligne",
   ["it"]="iniziacorrezioneriga",
   ["nl"]="startregelcorrectie",
   ["pe"]="شروع‌تصحیح‌خط",
   ["ro"]="startcorectielinie",
  },
  ["startlinenote"]={
   ["en"]="startlinenote",
   ["fr"]="débutnotedeligne",
  },
  ["startlinenumbering"]={
   ["cs"]="startcislovaniradku",
   ["de"]="startzeilennumerierung",
   ["en"]="startlinenumbering",
   ["fr"]="débutnumérotationligne",
   ["it"]="inizianumerazionerighe",
   ["nl"]="startregelnummeren",
   ["pe"]="شروع‌شماره‌گذاری‌خط",
   ["ro"]="startnumerotarelinii",
  },
  ["startlines"]={
   ["cs"]="startradky",
   ["de"]="startzeilen",
   ["en"]="startlines",
   ["fr"]="débutlignes",
   ["it"]="iniziarighe",
   ["nl"]="startregels",
   ["pe"]="شروع‌خطها",
   ["ro"]="startlinii",
  },
  ["startlocal"]={
   ["cs"]="startlokalni",
   ["de"]="startlokal",
   ["en"]="startlocal",
   ["fr"]="débutlocal",
   ["it"]="inizialocale",
   ["nl"]="startlokaal",
   ["pe"]="شروع‌موضعی",
   ["ro"]="startlocal",
  },
  ["startlocalfootnotes"]={
   ["cs"]="startlokalnipoznamkypodcarou",
   ["de"]="startlokalefussnoten",
   ["en"]="startlocalfootnotes",
   ["fr"]="débutnotesbdplocal",
   ["it"]="inizianotepdplocali",
   ["nl"]="startlokalevoetnoten",
   ["pe"]="شروع‌پانوشتهای‌موضعی",
   ["ro"]="startnotesubsollocale",
  },
  ["startmakeup"]={
   ["cs"]="startuprava",
   ["de"]="startumbruch",
   ["en"]="startmakeup",
   ["fr"]="débutcomposition",
   ["it"]="iniziamakeup",
   ["nl"]="startopmaak",
   ["pe"]="شروع‌آرایش",
   ["ro"]="startmakeup",
  },
  ["startmarginblock"]={
   ["cs"]="startmarginalniblok",
   ["de"]="startmarginalblock",
   ["en"]="startmarginblock",
   ["fr"]="débutblocmarge",
   ["it"]="iniziabloccomargine",
   ["nl"]="startmargeblok",
   ["pe"]="شروع‌بلوک‌حاشیه",
   ["ro"]="startblocmarginal",
  },
  ["startmarginrule"]={
   ["cs"]="startmarginalnilinka",
   ["de"]="startmarginallinie",
   ["en"]="startmarginrule",
   ["fr"]="débuttraitmarge",
   ["it"]="inizialineamargine",
   ["nl"]="startkantlijn",
   ["pe"]="شروع‌خط‌حاشیه",
   ["ro"]="startliniemargine",
  },
  ["startnarrower"]={
   ["cs"]="startzuzeni",
   ["de"]="startenger",
   ["en"]="startnarrower",
   ["fr"]="débutplusetroit",
   ["it"]="iniziapiustretto",
   ["nl"]="startsmaller",
   ["pe"]="شروع‌باریکتر",
   ["ro"]="startingust",
  },
  ["startopposite"]={
   ["cs"]="startprotejsek",
   ["de"]="startgegenueber",
   ["en"]="startopposite",
   ["fr"]="débutopposition",
   ["it"]="iniziaopposto",
   ["nl"]="startnaast",
   ["pe"]="شروع‌مخالف",
   ["ro"]="startopozit",
  },
  ["startoverlay"]={
   ["cs"]="startprekryv",
   ["de"]="startoverlay",
   ["en"]="startoverlay",
   ["fr"]="débutsuperposition",
   ["it"]="iniziasovrapposizione",
   ["nl"]="startoverlay",
   ["pe"]="شروع‌پوشش",
   ["ro"]="startoverlay",
  },
  ["startoverview"]={
   ["cs"]="startprehled",
   ["de"]="startueberblick",
   ["en"]="startoverview",
   ["fr"]="débutvuedensemble",
   ["it"]="iniziaoverview",
   ["nl"]="startoverzicht",
   ["pe"]="شروع‌بازبینی",
   ["ro"]="startoverview",
  },
  ["startpacked"]={
   ["cs"]="startzhustene",
   ["de"]="startkleinerdurchschuss",
   ["en"]="startpacked",
   ["fr"]="débutemballage",
   ["it"]="iniziaimpaccato",
   ["nl"]="startopelkaar",
   ["pe"]="شروع‌فشرده",
   ["ro"]="startimpachetat",
  },
  ["startpositioning"]={
   ["cs"]="startumistovani",
   ["de"]="startpositionieren",
   ["en"]="startpositioning",
   ["fr"]="débutpositionnement",
   ["it"]="iniziaposizionamento",
   ["nl"]="startpositioneren",
   ["pe"]="شروع‌مکان‌گذاری",
   ["ro"]="startpozitionare",
  },
  ["startproduct"]={
   ["cs"]="startprodukt",
   ["de"]="startprodukt",
   ["en"]="startproduct",
   ["fr"]="débutproduit",
   ["it"]="iniziaprodotto",
   ["nl"]="startprodukt",
   ["pe"]="شروع‌تولید",
   ["ro"]="startprodus",
  },
  ["startprofile"]={
   ["cs"]="startprofil",
   ["de"]="startprofil",
   ["en"]="startprofile",
   ["fr"]="débutprofil",
   ["it"]="iniziaprofilo",
   ["nl"]="startprofiel",
   ["pe"]="شروع‌پروفایل",
   ["ro"]="startprofil",
  },
  ["startproject"]={
   ["cs"]="startprojekt",
   ["de"]="startprojekt",
   ["en"]="startproject",
   ["fr"]="débutprojet",
   ["it"]="iniziaprogetto",
   ["nl"]="startprojekt",
   ["pe"]="شروع‌پروژه",
   ["ro"]="startproiect",
  },
  ["startquotation"]={
   ["cs"]="startcitace",
   ["de"]="startzitat",
   ["en"]="startquotation",
   ["fr"]="débutcitation",
   ["it"]="iniziacitazione",
   ["nl"]="startcitaat",
   ["pe"]="شروع‌نقل‌قول",
   ["ro"]="startcitat",
  },
  ["startraster"]={
   ["cs"]="startrastr",
   ["de"]="startraster",
   ["en"]="startraster",
   ["fr"]="débutraster",
   ["it"]="iniziaraster",
   ["nl"]="startraster",
   ["pe"]="startraster",
   ["ro"]="startraster",
  },
  ["startsymbolset"]={
   ["cs"]="startsadasymbolu",
   ["de"]="startsymbolset",
   ["en"]="startsymbolset",
   ["fr"]="débutjeusymboles",
   ["it"]="iniziasetsimboli",
   ["nl"]="startsymboolset",
   ["pe"]="شروع‌مجموعه‌نماد",
   ["ro"]="startsetsimboluri",
  },
  ["startsynchronization"]={
   ["cs"]="startsynchronizace",
   ["de"]="startsynchronisation",
   ["en"]="startsynchronization",
   ["fr"]="débutsynchronisation",
   ["it"]="iniziasincronizzazione",
   ["nl"]="startsynchronisatie",
   ["pe"]="شروع‌تطابق",
   ["ro"]="startsincronizare",
  },
  ["starttable"]={
   ["cs"]="starttabulka",
   ["de"]="starttabelle",
   ["en"]="starttable",
   ["fr"]="débuttable",
   ["it"]="iniziatabella",
   ["nl"]="starttabel",
   ["pe"]="شروع‌جدول",
   ["ro"]="starttabel",
  },
  ["starttables"]={
   ["cs"]="starttabulky",
   ["de"]="starttabellen",
   ["en"]="starttables",
   ["fr"]="débuttables",
   ["it"]="iniziatabelle",
   ["nl"]="starttabellen",
   ["pe"]="شروع‌جدولها",
   ["ro"]="starttabele",
  },
  ["starttext"]={
   ["cs"]="starttext",
   ["de"]="starttext",
   ["en"]="starttext",
   ["fr"]="débuttexte",
   ["it"]="iniziatesto",
   ["nl"]="starttekst",
   ["pe"]="شروع‌متن",
   ["ro"]="starttext",
  },
  ["starttextbackground"]={
   ["en"]="starttextbackground",
   ["fr"]="débuttextearriereplan",
   ["nl"]="starttekstachtergrond",
  },
  ["starttextrule"]={
   ["cs"]="starttextovalinka",
   ["de"]="starttextlinie",
   ["en"]="starttextrule",
   ["fr"]="débuttextetrait",
   ["it"]="inizialineatesto",
   ["nl"]="starttekstlijn",
   ["pe"]="شروع‌خط‌متن",
   ["ro"]="startriglatext",
  },
  ["startunpacked"]={
   ["cs"]="startnezhustene",
   ["de"]="startgrosserdurchschuss",
   ["en"]="startunpacked",
   ["fr"]="débutdesemballage",
   ["it"]="iniziaunpacked",
   ["nl"]="startvanelkaar",
   ["pe"]="شروع‌غیر‌فشرده",
   ["ro"]="startneimpachetat",
  },
  ["startversion"]={
   ["cs"]="startverze",
   ["de"]="startversion",
   ["en"]="startversion",
   ["fr"]="débutversion",
   ["it"]="iniziaversione",
   ["nl"]="startversie",
   ["pe"]="شروع‌نسخه",
   ["ro"]="startversiune",
  },
  ["stopalignment"]={
   ["cs"]="stopzarovnavani",
   ["de"]="stopausrichtung",
   ["en"]="stopalignment",
   ["fr"]="finalignement",
   ["it"]="terminaallineamento",
   ["nl"]="stopuitlijnen",
   ["pe"]="پایان‌تنظیم",
   ["ro"]="stopaliniere",
  },
  ["stopbackground"]={
   ["cs"]="stoppozadi",
   ["de"]="stophintergrund",
   ["en"]="stopbackground",
   ["fr"]="finarriereplan",
   ["it"]="terminasfondo",
   ["nl"]="stopachtergrond",
   ["pe"]="پایان‌پس‌زمینه",
   ["ro"]="stopfundal",
  },
  ["stopcoding"]={
   ["cs"]="stopkodovani",
   ["de"]="stopkodierung",
   ["en"]="stopcoding",
   ["fr"]="fincodage",
   ["it"]="terminacodifica",
   ["nl"]="stopcodering",
   ["pe"]="پایان‌کد",
   ["ro"]="stopcodificare",
  },
  ["stopcolor"]={
   ["cs"]="stopbarva",
   ["de"]="stopfarbe",
   ["en"]="stopcolor",
   ["fr"]="fincouleur",
   ["it"]="terminacolore",
   ["nl"]="stopkleur",
   ["pe"]="پایان‌رنگ",
   ["ro"]="stopculoare",
  },
  ["stopcolumnmakeup"]={
   ["cs"]="stopcolumnmakeup",
   ["de"]="stopcolumnmakeup",
   ["en"]="stopcolumnmakeup",
   ["fr"]="fincompositioncolonne",
   ["it"]="stopcolumnmakeup",
   ["nl"]="stopkolomopmaak",
   ["pe"]="پایان‌آرایش‌ستون",
   ["ro"]="stopcolumnmakeup",
  },
  ["stopcolumns"]={
   ["cs"]="stopsloupce",
   ["de"]="stopspalten",
   ["en"]="stopcolumns",
   ["fr"]="fincolonnes",
   ["it"]="terminacolonne",
   ["nl"]="stopkolommen",
   ["pe"]="پایان‌ستونها",
   ["ro"]="stopcoloane",
  },
  ["stopcolumnset"]={
   ["cs"]="stopcolumnset",
   ["de"]="stopcolumnset",
   ["en"]="stopcolumnset",
   ["fr"]="finjeucolonnes",
   ["it"]="terminagruppocolonne",
   ["nl"]="stopkolomgroep",
   ["pe"]="پایان‌مجموعه‌ستون",
   ["ro"]="stopcolumnset",
  },
  ["stopcombination"]={
   ["cs"]="stopspojeni",
   ["de"]="stopkombination",
   ["en"]="stopcombination",
   ["fr"]="fincombinaison",
   ["it"]="terminacombinazione",
   ["nl"]="stopcombinatie",
   ["pe"]="پایان‌ترکیب",
   ["ro"]="stopcombinare",
  },
  ["stopcomponent"]={
   ["cs"]="stopkomponenta",
   ["de"]="stopkomponente",
   ["en"]="stopcomponent",
   ["fr"]="fincomposant",
   ["it"]="terminacomponente",
   ["nl"]="stoponderdeel",
   ["pe"]="پایان‌مولفه",
   ["ro"]="stopcomponenta",
  },
  ["stopdocument"]={
   ["cs"]="stopdokument",
   ["de"]="stopdokument",
   ["en"]="stopdocument",
   ["fr"]="findocument",
   ["it"]="terminadocumento",
   ["nl"]="stopdocument",
   ["pe"]="پایان‌نوشتار",
   ["ro"]="stopdocument",
  },
  ["stopenvironment"]={
   ["cs"]="stopprostredi",
   ["de"]="stopumgebung",
   ["en"]="stopenvironment",
   ["fr"]="finenvironnement",
   ["it"]="terminaambiente",
   ["nl"]="stopomgeving",
   ["pe"]="پایان‌محیط",
   ["ro"]="stopmediu",
  },
  ["stopframed"]={
   ["en"]="stopframed",
   ["fr"]="finencadre",
   ["nl"]="stopomlijnd",
  },
  ["stopglobal"]={
   ["cs"]="stopglobalni",
   ["de"]="stopglobal",
   ["en"]="stopglobal",
   ["fr"]="finglobal",
   ["it"]="terminaglobale",
   ["nl"]="stopglobaal",
   ["pe"]="پایان‌سراسری",
   ["ro"]="stopblobal",
  },
  ["stopline"]={
   ["cs"]="stopradek",
   ["de"]="stopzeile",
   ["en"]="stopline",
   ["fr"]="finligneref",
   ["it"]="terminariga",
   ["nl"]="stopregel",
   ["pe"]="پایان‌خط",
   ["ro"]="stoplinie",
  },
  ["stoplinecorrection"]={
   ["cs"]="stopkorekceradku",
   ["de"]="stopzeilenkorrektur",
   ["en"]="stoplinecorrection",
   ["fr"]="fincorrectionligne",
   ["it"]="terminacorrezioneriga",
   ["nl"]="stopregelcorrectie",
   ["pe"]="پایان‌تصحیح‌خط",
   ["ro"]="stopcorectielinie",
  },
  ["stoplinenote"]={
   ["en"]="stoplinenote",
   ["fr"]="finnotedeligne",
  },
  ["stoplinenumbering"]={
   ["cs"]="stopcislovaniradku",
   ["de"]="stopzeilennumerierung",
   ["en"]="stoplinenumbering",
   ["fr"]="finnumérotationligne",
   ["it"]="terminanumerazionerighe",
   ["nl"]="stopregelnummeren",
   ["pe"]="پایان‌شماره‌گذاری‌خط",
   ["ro"]="stopnumerotarelinii",
  },
  ["stoplines"]={
   ["cs"]="stopradky",
   ["de"]="stopzeilen",
   ["en"]="stoplines",
   ["fr"]="finlignes",
   ["it"]="terminarighe",
   ["nl"]="stopregels",
   ["pe"]="پایان‌خطها",
   ["ro"]="stoplinii",
  },
  ["stoplocal"]={
   ["cs"]="stoplokalni",
   ["de"]="stoplokal",
   ["en"]="stoplocal",
   ["fr"]="finlocal",
   ["it"]="terminalocale",
   ["nl"]="stoplokaal",
   ["pe"]="پایان‌موضعی",
   ["ro"]="stoplocal",
  },
  ["stoplocalfootnotes"]={
   ["cs"]="stoplokalnipoznamkypodcarou",
   ["de"]="stoplokalefussnoten",
   ["en"]="stoplocalfootnotes",
   ["fr"]="finnotesbdplocal",
   ["it"]="terminanotepdplocali",
   ["nl"]="stoplokalevoetnoten",
   ["pe"]="پایان‌پانوشتهای‌موضعی",
   ["ro"]="stopnotesubsollocale",
  },
  ["stopmakeup"]={
   ["cs"]="stopuprava",
   ["de"]="stopumbruch",
   ["en"]="stopmakeup",
   ["fr"]="fincomposition",
   ["it"]="terminamakeup",
   ["nl"]="stopopmaak",
   ["pe"]="پایان‌آرایش",
   ["ro"]="stopmakeup",
  },
  ["stopmarginblock"]={
   ["cs"]="stopmarginalniblok",
   ["de"]="stopmarginalblock",
   ["en"]="stopmarginblock",
   ["fr"]="finblocmarge",
   ["it"]="terminabloccomargine",
   ["nl"]="stopmargeblok",
   ["pe"]="پایان‌بلوک‌حاشیه",
   ["ro"]="stopblocmarginal",
  },
  ["stopmarginrule"]={
   ["cs"]="stopmarginalnilinka",
   ["de"]="stopmarginallinie",
   ["en"]="stopmarginrule",
   ["fr"]="fintraitmarge",
   ["it"]="terminalineamargine",
   ["nl"]="stopkantlijn",
   ["pe"]="پایان‌خط‌حاشیه",
   ["ro"]="stopliniemargine",
  },
  ["stopnarrower"]={
   ["cs"]="stopzuzeni",
   ["de"]="stopenger",
   ["en"]="stopnarrower",
   ["fr"]="finplusetroit",
   ["it"]="terminapiustretto",
   ["nl"]="stopsmaller",
   ["pe"]="پایان‌نازکتر",
   ["ro"]="stopingust",
  },
  ["stopopposite"]={
   ["cs"]="stopprotejsek",
   ["de"]="stopgegenueber",
   ["en"]="stopopposite",
   ["fr"]="finopposition",
   ["it"]="terminaopposto",
   ["nl"]="stopnaast",
   ["pe"]="پایان‌مخالف",
   ["ro"]="stopopozit",
  },
  ["stopoverlay"]={
   ["cs"]="stopprekryv",
   ["de"]="stopoverlay",
   ["en"]="stopoverlay",
   ["fr"]="finsuperposition",
   ["it"]="terminasovrapposizione",
   ["nl"]="stopoverlay",
   ["pe"]="پایان‌پوشش",
   ["ro"]="stopoverlay",
  },
  ["stopoverview"]={
   ["cs"]="stopprehled",
   ["de"]="stopueberblick",
   ["en"]="stopoverview",
   ["fr"]="finvuedensemble",
   ["it"]="terminaoverview",
   ["nl"]="stopoverzicht",
   ["pe"]="پایان‌بازبینی",
   ["ro"]="stopoverview",
  },
  ["stoppacked"]={
   ["cs"]="stopzhustene",
   ["de"]="stopkleinerdurchschuss",
   ["en"]="stoppacked",
   ["fr"]="finemballage",
   ["it"]="terminaimpaccato",
   ["nl"]="stopopelkaar",
   ["pe"]="پایان‌فشرده",
   ["ro"]="stopimpachetat",
  },
  ["stoppositioning"]={
   ["cs"]="stopumistovani",
   ["de"]="stoppositionieren",
   ["en"]="stoppositioning",
   ["fr"]="finpositionnement",
   ["it"]="terminaposizionamento",
   ["nl"]="stoppositioneren",
   ["pe"]="پایان‌مکان‌گذاری",
   ["ro"]="stoppozitionare",
  },
  ["stopproduct"]={
   ["cs"]="stopprodukt",
   ["de"]="stopprodukt",
   ["en"]="stopproduct",
   ["fr"]="finproduit",
   ["it"]="terminaprodotto",
   ["nl"]="stopprodukt",
   ["pe"]="پایان‌تولید",
   ["ro"]="stopprodus",
  },
  ["stopprofile"]={
   ["cs"]="stopprofil",
   ["de"]="stopprofil",
   ["en"]="stopprofile",
   ["fr"]="finprofil",
   ["it"]="terminaprofili",
   ["nl"]="stopprofiel",
   ["pe"]="پایان‌پروفایل",
   ["ro"]="stopprofil",
  },
  ["stopproject"]={
   ["cs"]="stopprojekt",
   ["de"]="stopprojekt",
   ["en"]="stopproject",
   ["fr"]="finprojet",
   ["it"]="terminaprogetto",
   ["nl"]="stopprojekt",
   ["pe"]="پایان‌پروژه",
   ["ro"]="stopproiect",
  },
  ["stopquotation"]={
   ["cs"]="stopcitace",
   ["de"]="stopzitat",
   ["en"]="stopquotation",
   ["fr"]="fincitation",
   ["it"]="terminacitazione",
   ["nl"]="stopcitaat",
   ["pe"]="پایان‌نقل‌قول",
   ["ro"]="stopcitat",
  },
  ["stopraster"]={
   ["cs"]="stoprastr",
   ["de"]="stopraster",
   ["en"]="stopraster",
   ["fr"]="finraster",
   ["it"]="terminaraster",
   ["nl"]="stopraster",
   ["pe"]="stopraster",
   ["ro"]="stopraster",
  },
  ["stopsynchronization"]={
   ["cs"]="stopsynchronizace",
   ["de"]="stopsynchronisation",
   ["en"]="stopsynchronization",
   ["fr"]="finsynchronisation",
   ["it"]="terminasincronizzazione",
   ["nl"]="stopsynchronisatie",
   ["pe"]="پایان‌تطابق",
   ["ro"]="stopsincronizare",
  },
  ["stoptable"]={
   ["cs"]="stoptabulka",
   ["de"]="stoptabelle",
   ["en"]="stoptable",
   ["fr"]="fintable",
   ["it"]="terminatabella",
   ["nl"]="stoptabel",
   ["pe"]="پایان‌جدول",
   ["ro"]="stoptabel",
  },
  ["stoptables"]={
   ["cs"]="stoptabulky",
   ["de"]="stoptabellen",
   ["en"]="stoptables",
   ["fr"]="fintables",
   ["it"]="terminatabelle",
   ["nl"]="stoptabellen",
   ["pe"]="پایان‌جدولها",
   ["ro"]="stoptabele",
  },
  ["stoptext"]={
   ["cs"]="stoptext",
   ["de"]="stoptext",
   ["en"]="stoptext",
   ["fr"]="fintexte",
   ["it"]="terminatesto",
   ["nl"]="stoptekst",
   ["pe"]="پایان‌متن",
   ["ro"]="stoptext",
  },
  ["stoptextbackground"]={
   ["en"]="stoptextbackground",
   ["fr"]="fintextearriereplan",
   ["nl"]="stoptekstachtergrond",
  },
  ["stoptextrule"]={
   ["cs"]="stoptextovalinka",
   ["de"]="stoptextlinie",
   ["en"]="stoptextrule",
   ["fr"]="fintextetrait",
   ["it"]="terminalineatesto",
   ["nl"]="stoptekstlijn",
   ["pe"]="پایان‌خط‌متن",
   ["ro"]="stopriglatext",
  },
  ["stopunpacked"]={
   ["cs"]="stopnezhustene",
   ["de"]="stopgrosserdurchschuss",
   ["en"]="stopunpacked",
   ["fr"]="findesemballage",
   ["it"]="terminaunpacked",
   ["nl"]="stopvanelkaar",
   ["pe"]="پایان‌غیرفشرده",
   ["ro"]="stopneimpachetat",
  },
  ["stopversion"]={
   ["cs"]="stopverze",
   ["de"]="stopversion",
   ["en"]="stopversion",
   ["fr"]="finversion",
   ["it"]="terminaversioni",
   ["nl"]="stopversie",
   ["pe"]="پایان‌نسخه",
   ["ro"]="stopversiune",
  },
  ["stretched"]={
   ["cs"]="roztazene",
   ["de"]="gestreckt",
   ["en"]="stretched",
   ["fr"]="etire",
   ["it"]="stirato",
   ["nl"]="uitgerekt",
   ["pe"]="کشیده",
   ["ro"]="intins",
  },
  ["strictinspectnextcharacter"]={
   ["en"]="strictinspectnextcharacter",
   ["fr"]="inspecterstrictementcaracteresuivant",
  },
  ["stripcharacter"]={
   ["en"]="stripcharacter",
   ["fr"]="demontercaractere",
  },
  ["subformulanumber"]={
   ["cs"]="cislopodrovnice",
   ["de"]="unterformelnummer",
   ["en"]="subformulanumber",
   ["fr"]="numerosousformule",
   ["it"]="numerosottoformula",
   ["nl"]="subformulenummer",
   ["pe"]="شماره‌زیرفرمول",
   ["ro"]="numarsubformula",
  },
  ["subpagenumber"]={
   ["cs"]="subpagenumber",
   ["de"]="subpagenumber",
   ["en"]="subpagenumber",
   ["fr"]="numerosouspage",
   ["it"]="subpagenumber",
   ["nl"]="subpaginanummer",
   ["pe"]="شماره‌زیرصفحه",
   ["ro"]="subpagenumber",
  },
  ["subtractfeature"]={
   ["en"]="subtractfeature",
   ["fr"]="enleverfonctionnalite",
  },
  ["switchtobodyfont"]={
   ["cs"]="prepninazakladnifont",
   ["de"]="wechselezumfliesstext",
   ["en"]="switchtobodyfont",
   ["fr"]="passerapolicedecorps",
   ["it"]="cambiaafontdeltesto",
   ["nl"]="switchnaarkorps",
   ["pe"]="تغییربه‌قلم‌بدنه",
   ["ro"]="trecilafonttext",
  },
  ["switchtorawfont"]={
   ["cs"]="switchtorawfont",
   ["de"]="switchtorawfont",
   ["en"]="switchtorawfont",
   ["fr"]="passerapolicebrute",
   ["it"]="passaafontgrezzo",
   ["nl"]="switchtorawfont",
   ["pe"]="تغییربه‌قلم‌خام",
   ["ro"]="trecilafontraw",
  },
  ["sym"]={
   ["cs"]="sym",
   ["de"]="sym",
   ["en"]="sym",
   ["fr"]="sym",
   ["it"]="sim",
   ["nl"]="sym",
   ["pe"]="نم",
   ["ro"]="sim",
  },
  ["symbol"]={
   ["cs"]="symbol",
   ["de"]="symbol",
   ["en"]="symbol",
   ["fr"]="symbole",
   ["it"]="simbolo",
   ["nl"]="symbool",
   ["pe"]="نماد",
   ["ro"]="simbol",
  },
  ["symoffset"]={
   ["en"]="symoffset",
   ["fr"]="decalagesym",
   ["nl"]="symoffset",
  },
  ["synchronizationbar"]={
   ["cs"]="synchronizacnilista",
   ["de"]="synchronisationsbalken",
   ["en"]="synchronizationbar",
   ["fr"]="barresynchronisation",
   ["it"]="barrasincronizzazione",
   ["nl"]="synchronisatiebalk",
   ["pe"]="میله‌تطابق",
   ["ro"]="barasincronizare",
  },
  ["synchronize"]={
   ["cs"]="synchronizovat",
   ["de"]="synchronisieren",
   ["en"]="synchronize",
   ["fr"]="synchroniser",
   ["it"]="sincronizza",
   ["nl"]="synchroniseer",
   ["pe"]="تطابق",
   ["ro"]="sincronizeaza",
  },
  ["tab"]={
   ["cs"]="tab",
   ["de"]="tab",
   ["en"]="tab",
   ["fr"]="tab",
   ["it"]="tab",
   ["nl"]="tab",
   ["pe"]="تب",
   ["ro"]="tab",
  },
  ["testcolumn"]={
   ["cs"]="testcolumn",
   ["de"]="testcolumn",
   ["en"]="testcolumn",
   ["fr"]="testcolonne",
   ["it"]="testcolumn",
   ["nl"]="testkolom",
   ["pe"]="ستون‌امتحان",
   ["ro"]="testcolumn",
  },
  ["testfeature"]={
   ["en"]="testfeature",
   ["fr"]="testerfonctionnalite",
  },
  ["testfeatureonce"]={
   ["en"]="testfeatureonce",
   ["fr"]="testerfonctionnaliteunfois",
  },
  ["testpage"]={
   ["cs"]="testpage",
   ["de"]="testpage",
   ["en"]="testpage",
   ["fr"]="testpage",
   ["it"]="testpage",
   ["nl"]="testpagina",
   ["pe"]="صفحه‌تست",
   ["ro"]="testpage",
  },
  ["tex"]={
   ["cs"]="tex",
   ["de"]="tex",
   ["en"]="tex",
   ["fr"]="tex",
   ["it"]="tex",
   ["nl"]="tex",
   ["pe"]="تک",
   ["ro"]="tex",
  },
  ["textbackground"]={
   ["en"]="textbackground",
   ["fr"]="textearriereplan",
   ["nl"]="tekstachtergrond",
  },
  ["textcolor"]={
   ["en"]="textcolor",
   ["fr"]="textecouleura",
  },
  ["textheight"]={
   ["cs"]="vyskatextu",
   ["de"]="texthoehe",
   ["en"]="textheight",
   ["fr"]="hauteurtexte",
   ["it"]="altezzatesto",
   ["nl"]="teksthoogte",
   ["pe"]="ارتفاع‌متن",
   ["ro"]="inaltimetext",
  },
  ["textnote"]={
   ["en"]="textnote",
   ["fr"]="notetexte",
  },
  ["textreference"]={
   ["cs"]="odkaznatext",
   ["de"]="textreferenz",
   ["en"]="textreference",
   ["fr"]="referencetexte",
   ["it"]="riferimentotesto",
   ["nl"]="tekstreferentie",
   ["pe"]="مرجع‌متن",
   ["ro"]="referintatext",
  },
  ["textrule"]={
   ["cs"]="textovalinka",
   ["de"]="textlinie",
   ["en"]="textrule",
   ["fr"]="traittexte",
   ["it"]="lineatesto",
   ["nl"]="tekstlijn",
   ["pe"]="خط‌متن",
   ["ro"]="riglatext",
  },
  ["textvariable"]={
   ["cs"]="textvariable",
   ["de"]="textvariable",
   ["en"]="textvariable",
   ["fr"]="variabletexte",
   ["it"]="variabiletesto",
   ["nl"]="tekstvariabele",
   ["pe"]="متغیر متن",
   ["ro"]="variabilatext",
  },
  ["textwidth"]={
   ["cs"]="sirkatextu",
   ["de"]="textbreite",
   ["en"]="textwidth",
   ["fr"]="largeurtexte",
   ["it"]="ampiezzatesto",
   ["nl"]="tekstbreedte",
   ["pe"]="عرض‌متن",
   ["ro"]="latimetext",
  },
  ["thefirstcharacter"]={
   ["en"]="thefirstcharacter",
   ["fr"]="lepremiercaractere",
  },
  ["thenormalizedbodyfontsize"]={
   ["en"]="thenormalizedbodyfontsize",
   ["fr"]="lataillenormaliseepolicedecorps",
  },
  ["theremainingcharacters"]={
   ["en"]="theremainingcharacters",
   ["fr"]="lescaracteresrestants",
  },
  ["thinrule"]={
   ["cs"]="tenkalinka",
   ["de"]="duennelinie",
   ["en"]="thinrule",
   ["fr"]="traitfin",
   ["it"]="lineasottile",
   ["nl"]="dunnelijn",
   ["pe"]="خط‌نازک",
   ["ro"]="liniesubtire",
  },
  ["thinrules"]={
   ["cs"]="tenkelinky",
   ["de"]="duennerumriss",
   ["en"]="thinrules",
   ["fr"]="traitsfins",
   ["it"]="lineesottili",
   ["nl"]="dunnelijnen",
   ["pe"]="خطهای‌نازک",
   ["ro"]="liniisubtiri",
  },
  ["tolinenote"]={
   ["en"]="tolinenote",
   ["fr"]="alanotedeligne",
  },
  ["tooltip"]={
   ["cs"]="bublinkovanapoveda",
   ["de"]="tooltip",
   ["en"]="tooltip",
   ["fr"]="infobulle",
   ["it"]="tooltip",
   ["nl"]="suggestie",
   ["pe"]="tooltip",
   ["ro"]="tooltip",
  },
  ["topdistance"]={
   ["cs"]="vzdalenosthorejsku",
   ["de"]="abstandoben",
   ["en"]="topdistance",
   ["fr"]="distancehaut",
   ["it"]="distanzacima",
   ["nl"]="bovenafstand",
   ["pe"]="فاصله‌بالا",
   ["ro"]="distantasus",
  },
  ["topheight"]={
   ["cs"]="vyskahorejsku",
   ["de"]="hoeheoben",
   ["en"]="topheight",
   ["fr"]="hauteurhaut",
   ["it"]="altezzacima",
   ["nl"]="bovenhoogte",
   ["pe"]="ارتفاع‌بالا",
   ["ro"]="inaltimesus",
  },
  ["topspace"]={
   ["cs"]="odsazenishora",
   ["de"]="kopfweite",
   ["en"]="topspace",
   ["fr"]="espacehaut",
   ["it"]="spaziocima",
   ["nl"]="kopwit",
   ["pe"]="فضای‌بالا",
   ["ro"]="spatiusus",
  },
  ["totalnumberofpages"]={
   ["cs"]="celkovypocetstran",
   ["de"]="gesamtseitenanzahl",
   ["en"]="totalnumberofpages",
   ["fr"]="nombretotaldepages",
   ["it"]="numerototaledipagine",
   ["nl"]="totaalaantalpaginas",
   ["pe"]="شماره‌کل‌صفحه‌ها",
   ["ro"]="numartotalpagini",
  },
  ["translate"]={
   ["cs"]="prelozit",
   ["de"]="uebersetzten",
   ["en"]="translate",
   ["fr"]="traduire",
   ["it"]="traduci",
   ["nl"]="vertaal",
   ["pe"]="ترجمه",
   ["ro"]="traduce",
  },
  ["txt"]={
   ["cs"]="txt",
   ["de"]="txt",
   ["en"]="txt",
   ["fr"]="txt",
   ["it"]="txt",
   ["nl"]="txt",
   ["pe"]="txt",
   ["ro"]="txt",
  },
  ["typ"]={
   ["cs"]="pis",
   ["de"]="tip",
   ["en"]="typ",
   ["fr"]="transcrireaveccesure",
   ["it"]="typ",
   ["nl"]="typ",
   ["pe"]="typ",
   ["ro"]="typ",
  },
  ["type"]={
   ["cs"]="opis",
   ["de"]="tippen",
   ["en"]="type",
   ["fr"]="transcrire",
   ["it"]="type",
   ["nl"]="type",
   ["pe"]="تایپ",
   ["ro"]="type",
  },
  ["typebuffer"]={
   ["cs"]="typebuffer",
   ["de"]="tippepuffer",
   ["en"]="typebuffer",
   ["fr"]="transcriretampon",
   ["it"]="typebuffer",
   ["nl"]="typebuffer",
   ["pe"]="تایپ‌بافر",
   ["ro"]="scriebuffer",
  },
  ["typedefinedbuffer"]={
   ["en"]="typedefinedbuffer",
   ["fr"]="transcriretampondefini",
  },
  ["typefile"]={
   ["cs"]="opissoubor",
   ["de"]="tippedatei",
   ["en"]="typefile",
   ["fr"]="transcrirefichier",
   ["it"]="typefile",
   ["nl"]="typefile",
   ["pe"]="تایپ‌پرونده",
   ["ro"]="typefile",
  },
  ["typeinlinebuffer"]={
   ["en"]="typeinlinebuffer",
   ["fr"]="transcriretamponenligne",
  },
  ["typing"]={
   ["en"]="typing",
   ["fr"]="transcription",
  },
  ["unitmeaning"]={
   ["cs"]="vyznam",
   ["de"]="bedeutung",
   ["en"]="unitmeaning",
   ["fr"]="significationunite",
   ["it"]="nomeunita",
   ["nl"]="betekenis",
   ["pe"]="معنی‌واحد",
   ["ro"]="numeunitate",
  },
  ["unknown"]={
   ["cs"]="neznamo",
   ["de"]="unbekant",
   ["en"]="unknown",
   ["fr"]="inconnu",
   ["it"]="ignoto",
   ["nl"]="onbekend",
   ["pe"]="ناشناس",
   ["ro"]="necunoscut",
  },
  ["unregisterhyphenationpattern"]={
   ["en"]="unregisterhyphenationpattern",
   ["fr"]="desenregistrermotifcesure",
  },
  ["useJSscripts"]={
   ["cs"]="uzijJSscripts",
   ["de"]="verwendeJSscript",
   ["en"]="useJSscripts",
   ["fr"]="utiliserJSscript",
   ["it"]="usaJSscripts",
   ["nl"]="gebruikJSscripts",
   ["pe"]="useJSscripts",
   ["ro"]="folosestescriptJS",
  },
  ["useURL"]={
   ["cs"]="uzijURL",
   ["de"]="verwendeURL",
   ["en"]="useURL",
   ["fr"]="utiliserURL",
   ["it"]="usaURL",
   ["nl"]="gebruikURL",
   ["pe"]="useURL",
   ["ro"]="folosesteURL",
  },
  ["useXMLfilter"]={
   ["cs"]="useXMLfilter",
   ["de"]="useXMLfilter",
   ["en"]="useXMLfilter",
   ["fr"]="utiliserfiltreXML",
   ["it"]="useXMLfilter",
   ["nl"]="gebruikXMLfilter",
   ["pe"]="useXMLfilter",
   ["ro"]="useXMLfilter",
  },
  ["useblocks"]={
   ["cs"]="uzijbloky",
   ["de"]="verwendebloecke",
   ["en"]="useblocks",
   ["fr"]="utiliserblocs",
   ["it"]="usablocco",
   ["nl"]="gebruikblokken",
   ["pe"]="استفاده‌بلوکها",
   ["ro"]="folosestebloc",
  },
  ["usebodyfont"]={
   ["en"]="usebodyfont",
   ["fr"]="utiliserpolicedecorps",
  },
  ["usebodyfontparameter"]={
   ["en"]="usebodyfontparameter",
   ["fr"]="utiliserparametrepolicedecorps",
  },
  ["usecommands"]={
   ["cs"]="uzijprikazy",
   ["de"]="verwendebefehl",
   ["en"]="usecommands",
   ["fr"]="utilisercommandes",
   ["it"]="usacomandi",
   ["nl"]="gebruikcommandos",
   ["pe"]="استفاده‌فرمانها",
   ["ro"]="folosestecomenzi",
  },
  ["useencoding"]={
   ["cs"]="uzijkodovani",
   ["de"]="benutzekodierung",
   ["en"]="useencoding",
   ["fr"]="utilisercodage",
   ["it"]="usacodifica",
   ["nl"]="usecodering",
   ["pe"]="استفاده‌رمزینه",
   ["ro"]="folosestecodificarea",
  },
  ["useexternaldocument"]={
   ["cs"]="uzijexternidokument",
   ["de"]="verwendeexteresdokument",
   ["en"]="useexternaldocument",
   ["fr"]="utiliserdocumentexterne",
   ["it"]="usadocumentoesterno",
   ["nl"]="gebruikexterndocument",
   ["pe"]="استفاده‌نوشتارخارجی",
   ["ro"]="folosestedocumentextern",
  },
  ["useexternalfigure"]={
   ["cs"]="uzijexterniobraz",
   ["de"]="verwendeexterneabbildung",
   ["en"]="useexternalfigure",
   ["fr"]="utiliserfigureexterne",
   ["it"]="usafiguraesterna",
   ["nl"]="gebruikexternfiguur",
   ["pe"]="استفاده‌شکل‌خارجی",
   ["ro"]="folosestefiguraexterna",
  },
  ["useexternalfile"]={
   ["cs"]="uzijexternisoubor",
   ["de"]="verwendeexternedatei",
   ["en"]="useexternalfile",
   ["fr"]="utiliserfichierexterne",
   ["it"]="usafileesterno",
   ["nl"]="gebruikexternefile",
   ["pe"]="استفاده‌پرونده‌خارجی",
   ["ro"]="folosestefisierextern",
  },
  ["useexternalfiles"]={
   ["cs"]="uzijexternisoubory",
   ["de"]="verwendeexternedateien",
   ["en"]="useexternalfiles",
   ["fr"]="utiliserfichiersexternes",
   ["it"]="usafileesterni",
   ["nl"]="gebruikexternefiles",
   ["pe"]="استفاده‌پرونده‌های‌خارجی",
   ["ro"]="folosestefisiereexterne",
  },
  ["useexternalsoundtrack"]={
   ["cs"]="uzijexternizvuk",
   ["de"]="verwendeexternestonstueck",
   ["en"]="useexternalsoundtrack",
   ["fr"]="utilisersonexterne",
   ["it"]="usacolonnasonoraesterna",
   ["nl"]="gebruikexterngeluidsfragment",
   ["pe"]="استفاده‌قطعه‌موزیک‌خارجی",
   ["ro"]="folosestemuzicaexterna",
  },
  ["usemodule"]={
   ["cs"]="uzijmodul",
   ["de"]="verwendemodul",
   ["en"]="usemodule",
   ["fr"]="utilisermodule",
   ["it"]="usamodulo",
   ["nl"]="gebruikmodule",
   ["pe"]="استفاده‌مدول",
   ["ro"]="folosestemodul",
  },
  ["usemodules"]={
   ["cs"]="uzijmoduly",
   ["de"]="verwendemodule",
   ["en"]="usemodules",
   ["fr"]="utilisermodules",
   ["it"]="usamoduli",
   ["nl"]="gebruikmodules",
   ["pe"]="استفاده‌مدولها",
   ["ro"]="folosestemodule",
  },
  ["usepath"]={
   ["cs"]="uzijadresar",
   ["de"]="benutzeverzeichnis",
   ["en"]="usedirectory",
   ["fr"]="utiliserchemin",
   ["it"]="usacartella",
   ["nl"]="gebruikpad",
   ["pe"]="استفاده‌مسیر",
   ["ro"]="folosestedirector",
  },
  ["usereferences"]={
   ["cs"]="uzijodkazy",
   ["de"]="verwendereferenzen",
   ["en"]="usereferences",
   ["fr"]="utiliserreferences",
   ["it"]="usariferimenti",
   ["nl"]="gebruikreferenties",
   ["pe"]="استفاده‌مرجعها",
   ["ro"]="folosestereferinte",
  },
  ["usespecials"]={
   ["cs"]="uzijspeciality",
   ["de"]="benutzespezielles",
   ["en"]="usespecials",
   ["fr"]="utiliserspeciaux",
   ["it"]="usaspecialita",
   ["nl"]="gebruikspecials",
   ["pe"]="استفاده‌ویژگیها",
   ["ro"]="folosestespeciale",
  },
  ["usesymbols"]={
   ["cs"]="uzijsymbol",
   ["de"]="verwendesymbole",
   ["en"]="usesymbols",
   ["fr"]="utilisersymboles",
   ["it"]="usasimboli",
   ["nl"]="gebruiksymbolen",
   ["pe"]="استفاده‌نمادها",
   ["ro"]="folosestesimboluri",
  },
  ["usetypescript"]={
   ["cs"]="usetypescript",
   ["de"]="usetypescript",
   ["en"]="usetypescript",
   ["fr"]="utiliserscriptdepolices",
   ["it"]="usetypescript",
   ["nl"]="gebruiktypescript",
   ["pe"]="استفاده‌دستخط‌تایپ",
   ["ro"]="usetypescript",
  },
  ["usetypescriptfile"]={
   ["cs"]="usetypescriptfile",
   ["de"]="usetypescriptfile",
   ["en"]="usetypescriptfile",
   ["fr"]="utiliserfichierscriptdepolices",
   ["it"]="usetypescriptfile",
   ["nl"]="gebruiktypescriptfile",
   ["pe"]="استفاده‌پرونده‌دستخط‌تایپ",
   ["ro"]="usetypescriptfile",
  },
  ["useurl"]={
   ["cs"]="uzijurl",
   ["de"]="verwendeurl",
   ["en"]="useurl",
   ["fr"]="utiliserurl",
   ["it"]="usaurl",
   ["nl"]="gebruikurl",
   ["pe"]="useurl",
   ["ro"]="folosesteurl",
  },
  ["verbatim"]={
   ["en"]="verbatim",
   ["fr"]="verbatim",
  },
  ["version"]={
   ["cs"]="verze",
   ["de"]="version",
   ["en"]="version",
   ["fr"]="version",
   ["it"]="versione",
   ["nl"]="versie",
   ["pe"]="نسخه",
   ["ro"]="versiune",
  },
  ["vl"]={
   ["cs"]="vl",
   ["de"]="vl",
   ["en"]="vl",
   ["fr"]="lv",
   ["it"]="vl",
   ["nl"]="vl",
   ["pe"]="خ‌ع",
   ["ro"]="vl",
  },
  ["weekday"]={
   ["cs"]="vsedniden",
   ["de"]="wochentag",
   ["en"]="weekday",
   ["fr"]="joursemaine",
   ["it"]="giornosettimana",
   ["nl"]="weekdag",
   ["pe"]="روزهفته",
   ["ro"]="zidinsaptamana",
  },
  ["whitespace"]={
   ["cs"]="bilemisto",
   ["de"]="zwischenraum",
   ["en"]="whitespace",
   ["fr"]="espaceblanc",
   ["it"]="spaziobianco",
   ["nl"]="witruimte",
   ["pe"]="فضای‌سفید",
   ["ro"]="spatiualb",
  },
  ["widthspanningtext"]={
   ["en"]="widthspanningtext",
   ["fr"]="largeurtextecouvrant",
  },
  ["wordright"]={
   ["cs"]="slovovpravo",
   ["de"]="wortrechts",
   ["en"]="wordright",
   ["fr"]="ecrireadroite",
   ["it"]="paroladestra",
   ["nl"]="woordrechts",
   ["pe"]="کلمه‌راست",
   ["ro"]="cuvantdreapta",
  },
  ["writebetweenlist"]={
   ["cs"]="zapismeziseznam",
   ["de"]="schreibezwischenliste",
   ["en"]="writebetweenlist",
   ["fr"]="ecrireentreliste",
   ["it"]="scrividentroelenco",
   ["nl"]="schrijftussenlijst",
   ["pe"]="بنویس‌بین‌لیست",
   ["ro"]="scrieintreliste",
  },
  ["writetolist"]={
   ["cs"]="zapisdoseznamu",
   ["de"]="schreibezurliste",
   ["en"]="writetolist",
   ["fr"]="ecriredansliste",
   ["it"]="scriviinelenco",
   ["nl"]="schrijfnaarlijst",
   ["pe"]="بنویس‌در‌لیست",
   ["ro"]="scrieinlista",
  },
  ["writetoreferencelist"]={
   ["cs"]="zapisdoseznamuodkazu",
   ["de"]="schreibezurreferenzliste",
   ["en"]="writetoreferencelist",
   ["fr"]="ecriredanslistereference",
   ["it"]="scriviinlistariferimenti",
   ["nl"]="schrijfnaarreferentielijst",
   ["pe"]="بنویس‌درلیست‌مرجع",
   ["ro"]="scrieinlistareferinte",
  },
  ["writetoregister"]={
   ["cs"]="zapisdorejstriku",
   ["de"]="schreibezumregister",
   ["en"]="writetoregister",
   ["fr"]="ecriredansregistre",
   ["it"]="scriviinregistro",
   ["nl"]="schrijfnaarregister",
   ["pe"]="بنویس‌درثبت",
   ["ro"]="scrieinregistru",
  },
  -- we still need to add some new ones here as i completely forgot that it's
  -- needed ... maybe ws has a backlog for setups that we can use
  ["definesavebuffer"]={
   ["en"]="startsavebuffer",
  },
  ["startnamedtyping"]={
   ["en"]="startnamedtyping",
  },
  ["stopnamedtyping"]={
   ["en"]="stopnamedtyping",
  },
  ["namedconstruction"]={
   ["en"]="namedconstruction",
  },
  ["startnamedconstruction"]={
   ["en"]="startnamedconstruction",
  },
  ["stopnamedconstruction"]={
   ["en"]="stopnamedconstruction",
  },
  ["nameddescription"]={
   ["en"]="nameddescription",
  },
  ["startnameddescription"]={
   ["en"]="startnameddescription",
  },
  ["stopnameddescription"]={
   ["en"]="stopnameddescription",
  },
  ["namedenumeration"]={
   ["en"]="namedenumeration",
  },
  ["startnamedenumeration"]={
   ["en"]="startnamedenumeration",
  },
  ["stopnamedenumeration"]={
   ["en"]="stopnamedenumeration",
  },
  ["startnamedsection"]={
   ["en"]="startnamedsection",
  },
  ["stopnamedsection"]={
   ["en"]="stopnamedsection",
  },
  ["startnamedmatrix"]={
   ["en"]="startnamedmatrix",
  },
  ["stopnamedmatrix"]={
   ["en"]="stopnamedmatrix",
  },
  ["startnamedsubformulas"]={
   ["en"]="startnamedsubformulas",
  },
  ["stopnamedsubformulas"]={
   ["en"]="stopnamedsubformulas",
  },
 },
 ["constants"]={
  ["action"]={
   ["cs"]="akce",
   ["de"]="aktion",
   ["en"]="action",
   ["fr"]="action",
   ["it"]="azione",
   ["nl"]="actie",
   ["pe"]="کنش",
   ["ro"]="actiune",
  },
  ["adaptive"]={
   ["en"]="adaptive",
  },
  ["freezespacing"]={
   ["en"]="freezespacing",
  },
  ["address"]={
   ["cs"]="adresa",
   ["de"]="adresse",
   ["en"]="address",
   ["fr"]="adresse",
   ["it"]="indirizzo",
   ["nl"]="adres",
   ["pe"]="نشانی",
   ["ro"]="adresa",
  },
  ["after"]={
   ["cs"]="po",
   ["de"]="nach",
   ["en"]="after",
   ["fr"]="après",
   ["it"]="dopo",
   ["nl"]="na",
   ["pe"]="بعداز",
   ["ro"]="dupa",
  },
  ["afterhead"]={
   ["cs"]="pohlavicce",
   ["de"]="nachkopf",
   ["en"]="afterhead",
   ["fr"]="aprèstête",
   ["it"]="dopotesta",
   ["nl"]="kopna",
   ["pe"]="بعدازسر",
   ["ro"]="dupatitlu",
  },
  ["afterkey"]={
   ["cs"]="klavesapo",
   ["de"]="nachtaste",
   ["en"]="afterkey",
   ["fr"]="aprèsclé",
   ["it"]="dopotasto",
   ["nl"]="natoets",
   ["pe"]="بعدازکلید",
   ["ro"]="dupatasta",
  },
  ["aftersection"]={
   ["en"]="aftersection",
   ["fr"]="aprèssection",
   ["nl"]="nasectie",
  },
  ["align"]={
   ["cs"]="zarovnani",
   ["de"]="ausrichtung",
   ["en"]="align",
   ["fr"]="aligner",
   ["it"]="allinea",
   ["nl"]="uitlijnen",
   ["pe"]="تنظیم",
   ["ro"]="aliniere",
  },
  ["alignbottom"]={
   ["en"]="alignbottom",
   ["fr"]="alignerbas",
  },
  ["topalign"]={
   ["en"]="topalign",
  },
  ["bottomalign"]={
   ["en"]="bottomalign",
  },
  ["aligncharacter"]={
   ["cs"]="aligncharacter",
   ["de"]="aligncharacter",
   ["en"]="aligncharacter",
   ["fr"]="alignersurcaractere",
   ["it"]="allineacarattere",
   ["nl"]="karakteruitlijnen",
   ["pe"]="حرف‌تنظیم",
   ["ro"]="aliniazacaracter",
  },
  ["alignmentcharacter"]={
   ["cs"]="alignmentcharacter",
   ["de"]="alignmentcharacter",
   ["en"]="alignmentcharacter",
   ["fr"]="caracterealignement",
   ["it"]="carattereallineamento",
   ["nl"]="uitlijnkarakter",
   ["pe"]="حرف‌تنظیم‌کردن",
   ["ro"]="alierecaracter",
  },
  ["alignmentleftsample"]={
   ["en"]="alignmentleftsample",
   ["fr"]="echantillongauchealignement",
  },
  ["alignmentleftwidth"]={
   ["en"]="alignmentleftwidth",
   ["fr"]="largeurgauchealignement",
  },
  ["alignmentrightsample"]={
   ["en"]="alignmentrightsample",
   ["fr"]="echantillondroitealignement",
  },
  ["alignmentrightwidth"]={
   ["en"]="alignmentrightwidth",
   ["fr"]="largeurdroitealignement",
  },
  ["interscriptfactor"]={
    ["en"]="interscriptfactor",
  },
  ["alignscripts"]={
   ["en"]="alignscripts",
  },
  ["alignsymbol"]={
   ["en"]="alignsymbol",
   ["fr"]="alignersursymbol",
  },
  ["aligntitle"]={
   ["cs"]="zarovnejtitul",
   ["de"]="titelausrichten",
   ["en"]="aligntitle",
   ["fr"]="alignersurtitre",
   ["it"]="allineatitolo",
   ["nl"]="titeluitlijnen",
   ["pe"]="عنوان‌تنظیم",
   ["ro"]="alinieretitlu",
  },
  ["alternative"]={
   ["cs"]="alternativa",
   ["de"]="alternative",
   ["en"]="alternative",
   ["fr"]="variante",
   ["it"]="alternativa",
   ["nl"]="variant",
   ["pe"]="جایگزین",
   ["ro"]="alternativ",
  },
  ["alternativetext"]={
   ["en"]="alternativetext",
  },
  ["anchor"]={
   ["en"]="anchor",
   ["fr"]="ancre",
   ["nl"]="anker",
  },
  ["anchoring"]={
   ["en"]="anchoring",
   ["fr"]="ancrage",
   ["nl"]="verankering",
  },
  ["andtext"]={
   ["en"]="andtext",
   ["fr"]="ettexte",
  },
  ["apa"]={
   ["cs"]="apa",
   ["de"]="apa",
   ["en"]="apa",
   ["fr"]="apa",
   ["it"]="apa",
   ["nl"]="apa",
   ["pe"]="apa",
   ["ro"]="apa",
  },
  ["arguments"]={
   ["en"]="arguments",
   ["fr"]="arguments",
   ["nl"]="argumenten",
  },
  ["arrow"]={
   ["cs"]="sipka",
   ["de"]="pfeil",
   ["en"]="arrow",
   ["fr"]="fleche",
   ["it"]="freccia",
   ["nl"]="pijl",
   ["pe"]="پیکان",
   ["ro"]="sageata",
  },
  ["artauthor"]={
   ["en"]="artauthor",
   ["fr"]="artauteur",
  },
  ["artauthoretaldisplay"]={
   ["en"]="artauthoretaldisplay",
   ["fr"]="artauteuretalaffichage",
  },
  ["artauthoretallimit"]={
   ["en"]="artauthoretallimit",
   ["fr"]="artauteuretallimite",
  },
  ["artauthoretaltext"]={
   ["en"]="artauthoretaltext",
   ["fr"]="artauteuretaltexte",
  },
  ["artoffset"]={
   ["en"]="artoffset",
   ["fr"]="artdecalage",
  },
  ["at"]={
   ["cs"]="u",
   ["de"]="bei",
   ["en"]="at",
   ["fr"]="en",
   ["it"]="ad",
   ["nl"]="op",
   ["pe"]="در",
   ["ro"]="la",
  },
  ["author"]={
   ["cs"]="autor",
   ["de"]="autor",
   ["en"]="author",
   ["fr"]="auteur",
   ["it"]="autore",
   ["nl"]="auteur",
   ["pe"]="نویسنده",
   ["ro"]="autor",
  },
  ["authorconversion"]={
   ["en"]="authorconversion",
   ["fr"]="auteurconversion",
  },
  ["authoretaldisplay"]={
   ["en"]="authoretaldisplay",
   ["fr"]="auteuretalaffichage",
  },
  ["authoretallimit"]={
   ["en"]="authoretallimit",
   ["fr"]="auteuretallimite",
  },
  ["authoretaltext"]={
   ["en"]="authoretaltext",
   ["fr"]="auteuretaltexte",
  },
  ["auto"]={
   ["cs"]="auto",
   ["de"]="auto",
   ["en"]="auto",
   ["fr"]="auto",
   ["it"]="auto",
   ["nl"]="auto",
   ["pe"]="خودکار",
   ["ro"]="auto",
  },
  ["autocase"]={
   ["en"]="autocase",
   ["fr"]="autocase",
   ["nl"]="autocase",
  },
  ["autofile"]={
   ["cs"]="autofile",
   ["de"]="autofile",
   ["en"]="autofile",
   ["fr"]="autofichier",
   ["it"]="autofile",
   ["nl"]="autofile",
   ["pe"]="پرونده‌خودکار",
   ["ro"]="autofile",
  },
  ["autofocus"]={
   ["cs"]="autoostreni",
   ["de"]="autofocus",
   ["en"]="autofocus",
   ["fr"]="autofocus",
   ["it"]="autofocus",
   ["nl"]="autofocus",
   ["pe"]="تمرکز‌خودکار",
   ["ro"]="autofocus",
  },
  ["autohang"]={
   ["cs"]="autohang",
   ["de"]="autohang",
   ["en"]="autohang",
   ["fr"]="autosuspendre",
   ["it"]="autohang",
   ["nl"]="autohang",
   ["pe"]="آویزان‌خودکار",
   ["ro"]="autohang",
  },
  ["autostrut"]={
   ["cs"]="autostrut",
   ["de"]="autostrut",
   ["en"]="autostrut",
   ["fr"]="autoentretoise",
   ["it"]="autostrut",
   ["nl"]="autostrut",
   ["pe"]="بست‌خودکار",
   ["ro"]="autostrut",
  },
  ["autowidth"]={
   ["cs"]="autosirka",
   ["de"]="autobreite",
   ["en"]="autowidth",
   ["fr"]="autolargeur",
   ["it"]="autoampiezza",
   ["nl"]="autobreedte",
   ["pe"]="عرض‌خودکار",
   ["ro"]="autolatime",
  },
  ["autointervals"]={
   ["en"]="autointervals",
  },
  ["autopunctuation"]={
   ["en"]="autopunctuation",
   ["fr"]="autoponctuation",
  },
  ["autospacing"]={
   ["en"]="autospacing",
   ["fr"]="autospacing",
  },
  ["autofencing"]={
   ["en"]="autofencing",
   ["fr"]="autofencing",
  },
  ["autonumbers"]={
   ["en"]="autonumbers",
   ["fr"]="autonumbers",
  },
  ["availableheight"]={
   ["en"]="availableheight",
   ["fr"]="hauteurdisponible",
   ["nl"]="beschikbarehoogte",
  },
  ["availablewidth"]={
   ["en"]="availablewidth",
   ["fr"]="largeurdisponible",
   ["nl"]="beschikbarebreedte",
  },
  ["axis"]={
   ["cs"]="osa",
   ["de"]="achsen",
   ["en"]="axis",
   ["fr"]="axe",
   ["it"]="asse",
   ["nl"]="assenstelsel",
   ["pe"]="محورها",
   ["ro"]="axa",
  },
  ["background"]={
   ["cs"]="pozadi",
   ["de"]="hintergrund",
   ["en"]="background",
   ["fr"]="arriereplan",
   ["it"]="sfondo",
   ["nl"]="achtergrond",
   ["pe"]="پس‌زمینه",
   ["ro"]="fundal",
  },
  ["backgroundcolor"]={
   ["cs"]="barvapozadi",
   ["de"]="hintergrundfarbe",
   ["en"]="backgroundcolor",
   ["fr"]="couleurarriereplan",
   ["it"]="coloresfondo",
   ["nl"]="achtergrondkleur",
   ["pe"]="رنگ‌پس‌زمینه",
   ["ro"]="culoarefundal",
  },
  ["backgroundcorner"]={
   ["cs"]="rohpozadi",
   ["de"]="hintergrundwinkel",
   ["en"]="backgroundcorner",
   ["fr"]="coinarriereplan",
   ["it"]="angolosfondo",
   ["nl"]="achtergrondhoek",
   ["pe"]="گوشه‌پس‌زمینه",
   ["ro"]="coltfundal",
  },
  ["backgrounddepth"]={
   ["cs"]="hloubkapozadi",
   ["de"]="hintergrundtiefe",
   ["en"]="backgrounddepth",
   ["fr"]="profondeurarriereplan",
   ["it"]="profonditasfondo",
   ["nl"]="achtergronddiepte",
   ["pe"]="عمق‌پس‌زمینه",
   ["ro"]="adancimefundal",
  },
  ["backgroundoffset"]={
   ["cs"]="offsetpozadi",
   ["de"]="hintergrundoffset",
   ["en"]="backgroundoffset",
   ["fr"]="decalagearriereplan",
   ["it"]="offsetsfondo",
   ["nl"]="achtergrondoffset",
   ["pe"]="آفست‌پس‌زمینه",
   ["ro"]="offsetfundal",
  },
  ["backgroundradius"]={
   ["cs"]="polomerpozadi",
   ["de"]="hintergrundradius",
   ["en"]="backgroundradius",
   ["fr"]="rayonarriereplan",
   ["it"]="raggiosfondo",
   ["nl"]="achtergrondstraal",
   ["pe"]="شعاع‌پس‌زمینه",
   ["ro"]="razafundal",
  },
  ["backgroundscreen"]={
   ["cs"]="rastrpozadi",
   ["de"]="hintergrundraster",
   ["en"]="backgroundscreen",
   ["fr"]="ecranarriereplan",
   ["it"]="schermosfondo",
   ["nl"]="achtergrondraster",
   ["pe"]="پرده‌پس‌زمینه",
   ["ro"]="ecranfundal",
  },
  ["backreference"]={
   ["en"]="backreference",
   ["fr"]="retroreference",
   ["nl"]="terugreferentie",
  },
  ["backspace"]={
   ["cs"]="zpetnamezera",
   ["de"]="rumpfspatium",
   ["en"]="backspace",
   ["fr"]="espacearrière",
   ["it"]="backspace",
   ["nl"]="rugwit",
   ["pe"]="فضای‌پس",
   ["ro"]="spatiuspate",
  },
  ["balance"]={
   ["cs"]="rovnovaha",
   ["de"]="ausgleichen",
   ["en"]="balance",
   ["fr"]="equilibrer",
   ["it"]="bilanciamento",
   ["nl"]="balanceren",
   ["pe"]="تعادل",
   ["ro"]="balanta",
  },
  ["before"]={
   ["cs"]="pred",
   ["de"]="vor",
   ["en"]="before",
   ["fr"]="avant",
   ["it"]="prima",
   ["nl"]="voor",
   ["pe"]="قبل‌از",
   ["ro"]="inainte",
  },
  ["beforehead"]={
   ["cs"]="predhlavickou",
   ["de"]="vorkopf",
   ["en"]="beforehead",
   ["fr"]="avanttete",
   ["it"]="primaditesta",
   ["nl"]="kopvoor",
   ["pe"]="قبل‌ازسر",
   ["ro"]="inaintetitlu",
  },
  ["beforesection"]={
   ["en"]="beforesection",
   ["fr"]="avantsection",
   ["nl"]="voorsectie",
  },
  ["bet"]={
   ["cs"]="bet",
   ["de"]="bet",
   ["en"]="bet",
   ["fr"]="suj",
   ["it"]="bet",
   ["nl"]="bet",
   ["pe"]="bet",
   ["ro"]="bet",
  },
  ["bidi"]={
   ["en"]="bidi",
   ["fr"]="bidi",
  },
  ["big"]={
   ["cs"]="velky",
   ["de"]="gross",
   ["en"]="big",
   ["fr"]="important",
   ["it"]="grande",
   ["nl"]="groot",
   ["pe"]="بزرگ",
   ["ro"]="mare",
  },
  ["blank"]={
   ["cs"]="prazdny",
   ["de"]="blanko",
   ["en"]="blank",
   ["fr"]="vierge",
   ["it"]="rigovuoto",
   ["nl"]="blanko",
   ["pe"]="خالی",
   ["ro"]="blanc",
  },
  ["bleedoffset"]={
   ["en"]="bleedoffset",
   ["fr"]="decalagefuite",
  },
  ["blockway"]={
   ["cs"]="dobloku",
   ["de"]="blockauf",
   ["en"]="blockway",
   ["fr"]="faconbloc",
   ["it"]="blockway",
   ["nl"]="blokwijze",
   ["pe"]="راه‌بلوک",
   ["ro"]="blockway",
  },
  ["bodyfont"]={
   ["cs"]="zakladnifont",
   ["de"]="fliesstext",
   ["en"]="bodyfont",
   ["fr"]="policedecorps",
   ["it"]="fonttesto",
   ["nl"]="korps",
   ["pe"]="قلم‌بدنه",
   ["ro"]="fonttext",
  },
  ["boffset"]={
   ["en"]="boffset",
   ["fr"]="decalageb",
  },
  ["boldfeatures"]={
   ["en"]="boldfeatures",
   ["fr"]="fonctionnalitesgras",
  },
  ["boldfont"]={
   ["en"]="boldfont",
   ["fr"]="policegras",
  },
  ["bolditalicfeatures"]={
   ["en"]="bolditalicfeatures",
   ["fr"]="fonctionnalitesgrasitalique",
  },
  ["bolditalicfont"]={
   ["en"]="bolditalicfont",
   ["fr"]="policegrasitalique",
  },
  ["boldslantedfeatures"]={
   ["en"]="boldslantedfeatures",
   ["fr"]="fonctionnalitesgrasincline",
  },
  ["boldslantedfont"]={
   ["en"]="boldslantedfont",
   ["fr"]="policegrasincline",
  },
  ["bookmark"]={
   ["cs"]="zalozka",
   ["de"]="bookmark",
   ["en"]="bookmark",
   ["fr"]="marquepage",
   ["it"]="segnalibro",
   ["nl"]="bookmark",
   ["pe"]="چوبخط",
   ["ro"]="semncarte",
  },
  ["bottom"]={
   ["cs"]="spodek",
   ["de"]="unten",
   ["en"]="bottom",
   ["fr"]="bas",
   ["it"]="fondo",
   ["nl"]="onder",
   ["pe"]="پایین",
   ["ro"]="jos",
  },
  ["bottomafter"]={
   ["en"]="bottomafter",
   ["fr"]="basaprès",
  },
  ["bottombefore"]={
   ["en"]="bottombefore",
   ["fr"]="basavant",
  },
  ["bottomcommand"]={
   ["en"]="bottomcommand",
   ["fr"]="commandebas",
   ["nl"]="ondercommando",
  },
  ["bottomcolor"]={
   ["en"]="bottomcolor",
   ["nl"]="onderkleur",
  },
  ["bottomdistance"]={
   ["cs"]="vzdalenostspodku",
   ["de"]="abstandunten",
   ["en"]="bottomdistance",
   ["fr"]="distancebas",
   ["it"]="distanzafondo",
   ["nl"]="onderafstand",
   ["pe"]="فاصله‌پایین",
   ["ro"]="distantajos",
  },
  ["bottomframe"]={
   ["cs"]="ramecekdole",
   ["de"]="untenrahmen",
   ["en"]="bottomframe",
   ["fr"]="cadrebas",
   ["it"]="cornicefondo",
   ["nl"]="onderkader",
   ["pe"]="قالب‌پایین",
   ["ro"]="framejos",
  },
  ["bottomoffset"]={
   ["cs"]="offsetspodku",
   ["de"]="untenoffset",
   ["en"]="bottomoffset",
   ["fr"]="decalagebas",
   ["it"]="offsetfondo",
   ["nl"]="onderoffset",
   ["pe"]="آفست‌پایین",
   ["ro"]="offsetjos",
  },
  ["bottomspace"]={
   ["cs"]="bottomspace",
   ["de"]="bottomspace",
   ["en"]="bottomspace",
   ["fr"]="espacebas",
   ["it"]="spaziofondo",
   ["nl"]="bodemwit",
   ["pe"]="فضای‌پایین",
   ["ro"]="spatiujos",
  },
  ["bottomstate"]={
   ["cs"]="statusspodku",
   ["de"]="untenstatus",
   ["en"]="bottomstate",
   ["fr"]="etatbas",
   ["it"]="statofondo",
   ["nl"]="onderstatus",
   ["pe"]="وضعیت‌پایین",
   ["ro"]="starejos",
  },
  ["bottomstyle"]={
   ["en"]="bottomstyle",
  },
  ["break"]={
   ["en"]="break",
   ["fr"]="saut",
   ["nl"]="break",
  },
  ["buffer"]={
   ["en"]="buffer",
   ["fr"]="tampon",
   ["nl"]="buffer",
  },
  ["cache"]={
   ["cs"]="cache",
   ["de"]="cache",
   ["en"]="cache",
   ["fr"]="memoirecache",
   ["it"]="cache",
   ["nl"]="cache",
   ["pe"]="میانگیر",
   ["ro"]="cache",
  },
  ["calculate"]={
   ["cs"]="pocitat",
   ["de"]="berechnen",
   ["en"]="calculate",
   ["fr"]="calculer",
   ["it"]="calcola",
   ["nl"]="bereken",
   ["pe"]="محاسبه",
   ["ro"]="calculeaza",
  },
  ["category"]={
   ["en"]="category",
   ["fr"]="categorie",
   ["nl"]="categorie",
  },
  ["ccommand"]={
   ["cs"]="cprikaz",
   ["de"]="cbefehl",
   ["en"]="ccommand",
   ["fr"]="commandec",
   ["it"]="ccomando",
   ["nl"]="ccommando",
   ["pe"]="فرمان",
   ["ro"]="comandac",
  },
  ["character"]={
   ["en"]="character",
   ["fr"]="caractere",
  },
  ["characters"]={
   ["en"]="characters",
   ["fr"]="caracteres",
  },
  ["check"]={
   ["en"]="check",
   ["fr"]="verifier",
  },
  ["click"]={
   ["cs"]="klik",
   ["de"]="klick",
   ["en"]="click",
   ["fr"]="clic",
   ["it"]="click",
   ["nl"]="klik",
   ["pe"]="فشردن",
   ["ro"]="click",
  },
  ["clickin"]={
   ["cs"]="klikuvnitr",
   ["de"]="klickin",
   ["en"]="clickin",
   ["fr"]="clicinterieur",
   ["it"]="clickdentro",
   ["nl"]="klikin",
   ["pe"]="فشردن‌داخل",
   ["ro"]="clickintru",
  },
  ["clickout"]={
   ["cs"]="klikvne",
   ["de"]="klickaus",
   ["en"]="clickout",
   ["fr"]="clicexterieur",
   ["it"]="clickfuori",
   ["nl"]="klikuit",
   ["pe"]="فشردن‌خارج",
   ["ro"]="clickies",
  },
  ["clipoffset"]={
   ["cs"]="clipoffset",
   ["de"]="clipoffset",
   ["en"]="clipoffset",
   ["fr"]="decalagedecoupe",
   ["it"]="clipoffset",
   ["nl"]="clipoffset",
   ["pe"]="آفست‌کلیپ",
   ["ro"]="clipoffset",
  },
  ["closeaction"]={
   ["cs"]="zavriakci",
   ["de"]="schliessenaktion",
   ["en"]="closeaction",
   ["fr"]="fermeraction",
   ["it"]="azionechiudi",
   ["nl"]="sluitactie",
   ["pe"]="بستن‌کنش",
   ["ro"]="actiuneinchidere",
  },
  ["closecommand"]={
   ["cs"]="closecommand",
   ["de"]="closecommand",
   ["en"]="closecommand",
   ["fr"]="fermercommande",
   ["it"]="closecommand",
   ["nl"]="sluitcommando",
   ["pe"]="بستن‌فرمان",
   ["ro"]="closecommand",
  },
  ["closepage"]={
   ["en"]="closepage",
   ["fr"]="fermerpage",
   ["nl"]="sluitpagina",
  },
  ["closepageaction"]={
   ["cs"]="akcezavrenistranky",
   ["de"]="closepageaction",
   ["en"]="closepageaction",
   ["fr"]="fermeractionpage",
   ["it"]="azionechiudipagina",
   ["nl"]="sluitpaginaactie",
   ["pe"]="بستن‌عمل‌صفحه",
   ["ro"]="actiuneinchiderepagina",
  },
  ["closesymbol"]={
   ["cs"]="closesymbol",
   ["de"]="closesymbol",
   ["en"]="closesymbol",
   ["fr"]="fermersymbol",
   ["it"]="closesymbol",
   ["nl"]="sluitsymbool",
   ["pe"]="بستن‌نماد",
   ["ro"]="closesymbol",
  },
  ["color"]={
   ["cs"]="barva",
   ["de"]="farbe",
   ["en"]="color",
   ["fr"]="couleur",
   ["it"]="colore",
   ["nl"]="kleur",
   ["pe"]="رنگ",
   ["ro"]="culoare",
  },
  ["column"]={
   ["cs"]="sloupec",
   ["de"]="spalte",
   ["en"]="column",
   ["fr"]="colonne",
   ["it"]="colonna",
   ["nl"]="kolom",
   ["pe"]="ستون",
   ["ro"]="coloana",
  },
  ["columndistance"]={
   ["cs"]="vzdalenostsloupcu",
   ["de"]="spaltenabstand",
   ["en"]="columndistance",
   ["fr"]="distancecolonne",
   ["it"]="distanzacolonne",
   ["nl"]="kolomafstand",
   ["pe"]="فاصله‌ستون",
   ["ro"]="distantacoloane",
  },
  ["columns"]={
   ["cs"]="sloupce",
   ["de"]="spalten",
   ["en"]="columns",
   ["fr"]="colonnes",
   ["it"]="colonne",
   ["nl"]="kolommen",
   ["pe"]="ستونها",
   ["ro"]="coloane",
  },
  ["comma"]={
   ["en"]="comma",
   ["fr"]="virgule",
  },
  ["command"]={
   ["cs"]="prikaz",
   ["de"]="befehl",
   ["en"]="command",
   ["fr"]="commande",
   ["it"]="comando",
   ["nl"]="commando",
   ["pe"]="فرمان",
   ["ro"]="comanda",
  },
  ["commandafter"]={
   ["cs"]="prikazpo",
   ["de"]="zumbefehl",
   ["en"]="commandafter",
   ["fr"]="commandeaprès",
   ["it"]="comdandodopo",
   ["nl"]="nacommando",
   ["pe"]="فرمان‌بعداز",
   ["ro"]="comandadupa",
  },
  ["commandbefore"]={
   ["cs"]="predchoziprikaz",
   ["de"]="vorbefehl",
   ["en"]="commandbefore",
   ["fr"]="commandeavant",
   ["it"]="comandoprima",
   ["nl"]="voorcommando",
   ["pe"]="فرمان‌قبل‌از",
   ["ro"]="comandainainte",
  },
  ["commands"]={
   ["cs"]="prikazy",
   ["de"]="befehle",
   ["en"]="commands",
   ["fr"]="commandes",
   ["it"]="comandi",
   ["nl"]="commandos",
   ["pe"]="فرمانها",
   ["ro"]="comenzi",
  },
  ["comment"]={
   ["en"]="comment",
   ["fr"]="commentaire",
  },
  ["commentchar"]={
   ["en"]="commentchar",
   ["fr"]="caracommentaire",
  },
  ["commentoffset"]={
   ["en"]="commentoffset",
   ["fr"]="decalagecommentaire",
  },
  ["compact"]={
   ["en"]="compact",
   ["fr"]="compact",
   ["nl"]="compact",
  },
  ["component"]={
   ["cs"]="component",
   ["de"]="component",
   ["en"]="component",
   ["fr"]="composant",
   ["it"]="component",
   ["nl"]="component",
   ["pe"]="مولفه",
   ["ro"]="component",
  },
  ["compoundhyphen"]={
   ["cs"]="compoundhyphen",
   ["de"]="compoundhyphen",
   ["en"]="compoundhyphen",
   ["fr"]="traitdunioncompose",
   ["it"]="compoundhyphen",
   ["nl"]="koppelteken",
   ["pe"]="compoundhyphen",
   ["ro"]="compoundhyphen",
  },
  ["compress"]={
   ["cs"]="compress",
   ["de"]="compress",
   ["en"]="compress",
   ["fr"]="reduire",
   ["it"]="compress",
   ["nl"]="comprimeren",
   ["pe"]="فشردن",
   ["ro"]="compress",
  },
  ["compressdistance"]={
   ["en"]="compressdistance",
   ["fr"]="distancereduction",
  },
  ["compressmethod"]={
   ["en"]="compressmethod",
   ["fr"]="methodereduction",
  },
  ["compressseparator"]={
   ["en"]="compressseparator",
   ["fr"]="separateurreduction",
  },
  ["compressstopper"]={
   ["en"]="compressstopper",
   ["fr"]="stoppeurreduction",
  },
  ["concerns"]={
   ["en"]="concerns",
   ["fr"]="concerne",
   ["nl"]="betreft",
  },
  ["connector"]={
   ["en"]="connector",
   ["fr"]="connecteur",
  },
  ["continue"]={
   ["cs"]="pokracovat",
   ["de"]="fortsetzen",
   ["en"]="continue",
   ["fr"]="continuer",
   ["it"]="continua",
   ["nl"]="doorgaan",
   ["pe"]="ادامه",
   ["ro"]="continua",
  },
  ["contrastcolor"]={
   ["cs"]="kontrastnibarva",
   ["de"]="kontrastfarbe",
   ["en"]="contrastcolor",
   ["fr"]="couleurconstrast",
   ["it"]="colorecontrasto",
   ["nl"]="contrastkleur",
   ["pe"]="contrastcolor",
   ["ro"]="culoarecontrast",
  },
  ["controls"]={
   ["cs"]="controls",
   ["de"]="controls",
   ["en"]="controls",
   ["fr"]="controles",
   ["it"]="controlli",
   ["nl"]="sturing",
   ["pe"]="کنترلها",
   ["ro"]="controale",
  },
  ["conversion"]={
   ["cs"]="konverze",
   ["de"]="konversion",
   ["en"]="conversion",
   ["fr"]="conversion",
   ["it"]="conversione",
   ["nl"]="conversie",
   ["pe"]="تبدیل",
   ["ro"]="conversie",
  },
  ["convertfile"]={
   ["cs"]="konverzesouboru",
   ["de"]="konvertieredatei",
   ["en"]="convertfile",
   ["fr"]="convertirfichier",
   ["it"]="convertifile",
   ["nl"]="converteerfile",
   ["pe"]="پرونده‌تبدیل",
   ["ro"]="convertestefisier",
  },
  ["copies"]={
   ["en"]="copies",
   ["fr"]="copies",
   ["nl"]="kopieen",
  },
  ["corner"]={
   ["cs"]="roh",
   ["de"]="winkel",
   ["en"]="corner",
   ["fr"]="coin",
   ["it"]="angolo",
   ["nl"]="hoek",
   ["pe"]="گوشه",
   ["ro"]="colt",
  },
  ["coupling"]={
   ["cs"]="propojeni",
   ["de"]="verknuepfung",
   ["en"]="coupling",
   ["fr"]="couplage",
   ["it"]="accoppiamento",
   ["nl"]="koppeling",
   ["pe"]="تزویج",
   ["ro"]="cuplare",
  },
  ["couplingway"]={
   ["cs"]="zpusobpropojeni",
   ["de"]="verkopplungsart",
   ["en"]="couplingway",
   ["fr"]="faconcouplage",
   ["it"]="modoaccoppiamento",
   ["nl"]="koppelwijze",
   ["pe"]="روش‌تزویج",
   ["ro"]="modcuplare",
  },
  ["criterium"]={
   ["cs"]="kriterium",
   ["de"]="kriterium",
   ["en"]="criterium",
   ["fr"]="critere",
   ["it"]="criterio",
   ["nl"]="criterium",
   ["pe"]="criterium",
   ["ro"]="criteriu",
  },
  ["crop"]={
   ["en"]="crop",
   ["fr"]="recadre",
  },
  ["cropoffset"]={
   ["en"]="cropoffset",
   ["fr"]="decalagerecadre",
  },
  ["crossreference"]={
   ["en"]="crossreference",
   ["fr"]="renvoi",
  },
  ["cssfile"]={
   ["en"]="cssfile",
   ["fr"]="fichiercss",
  },
  ["current"]={
   ["cs"]="aktualni",
   ["de"]="aktuell",
   ["en"]="current",
   ["fr"]="courant",
   ["it"]="corrente",
   ["nl"]="huidige",
   ["pe"]="جاری",
   ["ro"]="curent",
  },
  ["cutspace"]={
   ["cs"]="cutspace",
   ["de"]="cutspace",
   ["en"]="cutspace",
   ["fr"]="espacecoupe",
   ["it"]="cutspace",
   ["nl"]="snijwit",
   ["pe"]="فضای‌برش",
   ["ro"]="cutspace",
  },
  ["dash"]={
   ["cs"]="pomlcka",
   ["de"]="strich",
   ["en"]="dash",
   ["fr"]="pointille",
   ["it"]="dash",
   ["nl"]="streep",
   ["pe"]="دش",
   ["ro"]="dash",
  },
  ["dashstep"]={
   ["en"]="dashstep",
  },
  ["dat"]={
   ["cs"]="dat",
   ["de"]="dat",
   ["en"]="dat",
   ["fr"]="dat",
   ["it"]="dat",
   ["nl"]="dat",
   ["pe"]="dat",
   ["ro"]="dat",
  },
  ["database"]={
   ["en"]="database",
   ["fr"]="basededonnees",
  },
  ["dataset"]={
   ["en"]="dataset",
   ["fr"]="jeudedonnees",
  },
  ["date"]={
   ["cs"]="datum",
   ["de"]="datum",
   ["en"]="date",
   ["fr"]="date",
   ["it"]="data",
   ["nl"]="datum",
   ["pe"]="تاریخ",
   ["ro"]="data",
  },
  ["deepnumbercommand"]={
   ["cs"]="deepnumbercommand",
   ["de"]="deepnumbercommand",
   ["en"]="deepnumbercommand",
   ["fr"]="commandeprofondenumero",
   ["it"]="deepnumbercommand",
   ["nl"]="diepnummercommando",
   ["pe"]="فرمان‌شماره‌عمیق",
   ["ro"]="deepnumbercommand",
  },
  ["deeptextcommand"]={
   ["cs"]="deeptextcommand",
   ["de"]="deeptextcommand",
   ["en"]="deeptextcommand",
   ["fr"]="commandeprofondetexte",
   ["it"]="deeptextcommand",
   ["nl"]="dieptekstcommando",
   ["pe"]="فرمان‌متن‌عمیق",
   ["ro"]="deeptextcommand",
  },
  ["define"]={
   ["en"]="define",
   ["nl"]="definieer",
  },
  ["default"]={
   ["cs"]="implicitni",
   ["de"]="default",
   ["en"]="default",
   ["fr"]="defaut",
   ["it"]="implicito",
   ["nl"]="default",
   ["pe"]="پیش‌فرض",
   ["ro"]="implicit",
  },
  ["defaultwidth"]={
   ["en"]="defaultwidth",
   ["nl"]="defaultbreedte",
  },
  ["defaultheight"]={
   ["en"]="defaultheight",
   ["nl"]="defaulthoogte",
  },
  ["delay"]={
   ["cs"]="prodleva",
   ["de"]="wartezeit",
   ["en"]="delay",
   ["fr"]="retard",
   ["it"]="attesa",
   ["nl"]="wachttijd",
   ["pe"]="تاخیر",
   ["ro"]="intarziere",
  },
  ["depth"]={
   ["cs"]="hloubka",
   ["de"]="tiefe",
   ["en"]="depth",
   ["fr"]="profondeur",
   ["it"]="profondita",
   ["nl"]="diepte",
   ["pe"]="عمق",
   ["ro"]="inaltime",
  },
  ["depthcorrection"]={
   ["cs"]="korekcehloubky",
   ["de"]="tiefekorrektur",
   ["en"]="depthcorrection",
   ["fr"]="correctionprofondeur",
   ["it"]="correzioneprofondita",
   ["nl"]="dieptecorrectie",
   ["pe"]="تصحیح‌عمق",
   ["ro"]="corectieadancime",
  },
  ["descriptiontext"]={
   ["en"]="descriptiontext",
  },
  ["direction"]={
   ["cs"]="smer",
   ["de"]="richtung",
   ["en"]="direction",
   ["fr"]="direction",
   ["it"]="direzione",
   ["nl"]="richting",
   ["pe"]="جهت",
   ["ro"]="directie",
  },
  ["directory"]={
   ["cs"]="adresar",
   ["de"]="verzeichnis",
   ["en"]="directory",
   ["fr"]="repertoire",
   ["it"]="directory",
   ["nl"]="directory",
   ["pe"]="پوشه",
   ["ro"]="director",
  },
  ["display"]={
   ["cs"]="obrazovka",
   ["de"]="bildschirm",
   ["en"]="display",
   ["fr"]="affichage",
   ["it"]="display",
   ["nl"]="scherm",
   ["pe"]="نمایش",
   ["ro"]="display",
  },
  ["displaythreshold"]={
   ["en"]="displaythreshold",
   ["fr"]="seuilaffichage",
  },
  ["displayfactor"]={
   ["en"]="displayfactor",
  },
  ["distance"]={
   ["cs"]="vzdalenost",
   ["de"]="abstand",
   ["en"]="distance",
   ["fr"]="distance",
   ["it"]="distanza",
   ["nl"]="afstand",
   ["pe"]="فاصله",
   ["ro"]="distanta",
  },
  ["domain"]={
   ["en"]="domain",
   ["fr"]="domaine",
  },
  ["dot"]={
   ["cs"]="tecka",
   ["de"]="punkt",
   ["en"]="dot",
   ["fr"]="point",
   ["it"]="punto",
   ["nl"]="punt",
   ["pe"]="نقطه",
   ["ro"]="punct",
  },
  ["doublesided"]={
   ["cs"]="oboustranne",
   ["de"]="doppelseitig",
   ["en"]="doublesided",
   ["fr"]="rectoverso",
   ["it"]="doppiafaccia",
   ["nl"]="dubbelzijdig",
   ["pe"]="دورو",
   ["ro"]="douafete",
  },
  ["down"]={
   ["en"]="down",
   ["fr"]="enbas",
  },
  ["dummy"]={
   ["cs"]="dummy",
   ["de"]="dummy",
   ["en"]="dummy",
   ["fr"]="factice",
   ["it"]="dummy",
   ["nl"]="dummy",
   ["pe"]="مصنوعی",
   ["ro"]="dummy",
  },
  ["dx"]={
   ["cs"]="dx",
   ["de"]="dx",
   ["en"]="dx",
   ["fr"]="dx",
   ["it"]="dx",
   ["nl"]="dx",
   ["pe"]="dx",
   ["ro"]="dx",
  },
  ["dy"]={
   ["cs"]="dy",
   ["de"]="dy",
   ["en"]="dy",
   ["fr"]="dy",
   ["it"]="dy",
   ["nl"]="dy",
   ["pe"]="dy",
   ["ro"]="dy",
  },
  ["edge"]={
   ["cs"]="hrana",
   ["de"]="kante",
   ["en"]="edge",
   ["fr"]="bord",
   ["it"]="bordo",
   ["nl"]="rand",
   ["pe"]="لبه",
   ["ro"]="bordura",
  },
  ["edgedistance"]={
   ["cs"]="vzdalenosthrany",
   ["de"]="kantenabstand",
   ["en"]="edgedistance",
   ["fr"]="distancebord",
   ["it"]="distanzabordo",
   ["nl"]="randafstand",
   ["pe"]="فاصله‌لبه",
   ["ro"]="distantabordura",
  },
  ["editor"]={
   ["en"]="editor",
   ["fr"]="editeur",
  },
  ["editoretaldisplay"]={
   ["en"]="editoretaldisplay",
   ["fr"]="editeurealaffichage",
  },
  ["editoretallimit"]={
   ["en"]="editoretallimit",
   ["fr"]="editeureallimite",
  },
  ["editoretaltext"]={
   ["en"]="editoretaltext",
   ["fr"]="editeurealtexte",
  },
  ["empty"]={
   ["cs"]="prazdne",
   ["de"]="leer",
   ["en"]="empty",
   ["fr"]="vide",
   ["it"]="vuoto",
   ["nl"]="leeg",
   ["pe"]="تهی",
   ["ro"]="gol",
  },
  ["entities"]={
   ["en"]="entities",
   ["fr"]="entitees",
   ["nl"]="entities",
  },
  ["entries"]={
   ["en"]="entries",
   ["fr"]="entrees",
   ["nl"]="ingangen",
  },
  ["extradata"]={
   ["en"]="extradata",
   ["nl"]="extragegevens",
  },
  ["processors"]={
   ["en"]="processors",
  },
  ["equalheight"]={
   ["cs"]="equalheight",
   ["de"]="equalheight",
   ["en"]="equalheight",
   ["fr"]="hauteuregale",
   ["it"]="equalheight",
   ["nl"]="equalheight",
   ["pe"]="ارتفاع‌یکسان",
   ["ro"]="equalheight",
  },
  ["equalwidth"]={
   ["cs"]="equalwidth",
   ["de"]="equalwidth",
   ["en"]="equalwidth",
   ["fr"]="margeuregale",
   ["it"]="equalwidth",
   ["nl"]="equalwidth",
   ["pe"]="عرض‌یکسان",
   ["ro"]="equalwidth",
  },
  ["escape"]={
   ["cs"]="escape",
   ["de"]="escape",
   ["en"]="escape",
   ["fr"]="echappatoire",
   ["it"]="escape",
   ["nl"]="escape",
   ["pe"]="فرار",
   ["ro"]="escape",
  },
  ["etaldisplay"]={
   ["en"]="etaldisplay",
   ["fr"]="etalaffichage",
  },
  ["etallimit"]={
   ["en"]="etallimit",
   ["fr"]="etallimite",
  },
  ["etaloption"]={
   ["en"]="etaloption",
   ["fr"]="etaloption",
  },
  ["etaltext"]={
   ["en"]="etaltext",
   ["fr"]="etalttexte",
  },
  ["evenmargin"]={
   ["cs"]="sudamarginalie",
   ["de"]="geraderand",
   ["en"]="evenmargin",
   ["fr"]="margepaire",
   ["it"]="marginepari",
   ["nl"]="evenmarge",
   ["pe"]="حاشیه‌زوج",
   ["ro"]="marginepara",
  },
  ["exact"]={
   ["en"]="exact",
   ["fr"]="exact",
  },
  ["exitoffset"]={
   ["en"]="exitoffset",
   ["fr"]="decalagesortie",
  },
  ["expansion"]={
   ["cs"]="expanzen",
   ["de"]="expansion",
   ["en"]="expansion",
   ["fr"]="expansion",
   ["it"]="espansione",
   ["nl"]="expansie",
   ["pe"]="گسترش",
   ["ro"]="expansiune",
  },
  ["export"]={
   ["en"]="export",
   ["fr"]="exporter",
   ["nl"]="exporteer",
  },
  ["extras"]={
   ["cs"]="extras",
   ["de"]="extras",
   ["en"]="extras",
   ["fr"]="extras",
   ["it"]="extras",
   ["nl"]="extras",
   ["pe"]="extras",
   ["ro"]="extras",
  },
  ["factor"]={
   ["cs"]="faktor",
   ["de"]="faktor",
   ["en"]="factor",
   ["fr"]="facteur",
   ["it"]="fattore",
   ["nl"]="factor",
   ["pe"]="عامل",
   ["ro"]="factor",
  },
  ["fallback"]={
   ["cs"]="fallback",
   ["de"]="fallback",
   ["en"]="fallback",
   ["fr"]="repli",
   ["it"]="fallback",
   ["nl"]="terugval",
   ["pe"]="عقب‌ریختن",
   ["ro"]="fallback",
  },
  ["fallbacks"]={
   ["en"]="fallbacks",
   ["fr"]="replis",
  },
  ["family"]={
   ["cs"]="rodina",
   ["de"]="familie",
   ["en"]="family",
   ["fr"]="famille",
   ["it"]="famiglia",
   ["nl"]="soort",
   ["pe"]="خانواده",
   ["ro"]="familie",
  },
  ["features"]={
   ["en"]="features",
   ["fr"]="fonctionnalites",
  },
  ["featureset"]={
   ["en"]="featureset",
   ["fr"]="jeufonctionnalite",
  },
  ["fences"]={
   ["en"]="fences",
   ["fr"]="barrieres",
  },
  ["field"]={
   ["en"]="field",
   ["fr"]="champ",
   ["nl"]="veld",
  },
  ["fieldbackgroundcolor"]={
   ["cs"]="barvapozadipole",
   ["de"]="feldhintergrundfarbe",
   ["en"]="fieldbackgroundcolor",
   ["fr"]="couleurarrierreplanchamp",
   ["it"]="coloresfondocampo",
   ["nl"]="veldachtergrondkleur",
   ["pe"]="رنگ‌پس‌زمینه‌میدان",
   ["ro"]="culoarefundalcamp",
  },
  ["fieldframecolor"]={
   ["cs"]="barvarameckupole",
   ["de"]="feldrahmenfarbe",
   ["en"]="fieldframecolor",
   ["fr"]="couleurcadrechamp",
   ["it"]="colorecornicecampo",
   ["nl"]="veldkaderkleur",
   ["pe"]="رنگ‌قالب‌میدان",
   ["ro"]="culoareframecamp",
  },
  ["fieldlayer"]={
   ["cs"]="fieldlayer",
   ["de"]="fieldlayer",
   ["en"]="fieldlayer",
   ["fr"]="calquechamp",
   ["it"]="fieldlayer",
   ["nl"]="veldlaag",
   ["pe"]="لایه‌میدان",
   ["ro"]="fieldlayer",
  },
  ["fieldoffset"]={
   ["cs"]="offsetpole",
   ["de"]="feldoffset",
   ["en"]="fieldoffset",
   ["fr"]="decalagechamp",
   ["it"]="offsetcampo",
   ["nl"]="veldoffset",
   ["pe"]="آفست‌میدان",
   ["ro"]="offsetcamp",
  },
  ["file"]={
   ["cs"]="soubor",
   ["de"]="datei",
   ["en"]="file",
   ["fr"]="fichier",
   ["it"]="file",
   ["nl"]="file",
   ["pe"]="پرونده",
   ["ro"]="fisier",
  },
  ["filler"]={
   ["en"]="filler",
   ["fr"]="remplisseur",
  },
  ["filter"]={
   ["en"]="filter",
   ["fr"]="filtre",
  },
  ["filtercommand"]={
   ["en"]="filtercommand",
   ["fr"]="commandefiltre",
  },
  ["finalnamesep"]={
   ["en"]="finalnamesep",
   ["fr"]="sepnomfinal",
  },
  ["finalpagesep"]={
   ["en"]="finalpagesep",
   ["fr"]="seppagefinale",
  },
  ["finalpubsep"]={
   ["en"]="finalpubsep",
   ["fr"]="seppubfinale",
  },
  ["first"]={
   ["cs"]="prvni",
   ["de"]="erste",
   ["en"]="first",
   ["fr"]="premier",
   ["it"]="primo",
   ["nl"]="eerste",
   ["pe"]="اولی",
   ["ro"]="primul",
  },
  ["firstnamesep"]={
   ["en"]="firstnamesep",
   ["fr"]="seppremiernom",
  },
  ["firstpage"]={
   ["cs"]="prvnistranka",
   ["de"]="ersteseite",
   ["en"]="firstpage",
   ["fr"]="premierepage",
   ["it"]="primapagina",
   ["nl"]="eerstepagina",
   ["pe"]="صفحه‌اول",
   ["ro"]="primapagina",
  },
  ["focus"]={
   ["cs"]="zaostreni",
   ["de"]="focus",
   ["en"]="focus",
   ["fr"]="focus",
   ["it"]="focus",
   ["nl"]="focus",
   ["pe"]="تمرکز",
   ["ro"]="focus",
  },
  ["focusin"]={
   ["cs"]="focusin",
   ["de"]="focusin",
   ["en"]="focusin",
   ["fr"]="focusin",
   ["it"]="focusin",
   ["nl"]="focusin",
   ["pe"]="تمرکزدرون",
   ["ro"]="focusin",
  },
  ["focusoffset"]={
   ["en"]="focusoffset",
   ["fr"]="decalagefocus",
   ["nl"]="focusoffset",
  },
  ["focusout"]={
   ["cs"]="focusout",
   ["de"]="focusout",
   ["en"]="focusout",
   ["fr"]="focusout",
   ["it"]="focusout",
   ["nl"]="focusuit",
   ["pe"]="تمرکزبیرون",
   ["ro"]="focusout",
  },
  ["font"]={
   ["en"]="font",
   ["nl"]="font",
  },
  ["footer"]={
   ["cs"]="upati",
   ["de"]="fusszeile",
   ["en"]="footer",
   ["fr"]="pdp",
   ["it"]="piedipagina",
   ["nl"]="voet",
   ["pe"]="ته‌برگ",
   ["ro"]="subsol",
  },
  ["footerdistance"]={
   ["cs"]="vzdalenostupati",
   ["de"]="fusszeilenabstand",
   ["en"]="footerdistance",
   ["fr"]="distancepdp",
   ["it"]="distanzapdp",
   ["nl"]="voetafstand",
   ["pe"]="فاصله‌ته‌برگ",
   ["ro"]="distantasubsol",
  },
  ["footerstate"]={
   ["cs"]="statusupati",
   ["de"]="fusszeilenstatus",
   ["en"]="footerstate",
   ["fr"]="etatpdp",
   ["it"]="statopdp",
   ["nl"]="voetstatus",
   ["pe"]="وضعیت‌ته‌برگ",
   ["ro"]="staresubsol",
  },
  ["force"]={
   ["cs"]="sila",
   ["de"]="zwinge",
   ["en"]="force",
   ["fr"]="forcer",
   ["it"]="forza",
   ["nl"]="forceer",
   ["pe"]="اجبار",
   ["ro"]="fortat",
  },
  ["foregroundcolor"]={
   ["cs"]="foregroundcolor",
   ["de"]="foregroundcolor",
   ["en"]="foregroundcolor",
   ["fr"]="couleurpremierplan",
   ["it"]="coloreprimopiano",
   ["nl"]="voorgrondkleur",
   ["pe"]="رنگ‌پیش‌زمینه",
   ["ro"]="foregroundcolor",
  },
  ["foregroundstyle"]={
   ["cs"]="foregroundstyle",
   ["de"]="foregroundstyle",
   ["en"]="foregroundstyle",
   ["fr"]="stylepremierplan",
   ["it"]="foregroundstyle",
   ["nl"]="voorgrondletter",
   ["pe"]="سبک‌پیش‌زمینه",
   ["ro"]="foregroundstyle",
  },
  ["format"]={
   ["cs"]="formatovat",
   ["de"]="formatieren",
   ["en"]="format",
   ["fr"]="formatter",
   ["it"]="format",
   ["nl"]="formatteer",
   ["pe"]="شمایل",
   ["ro"]="format",
  },
  ["fractions"]={
   ["en"]="fractions",
   ["fr"]="fractions",
   ["nl"]="breuken",
  },
  ["frame"]={
   ["cs"]="ramecek",
   ["de"]="rahmen",
   ["en"]="frame",
   ["fr"]="cadre",
   ["it"]="cornice",
   ["nl"]="kader",
   ["pe"]="قالب",
   ["ro"]="frame",
  },
  ["framecolor"]={
   ["cs"]="barvaramecku",
   ["de"]="rahmenfarbe",
   ["en"]="framecolor",
   ["fr"]="couleurcadre",
   ["it"]="colorecornice",
   ["nl"]="kaderkleur",
   ["pe"]="رنگ‌قالب",
   ["ro"]="culoareframe",
  },
  ["framecorner"]={
   ["cs"]="rohramecku",
   ["de"]="rahmenwinkel",
   ["en"]="framecorner",
   ["fr"]="coincadre",
   ["it"]="angolocornice",
   ["nl"]="kaderhoek",
   ["pe"]="گوشه‌قالب",
   ["ro"]="coltframe",
  },
  ["framedepth"]={
   ["cs"]="hloubkaramecku",
   ["de"]="rahmentiefe",
   ["en"]="framedepth",
   ["fr"]="profondeurcadre",
   ["it"]="profonditacornice",
   ["nl"]="kaderdiepte",
   ["pe"]="عمق‌قالب",
   ["ro"]="adancimeframe",
  },
  ["frameoffset"]={
   ["cs"]="offsetramecku",
   ["de"]="rahmenoffset",
   ["en"]="frameoffset",
   ["fr"]="decalagecadre",
   ["it"]="offsetcornice",
   ["nl"]="kaderoffset",
   ["pe"]="آفست‌قالب",
   ["ro"]="offsetframe",
  },
  ["frameradius"]={
   ["cs"]="polomerramecku",
   ["de"]="rahmenradius",
   ["en"]="frameradius",
   ["fr"]="rayoncadre",
   ["it"]="raggiocornice",
   ["nl"]="kaderstraal",
   ["pe"]="شعاع‌قالب",
   ["ro"]="razaframe",
  },
  ["frames"]={
   ["cs"]="ramecky",
   ["de"]="umrahmen",
   ["en"]="frames",
   ["fr"]="cadres",
   ["it"]="cornici",
   ["nl"]="hokjes",
   ["pe"]="قالبها",
   ["ro"]="frames",
  },
  ["freeregion"]={
   ["en"]="freeregion",
   ["fr"]="regionlibre",
   ["nl"]="vrijgebied",
  },
  ["from"]={
   ["cs"]="z",
   ["de"]="von",
   ["en"]="from",
   ["fr"]="de",
   ["it"]="da",
   ["nl"]="van",
   ["pe"]="از",
   ["ro"]="dela",
  },
  ["functioncolor"]={
   ["en"]="functioncolor",
   ["fr"]="couleurfonction",
  },
  ["functionstyle"]={
   ["en"]="functionstyle",
   ["fr"]="stylefonction",
  },
  ["get"]={
   ["cs"]="ziskat",
   ["de"]="hole",
   ["en"]="get",
   ["fr"]="obtenir",
   ["it"]="prendi",
   ["nl"]="haal",
   ["pe"]="بگیر",
   ["ro"]="adu",
  },
  ["global"]={
   ["cs"]="globalne",
   ["de"]="global",
   ["en"]="global",
   ["fr"]="global",
   ["it"]="globale",
   ["nl"]="globaal",
   ["pe"]="سراسری",
   ["ro"]="global",
  },
  ["goodies"]={
   ["en"]="goodies",
   ["fr"]="friandises",
  },
  ["grid"]={
   ["cs"]="mrizka",
   ["de"]="gritter",
   ["en"]="grid",
   ["fr"]="grille",
   ["it"]="griglia",
   ["nl"]="grid",
   ["pe"]="توری",
   ["ro"]="grid",
  },
  ["gridcorrection"]={
   ["en"]="gridcorrection",
   ["nl"]="gridcorrectie",
  },
  ["group"]={
   ["en"]="group",
   ["fr"]="groupe",
  },
  ["groupsuffix"]={
   ["en"]="groupsuffix",
   ["fr"]="suffixegroupe",
  },
  ["hang"]={
   ["cs"]="zaveseni",
   ["de"]="haengend",
   ["en"]="hang",
   ["fr"]="suspendre",
   ["it"]="sospendi",
   ["nl"]="hang",
   ["pe"]="بیاویز",
   ["ro"]="suspenda",
  },
  ["hcompact"]={
   ["en"]="hcompact",
   ["fr"]="compacth",
   ["nl"]="hcomprimeer",
  },
  ["headalign"]={
   ["en"]="headalign",
   ["fr"]="alignertete",
   ["nl"]="kopuitlijnen",
  },
  ["headcolor"]={
   ["cs"]="barvahlavicky",
   ["de"]="kopffarbe",
   ["en"]="headcolor",
   ["fr"]="couleurtete",
   ["it"]="coloretesta",
   ["nl"]="kopkleur",
   ["pe"]="رنگ‌سر",
   ["ro"]="culoaretitlu",
  },
  ["headcommand"]={
   ["cs"]="headcommand",
   ["de"]="headcommand",
   ["en"]="headcommand",
   ["fr"]="commandetete",
   ["it"]="headcommand",
   ["nl"]="kopcommando",
   ["pe"]="فرمان‌سر",
   ["ro"]="headcommand",
  },
  ["headconversion"]={
   ["cs"]="konverzehlavicky",
   ["de"]="headconversion",
   ["en"]="headconversion",
   ["fr"]="conversiontete",
   ["it"]="conversionetesta",
   ["nl"]="kopconversie",
   ["pe"]="تبدیل‌سر",
   ["ro"]="conversietitlu",
  },
  ["header"]={
   ["cs"]="zahlavi",
   ["de"]="kopfzeile",
   ["en"]="header",
   ["fr"]="entête",
   ["it"]="intestazione",
   ["nl"]="hoofd",
   ["pe"]="سربرگ",
   ["ro"]="antet",
  },
  ["headerdistance"]={
   ["cs"]="vzdalenostzahlavi",
   ["de"]="kopfzeilenabstand",
   ["en"]="headerdistance",
   ["fr"]="distanceentête",
   ["it"]="distanzaintestazione",
   ["nl"]="hoofdafstand",
   ["pe"]="فاصله‌سربرگ",
   ["ro"]="distantaantet",
  },
  ["headerstate"]={
   ["cs"]="statuszahlavi",
   ["de"]="kopfzeilenstatus",
   ["en"]="headerstate",
   ["fr"]="etatentête",
   ["it"]="statointestazione",
   ["nl"]="hoofdstatus",
   ["pe"]="وضعیت‌سربرگ",
   ["ro"]="stareantet",
  },
  ["headlabel"]={
   ["cs"]="popisekhlavicky",
   ["de"]="headlabel",
   ["en"]="headlabel",
   ["fr"]="etiquettetete",
   ["it"]="etichettatesta",
   ["nl"]="koplabel",
   ["pe"]="برچسب‌سر",
   ["ro"]="etichetatitlu",
  },
  ["headnumber"]={
   ["cs"]="cislonadpisu",
   ["de"]="kopfnummer",
   ["en"]="headnumber",
   ["fr"]="numerotete",
   ["it"]="numerotesta",
   ["nl"]="kopnummer",
   ["pe"]="شماره‌سر",
   ["ro"]="numartitlu",
  },
  ["headseparator"]={
   ["en"]="headseparator",
   ["fr"]="separateurtete",
   ["nl"]="kopscheider",
  },
  ["headindenting"]={
   ["en"]="headindenting",
   ["nl"]="kopinspringen",
  },
  ["headstyle"]={
   ["cs"]="stylhlavicky",
   ["de"]="kopfstil",
   ["en"]="headstyle",
   ["fr"]="styletete",
   ["it"]="stiletesta",
   ["nl"]="kopletter",
   ["pe"]="سبک‌سر",
   ["ro"]="stiltitlu",
  },
  ["height"]={
   ["cs"]="vyska",
   ["de"]="hoehe",
   ["en"]="height",
   ["fr"]="hauteur",
   ["it"]="altezza",
   ["nl"]="hoogte",
   ["pe"]="ارتفاع",
   ["ro"]="inaltime",
  },
  ["hfactor"]={
   ["cs"]="hfaktor",
   ["de"]="hfaktor",
   ["en"]="hfactor",
   ["fr"]="facteurh",
   ["it"]="hfactor",
   ["nl"]="hfactor",
   ["pe"]="عامل‌ارتفاع",
   ["ro"]="hfactor",
  },
  ["hfil"]={
   ["cs"]="hfil",
   ["de"]="hfil",
   ["en"]="hfil",
   ["fr"]="remplissageh",
   ["it"]="hfil",
   ["nl"]="hfil",
   ["pe"]="پرکردن‌ارتفاع",
   ["ro"]="hfil",
  },
  ["hidenumber"]={
   ["en"]="hidenumber",
   ["fr"]="cachernumero",
  },
  ["hoffset"]={
   ["cs"]="hoffset",
   ["de"]="hoffset",
   ["en"]="hoffset",
   ["fr"]="decalageh",
   ["it"]="hoffset",
   ["nl"]="hoffset",
   ["pe"]="آفست‌ا",
   ["ro"]="hoffset",
  },
  ["horizontal"]={
   ["cs"]="horizontalne",
   ["de"]="horizontal",
   ["en"]="horizontal",
   ["fr"]="horizontal",
   ["it"]="orizzontale",
   ["nl"]="horizontaal",
   ["pe"]="افقی",
   ["ro"]="orizontal",
  },
  ["horoffset"]={
   ["cs"]="horoffset",
   ["de"]="rumpfabstand",
   ["en"]="horoffset",
   ["fr"]="decalagehor",
   ["it"]="horoffset",
   ["nl"]="rugoffset",
   ["pe"]="آفست‌افق",
   ["ro"]="offsetoriz",
  },
  ["hyphen"]={
   ["cs"]="hyphen",
   ["de"]="hyphen",
   ["en"]="hyphen",
   ["fr"]="traitdunion",
   ["it"]="hyphen",
   ["nl"]="hyphen",
   ["pe"]="شکستن",
   ["ro"]="hyphen",
  },
  ["hyphens"]={
   ["en"]="hyphens",
   ["fr"]="traitsdunion",
  },
  ["hz"]={
   ["en"]="hz",
  },
  ["vz"]={
   ["en"]="vz",
  },
  ["icommand"]={
   ["cs"]="iprikaz",
   ["de"]="ibefehl",
   ["en"]="icommand",
   ["fr"]="commandei",
   ["it"]="icomando",
   ["nl"]="icommando",
   ["pe"]="icommand",
   ["ro"]="comandai",
  },
  ["ignore"]={
   ["en"]="ignore",
   ["fr"]="ignorer",
  },
  ["in"]={
   ["cs"]="v",
   ["de"]="in",
   ["en"]="in",
   ["fr"]="dans",
   ["it"]="in",
   ["nl"]="in",
   ["pe"]="درون",
   ["ro"]="in",
  },
  ["inbetween"]={
   ["cs"]="mezi",
   ["de"]="zwischen",
   ["en"]="inbetween",
   ["fr"]="entre",
   ["it"]="tra",
   ["nl"]="tussen",
   ["pe"]="دربین",
   ["ro"]="intre",
  },
  ["increment"]={
   ["cs"]="zvysit",
   ["de"]="erhoehen",
   ["en"]="increment",
   ["fr"]="incrementer",
   ["it"]="incremento",
   ["nl"]="verhoog",
   ["pe"]="افزایش",
   ["ro"]="increment",
  },
  ["incrementnumber"]={
   ["cs"]="zvysujicicislo",
   ["de"]="nummererhoehen",
   ["en"]="incrementnumber",
   ["fr"]="incrementernumero",
   ["it"]="incrementanumero",
   ["nl"]="verhoognummer",
   ["pe"]="شماره‌افزایش",
   ["ro"]="numarincrement",
  },
  ["indenting"]={
   ["cs"]="odsazovani",
   ["de"]="einziehen",
   ["en"]="indenting",
   ["fr"]="indentation",
   ["it"]="rientro",
   ["nl"]="inspringen",
   ["pe"]="تورفتگی",
   ["ro"]="aliniat",
  },
  ["indentlist"]={
   ["en"]="indentlist",
  },
  ["indentnext"]={
   ["cs"]="odsadpristi",
   ["de"]="ziehefolgendeein",
   ["en"]="indentnext",
   ["fr"]="indenterauuivant",
   ["it"]="rientrasuccessivo",
   ["nl"]="springvolgendein",
   ["pe"]="متن‌تورفته",
   ["ro"]="aliniaturmator",
  },
  ["index"]={
   ["en"]="index",
   ["fr"]="index",
  },
  ["indicator"]={
   ["cs"]="indikator",
   ["de"]="indikator",
   ["en"]="indicator",
   ["fr"]="indicateur",
   ["it"]="indicatore",
   ["nl"]="aanduiding",
   ["pe"]="اندیکاتور",
   ["ro"]="indicator",
  },
  ["initialsep"]={
   ["en"]="initialsep",
   ["fr"]="sepinitial",
  },
  ["inlinethreshold"]={
   ["en"]="inlinethreshold",
   ["fr"]="seuilenligne",
  },
  ["inlinefactor"]={
   ["en"]="inlinefactor",
  },
  ["inner"]={
   ["cs"]="vnitrni",
   ["de"]="innen",
   ["en"]="inner",
   ["fr"]="interieur",
   ["it"]="interno",
   ["nl"]="binnen",
   ["pe"]="داخلی",
   ["ro"]="intern",
  },
  ["innermargin"]={
   ["cs"]="innermargin",
   ["de"]="innermargin",
   ["en"]="innermargin",
   ["fr"]="margeinterieure",
   ["it"]="margineinterno",
   ["nl"]="binnenmarge",
   ["pe"]="حاشیه‌داخلی",
   ["ro"]="innermargin",
  },
  ["inputfile"]={
   ["cs"]="inputfile",
   ["de"]="inputfile",
   ["en"]="inputfile",
   ["fr"]="fichierentree",
   ["it"]="inputfile",
   ["nl"]="inputfile",
   ["pe"]="پرونده‌ورودی",
   ["ro"]="inputfile",
  },
  ["insidesection"]={
   ["en"]="insidesection",
   ["fr"]="auseinsection",
   ["nl"]="binnensectie",
  },
  ["instance"]={
   ["en"]="instance",
   ["fr"]="instance",
  },
  ["integral"]={
   ["en"]="integral",
   ["fr"]="integrale",
  },
  ["intent"]={
   ["en"]="intent",
   ["fr"]="intention",
  },
  ["interaction"]={
   ["cs"]="interakce",
   ["de"]="interaktion",
   ["en"]="interaction",
   ["fr"]="interaction",
   ["it"]="interazione",
   ["nl"]="interactie",
   ["pe"]="پانل",
   ["ro"]="interactiune",
  },
  ["interlinespace"]={
   ["cs"]="meziradkovamezera",
   ["de"]="zeilenabstande",
   ["en"]="interlinespace",
   ["fr"]="espaceinterligne",
   ["it"]="interlinea",
   ["nl"]="interlinie",
   ["pe"]="فضای‌بین‌خط",
   ["ro"]="spatiereinterliniara",
  },
  ["internalgrid"]={
   ["en"]="internalgrid",
   ["fr"]="grilleinterieure",
   ["nl"]="interngrid",
  },
  ["italicfeatures"]={
   ["en"]="italicfeatures",
   ["fr"]="fonctionnalitesitalique",
  },
  ["italicfont"]={
   ["en"]="italicfont",
   ["fr"]="policeitalique",
  },
  ["itemalign"]={
   ["cs"]="itemalign",
   ["de"]="itemalign",
   ["en"]="itemalign",
   ["fr"]="alignerelem",
   ["it"]="itemalign",
   ["nl"]="itemuitlijnen",
   ["pe"]="تنظیم‌آیتم",
   ["ro"]="itemalign",
  },
  ["items"]={
   ["cs"]="polozky",
   ["de"]="posten",
   ["en"]="items",
   ["fr"]="élément",
   ["it"]="elementi",
   ["nl"]="items",
   ["pe"]="آیتمها",
   ["ro"]="elemente",
  },
  ["joiners"]={
   ["en"]="joiners",
   ["fr"]="joins",
  },
  ["journalconversion"]={
   ["en"]="journalconversion",
   ["fr"]="conversionjournal",
  },
  ["juniorsep"]={
   ["en"]="juniorsep",
   ["fr"]="sepjunior",
  },
  ["keeptogether"]={
   ["en"]="keeptogether",
   ["fr"]="conserverensemble",
  },
  ["ken"]={
   ["cs"]="ken",
   ["de"]="ken",
   ["en"]="ken",
   ["fr"]="rep",
   ["it"]="ken",
   ["nl"]="ken",
   ["pe"]="ken",
   ["ro"]="ken",
  },
  ["keyexpansion"]={
   ["cs"]="keyexpansion",
   ["de"]="keyexpansion",
   ["en"]="keyexpansion",
   ["fr"]="expansioncle",
   ["it"]="keyexpansion",
   ["nl"]="sleutelexpansie",
   ["pe"]="گسترش‌کلید",
   ["ro"]="keyexpansion",
  },
  ["keys"]={
   ["en"]="keys",
   ["fr"]="cles",
   ["nl"]="sleutels",
  },
  ["keyword"]={
   ["cs"]="keyword",
   ["de"]="keyword",
   ["en"]="keyword",
   ["fr"]="motclé",
   ["it"]="parolachiave",
   ["nl"]="trefwoord",
   ["pe"]="کلید‌واژه",
   ["ro"]="keyword",
  },
  ["label"]={
   ["cs"]="popisek",
   ["de"]="label",
   ["en"]="label",
   ["fr"]="etiquette",
   ["it"]="etichetta",
   ["nl"]="label",
   ["pe"]="برچسب",
   ["ro"]="eticheta",
  },
  ["labelalternative"]={
   ["en"]="labelalternative",
   ["fr"]="varianteetiquette",
   ["nl"]="labelvariant",
  },
  ["labelcolor"]={
   ["en"]="labelcolor",
   ["fr"]="couleuretiquette",
   ["nl"]="labelkleur",
  },
  ["labeloffset"]={
   ["en"]="labeloffset",
   ["fr"]="decalageetiquette",
  },
  ["labelstyle"]={
   ["en"]="labelstyle",
   ["fr"]="styleetiquette",
   ["nl"]="labelletter",
  },
  ["language"]={
   ["en"]="language",
   ["fr"]="langue",
  },
  ["last"]={
   ["en"]="last",
   ["fr"]="dernier",
  },
  ["lastnamesep"]={
   ["en"]="lastnamesep",
   ["fr"]="sepderniernom",
  },
  ["lastpage"]={
   ["cs"]="poslednistrana",
   ["de"]="letzteseite",
   ["en"]="lastpage",
   ["fr"]="dernierepage",
   ["it"]="ultimapagina",
   ["nl"]="laatstepagina",
   ["pe"]="صفحه‌آخر",
   ["ro"]="ultimapagina",
  },
  ["lastpagesep"]={
   ["en"]="lastpagesep",
   ["fr"]="sepdernierepage",
  },
  ["lastpubsep"]={
   ["en"]="lastpubsep",
   ["fr"]="sepdernierepub",
  },
  ["layout"]={
   ["en"]="layout",
   ["fr"]="miseenpage",
   ["nl"]="layout",
  },
  ["left"]={
   ["cs"]="vlevo",
   ["de"]="links",
   ["en"]="left",
   ["fr"]="gauche",
   ["it"]="sinistra",
   ["nl"]="links",
   ["pe"]="چپ",
   ["ro"]="stanga",
  },
  ["leftcolor"]={
   ["cs"]="barvavlevo",
   ["de"]="linkerfarbe",
   ["en"]="leftcolor",
   ["fr"]="couleurgauche",
   ["it"]="coloresinistra",
   ["nl"]="linkerkleur",
   ["pe"]="رنگ‌چپ",
   ["ro"]="culoarestanga",
  },
  ["leftcompoundhyphen"]={
   ["cs"]="leftcompoundhyphen",
   ["de"]="leftcompoundhyphen",
   ["en"]="leftcompoundhyphen",
   ["fr"]="traitdunioncomposegauche",
   ["it"]="leftcompoundhyphen",
   ["nl"]="linkerkoppelteken",
   ["pe"]="leftcompoundhyphen",
   ["ro"]="leftcompoundhyphen",
  },
  ["leftedge"]={
   ["cs"]="levahrana",
   ["de"]="linkekante",
   ["en"]="leftedge",
   ["fr"]="bordgauche",
   ["it"]="bordosinistro",
   ["nl"]="linkerrand",
   ["pe"]="لبه‌چپ",
   ["ro"]="bordurastanga",
  },
  ["leftedgedistance"]={
   ["cs"]="vzdalenostlevehrany",
   ["de"]="linkerkantenabstand",
   ["en"]="leftedgedistance",
   ["fr"]="distancebordgauche",
   ["it"]="distanzabordosinistro",
   ["nl"]="linkerrandafstand",
   ["pe"]="فاصله‌لبه‌چپ",
   ["ro"]="distantabordurastanga",
  },
  ["leftframe"]={
   ["cs"]="ramecekvlevo",
   ["de"]="linkerrahmen",
   ["en"]="leftframe",
   ["fr"]="cadregauche",
   ["it"]="cornicesinistra",
   ["nl"]="linkerkader",
   ["pe"]="قالب‌چپ",
   ["ro"]="framestanga",
  },
  ["lefthyphen"]={
   ["cs"]="lefthyphen",
   ["de"]="lefthyphen",
   ["en"]="lefthyphen",
   ["fr"]="traitduniongauche",
   ["it"]="lefthyphen",
   ["nl"]="lefthyphen",
   ["pe"]="شکست‌چپ",
   ["ro"]="lefthyphen",
  },
  ["leftmargin"]={
   ["cs"]="levyokraj",
   ["de"]="linkerrand",
   ["en"]="leftmargin",
   ["fr"]="margegauche",
   ["it"]="marginesinistro",
   ["nl"]="linkermarge",
   ["pe"]="حاشیه‌چپ",
   ["ro"]="marginestanga",
  },
  ["leftmargindistance"]={
   ["cs"]="vzdalenostlevehookraje",
   ["de"]="linkerrandabstand",
   ["en"]="leftmargindistance",
   ["fr"]="distancemargegauche",
   ["it"]="distanzamarginesinistro",
   ["nl"]="linkermargeafstand",
   ["pe"]="فاصله‌حاشیه‌چپ",
   ["ro"]="distantamarginestanga",
  },
  ["leftoffset"]={
   ["cs"]="levyoffset",
   ["de"]="linkeroffset",
   ["en"]="leftoffset",
   ["fr"]="decalagegauche",
   ["it"]="offsetsinistro",
   ["nl"]="linkeroffset",
   ["pe"]="آفست‌چپ",
   ["ro"]="offsetstanga",
  },
  ["leftquotation"]={
   ["cs"]="citacevlevo",
   ["de"]="linkerzitat",
   ["en"]="leftquotation",
   ["fr"]="citationgauche",
   ["it"]="citazionesinistra",
   ["nl"]="linkercitaat",
   ["pe"]="نقل‌قول‌چپ",
   ["ro"]="citatstanga",
  },
  ["leftquote"]={
   ["cs"]="citovatvlevo",
   ["de"]="linkerzitieren",
   ["en"]="leftquote",
   ["fr"]="citergauche",
   ["it"]="menzionesinistra",
   ["nl"]="linkerciteer",
   ["pe"]="نقل‌چپ",
   ["ro"]="minicitatstanga",
  },
  ["leftsample"]={
   ["en"]="leftsample",
   ["fr"]="echantillongauche",
  },
  ["leftsentence"]={
   ["cs"]="vetavlevo",
   ["de"]="linkersatz",
   ["en"]="leftsentence",
   ["fr"]="phrasegauche",
   ["it"]="frasesinistra",
   ["nl"]="linkerzin",
   ["pe"]="جمله‌چپ",
   ["ro"]="propozitiestanga",
  },
  ["leftspeech"]={
   ["cs"]="leftspeech",
   ["de"]="leftspeech",
   ["en"]="leftspeech",
   ["fr"]="discoursgauche",
   ["it"]="leftspeech",
   ["nl"]="linkeruitspraak",
   ["pe"]="سخنرانی‌چپ",
   ["ro"]="leftspeech",
  },
  ["leftstyle"]={
   ["cs"]="stylvlevo",
   ["de"]="linkerstil",
   ["en"]="leftstyle",
   ["fr"]="stylegauche",
   ["it"]="stilesinistra",
   ["nl"]="linkerletter",
   ["pe"]="سبک‌چپ",
   ["ro"]="stilstanga",
  },
  ["leftsubsentence"]={
   ["cs"]="podvetavlevo",
   ["de"]="linkersubsatz",
   ["en"]="leftsubsentence",
   ["fr"]="sousphrasegauche",
   ["it"]="sottofrasesinistra",
   ["nl"]="linkersubzin",
   ["pe"]="زیرجمله‌چپ",
   ["ro"]="subpropozitiestanga",
  },
  ["lefttext"]={
   ["cs"]="textvlevo",
   ["de"]="linkertext",
   ["en"]="lefttext",
   ["fr"]="textegauche",
   ["it"]="testosinistro",
   ["nl"]="linkertekst",
   ["pe"]="متن‌چپ",
   ["ro"]="textstanga",
  },
  ["leftwidth"]={
   ["cs"]="sirkavlevo",
   ["de"]="linkerbreite",
   ["en"]="leftwidth",
   ["fr"]="largeurgauche",
   ["it"]="ampiezzasinistra",
   ["nl"]="linkerbreedte",
   ["pe"]="عرض‌خط",
   ["ro"]="latimestanga",
  },
  ["leftwords"]={
   ["en"]="leftwords",
   ["fr"]="motsgauche",
  },
  ["less"]={
   ["en"]="less",
   ["fr"]="moins",
   ["nl"]="minder",
  },
  ["level"]={
   ["cs"]="uroven",
   ["de"]="niveau",
   ["en"]="level",
   ["fr"]="niveau",
   ["it"]="livello",
   ["nl"]="niveau",
   ["pe"]="مرحله",
   ["ro"]="nivel",
  },
  ["levels"]={
   ["cs"]="urovne",
   ["de"]="niveaus",
   ["en"]="levels",
   ["fr"]="niveaux",
   ["it"]="livelli",
   ["nl"]="niveaus",
   ["pe"]="مرحله‌ها",
   ["ro"]="nivele",
  },
  ["limit"]={
   ["en"]="limit",
   ["nl"]="beperk",
  },
  ["limittext"]={
   ["cs"]="limittext",
   ["de"]="limittext",
   ["en"]="limittext",
   ["fr"]="textelimite",
   ["it"]="limittext",
   ["nl"]="limiettekst",
   ["pe"]="مرزمتن",
   ["ro"]="limittext",
  },
  ["limitstretch"]={
   ["en"]="limitstretch",
   ["nl"]="limietrek",
  },
  ["line"]={
   ["cs"]="radek",
   ["de"]="zeile",
   ["en"]="line",
   ["fr"]="ligne",
   ["it"]="riga",
   ["nl"]="regel",
   ["pe"]="خط",
   ["ro"]="linie",
  },
  ["linecorrection"]={
   ["cs"]="korekceradku",
   ["de"]="zeilekorrektur",
   ["en"]="linecorrection",
   ["fr"]="correctionligne",
   ["it"]="correzioneriga",
   ["nl"]="regelcorrectie",
   ["pe"]="تصحیح‌خط",
   ["ro"]="corectielinie",
  },
  ["linedirection"]={
   ["en"]="linedirection",
   ["nl"]="regelrichting",
  },
  ["lines"]={
   ["cs"]="radky",
   ["de"]="zeilen",
   ["en"]="lines",
   ["fr"]="lignes",
   ["it"]="righe",
   ["nl"]="regels",
   ["pe"]="خطها",
   ["ro"]="linii",
  },
  ["list"]={
   ["cs"]="seznam",
   ["de"]="liste",
   ["en"]="list",
   ["fr"]="liste",
   ["it"]="elenco",
   ["nl"]="lijst",
   ["pe"]="لیست",
   ["ro"]="lista",
  },
  ["listtext"]={
   ["cs"]="listtext",
   ["de"]="listtext",
   ["en"]="listtext",
   ["fr"]="texteliste",
   ["it"]="listtext",
   ["nl"]="lijsttekst",
   ["pe"]="متن‌لیست",
   ["ro"]="listtext",
  },
  ["local"]={
   ["cs"]="lokalne",
   ["de"]="lokal",
   ["en"]="local",
   ["fr"]="local",
   ["it"]="locale",
   ["nl"]="lokaal",
   ["pe"]="موضعی",
   ["ro"]="local",
  },
  ["location"]={
   ["cs"]="misto",
   ["de"]="platz",
   ["en"]="location",
   ["fr"]="emplacement",
   ["it"]="luogo",
   ["nl"]="plaats",
   ["pe"]="مکان",
   ["ro"]="locatie",
  },
  ["loffset"]={
   ["en"]="loffset",
   ["fr"]="decalagel",
  },
  ["logo"]={
   ["cs"]="logo",
   ["de"]="logo",
   ["en"]="logo",
   ["fr"]="logo",
   ["it"]="logo",
   ["nl"]="logo",
   ["pe"]="آرم",
   ["ro"]="logo",
  },
  ["logos"]={
   ["cs"]="loga",
   ["de"]="logos",
   ["en"]="logos",
   ["fr"]="logos",
   ["it"]="loghi",
   ["nl"]="logos",
   ["pe"]="آرمها",
   ["ro"]="logos",
  },
  ["marcolor"]={
   ["cs"]="barvaznacky",
   ["de"]="beschrfarbe",
   ["en"]="marcolor",
   ["fr"]="couleurrem",
   ["it"]="coloremarcatura",
   ["nl"]="markleur",
   ["pe"]="رنگ‌حاش",
   ["ro"]="culoaremarcaj",
  },
  ["margin"]={
   ["cs"]="marginalie",
   ["de"]="marginalie",
   ["en"]="margin",
   ["fr"]="marge",
   ["it"]="margine",
   ["nl"]="marge",
   ["pe"]="حاشیه",
   ["ro"]="margine",
  },
  ["margindistance"]={
   ["cs"]="vzdalenostokraje",
   ["de"]="marginalabstand",
   ["en"]="margindistance",
   ["fr"]="distancemarge",
   ["it"]="distanzamargine",
   ["nl"]="margeafstand",
   ["pe"]="فاصله‌حاشیه",
   ["ro"]="distantamargine",
  },
  ["marginedge"]={
   ["cs"]="textovahrana",
   ["de"]="marginalkante",
   ["en"]="marginedge",
   ["fr"]="margebord",
   ["it"]="bordomargine",
   ["nl"]="kantlijn",
   ["pe"]="لبه‌حاشیه",
   ["ro"]="coltbordura",
  },
  ["marginedgetext"]={
   ["cs"]="textmarginalnihookraje",
   ["de"]="marginalkantetext",
   ["en"]="marginedgetext",
   ["fr"]="textemargebord",
   ["it"]="testobordomargine",
   ["nl"]="kantlijntekst",
   ["pe"]="متن‌لبه‌حاشیه",
   ["ro"]="textcoltbordura",
  },
  ["margintext"]={
   ["cs"]="textmarginalie",
   ["de"]="marginaltext",
   ["en"]="margintext",
   ["fr"]="textemarge",
   ["it"]="testomargine",
   ["nl"]="margetekst",
   ["pe"]="متن‌حاشیه",
   ["ro"]="textmargine",
  },
  ["mark"]={
   ["en"]="mark",
   ["fr"]="marque",
   ["nl"]="kernmerk",
  },
  ["marking"]={
   ["cs"]="znaceni",
   ["de"]="beschriftung",
   ["en"]="marking",
   ["fr"]="marquage",
   ["it"]="marcatura",
   ["nl"]="markering",
   ["pe"]="نشانه‌گذاری",
   ["ro"]="marcaje",
  },
  ["marstyle"]={
   ["cs"]="stylsnacky",
   ["de"]="beschrstil",
   ["en"]="marstyle",
   ["fr"]="stylerem",
   ["it"]="stilemarcatura",
   ["nl"]="marletter",
   ["pe"]="سبک‌حاش",
   ["ro"]="stilmarcaj",
  },
  ["mask"]={
   ["en"]="mask",
   ["fr"]="masque",
   ["nl"]="masker",
  },
  ["mathclass"]={
   ["en"]="mathclass",
   ["fr"]="classemath",
  },
  ["mathmeaning"]={
   ["en"]="mathmeaning",
  },
  ["leftclass"]={
   ["en"]="leftclass",
  },
  ["rightclass"]={
   ["en"]="rightclass",
  },
  ["middleclass"]={
   ["en"]="middleclass",
  },
  ["mathconstants"]={
   ["en"]="mathconstants",
  },
  ["mathlimits"]={
   ["en"]="mathlimits",
   ["fr"]="limitesmath",
  },
  ["mathstyle"]={
   ["en"]="mathstyle",
   ["fr"]="stylemath",
  },
  ["mathnumeratorstyle"]={
   ["en"]="mathnumeratorstyle",
  },
  ["mathdenominatorstyle"]={
   ["en"]="mathdenominatorstyle",
  },
  ["mathematics"]={
   ["en"]="mathematics",
  },
  ["max"]={
   ["cs"]="max",
   ["de"]="max",
   ["en"]="max",
   ["fr"]="max",
   ["it"]="max",
   ["nl"]="max",
   ["pe"]="بیشترین",
   ["ro"]="max",
  },
  ["maxdepth"]={
   ["en"]="maxdepth",
   ["fr"]="profondeurmax",
  },
  ["maxheight"]={
   ["cs"]="maxvyska",
   ["de"]="maxhoehe",
   ["en"]="maxheight",
   ["fr"]="hauteurmax",
   ["it"]="altezzamax",
   ["nl"]="maxhoogte",
   ["pe"]="بیشترین‌ارتفاع",
   ["ro"]="inaltimemaxima",
  },
  ["maxwidth"]={
   ["cs"]="maxsirka",
   ["de"]="maxbreite",
   ["en"]="maxwidth",
   ["fr"]="largeurmax",
   ["it"]="ampiezzamax",
   ["nl"]="maxbreedte",
   ["pe"]="بیشترین‌عرض",
   ["ro"]="latimemaxima",
  },
  ["maybeyear"]={
   ["en"]="maybeyear",
   ["fr"]="anneepeutetre",
  },
  ["menu"]={
   ["cs"]="menu",
   ["de"]="menue",
   ["en"]="menu",
   ["fr"]="menu",
   ["it"]="menu",
   ["nl"]="menu",
   ["pe"]="منو",
   ["ro"]="meniu",
  },
  ["metadata"]={
   ["en"]="metadata",
  },
  ["method"]={
   ["cs"]="metoda",
   ["de"]="methode",
   ["en"]="method",
   ["fr"]="methode",
   ["it"]="metodo",
   ["nl"]="methode",
   ["pe"]="روش",
   ["ro"]="metoda",
  },
  ["middle"]={
   ["cs"]="stredni",
   ["de"]="mittig",
   ["en"]="middle",
   ["fr"]="milieu",
   ["it"]="centro",
   ["nl"]="midden",
   ["pe"]="میان",
   ["ro"]="mijloc",
  },
  ["middlecolor"]={
   ["en"]="middlecolor",
   ["fr"]="couleurmilieu",
   ["nl"]="middenkleur",
  },
  ["middlecommand"]={
   ["en"]="middlecommand",
   ["fr"]="commandemilieu",
   ["nl"]="middencommando",
  },
  ["middlespeech"]={
   ["cs"]="middlespeech",
   ["de"]="middlespeech",
   ["en"]="middlespeech",
   ["fr"]="discoursmilieu",
   ["it"]="middlespeech",
   ["nl"]="middenuitspraak",
   ["pe"]="سخنرانی‌میانی",
   ["ro"]="middlespeech",
  },
  ["middlestyle"]={
   ["en"]="middlestyle",
   ["fr"]="stylemilieu",
   ["nl"]="middenletter",
  },
  ["middletext"]={
   ["cs"]="strednitext",
   ["de"]="mittigertext",
   ["en"]="middletext",
   ["fr"]="textemilieu",
   ["it"]="testocentro",
   ["nl"]="middentekst",
   ["pe"]="متن‌میانی",
   ["ro"]="textmijloc",
  },
  ["middlewidth"]={
   ["en"]="middlewidth",
   ["fr"]="largeurmilieu",
   ["nl"]="middenbreedte",
  },
  ["midsentence"]={
   ["cs"]="midsentence",
   ["de"]="midsentence",
   ["en"]="midsentence",
   ["fr"]="milieuphrase",
   ["it"]="midsentence",
   ["nl"]="middenzin",
   ["pe"]="جمله‌میانی",
   ["ro"]="midsentence",
  },
  ["min"]={
   ["cs"]="min",
   ["de"]="min",
   ["en"]="min",
   ["fr"]="min",
   ["it"]="min",
   ["nl"]="min",
   ["pe"]="کمترین",
   ["ro"]="min",
  },
  ["mindepth"]={
   ["cs"]="mindepth",
   ["de"]="mindepth",
   ["en"]="mindepth",
   ["fr"]="profondeurmin",
   ["it"]="mindepth",
   ["nl"]="mindiepte",
   ["pe"]="کمترین‌عمق",
   ["ro"]="mindepth",
  },
  ["minheight"]={
   ["cs"]="minvyska",
   ["de"]="minhoehe",
   ["en"]="minheight",
   ["fr"]="hauteurmin",
   ["it"]="altezzamin",
   ["nl"]="minhoogte",
   ["pe"]="کمترین‌ارتفاع",
   ["ro"]="inaltimeminima",
  },
  ["minwidth"]={
   ["cs"]="minsirka",
   ["de"]="minbreite",
   ["en"]="minwidth",
   ["fr"]="largeurmin",
   ["it"]="ampiezzamin",
   ["nl"]="minbreedte",
   ["pe"]="کمترین‌عرض",
   ["ro"]="latimeminima",
  },
  ["moffset"]={
   ["cs"]="moffset",
   ["de"]="moffset",
   ["en"]="moffset",
   ["fr"]="decalagem",
   ["it"]="moffset",
   ["nl"]="moffset",
   ["pe"]="moffset",
   ["ro"]="moffset",
  },
  ["monthconversion"]={
   ["en"]="monthconversion",
   ["fr"]="conversionmois",
   ["nl"]="maandconversie",
  },
  ["more"]={
   ["en"]="more",
   ["fr"]="plus",
   ["nl"]="meer",
  },
  ["mpdepth"]={
   ["en"]="mpdepth",
   ["fr"]="profondeurmp",
   ["nl"]="mpdiepte",
  },
  ["mpheight"]={
   ["en"]="mpheight",
   ["fr"]="hauteurmp",
   ["nl"]="mphoogte",
  },
  ["mpoffset"]={
   ["en"]="mpoffset",
   ["fr"]="decalagemp",
   ["nl"]="mpoffset",
  },
  ["mpwidth"]={
   ["en"]="mpwidth",
   ["fr"]="largeurmp",
   ["nl"]="mpbreedte",
  },
  ["n"]={
   ["cs"]="n",
   ["de"]="n",
   ["en"]="n",
   ["fr"]="n",
   ["it"]="n",
   ["nl"]="n",
   ["pe"]="n",
   ["ro"]="n",
  },
  ["name"]={
   ["cs"]="jmeno",
   ["de"]="name",
   ["en"]="name",
   ["fr"]="nom",
   ["it"]="nome",
   ["nl"]="naam",
   ["pe"]="نام",
   ["ro"]="nume",
  },
  ["namesep"]={
   ["en"]="namesep",
   ["fr"]="sepnom",
  },
  ["nbottom"]={
   ["cs"]="nspodek",
   ["de"]="nunten",
   ["en"]="nbottom",
   ["fr"]="nbas",
   ["it"]="nfondo",
   ["nl"]="nonder",
   ["pe"]="nbottom",
   ["ro"]="njos",
  },
  ["nc"]={
   ["cs"]="nc",
   ["de"]="nc",
   ["en"]="nc",
   ["fr"]="nc",
   ["it"]="nc",
   ["nl"]="nc",
   ["pe"]="nc",
   ["ro"]="nc",
  },
  ["next"]={
   ["cs"]="dalsi",
   ["de"]="folgende",
   ["en"]="next",
   ["fr"]="suivant",
   ["it"]="successivo",
   ["nl"]="volgende",
   ["pe"]="بعدی",
   ["ro"]="urmatorul",
  },
  ["nextleft"]={
   ["en"]="nextleft",
   ["fr"]="suivantgauche",
  },
  ["nextleftquotation"]={
   ["en"]="nextleftquotation",
   ["fr"]="citationgauchesuivante",
  },
  ["nextright"]={
   ["en"]="nextright",
   ["fr"]="suivantdroite",
  },
  ["nextrightquotation"]={
   ["en"]="nextrightquotation",
   ["fr"]="citationdroitesuivante",
  },
  ["nl"]={
   ["cs"]="nl",
   ["de"]="nl",
   ["en"]="nl",
   ["fr"]="nl",
   ["it"]="nl",
   ["nl"]="nl",
   ["pe"]="nl",
   ["ro"]="nl",
  },
  ["nleft"]={
   ["cs"]="nvlevo",
   ["de"]="nlinks",
   ["en"]="nleft",
   ["fr"]="ngauche",
   ["it"]="nsinistra",
   ["nl"]="nlinks",
   ["pe"]="nleft",
   ["ro"]="nstanga",
  },
  ["nlines"]={
   ["cs"]="nradky",
   ["de"]="zzeile",
   ["en"]="nlines",
   ["fr"]="nlignes",
   ["it"]="nrighe",
   ["nl"]="nregels",
   ["pe"]="nlines",
   ["ro"]="nlinii",
  },
  ["norm"]={
   ["cs"]="norm",
   ["de"]="norm",
   ["en"]="norm",
   ["fr"]="norme",
   ["it"]="norma",
   ["nl"]="norm",
   ["pe"]="norm",
   ["ro"]="norm",
  },
  ["note"]={
   ["en"]="note",
   ["fr"]="note",
   ["nl"]="noot",
  },
  ["notes"]={
   ["en"]="notes",
   ["fr"]="notes",
   ["nl"]="noten",
  },
  ["nr"]={
   ["cs"]="nr",
   ["de"]="nr",
   ["en"]="nr",
   ["fr"]="nr",
   ["it"]="nr",
   ["nl"]="nr",
   ["pe"]="nr",
   ["ro"]="nr",
  },
  ["nright"]={
   ["cs"]="nvpravo",
   ["de"]="nrechts",
   ["en"]="nright",
   ["fr"]="ndroite",
   ["it"]="ndestra",
   ["nl"]="nrechts",
   ["pe"]="nright",
   ["ro"]="ndreapta",
  },
  ["ntop"]={
   ["cs"]="nvrsek",
   ["de"]="noben",
   ["en"]="ntop",
   ["fr"]="nhaut",
   ["it"]="ncima",
   ["nl"]="nboven",
   ["pe"]="ntop",
   ["ro"]="nsus",
  },
  ["number"]={
   ["cs"]="cislo",
   ["de"]="nummer",
   ["en"]="number",
   ["fr"]="numero",
   ["it"]="numero",
   ["nl"]="nummer",
   ["pe"]="شماره",
   ["ro"]="numar",
  },
  ["numberalign"]={
   ["en"]="numberalign",
   ["fr"]="alignernumero",
   ["nl"]="nummeruitlijnen",
  },
  ["numbercolor"]={
   ["cs"]="barvacisla",
   ["de"]="nummernfarbe",
   ["en"]="numbercolor",
   ["fr"]="couleurnumero",
   ["it"]="colorenumero",
   ["nl"]="nummerkleur",
   ["pe"]="رنگ‌شماره",
   ["ro"]="culoarenumar",
  },
  ["numbercommand"]={
   ["cs"]="ciselnyprikaz",
   ["de"]="nummerbefehl",
   ["en"]="numbercommand",
   ["fr"]="commandenumero",
   ["it"]="comandonumero",
   ["nl"]="nummercommando",
   ["pe"]="فرمان‌شماره",
   ["ro"]="comandanumar",
  },
  ["numberconversion"]={
   ["en"]="numberconversion",
   ["fr"]="conversionnumero",
  },
  ["numberconversionset"]={
   ["en"]="numberconversionset",
   ["fr"]="jeuconversionnumero",
  },
  ["numberdistance"]={
   ["cs"]="numberdistance",
   ["de"]="numberdistance",
   ["en"]="numberdistance",
   ["fr"]="distancenumero",
   ["it"]="numberdistance",
   ["nl"]="nummerafstand",
   ["pe"]="فاصله‌شماره",
   ["ro"]="numberdistance",
  },
  ["numberthreshold"]={
   ["en"]="numberthreshold",
  },
  ["numbering"]={
   ["cs"]="cislovani",
   ["de"]="nummerierung",
   ["en"]="numbering",
   ["fr"]="numérotation",
   ["it"]="numerazione",
   ["nl"]="nummeren",
   ["pe"]="شماره‌گذاری",
   ["ro"]="numerotare",
  },
  ["numberlocation"]={
   ["en"]="numberlocation",
  },
  ["numberorder"]={
   ["en"]="numberorder",
   ["fr"]="ordrenumero",
  },
  ["numberprefix"]={
   ["en"]="numberprefix",
   ["fr"]="prefixenumero",
  },
  ["numbersegments"]={
   ["en"]="numbersegments",
   ["fr"]="segmentsnumero",
  },
  ["numberseparator"]={
   ["cs"]="oddelovaccisla",
   ["de"]="nummernseperator",
   ["en"]="numberseparator",
   ["fr"]="separateurnumero",
   ["it"]="separatorenumero",
   ["nl"]="nummerscheider",
   ["pe"]="جداکننده‌شماره",
   ["ro"]="separatornumar",
  },
  ["numberseparatorset"]={
   ["en"]="numberseparatorset",
   ["fr"]="jeuseparateurnumero",
  },
  ["numberset"]={
   ["en"]="numberset",
   ["fr"]="jeunumero",
  },
  ["numberstarter"]={
   ["en"]="numberstarter",
   ["fr"]="demarreurnumero",
  },
  ["numberstate"]={
   ["en"]="numberstate",
  },
  ["numberstopper"]={
   ["en"]="numberstopper",
   ["fr"]="stoppeurnumero",
  },
  ["numberstrut"]={
   ["en"]="numberstrut",
   ["fr"]="entretoisenumero",
   ["nl"]="nummerstrut",
  },
  ["numberstyle"]={
   ["cs"]="stylcisla",
   ["de"]="nummernstil",
   ["en"]="numberstyle",
   ["fr"]="stylenumero",
   ["it"]="stilenumero",
   ["nl"]="nummerletter",
   ["pe"]="سبک‌شماره",
   ["ro"]="stilnumar",
  },
  ["numberwidth"]={
   ["cs"]="numberwidth",
   ["de"]="numberwidth",
   ["en"]="numberwidth",
   ["fr"]="largeurnumero",
   ["it"]="numberwidth",
   ["nl"]="nummerbreedte",
   ["pe"]="عرض‌شماره",
   ["ro"]="numberwidth",
  },
  ["nx"]={
   ["cs"]="nx",
   ["de"]="nx",
   ["en"]="nx",
   ["fr"]="nx",
   ["it"]="nx",
   ["nl"]="nx",
   ["pe"]="nx",
   ["ro"]="nx",
  },
  ["ny"]={
   ["cs"]="ny",
   ["de"]="ny",
   ["en"]="ny",
   ["fr"]="ny",
   ["it"]="ny",
   ["nl"]="ny",
   ["pe"]="ny",
   ["ro"]="ny",
  },
  ["object"]={
   ["cs"]="objekt",
   ["de"]="objekt",
   ["en"]="object",
   ["fr"]="objet",
   ["it"]="oggetto",
   ["nl"]="object",
   ["pe"]="شیئ",
   ["ro"]="obiect",
  },
  ["obstruction"]={
   ["cs"]="prekazka",
   ["de"]="gesperrt",
   ["en"]="obstruction",
   ["fr"]="obstruction",
   ["it"]="ostruzione",
   ["nl"]="blokkade",
   ["pe"]="انسداد",
   ["ro"]="obstructie",
  },
  ["oddmargin"]={
   ["cs"]="lichyokraj",
   ["de"]="ungeraderand",
   ["en"]="oddmargin",
   ["fr"]="margeimpaire",
   ["it"]="marginedispari",
   ["nl"]="onevenmarge",
   ["pe"]="حاشیه‌فرد",
   ["ro"]="margineimpara",
  },
  ["off"]={
   ["cs"]="vyp",
   ["de"]="aus",
   ["en"]="off",
   ["fr"]="arret",
   ["it"]="disattivo",
   ["nl"]="uit",
   ["pe"]="خاموش",
   ["ro"]="dezactivat",
  },
  ["offset"]={
   ["cs"]="offset",
   ["de"]="offset",
   ["en"]="offset",
   ["fr"]="decalage",
   ["it"]="offset",
   ["nl"]="offset",
   ["pe"]="آفست",
   ["ro"]="offset",
  },
  ["on"]={
   ["cs"]="zap",
   ["de"]="an",
   ["en"]="on",
   ["fr"]="marche",
   ["it"]="attivo",
   ["nl"]="aan",
   ["pe"]="روی",
   ["ro"]="on",
  },
  ["openaction"]={
   ["cs"]="otevriakci",
   ["de"]="oeffenaktion",
   ["en"]="openaction",
   ["fr"]="actionouvrir",
   ["it"]="azioneapri",
   ["nl"]="openactie",
   ["pe"]="عمل‌باز",
   ["ro"]="actiunedeschidere",
  },
  ["openpage"]={
   ["en"]="openpage",
   ["fr"]="ouvrirpage",
   ["nl"]="openpagina",
  },
  ["openpageaction"]={
   ["cs"]="akceotevrenistranky",
   ["de"]="openpageaction",
   ["en"]="openpageaction",
   ["fr"]="actionouvrirpage",
   ["it"]="azioneapripagina",
   ["nl"]="openpaginaactie",
   ["pe"]="عمل‌صفحه‌باز",
   ["ro"]="actiunedeschiderepagina",
  },
  ["openup"]={
   ["en"]="openup",
   ["fr"]="ouverture",
  },
  ["opticalsize"]={
   ["en"]="opticalsize",
   ["fr"]="tailleoptique",
  },
  ["option"]={
   ["cs"]="volba",
   ["de"]="option",
   ["en"]="option",
   ["fr"]="option",
   ["it"]="opzione",
   ["nl"]="optie",
   ["pe"]="گزینه",
   ["ro"]="optiune",
  },
  ["order"]={
   ["en"]="order",
   ["fr"]="ordre",
   ["nl"]="volgorde",
  },
  ["orientation"]={
   ["cs"]="orientation",
   ["de"]="orientation",
   ["en"]="orientation",
   ["fr"]="orientation",
   ["it"]="orientation",
   ["nl"]="orientatie",
   ["pe"]="جهت‌دهی",
   ["ro"]="orientation",
  },
  ["otherstext"]={
   ["en"]="otherstext",
   ["fr"]="texteautres",
  },
  ["outermargin"]={
   ["cs"]="outermargin",
   ["de"]="outermargin",
   ["en"]="outermargin",
   ["fr"]="margeexterieure",
   ["it"]="margineesterno",
   ["nl"]="buitenmarge",
   ["pe"]="حاشیه‌خارجی",
   ["ro"]="outermargin",
  },
  ["overflow"]={
   ["en"]="overflow",
  },
  ["overprint"]={
   ["cs"]="overprint",
   ["de"]="overprint",
   ["en"]="overprint",
   ["fr"]="surimpression",
   ["it"]="overprint",
   ["nl"]="overprint",
   ["pe"]="overprint",
   ["ro"]="overprint",
  },
  ["ownerpassword"]={
   ["en"]="ownerpassword",
   ["fr"]="motdepasseproprietaire",
  },
  ["ownnumber"]={
   ["cs"]="vlastnicislo",
   ["de"]="eigenenummer",
   ["en"]="ownnumber",
   ["fr"]="numeropropre",
   ["it"]="numeroproprio",
   ["nl"]="eigennummer",
   ["pe"]="شماره‌خود",
   ["ro"]="numarpropriu",
  },
  ["packcriterium"]={
   ["en"]="packcriterium",
   ["fr"]="criterepack",
   ["nl"]="opelkaarcriterium",
  },
  ["page"]={
   ["cs"]="stranka",
   ["de"]="seite",
   ["en"]="page",
   ["fr"]="page",
   ["it"]="pagina",
   ["nl"]="pagina",
   ["pe"]="صفحه",
   ["ro"]="pagina",
  },
  ["pageboundaries"]={
   ["cs"]="hranicestranky",
   ["de"]="seitenbegrenzung",
   ["en"]="pageboundaries",
   ["fr"]="frontierespage",
   ["it"]="limitipagina",
   ["nl"]="paginaovergangen",
   ["pe"]="مرزهای‌صفحه",
   ["ro"]="marginipagina",
  },
  ["pagecolor"]={
   ["cs"]="barvastranky",
   ["de"]="seitenfarbe",
   ["en"]="pagecolor",
   ["fr"]="couleurpage",
   ["it"]="colorepagina",
   ["nl"]="paginakleur",
   ["pe"]="رنگ‌صفحه",
   ["ro"]="culoarepagina",
  },
  ["pagecolormodel"]={
   ["en"]="pagecolormodel",
   ["fr"]="modelecouleurpage",
   ["nl"]="paginakleurmodel",
  },
  ["pagecommand"]={
   ["cs"]="strankovyprikaz",
   ["de"]="seitenbefehl",
   ["en"]="pagecommand",
   ["fr"]="commandepage",
   ["it"]="comandopagina",
   ["nl"]="paginacommando",
   ["pe"]="فرمان‌صفحه",
   ["ro"]="comandapagina",
  },
  ["pageconnector"]={
   ["en"]="pageconnector",
   ["fr"]="connecteurpage",
  },
  ["pageconversion"]={
   ["en"]="pageconversion",
   ["fr"]="conversionpage",
  },
  ["pageconversionset"]={
   ["en"]="pageconversionset",
   ["fr"]="jeuconversionpage",
  },
  ["pageleft"]={
   ["en"]="pageleft",
   ["fr"]="pagegauche",
  },
  ["pagemethod"]={
   ["en"]="pagemethod",
   ["fr"]="methodepage",
   ["nl"]="paginamethode",
  },
  ["pagenumber"]={
   ["cs"]="cislostranky",
   ["de"]="seitennummer",
   ["en"]="pagenumber",
   ["fr"]="numeropage",
   ["it"]="numeropagina",
   ["nl"]="paginanummer",
   ["pe"]="شماره‌صفحه",
   ["ro"]="numarpagina",
  },
  ["pageprefix"]={
   ["en"]="pageprefix",
   ["fr"]="prefixepage",
  },
  ["pageprefixconnector"]={
   ["en"]="pageprefixconnector",
   ["fr"]="connecteurprefixepage",
  },
  ["pageprefixconversion"]={
   ["en"]="pageprefixconversion",
   ["fr"]="conversionprefixepage",
  },
  ["pageprefixconversionset"]={
   ["en"]="pageprefixconversionset",
   ["fr"]="jeuconversionprefixepage",
  },
  ["pageprefixsegments"]={
   ["en"]="pageprefixsegments",
   ["fr"]="segmentsprefixepage",
  },
  ["pageprefixseparatorset"]={
   ["en"]="pageprefixseparatorset",
   ["fr"]="jeuseparateurprefixepage",
  },
  ["pageprefixset"]={
   ["en"]="pageprefixset",
   ["fr"]="jeuprefixepage",
  },
  ["pageprefixstarter"]={
   ["en"]="pageprefixstarter",
   ["fr"]="demarreurprefixepage",
  },
  ["pageprefixstopper"]={
   ["en"]="pageprefixstopper",
   ["fr"]="stoppeurprefixpage",
  },
  ["pageright"]={
   ["en"]="pageright",
   ["fr"]="pagedroite",
  },
  ["pagesegments"]={
   ["en"]="pagesegments",
   ["fr"]="segmentspage",
  },
  ["pagesep"]={
   ["en"]="pagesep",
   ["fr"]="seppage",
  },
  ["pageseparatorset"]={
   ["en"]="pageseparatorset",
   ["fr"]="jeuseparateurpage",
  },
  ["pageset"]={
   ["en"]="pageset",
   ["fr"]="jeupage",
  },
  ["pagestarter"]={
   ["en"]="pagestarter",
   ["fr"]="demarreurpage",
  },
  ["pagestate"]={
   ["cs"]="pagestate",
   ["de"]="pagestate",
   ["en"]="pagestate",
   ["fr"]="etatpage",
   ["it"]="statopagina",
   ["nl"]="paginastatus",
   ["pe"]="وضعیت‌صفحه",
   ["ro"]="pagestate",
  },
  ["pagestopper"]={
   ["en"]="pagestopper",
   ["fr"]="stoppeurpage",
  },
  ["pagestyle"]={
   ["cs"]="stylstranky",
   ["de"]="seitenstil",
   ["en"]="pagestyle",
   ["fr"]="stylepage",
   ["it"]="stilepagina",
   ["nl"]="paginaletter",
   ["pe"]="سبک‌صفحه",
   ["ro"]="stilpagina",
  },
  ["palet"]={
   ["cs"]="paleta",
   ["de"]="palette",
   ["en"]="palet",
   ["fr"]="palette",
   ["it"]="tavolozza",
   ["nl"]="palet",
   ["pe"]="لوح",
   ["ro"]="paleta",
  },
  ["paper"]={
   ["cs"]="papir",
   ["de"]="papier",
   ["en"]="paper",
   ["fr"]="papier",
   ["it"]="carta",
   ["nl"]="papier",
   ["pe"]="برگ",
   ["ro"]="hartie",
  },
  ["paragraph"]={
   ["cs"]="odstavec",
   ["de"]="absatz",
   ["en"]="paragraph",
   ["fr"]="paragraphe",
   ["it"]="capoverso",
   ["nl"]="alinea",
   ["pe"]="پاراگراف",
   ["ro"]="paragraf",
  },
  ["penalties"]={
   ["en"]="penalties",
  },
  ["period"]={
   ["en"]="period",
   ["fr"]="periode",
  },
  ["place"]={
   ["cs"]="umistit",
   ["de"]="platziere",
   ["en"]="place",
   ["fr"]="placer",
   ["it"]="metti",
   ["nl"]="plaatsen",
   ["pe"]="بگذار",
   ["ro"]="pune",
  },
  ["placehead"]={
   ["cs"]="umistihlavicku",
   ["de"]="setzekopf",
   ["en"]="placehead",
   ["fr"]="placertete",
   ["it"]="mettitesta",
   ["nl"]="plaatskop",
   ["pe"]="بگذارسر",
   ["ro"]="punetitlu",
  },
  ["placeholder"]={
   ["en"]="placeholder",
  },
  ["placestopper"]={
   ["cs"]="predelmista",
   ["de"]="setzetrenner",
   ["en"]="placestopper",
   ["fr"]="placerstoppeur",
   ["it"]="mettistopper",
   ["nl"]="plaatsafsluiter",
   ["pe"]="بگذارایست",
   ["ro"]="punestopper",
  },
  ["plugin"]={
   ["en"]="plugin",
  },
  ["position"]={
   ["cs"]="position",
   ["de"]="position",
   ["en"]="position",
   ["fr"]="position",
   ["it"]="posizione",
   ["nl"]="positie",
   ["pe"]="موقعیت",
   ["ro"]="pozitie",
  },
  ["prefix"]={
   ["cs"]="prefix",
   ["de"]="prefix",
   ["en"]="prefix",
   ["fr"]="prefixe",
   ["it"]="prefisso",
   ["nl"]="prefix",
   ["pe"]="پیشوند",
   ["ro"]="prefix",
  },
  ["prefixconnector"]={
   ["en"]="prefixconnector",
   ["fr"]="connecteurprefixe",
  },
  ["prefixconversion"]={
   ["en"]="prefixconversion",
   ["fr"]="conversionprefixe",
  },
  ["prefixconversionset"]={
   ["en"]="prefixconversionset",
   ["fr"]="jeuconversionprefixe",
  },
  ["prefixsegments"]={
   ["en"]="prefixsegments",
   ["fr"]="segmentsprefixe",
  },
  ["prefixseparatorset"]={
   ["en"]="prefixseparatorset",
   ["fr"]="jeuseparateurprefixe",
  },
  ["prefixset"]={
   ["en"]="prefixset",
   ["fr"]="jeuprefixe",
  },
  ["prefixstarter"]={
   ["en"]="prefixstarter",
   ["fr"]="demarreurprefixe",
  },
  ["prefixstopper"]={
   ["en"]="prefixstopper",
   ["fr"]="stoppeurprefixe",
  },
  ["preset"]={
   ["cs"]="prednastaveni",
   ["de"]="voreinstellung",
   ["en"]="preset",
   ["fr"]="preregler",
   ["it"]="preimpostato",
   ["nl"]="preset",
   ["pe"]="preset",
   ["ro"]="preset",
  },
  ["preview"]={
   ["cs"]="nahled",
   ["de"]="vorschau",
   ["en"]="preview",
   ["fr"]="apercu",
   ["it"]="anteprima",
   ["nl"]="preview",
   ["pe"]="پیش‌دید",
   ["ro"]="previzualizare",
  },
  ["previous"]={
   ["cs"]="predchozi",
   ["de"]="vorige",
   ["en"]="previous",
   ["fr"]="precedent",
   ["it"]="precedente",
   ["nl"]="vorige",
   ["pe"]="قبلی",
   ["ro"]="precendent",
  },
  ["previousnumber"]={
   ["cs"]="predchozicislo",
   ["de"]="vorigenummer",
   ["en"]="previousnumber",
   ["fr"]="numeroprecedent",
   ["it"]="numeroprecedente",
   ["nl"]="vorigenummer",
   ["pe"]="شماره‌قبلی",
   ["ro"]="numarprecedent",
  },
  ["print"]={
   ["en"]="print",
   ["fr"]="imprimer",
   ["nl"]="print",
  },
  ["printable"]={
   ["cs"]="tisknutelne",
   ["de"]="druckbar",
   ["en"]="printable",
   ["fr"]="imprimable",
   ["it"]="stampabile",
   ["nl"]="printbaar",
   ["pe"]="قابل‌چاپ",
   ["ro"]="tiparibil",
  },
  ["process"]={
   ["cs"]="process",
   ["de"]="process",
   ["en"]="process",
   ["fr"]="traiter",
   ["it"]="process",
   ["nl"]="proces",
   ["pe"]="پردازش",
   ["ro"]="process",
  },
  ["profile"]={
   ["en"]="profile",
   ["fr"]="profil",
  },
  ["properties"]={
   ["en"]="properties",
   ["fr"]="proprietes",
  },
  ["pubsep"]={
   ["en"]="pubsep",
   ["fr"]="seppub",
  },
  ["quotechar"]={
   ["en"]="quotechar",
   ["fr"]="caracitation",
  },
  ["radius"]={
   ["cs"]="polomer",
   ["de"]="radius",
   ["en"]="radius",
   ["fr"]="rayon",
   ["it"]="raggio",
   ["nl"]="straal",
   ["pe"]="شعاع",
   ["ro"]="raza",
  },
  ["random"]={
   ["cs"]="nahodne",
   ["de"]="zufaellig",
   ["en"]="random",
   ["fr"]="aleatoire",
   ["it"]="casuale",
   ["nl"]="willekeur",
   ["pe"]="تصادفی",
   ["ro"]="aleator",
  },
  ["range"]={
   ["en"]="range",
   ["fr"]="gamme",
   ["nl"]="bereik",
  },
  ["reduction"]={
   ["cs"]="redukce",
   ["de"]="reduktion",
   ["en"]="reduction",
   ["fr"]="reduction",
   ["it"]="riduzione",
   ["nl"]="reductie",
   ["pe"]="کاهش",
   ["ro"]="reducere",
  },
  ["ref"]={
   ["cs"]="ref",
   ["de"]="ref",
   ["en"]="ref",
   ["fr"]="ref",
   ["it"]="ref",
   ["nl"]="ref",
   ["pe"]="رج",
   ["ro"]="ref",
  },
  ["refcommand"]={
   ["en"]="refcommand",
   ["fr"]="commanderef",
  },
  ["reference"]={
   ["cs"]="odkaz",
   ["de"]="referenz",
   ["en"]="reference",
   ["fr"]="reference",
   ["it"]="riferimento",
   ["nl"]="referentie",
   ["pe"]="مرجع",
   ["ro"]="referinta",
  },
  ["referencemethod"]={
   ["en"]="referencemethod",
   ["fr"]="methodereference",
  },
  ["referencetext"]={
   ["en"]="referencetext",
  },
  ["referenceprefix"]={
   ["en"]="referenceprefix",
   ["fr"]="prefixereference",
  },
  ["referencing"]={
   ["cs"]="odkazujici",
   ["de"]="referieren",
   ["en"]="referencing",
   ["fr"]="referencement",
   ["it"]="referencing",
   ["nl"]="refereren",
   ["pe"]="مراجعه",
   ["ro"]="referinta",
  },
  ["region"]={
   ["en"]="region",
   ["fr"]="region",
   ["nl"]="gebied",
  },
  ["regionin"]={
   ["cs"]="oblastuvnitr",
   ["de"]="regionin",
   ["en"]="regionin",
   ["fr"]="regiondedans",
   ["it"]="entraregione",
   ["nl"]="gebiedin",
   ["pe"]="ناحیه‌درون",
   ["ro"]="regiuneintrare",
  },
  ["regionout"]={
   ["cs"]="oblastvne",
   ["de"]="regionaus",
   ["en"]="regionout",
   ["fr"]="regiondehors",
   ["it"]="esciregione",
   ["nl"]="gebieduit",
   ["pe"]="ناحیه‌بیرون",
   ["ro"]="regiuneiesire",
  },
  ["register"]={
   ["en"]="register",
   ["fr"]="registre",
  },
  ["regularfeatures"]={
   ["en"]="regularfeatures",
   ["fr"]="fonctionnalitesordinaires",
  },
  ["regularfont"]={
   ["en"]="regularfont",
   ["fr"]="policereordinaire",
  },
  ["renderingsetup"]={
   ["en"]="renderingsetup",
   ["fr"]="configurationrendu",
  },
  ["repeat"]={
   ["cs"]="opakovat",
   ["de"]="wiederholen",
   ["en"]="repeat",
   ["fr"]="repeter",
   ["it"]="ripeti",
   ["nl"]="herhaal",
   ["pe"]="تکرار",
   ["ro"]="repeta",
  },
  ["reset"]={
   ["cs"]="reset",
   ["de"]="reset",
   ["en"]="reset",
   ["fr"]="raz",
   ["it"]="reset",
   ["nl"]="reset",
   ["pe"]="بازنشانی",
   ["ro"]="reset",
  },
  ["resetnumber"]={
   ["cs"]="resetnumber",
   ["de"]="resetnumber",
   ["en"]="resetnumber",
   ["fr"]="raznumero",
   ["it"]="resetnumber",
   ["nl"]="resetnummer",
   ["pe"]="بازنشانی‌شماره",
   ["ro"]="resetnumber",
  },
  ["resolution"]={
   ["cs"]="rozliseni",
   ["de"]="aufloesung",
   ["en"]="resolution",
   ["fr"]="resolution",
   ["it"]="risoluzione",
   ["nl"]="resolutie",
   ["pe"]="کیفیت",
   ["ro"]="rezolutie",
  },
  ["resources"]={
   ["en"]="resources",
   ["fr"]="ressources",
  },
  ["reverse"]={
   ["cs"]="reverse",
   ["de"]="reverse",
   ["en"]="reverse",
   ["fr"]="inverser",
   ["it"]="invertito",
   ["nl"]="omgekeerd",
   ["pe"]="برعکس",
   ["ro"]="reverse",
  },
  ["right"]={
   ["cs"]="vpravo",
   ["de"]="rechts",
   ["en"]="right",
   ["fr"]="droite",
   ["it"]="destra",
   ["nl"]="rechts",
   ["pe"]="راست",
   ["ro"]="dreapta",
  },
  ["rightchars"]={
   ["en"]="rightchars",
   ["fr"]="caracteresdroite",
  },
  ["rightcolor"]={
   ["cs"]="barvavpravo",
   ["de"]="rechterfarbe",
   ["en"]="rightcolor",
   ["fr"]="couleurdroite",
   ["it"]="coloredestra",
   ["nl"]="rechterkleur",
   ["pe"]="رنگ‌راست",
   ["ro"]="culoaredreapta",
  },
  ["rightcompoundhyphen"]={
   ["cs"]="rightcompoundhyphen",
   ["de"]="rightcompoundhyphen",
   ["en"]="rightcompoundhyphen",
   ["fr"]="traitdunioncomposedroite",
   ["it"]="rightcompoundhyphen",
   ["nl"]="rechterkoppelteken",
   ["pe"]="rightcompoundhyphen",
   ["ro"]="rightcompoundhyphen",
  },
  ["rightedge"]={
   ["cs"]="pravahrana",
   ["de"]="rechtekante",
   ["en"]="rightedge",
   ["fr"]="borddroit",
   ["it"]="bordodestro",
   ["nl"]="rechterrand",
   ["pe"]="لبه‌راست",
   ["ro"]="borduradreapta",
  },
  ["rightedgedistance"]={
   ["cs"]="vzdalenostpravehrany",
   ["de"]="rechterkantenabstand",
   ["en"]="rightedgedistance",
   ["fr"]="distanceborddroit",
   ["it"]="distanzabordodestro",
   ["nl"]="rechterrandafstand",
   ["pe"]="فاصله‌لبه‌راست",
   ["ro"]="distantacoltdreapta",
  },
  ["rightframe"]={
   ["cs"]="ramecekvpravo",
   ["de"]="rechterrahmen",
   ["en"]="rightframe",
   ["fr"]="cadredroite",
   ["it"]="cornicedestra",
   ["nl"]="rechterkader",
   ["pe"]="قالب‌راست",
   ["ro"]="framedreapta",
  },
  ["righthyphen"]={
   ["cs"]="righthyphen",
   ["de"]="righthyphen",
   ["en"]="righthyphen",
   ["fr"]="traitduniondroite",
   ["it"]="righthyphen",
   ["nl"]="righthyphen",
   ["pe"]="righthyphen",
   ["ro"]="righthyphen",
  },
  ["rightmargin"]={
   ["cs"]="pravyokraj",
   ["de"]="rechterrand",
   ["en"]="rightmargin",
   ["fr"]="margedroite",
   ["it"]="marginedestro",
   ["nl"]="rechtermarge",
   ["pe"]="حاشیه‌راست",
   ["ro"]="marginedreapta",
  },
  ["rightmargindistance"]={
   ["cs"]="vzdalenostpravehookraje",
   ["de"]="rechterrandabstand",
   ["en"]="rightmargindistance",
   ["fr"]="distancemargedroite",
   ["it"]="distanzamarginedestro",
   ["nl"]="rechtermargeafstand",
   ["pe"]="فاصله‌حاشیه‌راست",
   ["ro"]="distantamarginedreapta",
  },
  ["rightoffset"]={
   ["cs"]="pravyoffset",
   ["de"]="rechterabstand",
   ["en"]="rightoffset",
   ["fr"]="decalagedroite",
   ["it"]="offsetdestro",
   ["nl"]="rechteroffset",
   ["pe"]="آفست‌راست",
   ["ro"]="offsetdreapta",
  },
  ["rightquotation"]={
   ["cs"]="citacevpravo",
   ["de"]="rechterzitat",
   ["en"]="rightquotation",
   ["fr"]="citationdroite",
   ["it"]="citazionedestra",
   ["nl"]="rechtercitaat",
   ["pe"]="نقل‌قول‌راست",
   ["ro"]="citatdreapta",
  },
  ["rightquote"]={
   ["cs"]="citovatvpravo",
   ["de"]="rechterzitieren",
   ["en"]="rightquote",
   ["fr"]="citerdroite",
   ["it"]="menzionedestra",
   ["nl"]="rechterciteer",
   ["pe"]="نقل‌راست",
   ["ro"]="minicitatdreapta",
  },
  ["rightsample"]={
   ["en"]="rightsample",
   ["fr"]="echantillondroite",
  },
  ["rightsentence"]={
   ["cs"]="vetavpravo",
   ["de"]="rechtersatz",
   ["en"]="rightsentence",
   ["fr"]="phrasedroite",
   ["it"]="frasedestra",
   ["nl"]="rechterzin",
   ["pe"]="جمله‌راست",
   ["ro"]="propozitiedreapta",
  },
  ["rightspeech"]={
   ["cs"]="rightspeech",
   ["de"]="rightspeech",
   ["en"]="rightspeech",
   ["fr"]="discoursdroite",
   ["it"]="rightspeech",
   ["nl"]="rechteruitspraak",
   ["pe"]="سخنرانی‌راست",
   ["ro"]="rightspeech",
  },
  ["rightstyle"]={
   ["cs"]="stylvpravo",
   ["de"]="rechterstil",
   ["en"]="rightstyle",
   ["fr"]="styledroit",
   ["it"]="stiledestra",
   ["nl"]="rechterletter",
   ["pe"]="سبک‌راست",
   ["ro"]="stildreapta",
  },
  ["rightsubsentence"]={
   ["cs"]="podvetavpravo",
   ["de"]="rechtersubsatz",
   ["en"]="rightsubsentence",
   ["fr"]="sousphrasedroite",
   ["it"]="sottofrasedestra",
   ["nl"]="rechtersubzin",
   ["pe"]="زیرجمله‌راست",
   ["ro"]="subpropozitiedreapta",
  },
  ["righttext"]={
   ["cs"]="textvpravo",
   ["de"]="rechtertext",
   ["en"]="righttext",
   ["fr"]="textedroite",
   ["it"]="testodestro",
   ["nl"]="rechtertekst",
   ["pe"]="متن‌راست",
   ["ro"]="textdreapta",
  },
  ["rightwidth"]={
   ["cs"]="sirkavpravo",
   ["de"]="rechterbreite",
   ["en"]="rightwidth",
   ["fr"]="largeurdroite",
   ["it"]="ampiezzadestra",
   ["nl"]="rechterbreedte",
   ["pe"]="عرض‌راست",
   ["ro"]="marginedreapta",
  },
  ["rightwords"]={
   ["en"]="rightwords",
   ["fr"]="motsdroite",
  },
  ["roffset"]={
   ["en"]="roffset",
   ["fr"]="decalager",
  },
  ["rotation"]={
   ["cs"]="rotace",
   ["de"]="rotation",
   ["en"]="rotation",
   ["fr"]="rotation",
   ["it"]="rotazione",
   ["nl"]="rotatie",
   ["pe"]="دوران",
   ["ro"]="rotatie",
  },
  ["rowdistance"]={
   ["en"]="rowdistance",
   ["nl"]="rijafstand",
  },
  ["rows"]={
   ["en"]="rows",
   ["nl"]="rijen",
  },
  ["rule"]={
   ["cs"]="linka",
   ["de"]="linie",
   ["en"]="rule",
   ["fr"]="trait",
   ["it"]="linea",
   ["nl"]="lijn",
   ["pe"]="خط",
   ["ro"]="rigla",
  },
  ["rulecolor"]={
   ["cs"]="barvalinky",
   ["de"]="linienfarbe",
   ["en"]="rulecolor",
   ["fr"]="couleurtrait",
   ["it"]="colorelinea",
   ["nl"]="lijnkleur",
   ["pe"]="رنگ‌خط",
   ["ro"]="culoarerigla",
  },
  ["rulecommand"]={
   ["en"]="rulecommand",
   ["fr"]="commandetrait",
   ["nl"]="lijncommando",
  },
  ["rulethickness"]={
   ["cs"]="tloustkalinky",
   ["de"]="liniendicke",
   ["en"]="rulethickness",
   ["fr"]="epaisseurtrait",
   ["it"]="spessorelinea",
   ["nl"]="lijndikte",
   ["pe"]="ضخامت‌خط",
   ["ro"]="grosimerigla",
  },
  ["rulewidth"]={
   ["en"]="rulewidth",
   ["nl"]="lijnbreedte",
  },
  ["ruledistance"]={
   ["en"]="ruledistance",
   ["nl"]="lijnafstand",
  },
  ["samepage"]={
   ["cs"]="stejnastranka",
   ["de"]="selbeseite",
   ["en"]="samepage",
   ["fr"]="memepage",
   ["it"]="stessapagina",
   ["nl"]="zelfdepagina",
   ["pe"]="همان‌صفحه",
   ["ro"]="aceeasipagina",
  },
  ["sample"]={
   ["cs"]="vzor",
   ["de"]="muster",
   ["en"]="sample",
   ["fr"]="echantillon",
   ["it"]="campione",
   ["nl"]="monster",
   ["pe"]="نمونه",
   ["ro"]="exemplu",
  },
  ["samplesize"]={
   ["en"]="samplesize",
   ["fr"]="tailleechantillon",
  },
  ["saveinlist"]={
   ["en"]="saveinlist",
   ["fr"]="enregistrerdansliste",
  },
  ["scale"]={
   ["cs"]="meritko",
   ["de"]="format",
   ["en"]="scale",
   ["fr"]="echelle",
   ["it"]="scala",
   ["nl"]="schaal",
   ["pe"]="مقیاس",
   ["ro"]="scala",
  },
  ["scope"]={
   ["cs"]="rozsah",
   ["de"]="bereich",
   ["en"]="scope",
   ["fr"]="perimetre",
   ["it"]="scope",
   ["nl"]="scope",
   ["pe"]="طرح",
   ["ro"]="scop",
  },
  ["screen"]={
   ["cs"]="rastr",
   ["de"]="raster",
   ["en"]="screen",
   ["fr"]="ecran",
   ["it"]="schermo",
   ["nl"]="raster",
   ["pe"]="پرده",
   ["ro"]="ecran",
  },
  ["section"]={
   ["cs"]="oddil",
   ["de"]="abschnitt",
   ["en"]="section",
   ["fr"]="section",
   ["it"]="sezione",
   ["nl"]="sectie",
   ["pe"]="بخش",
   ["ro"]="sectiune",
  },
  ["sectionblock"]={
   ["en"]="sectionblock",
   ["fr"]="blocsection",
  },
  ["sectionconversion"]={
   ["en"]="sectionconversion",
   ["fr"]="conversionsection",
  },
  ["sectionconversionset"]={
   ["en"]="sectionconversionset",
   ["fr"]="jeuconversionsection",
  },
  ["sectionnumber"]={
   ["cs"]="cislooddilu",
   ["de"]="abschnittsnummer",
   ["en"]="sectionnumber",
   ["fr"]="numerosection",
   ["it"]="numerosezione",
   ["nl"]="sectienummer",
   ["pe"]="شماره‌بخش",
   ["ro"]="numarsectiune",
  },
  ["sectionresetset"]={
   ["en"]="sectionresetset",
   ["fr"]="jeurazsection",
  },
  ["sectionsegments"]={
   ["en"]="sectionsegments",
   ["fr"]="segmentssection",
  },
  ["sectionseparatorset"]={
   ["en"]="sectionseparatorset",
   ["fr"]="jeuseparateursection",
  },
  ["sectionset"]={
   ["en"]="sectionset",
   ["fr"]="jeusection",
  },
  ["sectionstarter"]={
   ["en"]="sectionstarter",
   ["fr"]="demarreursection",
  },
  ["sectionstopper"]={
   ["en"]="sectionstopper",
   ["fr"]="stoppeursection",
  },
  ["separator"]={
   ["cs"]="oddelovac",
   ["de"]="seperator",
   ["en"]="separator",
   ["fr"]="separateur",
   ["it"]="separatore",
   ["nl"]="scheider",
   ["pe"]="جداکننده",
   ["ro"]="separator",
  },
  ["separatorcolor"]={
   ["en"]="separatorcolor",
   ["fr"]="couleurseparateur",
   ["nl"]="scheiderkleur",
  },
  ["separatorstyle"]={
   ["en"]="separatorstyle",
   ["fr"]="styleseparateur",
   ["nl"]="scheiderletter",
  },
  ["set"]={
   ["cs"]="set",
   ["de"]="set",
   ["en"]="set",
   ["fr"]="regler",
   ["it"]="set",
   ["nl"]="set",
   ["pe"]="قراربده",
   ["ro"]="set",
  },
  ["setups"]={
   ["en"]="setups",
  },
  ["sheet"]={
   ["cs"]="sheet",
   ["de"]="sheet",
   ["en"]="sheet",
   ["fr"]="feuille",
   ["it"]="sheet",
   ["nl"]="sheet",
   ["pe"]="ورقه",
   ["ro"]="sheet",
  },
  ["shrink"]={
   ["en"]="shrink",
   ["fr"]="retrecir",
   ["nl"]="krimp",
  },
  ["side"]={
   ["cs"]="pocitat",
   ["de"]="objektabstand",
   ["en"]="side",
   ["fr"]="cote",
   ["it"]="lato",
   ["nl"]="zij",
   ["pe"]="کنار",
   ["ro"]="parte",
  },
  ["sidealign"]={
   ["cs"]="sidealign",
   ["de"]="sidealign",
   ["en"]="sidealign",
   ["fr"]="alignercote",
   ["it"]="sidealign",
   ["nl"]="zijuitlijnen",
   ["pe"]="تنظیم‌کنار",
   ["ro"]="sidealign",
  },
  ["sidemethod"]={
   ["cs"]="sidemethod",
   ["de"]="sidemethod",
   ["en"]="sidemethod",
   ["fr"]="methodecote",
   ["it"]="sidemethod",
   ["nl"]="zijmethode",
   ["pe"]="روش‌کنار",
   ["ro"]="sidemethod",
  },
  ["sidespaceafter"]={
   ["cs"]="bocnimezeraza",
   ["de"]="nebennachspatium",
   ["en"]="sidespaceafter",
   ["fr"]="espacelatéralaprès",
   ["it"]="spaziolateraledopo",
   ["nl"]="zijnawit",
   ["pe"]="فضای‌کناری‌بعد",
   ["ro"]="spatiulateraldupa",
  },
  ["sidespacebefore"]={
   ["cs"]="bocnimezerapred",
   ["de"]="nebenvorspatium",
   ["en"]="sidespacebefore",
   ["fr"]="espacelateralavant",
   ["it"]="spaziolateraleprima",
   ["nl"]="zijvoorwit",
   ["pe"]="فضای‌کناری‌قبل",
   ["ro"]="spatiulateralinainte",
  },
  ["sidespaceinbetween"]={
   ["en"]="sidespaceinbetween",
   ["fr"]="espacelateralentre",
   ["nl"]="zijtussenwit",
  },
  ["sidethreshold"]={
   ["en"]="sidethreshold",
   ["fr"]="seuilcote",
  },
  ["sign"]={
   ["cs"]="znak",
   ["de"]="zeichen",
   ["en"]="sign",
   ["fr"]="signe",
   ["it"]="segno",
   ["nl"]="teken",
   ["pe"]="علامت",
   ["ro"]="semn",
  },
  ["simplecommand"]={
   ["en"]="simplecommand",
   ["fr"]="commandesimple",
   ["nl"]="simpelcommando",
  },
  ["size"]={
   ["cs"]="velikost",
   ["de"]="groesse",
   ["en"]="size",
   ["fr"]="taille",
   ["it"]="dimensione",
   ["nl"]="formaat",
   ["pe"]="اندازه",
   ["ro"]="dimensiune",
  },
  ["slantedfeatures"]={
   ["en"]="slantedfeatures",
   ["fr"]="fonctionnalitesincline",
  },
  ["slantedfont"]={
   ["en"]="slantedfont",
   ["fr"]="policeinclinee",
  },
  ["small"]={
   ["cs"]="male",
   ["de"]="klein",
   ["en"]="small",
   ["fr"]="petit",
   ["it"]="piccolo",
   ["nl"]="klein",
   ["pe"]="کوچک",
   ["ro"]="mic",
  },
  ["smallcapsfeatures"]={
   ["en"]="smallcapsfeatures",
   ["fr"]="fonctionnalitespetitescaps",
  },
  ["smallcapsfont"]={
   ["en"]="smallcapsfont",
   ["fr"]="policepetitescaps",
  },
  ["snap"]={
   ["en"]="snap",
  },
  ["snapping"]={
    ["en"]="snapping",
  },
  ["snapstep"]={
   ["en"]="snapstep",
  },
  ["solution"]={
   ["en"]="solution",
   ["fr"]="solution",
   ["nl"]="oplossing",
  },
  ["sort"]={
   ["en"]="sort",
   ["fr"]="trier",
  },
  ["sorttype"]={
   ["cs"]="sorttype",
   ["de"]="sorttype",
   ["en"]="sorttype",
   ["fr"]="modetri",
   ["it"]="sorttype",
   ["nl"]="sortering",
   ["pe"]="ترتیب‌تایپ",
   ["ro"]="sorttype",
  },
  ["source"]={
   ["cs"]="zdroj",
   ["de"]="quelle",
   ["en"]="source",
   ["fr"]="source",
   ["it"]="origine",
   ["nl"]="bron",
   ["pe"]="منبع",
   ["ro"]="sursa",
  },
  ["space"]={
   ["cs"]="mezera",
   ["de"]="spatium",
   ["en"]="space",
   ["fr"]="espace",
   ["it"]="spazio",
   ["nl"]="spatie",
   ["pe"]="فضا",
   ["ro"]="spatiu",
  },
  ["spaceafter"]={
   ["cs"]="mezeraza",
   ["de"]="nachspatium",
   ["en"]="spaceafter",
   ["fr"]="espaceaprès",
   ["it"]="spaziodopo",
   ["nl"]="nawit",
   ["pe"]="فضا‌بعداز",
   ["ro"]="spatiudupa",
  },
  ["spaceafterside"]={
   ["en"]="spaceafterside",
   ["fr"]="espaceaprèscoté",
   ["nl"]="witnazij",
  },
  ["spacebefore"]={
   ["cs"]="mezerapred",
   ["de"]="vorspatium",
   ["en"]="spacebefore",
   ["fr"]="espaceavant",
   ["it"]="spazioprima",
   ["nl"]="voorwit",
   ["pe"]="فضا‌قبل‌از",
   ["ro"]="spatiuinainte",
  },
  ["spacebeforeside"]={
   ["en"]="spacebeforeside",
   ["fr"]="espaceavantcote",
   ["nl"]="witvoorzij",
  },
  ["spaceinbetween"]={
   ["cs"]="spaceinbetween",
   ["de"]="spaceinbetween",
   ["en"]="spaceinbetween",
   ["fr"]="espaceentre",
   ["it"]="spaceinbetween",
   ["nl"]="tussenwit",
   ["pe"]="فضا‌دربین",
   ["ro"]="spaceinbetween",
  },
  ["spacing"]={
   ["cs"]="mezerovani",
   ["de"]="spatiumausgleich",
   ["en"]="spacing",
   ["fr"]="espacement",
   ["it"]="spaziatura",
   ["nl"]="spatiering",
   ["pe"]="فضاگذاری",
   ["ro"]="spatiere",
  },
  ["specification"]={
   ["en"]="specification",
   ["fr"]="specification",
  },
  ["split"]={
   ["cs"]="split",
   ["de"]="split",
   ["en"]="split",
   ["fr"]="partager",
   ["it"]="separa",
   ["nl"]="splitsen",
   ["pe"]="شکافتن",
   ["ro"]="split",
  },
  ["splitcolor"]={
   ["cs"]="splitcolor",
   ["de"]="splitcolor",
   ["en"]="splitcolor",
   ["fr"]="couleurpartage",
   ["it"]="splitcolor",
   ["nl"]="splitskleur",
   ["pe"]="شکافتن‌رنگ",
   ["ro"]="splitcolor",
  },
  ["splitmethod"]={
   ["cs"]="splitmethod",
   ["de"]="splitmethod",
   ["en"]="splitmethod",
   ["fr"]="methodepartage",
   ["it"]="splitmethod",
   ["nl"]="splitsmethode",
   ["pe"]="روش‌شکافتن",
   ["ro"]="splitmethod",
  },
  ["splitoffset"]={
   ["cs"]="splitoffset",
   ["de"]="splitoffset",
   ["en"]="splitoffset",
   ["fr"]="decalagepartage",
   ["it"]="splitoffset",
   ["nl"]="splitsoffset",
   ["pe"]="شکافتن‌آفست",
   ["ro"]="splitoffset",
  },
  ["spot"]={
   ["cs"]="spot",
   ["de"]="spot",
   ["en"]="spot",
   ["fr"]="tondirect",
   ["it"]="spot",
   ["nl"]="spot",
   ["pe"]="لکه",
   ["ro"]="spot",
  },
  ["stack"]={
   ["cs"]="stack",
   ["de"]="stack",
   ["en"]="stack",
   ["fr"]="pile",
   ["it"]="stack",
   ["nl"]="stapel",
   ["pe"]="توده",
   ["ro"]="stack",
  },
  ["stacking"]={
   ["en"]="stacking",
   ["nl"]="stapels",
  },
  ["stackname"]={
   ["en"]="stackname",
   ["fr"]="nompile",
   ["nl"]="stapelnaam",
  },
  ["start"]={
   ["cs"]="start",
   ["de"]="start",
   ["en"]="start",
   ["fr"]="début",
   ["it"]="inizia",
   ["nl"]="start",
   ["pe"]="شروع",
   ["ro"]="start",
  },
  ["starter"]={
   ["en"]="starter",
   ["fr"]="demarreur",
   ["nl"]="opener",
  },
  ["state"]={
   ["cs"]="status",
   ["de"]="status",
   ["en"]="state",
   ["fr"]="état",
   ["it"]="stato",
   ["nl"]="status",
   ["pe"]="وضعیت",
   ["ro"]="stare",
  },
  ["step"]={
   ["cs"]="krok",
   ["de"]="schritt",
   ["en"]="step",
   ["fr"]="etape",
   ["it"]="passo",
   ["nl"]="stap",
   ["pe"]="گام",
   ["ro"]="pas",
  },
  ["stop"]={
   ["cs"]="stop",
   ["de"]="stop",
   ["en"]="stop",
   ["fr"]="fin",
   ["it"]="termina",
   ["nl"]="stop",
   ["pe"]="پایان",
   ["ro"]="stop",
  },
  ["stopper"]={
   ["cs"]="predel",
   ["de"]="abschnitttrenner",
   ["en"]="stopper",
   ["fr"]="stoppeur",
   ["it"]="stopper",
   ["nl"]="afsluiter",
   ["pe"]="ایست",
   ["ro"]="stopper",
  },
  ["stretch"]={
   ["cs"]="natahnout",
   ["de"]="strecken",
   ["en"]="stretch",
   ["fr"]="etirer",
   ["it"]="dilata",
   ["nl"]="rek",
   ["pe"]="کشیدن",
   ["ro"]="dilatat",
  },
  ["strip"]={
   ["en"]="strip",
   ["fr"]="bande",
  },
  ["strut"]={
   ["cs"]="strut",
   ["de"]="strut",
   ["en"]="strut",
   ["fr"]="entretoise",
   ["it"]="strut",
   ["nl"]="strut",
   ["pe"]="بست",
   ["ro"]="strut",
  },
  ["style"]={
   ["cs"]="pismeno",
   ["de"]="stil",
   ["en"]="style",
   ["fr"]="style",
   ["it"]="stile",
   ["nl"]="letter",
   ["pe"]="سبک",
   ["ro"]="stil",
  },
  ["stylealternative"]={
   ["en"]="stylealternative",
   ["fr"]="variantestyle",
   ["nl"]="stylevariant",
  },
  ["sub"]={
   ["cs"]="pod",
   ["de"]="unter",
   ["en"]="sub",
   ["fr"]="sous",
   ["it"]="sotto",
   ["nl"]="sub",
   ["pe"]="زیر",
   ["ro"]="sub",
  },
  ["subtitle"]={
   ["cs"]="podtitulek",
   ["de"]="untertitel",
   ["en"]="subtitle",
   ["fr"]="soustitre",
   ["it"]="sottotitolo",
   ["nl"]="subtitel",
   ["pe"]="زیرعنوان",
   ["ro"]="subtitlu",
  },
  ["suffix"]={
   ["cs"]="suffix",
   ["de"]="suffix",
   ["en"]="suffix",
   ["fr"]="suffixe",
   ["it"]="suffix",
   ["nl"]="suffix",
   ["pe"]="پسوند",
   ["ro"]="suffix",
  },
  ["suffixseparator"]={
   ["en"]="suffixseparator",
   ["fr"]="separateursuffixe",
   ["nl"]="suffixscheider",
  },
  ["suffixstopper"]={
   ["en"]="suffixstopper",
   ["fr"]="stoppeursuffixe",
   ["nl"]="suffixafsluiter",
  },
  ["surnamefirstnamesep"]={
   ["en"]="surnamefirstnamesep",
   ["fr"]="sepnomfamilleprenom",
  },
  ["surnameinitialsep"]={
   ["en"]="surnameinitialsep",
   ["fr"]="sepnomfamilleinitial",
  },
  ["surnamesep"]={
   ["en"]="surnamesep",
   ["fr"]="seprenom",
  },
  ["synchronize"]={
   ["en"]="synchronize",
  },
  ["svgstyle"]={
   ["en"]="svgstyle",
   ["fr"]="stylesvg",
  },
  ["sx"]={
   ["cs"]="sx",
   ["de"]="sx",
   ["en"]="sx",
   ["fr"]="sx",
   ["it"]="sx",
   ["nl"]="sx",
   ["pe"]="sx",
   ["ro"]="sx",
  },
  ["sy"]={
   ["cs"]="sy",
   ["de"]="sy",
   ["en"]="sy",
   ["fr"]="sy",
   ["it"]="sy",
   ["nl"]="sy",
   ["pe"]="sy",
   ["ro"]="sy",
  },
  ["symalign"]={
   ["cs"]="symzarovnani",
   ["de"]="symausrichtung",
   ["en"]="symalign",
   ["fr"]="alignersym",
   ["it"]="symallinea",
   ["nl"]="symuitlijnen",
   ["pe"]="تنظیم‌نماد",
   ["ro"]="symaliniere",
  },
  ["symbol"]={
   ["cs"]="symbol",
   ["de"]="symbol",
   ["en"]="symbol",
   ["fr"]="symbole",
   ["it"]="simbolo",
   ["nl"]="symbool",
   ["pe"]="نماد",
   ["ro"]="simbol",
  },
  ["symbolcolor"]={
   ["en"]="symbolcolor",
  },
  ["symbolcommand"]={
   ["en"]="symbolcommand",
   ["fr"]="commandesymbole",
  },
  ["symbolset"]={
   ["cs"]="sadasymbolu",
   ["de"]="symbolset",
   ["en"]="symbolset",
   ["fr"]="jeusymbole",
   ["it"]="setsimboli",
   ["nl"]="symboolset",
   ["pe"]="مجموعه‌نماد",
   ["ro"]="setsimbol",
  },
  ["symcolor"]={
   ["cs"]="barvasymbolu",
   ["de"]="symfarbe",
   ["en"]="symcolor",
   ["fr"]="couleursym",
   ["it"]="coloresimbolo",
   ["nl"]="symkleur",
   ["pe"]="رنگ‌نماد",
   ["ro"]="culoaresimbol",
  },
  ["symstyle"]={
   ["cs"]="stylsymboly",
   ["de"]="symstil",
   ["en"]="symstyle",
   ["fr"]="stylesym",
   ["it"]="stilesimbolo",
   ["nl"]="symletter",
   ["pe"]="سبک‌نماد",
   ["ro"]="stilsimbol",
  },
  ["synonym"]={
   ["cs"]="synonymum",
   ["de"]="synonym",
   ["en"]="synonym",
   ["fr"]="synonyme",
   ["it"]="sinonimo",
   ["nl"]="synoniem",
   ["pe"]="مترادف",
   ["ro"]="sinonim",
  },
  ["synonymcolor"]={
   ["cs"]="barvasynonyma",
   ["de"]="synonymfarbe",
   ["en"]="synonymcolor",
   ["fr"]="couleursynonyme",
   ["it"]="coloresinonimi",
   ["nl"]="synoniemkleur",
   ["pe"]="رنگ‌مترادف",
   ["ro"]="culoaresinonim",
  },
  ["synonymcommand"]={
   ["cs"]="synonymcommand",
   ["de"]="synonymbefehl",
   ["en"]="synonymcommand",
   ["fr"]="commandesynonyme",
   ["it"]="synonymcommand",
   ["nl"]="synoniemcommando",
   ["pe"]="synonymcommand",
   ["ro"]="synonymcommand",
  },
  ["synonymstyle"]={
   ["cs"]="stylsynonyma",
   ["de"]="synonymstil",
   ["en"]="synonymstyle",
   ["fr"]="stylesynonyme",
   ["it"]="stilesinonimi",
   ["nl"]="synoniemletter",
   ["pe"]="سبک‌مترادف",
   ["ro"]="stilsinonim",
  },
  ["tab"]={
   ["cs"]="tab",
   ["de"]="tab",
   ["en"]="tab",
   ["fr"]="tab",
   ["it"]="tab",
   ["nl"]="tab",
   ["pe"]="تب",
   ["ro"]="tab",
  },
  ["text"]={
   ["cs"]="text",
   ["de"]="text",
   ["en"]="text",
   ["fr"]="texte",
   ["it"]="testo",
   ["nl"]="tekst",
   ["pe"]="متن",
   ["ro"]="text",
  },
  ["textalign"]={
   ["en"]="textalign",
   ["fr"]="alignertexte",
  },
  ["textalternative"]={
   ["en"]="textalternative",
   ["nl"]="tekstvariant",
  },
  ["textcolor"]={
   ["cs"]="barvatextu",
   ["de"]="textfarbe",
   ["en"]="textcolor",
   ["fr"]="couleurtexte",
   ["it"]="coloretesto",
   ["nl"]="tekstkleur",
   ["pe"]="رنگ‌متن",
   ["ro"]="culoaretext",
  },
  ["textcommand"]={
   ["cs"]="textovyprikaz",
   ["de"]="textbefehl",
   ["en"]="textcommand",
   ["fr"]="commandetexte",
   ["it"]="comandotesto",
   ["nl"]="tekstcommando",
   ["pe"]="فرمان‌متن",
   ["ro"]="comandatext",
  },
  ["textdistance"]={
   ["cs"]="textdistance",
   ["de"]="textdistance",
   ["en"]="textdistance",
   ["fr"]="distancetexte",
   ["it"]="distanzatesto",
   ["nl"]="tekstafstand",
   ["pe"]="فاصله‌متن",
   ["ro"]="textdistance",
  },
  ["textheight"]={
   ["cs"]="vyskatextu",
   ["de"]="texthoehe",
   ["en"]="textheight",
   ["fr"]="hauteurtexte",
   ["it"]="altezzatesto",
   ["nl"]="teksthoogte",
   ["pe"]="ارتفاع‌متن",
   ["ro"]="inaltimetext",
  },
  ["textlayer"]={
   ["cs"]="textlayer",
   ["de"]="textlayer",
   ["en"]="textlayer",
   ["fr"]="calquetexte",
   ["it"]="textlayer",
   ["nl"]="tekstlaag",
   ["pe"]="لایه‌متن",
   ["ro"]="textlayer",
  },
  ["textmargin"]={
   ["cs"]="textmargin",
   ["de"]="textmargin",
   ["en"]="textmargin",
   ["fr"]="margetexte",
   ["it"]="textmargin",
   ["nl"]="tekstmarge",
   ["pe"]="حاشیه‌متن",
   ["ro"]="textmargin",
  },
  ["numbermethod"]={
   ["en"]="numbermethod",
   ["nl"]="nummermethode",
  },
  ["textmethod"]={
   ["cs"]="textmethod",
   ["de"]="textmethod",
   ["en"]="textmethod",
   ["fr"]="methodetexte",
   ["it"]="textmethod",
   ["nl"]="tekstmethode",
   ["pe"]="روش‌متن",
   ["ro"]="textmethod",
  },
  ["lasttextseparator"]={
    ["en"]="lasttextseparator",
    ["nl"]="laatstetekstscheider",
  },
  ["textseparator"]={
   ["cs"]="oddelovactextu",
   ["de"]="textseparator",
   ["en"]="textseparator",
   ["fr"]="separateurtexte",
   ["it"]="separatoretesto",
   ["nl"]="tekstscheider",
   ["pe"]="جداکننده‌متن",
   ["ro"]="separatortext",
  },
  ["textsize"]={
   ["cs"]="velikosttextu",
   ["de"]="textgroesse",
   ["en"]="textsize",
   ["fr"]="tailletexte",
   ["it"]="dimensionetesto",
   ["nl"]="tekstformaat",
   ["pe"]="اندازه‌متن",
   ["ro"]="dimensiunetext",
  },
  ["textstate"]={
   ["cs"]="statustextu",
   ["de"]="textstatus",
   ["en"]="textstate",
   ["fr"]="etattexte",
   ["it"]="statotesto",
   ["nl"]="tekststatus",
   ["pe"]="وضعیت‌متن",
   ["ro"]="staretext",
  },
  ["textstyle"]={
   ["cs"]="styltextu",
   ["de"]="textstil",
   ["en"]="textstyle",
   ["fr"]="styletexte",
   ["it"]="stiletesto",
   ["nl"]="tekstletter",
   ["pe"]="سبک‌متن",
   ["ro"]="stiltext",
  },
  ["textwidth"]={
   ["cs"]="sirkatextu",
   ["de"]="textbreite",
   ["en"]="textwidth",
   ["fr"]="largeurtexte",
   ["it"]="ampiezzatesto",
   ["nl"]="tekstbreedte",
   ["pe"]="عرض‌متن",
   ["ro"]="latimetext",
  },
  ["threshold"]={
   ["en"]="threshold",
   ["fr"]="seuil",
  },
  ["time"]={
   ["en"]="time",
   ["fr"]="temps",
   ["nl"]="tijd",
  },
  ["title"]={
   ["cs"]="titul",
   ["de"]="titel",
   ["en"]="title",
   ["fr"]="titre",
   ["it"]="titolo",
   ["nl"]="titel",
   ["pe"]="عنوان",
   ["ro"]="titlu",
  },
  ["titlecolor"]={
   ["cs"]="barvatitulek",
   ["de"]="titelfarbe",
   ["en"]="titlecolor",
   ["fr"]="couleurtitre",
   ["it"]="coloretitolo",
   ["nl"]="titelkleur",
   ["pe"]="رنگ‌عنوان",
   ["ro"]="culoaretitlu",
  },
  ["titlecommand"]={
   ["cs"]="titlecommand",
   ["de"]="titlecommand",
   ["en"]="titlecommand",
   ["fr"]="commandetitre",
   ["it"]="titlecommand",
   ["nl"]="titelcommando",
   ["pe"]="فرمان‌عنوان",
   ["ro"]="titlecommand",
  },
  ["titledistance"]={
   ["cs"]="vzdalenosttitulek",
   ["de"]="titelabstand",
   ["en"]="titledistance",
   ["fr"]="distancetitre",
   ["it"]="distanzatitolo",
   ["nl"]="titelafstand",
   ["pe"]="فاصله‌عنوان",
   ["ro"]="distantatitlu",
  },
  ["titleleft"]={
   ["cs"]="titleleft",
   ["de"]="titleleft",
   ["en"]="titleleft",
   ["fr"]="gauchetitre",
   ["it"]="titleleft",
   ["nl"]="titellinks",
   ["pe"]="عنوان‌چپ",
   ["ro"]="titleleft",
  },
  ["titleright"]={
   ["cs"]="titleright",
   ["de"]="titleright",
   ["en"]="titleright",
   ["fr"]="droitetitre",
   ["it"]="titleright",
   ["nl"]="titelrechts",
   ["pe"]="عنوان‌راست",
   ["ro"]="titleright",
  },
  ["titlestrut"]={
   ["en"]="titlestrut",
   ["fr"]="entretoisetitre",
   ["nl"]="titelstrut",
  },
  ["titlestyle"]={
   ["cs"]="styltitulek",
   ["de"]="titelstil",
   ["en"]="titlestyle",
   ["fr"]="styletitre",
   ["it"]="stiletitolo",
   ["nl"]="titelletter",
   ["pe"]="سبک‌عنوان",
   ["ro"]="stiltitlu",
  },
  ["to"]={
   ["cs"]="na",
   ["de"]="zu",
   ["en"]="to",
   ["fr"]="vers",
   ["it"]="verso",
   ["nl"]="aan",
   ["pe"]="به",
   ["ro"]="catre",
  },
  ["toffset"]={
   ["en"]="toffset",
   ["fr"]="decalaget",
  },
  ["tolerance"]={
   ["cs"]="tolerance",
   ["de"]="toleranz",
   ["en"]="tolerance",
   ["fr"]="tolerance",
   ["it"]="tolleranza",
   ["nl"]="tolerantie",
   ["pe"]="بردباری",
   ["ro"]="toleranta",
  },
  ["top"]={
   ["cs"]="vrsek",
   ["de"]="oben",
   ["en"]="top",
   ["fr"]="haut",
   ["it"]="cima",
   ["nl"]="boven",
   ["pe"]="بالا",
   ["ro"]="sus",
  },
  ["topcommand"]={
   ["en"]="topcommand",
   ["fr"]="commandehaut",
   ["nl"]="bovencommando",
  },
  ["topcolor"]={
   ["en"]="topcolor",
   ["nl"]="bovenkleur",
  },
  ["topdistance"]={
   ["cs"]="vzdalenostvrsku",
   ["de"]="obenabstand",
   ["en"]="topdistance",
   ["fr"]="distancehaut",
   ["it"]="distanzacima",
   ["nl"]="bovenafstand",
   ["pe"]="فاصله‌بالا",
   ["ro"]="distantasus",
  },
  ["topframe"]={
   ["cs"]="rameceknahore",
   ["de"]="obenrahmen",
   ["en"]="topframe",
   ["fr"]="cadrehaut",
   ["it"]="cornicecima",
   ["nl"]="bovenkader",
   ["pe"]="قالب‌راست",
   ["ro"]="framesus",
  },
  ["topoffset"]={
   ["cs"]="offsetvrsku",
   ["de"]="obenoffset",
   ["en"]="topoffset",
   ["fr"]="decalagehaut",
   ["it"]="offsetcima",
   ["nl"]="bovenoffset",
   ["pe"]="آفست‌بالا",
   ["ro"]="offsetsus",
  },
  ["topspace"]={
   ["cs"]="svrchnimezera",
   ["de"]="kopfspatium",
   ["en"]="topspace",
   ["fr"]="espacehaut",
   ["it"]="spaziocima",
   ["nl"]="kopwit",
   ["pe"]="فضای‌بالا",
   ["ro"]="spatiusus",
  },
  ["topstate"]={
   ["cs"]="statusvrsku",
   ["de"]="statusoben",
   ["en"]="topstate",
   ["fr"]="etathaut",
   ["it"]="statocima",
   ["nl"]="bovenstatus",
   ["pe"]="وضعیت‌بالا",
   ["ro"]="staresus",
  },
  ["topstyle"]={
   ["en"]="topstyle",
  },
  ["totalnumber"]={
   ["en"]="totalnumber",
   ["fr"]="nombretotal",
  },
  ["transform"]={
   ["en"]="transform",
   ["fr"]="transformer",
   ["nl"]="transformatie",
  },
  ["translate"]={
   ["en"]="translate",
   ["fr"]="traduire",
  },
  ["trimoffset"]={
   ["en"]="trimoffset",
   ["fr"]="decalagecoupe",
  },
  ["type"]={
   ["cs"]="typ",
   ["de"]="typ",
   ["en"]="type",
   ["fr"]="transcrire",
   ["it"]="type",
   ["nl"]="type",
   ["pe"]="تایپ",
   ["ro"]="type",
  },
  ["unit"]={
   ["cs"]="jednotka",
   ["de"]="einheit",
   ["en"]="unit",
   ["fr"]="unite",
   ["it"]="unita",
   ["nl"]="eenheid",
   ["pe"]="واحد",
   ["ro"]="unitate",
  },
  ["unknownreference"]={
   ["cs"]="neznamyodkaz",
   ["de"]="unbekantereferenz",
   ["en"]="unknownreference",
   ["fr"]="referenceinconnue",
   ["it"]="riferimentoingoto",
   ["nl"]="onbekendereferentie",
   ["pe"]="مرجع‌ناشناس",
   ["ro"]="referintanecunoscuta",
  },
  ["up"]={
   ["en"]="up",
   ["fr"]="enhaut",
  },
  ["urlalternative"]={
   ["cs"]="urlalternativa",
   ["de"]="urlalternative",
   ["en"]="urlalternative",
   ["fr"]="varianteurl",
   ["it"]="alternativaurl",
   ["nl"]="urlvariant",
   ["pe"]="urlalternative",
   ["ro"]="urlalternativ",
  },
  ["urlspace"]={
   ["cs"]="prostorurl",
   ["de"]="urlspatium",
   ["en"]="urlspace",
   ["fr"]="espaceurl",
   ["it"]="spaziourl",
   ["nl"]="urlspatie",
   ["pe"]="urlspace",
   ["ro"]="spatiuurl",
  },
  ["userpassword"]={
   ["en"]="userpassword",
   ["fr"]="motdepasseutilisateur",
  },
  ["validate"]={
   ["cs"]="validovat",
   ["de"]="validieren",
   ["en"]="validate",
   ["fr"]="valider",
   ["it"]="verifica",
   ["nl"]="valideer",
   ["pe"]="تاییداعتبار",
   ["ro"]="verifica",
  },
  ["values"]={
   ["en"]="values",
   ["fr"]="valeurs",
   ["nl"]="waarden",
  },
  ["vcommand"]={
   ["cs"]="vprikaz",
   ["de"]="vbefehl",
   ["en"]="vcommand",
   ["fr"]="commandev",
   ["it"]="vcomando",
   ["nl"]="vcommando",
   ["pe"]="vcommand",
   ["ro"]="comandav",
  },
  ["vcompact"]={
   ["en"]="vcompact",
   ["fr"]="compactv",
   ["nl"]="vcomprimeer",
  },
  ["vector"]={
   ["en"]="vector",
   ["fr"]="vecteur",
   ["nl"]="vector",
  },
  ["veroffset"]={
   ["cs"]="offsethlavicky",
   ["de"]="kopfoffset",
   ["en"]="veroffset",
   ["fr"]="decalagever",
   ["it"]="veroffset",
   ["nl"]="kopoffset",
   ["pe"]="آفست‌عم",
   ["ro"]="veroffset",
  },
  ["vertical"]={
   ["cs"]="vertikalne",
   ["de"]="vertikal",
   ["en"]="vertical",
   ["fr"]="vertical",
   ["it"]="verticale",
   ["nl"]="vertikaal",
   ["pe"]="عمودی",
   ["ro"]="vertical",
  },
  ["vfactor"]={
   ["cs"]="vfaktor",
   ["de"]="vfaktor",
   ["en"]="vfactor",
   ["fr"]="facteurv",
   ["it"]="vfactor",
   ["nl"]="vfactor",
   ["ro"]="vfactor",
  },
  ["vfil"]={
   ["cs"]="vfil",
   ["de"]="vfil",
   ["en"]="vfil",
   ["fr"]="remplissagev",
   ["it"]="vfil",
   ["nl"]="vfil",
   ["pe"]="vfil",
   ["ro"]="vfil",
  },
  ["viewerprefix"]={
   ["en"]="viewerprefix",
   ["fr"]="prefixevisualisateur",
  },
  ["voffset"]={
   ["cs"]="voffset",
   ["de"]="voffset",
   ["en"]="voffset",
   ["fr"]="decalagev",
   ["it"]="voffset",
   ["nl"]="voffset",
   ["pe"]="آفست‌ع",
   ["ro"]="voffset",
  },
  ["vonsep"]={
   ["en"]="vonsep",
   ["fr"]="sepvon",
  },
  ["way"]={
   ["cs"]="zpusob",
   ["de"]="art",
   ["en"]="way",
   ["fr"]="maniere",
   ["it"]="modo",
   ["nl"]="wijze",
   ["pe"]="راه",
   ["ro"]="mod",
  },
  ["wfactor"]={
   ["cs"]="sfaktor",
   ["de"]="bfaktor",
   ["en"]="wfactor",
   ["fr"]="facteurw",
   ["it"]="wfactor",
   ["nl"]="bfactor",
   ["pe"]="wfactor",
   ["ro"]="factorw",
  },
  ["white"]={
   ["cs"]="bily",
   ["de"]="weiss",
   ["en"]="white",
   ["fr"]="blanc",
   ["it"]="bianco",
   ["nl"]="wit",
   ["pe"]="سفید",
   ["ro"]="alb",
  },
  ["width"]={
   ["cs"]="sirka",
   ["de"]="breite",
   ["en"]="width",
   ["fr"]="largeur",
   ["it"]="ampiezza",
   ["nl"]="breedte",
   ["pe"]="عرض",
   ["ro"]="latime",
  },
  ["weight"]={
   ["en"]="weight",
  },
  ["words"]={
   ["en"]="words",
   ["fr"]="mots",
  },
  ["xanchor"]={
   ["en"]="xanchor",
   ["fr"]="ancrex",
   ["nl"]="xanker",
  },
  ["xfactor"]={
   ["cs"]="xfaktor",
   ["de"]="xfaktor",
   ["en"]="xfactor",
   ["fr"]="facteurx",
   ["it"]="xfactor",
   ["nl"]="xfactor",
   ["pe"]="فاکتورایکس",
   ["ro"]="xfactor",
  },
  ["xhtml"]={
   ["en"]="xhtml",
   ["fr"]="xhtml",
  },
  ["xmax"]={
   ["cs"]="xmax",
   ["de"]="xmax",
   ["en"]="xmax",
   ["fr"]="xmax",
   ["it"]="xmax",
   ["nl"]="xmax",
   ["pe"]="xmax",
   ["ro"]="xmax",
  },
  ["xmlsetup"]={
   ["en"]="xmlsetup",
   ["fr"]="configurationxml",
  },
  ["xoffset"]={
   ["cs"]="xoffset",
   ["de"]="xoffset",
   ["en"]="xoffset",
   ["fr"]="decalagex",
   ["it"]="xoffset",
   ["nl"]="xoffset",
   ["pe"]="آفست‌ایکس",
   ["ro"]="xoffset",
  },
  ["xscale"]={
   ["cs"]="xmeritko",
   ["de"]="xformat",
   ["en"]="xscale",
   ["fr"]="echellex",
   ["it"]="xscale",
   ["nl"]="xschaal",
   ["pe"]="مقیاس‌ایکس",
   ["ro"]="xscala",
  },
  ["xstep"]={
   ["cs"]="xkrok",
   ["de"]="xschritt",
   ["en"]="xstep",
   ["fr"]="etapex",
   ["it"]="xstep",
   ["nl"]="xstap",
   ["pe"]="گام‌ایکس",
   ["ro"]="xstep",
  },
  ["yanchor"]={
   ["en"]="yanchor",
   ["fr"]="ancrey",
   ["nl"]="yanker",
  },
  ["yfactor"]={
   ["cs"]="yfaktor",
   ["de"]="yfaktor",
   ["en"]="yfactor",
   ["fr"]="facteury",
   ["it"]="yfactor",
   ["nl"]="yfactor",
   ["pe"]="فاکتوروای",
   ["ro"]="yfactor",
  },
  ["ymax"]={
   ["cs"]="ymax",
   ["de"]="ymax",
   ["en"]="ymax",
   ["fr"]="ymax",
   ["it"]="ymax",
   ["nl"]="ymax",
   ["pe"]="ymax",
   ["ro"]="ymax",
  },
  ["yoffset"]={
   ["cs"]="yoffset",
   ["de"]="yoffset",
   ["en"]="yoffset",
   ["fr"]="decalagey",
   ["it"]="yoffset",
   ["nl"]="yoffset",
   ["pe"]="آفست‌وای",
   ["ro"]="yoffset",
  },
  ["yscale"]={
   ["cs"]="ymeritko",
   ["de"]="yformat",
   ["en"]="yscale",
   ["fr"]="echelley",
   ["it"]="yscale",
   ["nl"]="yschaal",
   ["pe"]="مقیاس‌وای",
   ["ro"]="yscala",
  },
  ["ystep"]={
   ["cs"]="ykrok",
   ["de"]="yschritt",
   ["en"]="ystep",
   ["fr"]="etapey",
   ["it"]="ystep",
   ["nl"]="ystap",
   ["pe"]="گام‌وای",
   ["ro"]="ystep",
  },
  ["leftsource"]={
   ["en"]="leftsource",
  },
  ["rightsource"]={
   ["en"]="rightsource",
  },
  ["middlesource"]={
   ["en"]="middlesource",
  },
  ["source"]={
    ["en"]="source",
  },
  ["target"]={
    ["en"]="target",
  },
 },
 ["elements"]={
  ["answerlines"]={
   ["cs"]="answerlines",
   ["de"]="answerlines",
   ["en"]="answerlines",
   ["fr"]="lignesreponse",
   ["it"]="answerlines",
   ["nl"]="antwoordregels",
   ["pe"]="answerlines",
   ["ro"]="answerlines",
  },
  ["answerspace"]={
   ["cs"]="answerspace",
   ["de"]="answerspace",
   ["en"]="answerspace",
   ["fr"]="espacereponse",
   ["it"]="answerspace",
   ["nl"]="antwoordruimte",
   ["pe"]="answerspace",
   ["ro"]="answerspace",
  },
  ["begin"]={
   ["cs"]="zacatek",
   ["de"]="anfang",
   ["en"]="begin",
   ["fr"]="commencer",
   ["it"]="inizio",
   ["nl"]="beginvan",
   ["pe"]="عنصرها",
   ["ro"]="inceput",
  },
  ["complete"]={
   ["cs"]="uplny",
   ["de"]="vollende",
   ["en"]="complete",
   ["fr"]="complet",
   ["it"]="completo",
   ["nl"]="volledige",
   ["pe"]="کامل",
   ["ro"]="complet",
  },
  ["coupled"]={
   ["cs"]="propojene",
   ["de"]="verknuepft",
   ["en"]="coupled",
   ["fr"]="couple",
   ["it"]="accoppiato",
   ["nl"]="gekoppelde",
   ["pe"]="مزدوج",
   ["ro"]="cuplat",
  },
  ["currentlocal"]={
   ["cs"]="aktualnelokalni",
   ["de"]="aktuelllokal",
   ["en"]="currentlocal",
   ["fr"]="localcourant",
   ["it"]="correntelocale",
   ["nl"]="huidigelokale",
   ["pe"]="موضعی‌جاری",
   ["ro"]="localcurent",
  },
  ["end"]={
   ["cs"]="konec",
   ["de"]="ende",
   ["en"]="end",
   ["fr"]="terminer",
   ["it"]="fine",
   ["nl"]="eindvan",
   ["pe"]="انتها",
   ["ro"]="sfarsit",
  },
  ["endsetup"]={
   ["cs"]="",
   ["de"]="ein",
   ["en"]="",
   ["fr"]="",
   ["it"]="",
   ["nl"]="in",
   ["pe"]="انتهای‌بارگذاری",
   ["ro"]="",
  },
  ["get"]={
   ["cs"]="ziskat",
   ["de"]="get",
   ["en"]="get",
   ["fr"]="obtenir",
   ["it"]="prendi",
   ["nl"]="haal",
   ["pe"]="بگیر",
   ["ro"]="get",
  },
  ["increment"]={
   ["cs"]="zvysit",
   ["de"]="erhoehen",
   ["en"]="increment",
   ["fr"]="incrementer",
   ["it"]="incrementa",
   ["nl"]="verhoog",
   ["pe"]="افزایش",
   ["ro"]="increment",
  },
  ["list"]={
   ["cs"]="seznam",
   ["de"]="auflisten",
   ["en"]="list",
   ["fr"]="liste",
   ["it"]="elenco",
   ["nl"]="lijst",
   ["pe"]="لیست",
   ["ro"]="lista",
  },
  ["listof"]={
   ["cs"]="seznam",
   ["de"]="auflistenvon",
   ["en"]="listof",
   ["fr"]="listede",
   ["it"]="elencodi",
   ["nl"]="lijstmet",
   ["pe"]="لیست‌از",
   ["ro"]="listade",
  },
  ["load"]={
   ["cs"]="nacist",
   ["de"]="laden",
   ["en"]="load",
   ["fr"]="charger",
   ["it"]="carica",
   ["nl"]="laad",
   ["pe"]="بارگذاشتن",
   ["ro"]="incarca",
  },
  ["local"]={
   ["cs"]="lokalne",
   ["de"]="lokal",
   ["en"]="local",
   ["fr"]="local",
   ["it"]="locale",
   ["nl"]="lokale",
   ["pe"]="موضعی",
   ["ro"]="local",
  },
  ["makeup"]={
   ["cs"]="zlom",
   ["de"]="umbruch",
   ["en"]="makeup",
   ["fr"]="composition",
   ["it"]="makeup",
   ["nl"]="opmaak",
   ["pe"]="آرایش",
   ["ro"]="makeup",
  },
  ["namedtyping"]={
   ["en"]="namedtyping",
  },
  ["namednotation"]={
   ["en"]="namednotation",
  },
  ["next"]={
   ["cs"]="dalsi",
   ["de"]="folgende",
   ["en"]="next",
   ["fr"]="suivant",
   ["it"]="successivo",
   ["nl"]="volgende",
   ["pe"]="بعدی",
   ["ro"]="urmatorul",
  },
  ["place"]={
   ["cs"]="umisti",
   ["de"]="platziere",
   ["en"]="place",
   ["fr"]="placer",
   ["it"]="metti",
   ["nl"]="plaats",
   ["pe"]="مکان",
   ["ro"]="pune",
  },
  ["previous"]={
   ["cs"]="predchozi",
   ["de"]="vorige",
   ["en"]="previous",
   ["fr"]="precedent",
   ["it"]="precedente",
   ["nl"]="vorige",
   ["pe"]="قبلی",
   ["ro"]="precedent",
  },
  ["previouslocal"]={
   ["cs"]="predchozilokalni",
   ["de"]="voriglokal",
   ["en"]="previouslocal",
   ["fr"]="localprecedent",
   ["it"]="precedentelocale",
   ["nl"]="vorigelokale",
   ["pe"]="موضع‌قبلی",
   ["ro"]="precedentlocal",
  },
  ["reserve"]={
   ["cs"]="rezervovat",
   ["de"]="reservieren",
   ["en"]="reserve",
   ["fr"]="reserver",
   ["it"]="riserva",
   ["nl"]="reserveer",
   ["pe"]="رزرو",
   ["ro"]="rezervat",
  },
  ["see"]={
   ["cs"]="viz",
   ["de"]="sieh",
   ["en"]="see",
   ["fr"]="voir",
   ["it"]="vedi",
   ["nl"]="zie",
   ["pe"]="ببینید",
   ["ro"]="vezi",
  },
  ["setup"]={
   ["cs"]="nastaveni",
   ["de"]="stelle",
   ["en"]="setup",
   ["fr"]="configurer",
   ["it"]="imposta",
   ["nl"]="stel",
   ["pe"]="بارگذاری",
   ["ro"]="setare",
  },
  ["start"]={
   ["cs"]="start",
   ["de"]="start",
   ["en"]="start",
   ["fr"]="début",
   ["it"]="inizia",
   ["nl"]="start",
   ["pe"]="شروع",
   ["ro"]="start",
  },
  ["stop"]={
   ["cs"]="stop",
   ["de"]="stop",
   ["en"]="stop",
   ["fr"]="fin",
   ["it"]="termina",
   ["nl"]="stop",
   ["pe"]="پایان",
   ["ro"]="stop",
  },
  ["text"]={
   ["cs"]="text",
   ["de"]="text",
   ["en"]="text",
   ["fr"]="texte",
   ["it"]="testo",
   ["nl"]="tekst",
   ["pe"]="متن",
   ["ro"]="text",
  },
  ["type"]={
   ["cs"]="opis",
   ["de"]="type",
   ["en"]="type",
   ["fr"]="transcrire",
   ["it"]="type",
   ["nl"]="type",
   ["pe"]="تایپ",
   ["ro"]="type",
  },
 },
 ["setupstrings"]={
  ["cd:angle-l"]={
   ["en"]="<<...>>",
  },
  ["cd:angle-s"]={
   ["en"]="<<...>>",
  },
  ["cd:apply-l"]={
   ["en"]="[..,..=>..,..]",
  },
  ["cd:apply-s"]={
   ["en"]="[..=>..]",
  },
  ["cd:assignment-l"]={
   ["en"]="[..,..=..,..]",
   ["lua"]="{ ..,..=..,.. } ",
  },
  ["cd:assignment-s"]={
   ["en"]="[..=..]",
   ["lua"]="{ ..=.. } ",
  },
  ["cd:assignmentbraces-l"]={
   ["en"]="{..,..=..,..}",
  },
  ["cd:assignmentbraces-s"]={
   ["en"]="{..=..}",
  },
  ["cd:assignmentbrackets-l"]={
   ["en"]="[..,..=..,..]",
   ["lua"]="{ ..,..=..,.. } ",
  },
  ["cd:assignmentbrackets-s"]={
   ["en"]="[..=..]",
   ["lua"]="{ ..=.. } ",
  },
  ["cd:assignmentparentheses-l"]={
   ["en"]="(..,..=..,..)",
  },
  ["cd:assignmentparentheses-s"]={
   ["en"]="(..=..)",
  },
  ["cd:braces-l"]={
   ["en"]="{...,...}",
   ["lua"]=".. ... ..",
  },
  ["cd:braces-s"]={
   ["en"]="{...}",
   ["lua"]="\"...\"",
  },
  ["cd:brackets-l"]={
   ["en"]="[...,...]",
   ["lua"]="{..., ...}",
  },
  ["cd:brackets-s"]={
   ["en"]="[...]",
   ["lua"]="{ ... }",
  },
  ["cd:buffer"]={
   ["cs"]="buffer",
   ["de"]="buffer",
   ["en"]="buffer",
   ["fr"]="tampon",
   ["it"]="buffer",
   ["nl"]="buffer",
   ["ro"]="buffer",
  },
  ["cd:case-l"]={
   ["en"]="\\case ",
  },
  ["cd:case-s"]={
   ["en"]="\\case ",
  },
  ["cd:category"]={
   ["cs"]="category",
   ["de"]="category",
   ["en"]="category",
   ["fr"]="categorie",
   ["it"]="category",
   ["nl"]="category",
   ["ro"]="category",
  },
  ["cd:character"]={
   ["cs"]="character",
   ["de"]="character",
   ["en"]="character",
   ["fr"]="caractere",
   ["it"]="character",
   ["nl"]="karakter",
   ["ro"]="character",
  },
  ["cd:color"]={
   ["cs"]="color",
   ["de"]="color",
   ["en"]="color",
   ["fr"]="couleur",
   ["it"]="color",
   ["nl"]="kleur",
   ["ro"]="color",
  },
  ["cd:column"]={
   ["cs"]="column",
   ["de"]="column",
   ["en"]="column",
   ["fr"]="colonne",
   ["it"]="column",
   ["nl"]="kolom",
   ["ro"]="column",
  },
  ["cd:command"]={
   ["cs"]="Befehl",
   ["de"]="Befehl",
   ["en"]="command",
   ["fr"]="commande",
   ["it"]="Befehl",
   ["nl"]="commando",
   ["ro"]="Befehl",
  },
  ["cd:content"]={
   ["cs"]="content",
   ["de"]="content",
   ["en"]="content",
   ["fr"]="contenu",
   ["it"]="content",
   ["nl"]="inhoud",
   ["ro"]="content",
  },
  ["cd:content-l"]={
   ["en"]="{.. ... ..}",
   ["lua"]="\".. ... ..\"",
  },
  ["cd:content-s"]={
   ["en"]="{...}",
   ["lua"]="\"...\"",
  },
  ["cd:csname"]={
   ["cs"]="csname",
   ["de"]="csname",
   ["en"]="csname",
   ["fr"]="nomsc",
   ["it"]="csname",
   ["nl"]="naam",
   ["ro"]="csname",
  },
  ["cd:csname-l"]={
   ["en"]="\\...",
  },
  ["cd:csname-s"]={
   ["en"]="\\...",
  },
  ["cd:delimiter"]={
   ["cs"]="delimiter",
   ["de"]="delimiter",
   ["en"]="delimiter",
   ["fr"]="delimiteur",
   ["it"]="delimiter",
   ["nl"]="delimiter",
   ["ro"]="delimiter",
  },
  ["cd:destination"]={
   ["cs"]="destination",
   ["de"]="destination",
   ["en"]="destination",
   ["fr"]="destination",
   ["it"]="destination",
   ["nl"]="bestemming",
   ["ro"]="destination",
  },
  ["cd:destination-l"]={
   ["en"]="[..,{..[ref,..]},..]",
  },
  ["cd:destination-s"]={
   ["en"]="[{..[ref]}]",
  },
  ["cd:dimension"]={
   ["cs"]="Dimension",
   ["de"]="Dimension",
   ["en"]="dimension",
   ["fr"]="dimension",
   ["it"]="Dimension",
   ["nl"]="maat",
   ["ro"]="Dimension",
  },
  ["cd:displaymath"]={
   ["cs"]="formula",
   ["de"]="formula",
   ["en"]="formula",
   ["fr"]="affichemath",
   ["it"]="formula",
   ["nl"]="formule",
   ["ro"]="formula",
  },
  ["cd:displaymath-l"]={
   ["en"]="$$...$$",
  },
  ["cd:displaymath-s"]={
   ["en"]="$$...$$",
  },
  ["cd:else-l"]={
   ["en"]="\\else ",
  },
  ["cd:else-s"]={
   ["en"]="\\else ",
  },
  ["cd:false"]={
   ["cs"]="false",
   ["de"]="false",
   ["en"]="false",
   ["fr"]="faux",
   ["it"]="false",
   ["nl"]="false",
   ["ro"]="false",
  },
  ["cd:fi-l"]={
   ["en"]="\\fi ",
  },
  ["cd:fi-s"]={
   ["en"]="\\fi ",
  },
  ["cd:file"]={
   ["cs"]="file",
   ["de"]="file",
   ["en"]="file",
   ["fr"]="fichier",
   ["it"]="file",
   ["nl"]="file",
   ["ro"]="file",
  },
  ["cd:file-l"]={
   ["en"]=" ... ",
  },
  ["cd:file-s"]={
   ["en"]=" ... ",
  },
  ["cd:first"]={
   ["cs"]="first",
   ["de"]="first",
   ["en"]="first",
   ["fr"]="premier",
   ["it"]="first",
   ["nl"]="eerste",
   ["ro"]="first",
  },
  ["cd:font"]={
   ["cs"]="font",
   ["de"]="font",
   ["en"]="font",
   ["fr"]="police",
   ["it"]="font",
   ["nl"]="font",
   ["ro"]="font",
  },
  ["cd:formula"]={
   ["cs"]="Formel",
   ["de"]="Formel",
   ["en"]="formula",
   ["fr"]="formule",
   ["it"]="Formel",
   ["nl"]="formule",
   ["ro"]="Formel",
  },
  ["cd:identifier"]={
   ["cs"]="Name",
   ["de"]="Name",
   ["en"]="identifier",
   ["fr"]="identifiant",
   ["it"]="Name",
   ["nl"]="naam",
   ["ro"]="Name",
  },
  ["cd:if-l"]={
   ["en"]="\\if... ",
  },
  ["cd:if-s"]={
   ["en"]="\\if... ",
  },
  ["cd:index"]={
   ["cs"]="entry",
   ["de"]="entry",
   ["en"]="entry",
   ["fr"]="index",
   ["it"]="entry",
   ["nl"]="ingang",
   ["ro"]="entry",
  },
  ["cd:index-l"]={
   ["en"]="{..+...+..}",
  },
  ["cd:index-s"]={
   ["en"]="{...}",
  },
  ["cd:inherits"]={
   ["cs"]="inherits",
   ["de"]="inherits",
   ["en"]="inherits",
   ["fr"]="heritede",
   ["it"]="inherits",
   ["nl"]="erft",
   ["ro"]="inherits",
  },
  ["cd:inlinemath-l"]={
   ["en"]="$...$",
  },
  ["cd:inlinemath-s"]={
   ["en"]="$...$",
  },
  ["cd:key"]={
   ["cs"]="key",
   ["de"]="key",
   ["en"]="key",
   ["fr"]="cle",
   ["it"]="key",
   ["nl"]="parameter",
   ["ro"]="key",
  },
  ["cd:language"]={
   ["cs"]="sprache",
   ["de"]="sprache",
   ["en"]="language",
   ["fr"]="langage",
   ["it"]="sprache",
   ["nl"]="taal",
   ["ro"]="sprache",
  },
  ["cd:last"]={
   ["cs"]="last",
   ["de"]="last",
   ["en"]="last",
   ["fr"]="dernier",
   ["it"]="last",
   ["nl"]="laatste",
   ["ro"]="last",
  },
  ["cd:list"]={
   ["cs"]="Liste",
   ["de"]="Liste",
   ["en"]="list",
   ["fr"]="liste",
   ["it"]="Liste",
   ["nl"]="lijst",
   ["ro"]="Liste",
  },
  ["cd:lpath"]={
   ["cs"]="lpath",
   ["de"]="lpath",
   ["en"]="lpath",
   ["fr"]="cheminl",
   ["it"]="lpath",
   ["nl"]="lpath",
   ["ro"]="lpath",
  },
  ["cd:luafunction"]={
   ["cs"]="luafunction",
   ["de"]="luafunction",
   ["en"]="luafunction",
   ["fr"]="fonctionlua",
   ["it"]="luafunction",
   ["nl"]="luafunction",
   ["ro"]="luafunction",
  },
  ["cd:macro-l"]={
   ["en"]="\\... ",
  },
  ["cd:macro-s"]={
   ["en"]="\\... ",
  },
  ["cd:mark"]={
   ["cs"]="Beschriftung",
   ["de"]="Beschriftung",
   ["en"]="mark",
   ["fr"]="marque",
   ["it"]="Beschriftung",
   ["nl"]="markering",
   ["ro"]="Beschriftung",
  },
  ["cd:marking"]={
   ["cs"]="marking",
   ["de"]="marking",
   ["en"]="marking",
   ["fr"]="marquage",
   ["it"]="marking",
   ["nl"]="markering",
   ["ro"]="marking",
  },
  ["cd:math"]={
   ["cs"]="formula",
   ["de"]="formula",
   ["en"]="formula",
   ["fr"]="math",
   ["it"]="formula",
   ["nl"]="formule",
   ["ro"]="formula",
  },
  ["cd:math-l"]={
   ["en"]="$...$",
  },
  ["cd:math-s"]={
   ["en"]="$...$",
  },
  ["cd:matrix"]={
   ["cs"]="n*m",
   ["de"]="n*m",
   ["en"]="n*m",
   ["fr"]="matrice",
   ["it"]="n*m",
   ["nl"]="n*m",
   ["ro"]="n*m",
  },
  ["cd:name"]={
   ["cs"]="Name",
   ["de"]="Name",
   ["en"]="name",
   ["fr"]="nom",
   ["it"]="Name",
   ["nl"]="naam",
   ["ro"]="Name",
  },
  ["cd:noargument-l"]={
   ["en"]="\\...",
  },
  ["cd:noargument-s"]={
   ["en"]="\\...",
  },
  ["cd:node"]={
   ["cs"]="node",
   ["de"]="node",
   ["en"]="node",
   ["fr"]="noeud",
   ["it"]="node",
   ["nl"]="node",
   ["ro"]="node",
  },
  ["cd:none-l"]={
   ["en"]=".. ... ..",
  },
  ["cd:none-s"]={
   ["en"]="...",
  },
  ["cd:nothing"]={
   ["cs"]="empty",
   ["de"]="empty",
   ["en"]="empty",
   ["fr"]="rien",
   ["it"]="empty",
   ["nl"]="leeg",
   ["ro"]="empty",
  },
  ["cd:nothing-l"]={
   ["en"]="...",
  },
  ["cd:nothing-s"]={
   ["en"]="...",
  },
  ["cd:number"]={
   ["cs"]="Nummer",
   ["de"]="Nummer",
   ["en"]="number",
   ["fr"]="nombre",
   ["it"]="Nummer",
   ["nl"]="getal",
   ["ro"]="Nummer",
  },
  ["cd:oneargument"]={
   ["en"]="\\...#1",
  },
  ["cd:optional"]={
   ["cs"]="opt",
   ["de"]="opt",
   ["en"]="opt",
   ["fr"]="optionnel",
   ["it"]="opt",
   ["nl"]="opt",
   ["ro"]="opt",
  },
  ["cd:or-l"]={
   ["en"]="\\or ",
  },
  ["cd:or-s"]={
   ["en"]="\\or ",
  },
  ["cd:par-l"]={
   ["en"]="\\par ",
  },
  ["cd:par-s"]={
   ["en"]="\\par ",
  },
  ["cd:parenthesis-l"]={
   ["en"]="(...,...)",
  },
  ["cd:parenthesis-s"]={
   ["en"]="(...)",
  },
  ["cd:path"]={
   ["cs"]="path",
   ["de"]="path",
   ["en"]="path",
   ["fr"]="chemin",
   ["it"]="path",
   ["nl"]="path",
   ["ro"]="path",
  },
  ["cd:plural"]={
   ["cs"]="plural",
   ["de"]="plural",
   ["en"]="plural",
   ["fr"]="pluriel",
   ["it"]="plural",
   ["nl"]="naam",
   ["ro"]="plural",
  },
  ["cd:position"]={
   ["cs"]="position",
   ["de"]="position",
   ["en"]="position",
   ["fr"]="position",
   ["it"]="position",
   ["nl"]="positie",
   ["ro"]="position",
  },
  ["cd:position-l"]={
   ["en"]="(...,...)",
  },
  ["cd:position-s"]={
   ["en"]="(...)",
  },
  ["cd:processor"]={
   ["cs"]="processor",
   ["de"]="processor",
   ["en"]="processor",
   ["fr"]="processeur",
   ["it"]="processor",
   ["nl"]="verwerker",
   ["ro"]="processor",
  },
  ["cd:reference"]={
   ["cs"]="reference",
   ["de"]="reference",
   ["en"]="reference",
   ["fr"]="reference",
   ["it"]="reference",
   ["nl"]="verwijzing",
   ["ro"]="reference",
  },
  ["cd:reference-l"]={
   ["en"]="[...,...]",
   ["lua"]="{..., ...}",
  },
  ["cd:reference-s"]={
   ["en"]="[...]",
   ["lua"]="{ ... }",
  },
  ["cd:row"]={
   ["cs"]="row",
   ["de"]="row",
   ["en"]="row",
   ["fr"]="rang",
   ["it"]="row",
   ["nl"]="rij",
   ["ro"]="row",
  },
  ["cd:section"]={
   ["cs"]="Abschnitt",
   ["de"]="Abschnitt",
   ["en"]="section",
   ["fr"]="section",
   ["it"]="Abschnitt",
   ["nl"]="sectie",
   ["ro"]="Abschnitt",
  },
  ["cd:sectionblock"]={
   ["cs"]="sectionblock",
   ["de"]="sectionblock",
   ["en"]="sectionblock",
   ["fr"]="blocsection",
   ["it"]="sectionblock",
   ["nl"]="sectieblok",
   ["ro"]="sectionblock",
  },
  ["cd:see"]={
   ["cs"]="siehe",
   ["de"]="siehe",
   ["en"]="see",
   ["fr"]="voir",
   ["it"]="siehe",
   ["nl"]="zie",
   ["ro"]="siehe",
  },
  ["cd:sep-l"]={
   ["en"]="\\",
  },
  ["cd:sep-s"]={
   ["en"]="\\",
  },
  ["cd:setup"]={
   ["cs"]="setup",
   ["de"]="setup",
   ["en"]="setup",
   ["fr"]="configurer",
   ["it"]="setup",
   ["nl"]="setup",
   ["ro"]="setup",
  },
  ["cd:sign"]={
   ["en"]="[-+]",
  },
  ["cd:singular"]={
   ["cs"]="singular",
   ["de"]="singular",
   ["en"]="singular",
   ["fr"]="singulier",
   ["it"]="singular",
   ["nl"]="naam",
   ["ro"]="singular",
  },
  ["cd:style"]={
   ["cs"]="style",
   ["de"]="style",
   ["en"]="style",
   ["fr"]="style",
   ["it"]="style",
   ["nl"]="letter",
   ["ro"]="style",
  },
  ["cd:template"]={
   ["cs"]="template",
   ["de"]="template",
   ["en"]="template",
   ["fr"]="gabarit",
   ["it"]="template",
   ["nl"]="sjabloon",
   ["ro"]="template",
  },
  ["cd:template-l"]={
   ["en"]="[|...|...|]",
  },
  ["cd:template-s"]={
   ["en"]="[|...|]",
  },
  ["cd:text"]={
   ["cs"]="Text",
   ["de"]="Text",
   ["en"]="text",
   ["fr"]="texte",
   ["it"]="Text",
   ["nl"]="tekst",
   ["ro"]="Text",
  },
  ["cd:textual-l"]={
   ["en"]=".. ... ..",
  },
  ["cd:textual-s"]={
   ["en"]="...",
  },
  ["cd:threearguments"]={
   ["en"]="\\...#1#2#3",
  },
  ["cd:threewords-l"]={
   ["en"]="[..+..+..]",
  },
  ["cd:threewords-s"]={
   ["en"]="[..+..+..]",
  },
  ["cd:title"]={
   ["cs"]="Setup",
   ["de"]="Setup",
   ["en"]="setup",
   ["fr"]="titre",
   ["it"]="Setup",
   ["nl"]="setup",
   ["ro"]="Setup",
  },
  ["cd:to-l"]={
   ["en"]="\\to ",
  },
  ["cd:to-s"]={
   ["en"]="\\to ",
  },
  ["cd:triplet"]={
   ["cs"]="triplet",
   ["de"]="triplet",
   ["en"]="triplet",
   ["fr"]="triplet",
   ["it"]="triplet",
   ["nl"]="triplet",
   ["ro"]="triplet",
  },
  ["cd:triplet-l"]={
   ["en"]="[x:y:z,..]",
  },
  ["cd:triplet-s"]={
   ["en"]="[x:y:z]",
  },
  ["cd:true"]={
   ["cs"]="true",
   ["de"]="true",
   ["en"]="true",
   ["fr"]="vrai",
   ["it"]="true",
   ["nl"]="true",
   ["ro"]="true",
  },
  ["cd:twoarguments"]={
   ["en"]="\\...#1#2",
  },
  ["cd:twowords-l"]={
   ["en"]="[..+..]",
  },
  ["cd:twowords-s"]={
   ["en"]="[..+..]",
  },
  ["cd:url"]={
   ["cs"]="url",
   ["de"]="url",
   ["en"]="url",
   ["fr"]="url",
   ["it"]="url",
   ["nl"]="url",
   ["ro"]="url",
  },
  ["cd:userdata"]={
   ["cs"]="userdata",
   ["de"]="userdata",
   ["en"]="userdata",
   ["fr"]="donneesutilisateur",
   ["it"]="userdata",
   ["nl"]="gebruikersdata",
   ["ro"]="userdata",
  },
  ["cd:value"]={
   ["cs"]="value",
   ["de"]="value",
   ["en"]="value",
   ["fr"]="valeur",
   ["it"]="value",
   ["nl"]="waarde",
   ["ro"]="value",
  },
  ["cd:word"]={
   ["cs"]="word",
   ["de"]="word",
   ["en"]="word",
   ["fr"]="mot",
   ["it"]="word",
   ["nl"]="woord",
   ["ro"]="word",
  },
  ["cd:word-l"]={
   ["en"]="{.. ... ..}",
   ["lua"]="\".. ... ..\"",
  },
  ["cd:word-s"]={
   ["en"]="{...}",
   ["lua"]="\"...\"",
  },
  ["cd:xmlsetup"]={
   ["cs"]="xmlsetup",
   ["de"]="xmlsetup",
   ["en"]="xmlsetup",
   ["fr"]="configurationxml",
   ["it"]="xmlsetup",
   ["nl"]="xmlsetup",
   ["ro"]="xmlsetup",
  },
 },
 ["variables"]={
  ["Addition"]={
   ["cs"]="Pridavek",
   ["de"]="Zusatz",
   ["en"]="Addition",
   ["fr"]="Ajout",
   ["it"]="Aggiunta",
   ["nl"]="Toevoeging",
   ["pe"]="جمع",
   ["ro"]="Adaugare",
  },
  ["Alphabetic"]={
   ["en"]="Alphabetic",
   ["fr"]="Alphabetique",
  },
  ["Balloon"]={
   ["cs"]="Bublinka",
   ["de"]="Ballon",
   ["en"]="Balloon",
   ["fr"]="Ballon",
   ["it"]="Pallone",
   ["nl"]="Ballon",
   ["pe"]="بادکنک",
   ["ro"]="Balon",
  },
  ["Cap"]={
   ["cs"]="Kap",
   ["de"]="Kap",
   ["en"]="Cap",
   ["fr"]="Cap",
   ["it"]="Cap",
   ["nl"]="Kap",
   ["pe"]="Cap",
   ["ro"]="Cap",
  },
  ["Capital"]={
   ["cs"]="Kapitalky",
   ["de"]="Versalien",
   ["en"]="Capital",
   ["fr"]="Capitales",
   ["it"]="Maiuscolo",
   ["nl"]="Kapitaal",
   ["pe"]="Capital",
   ["ro"]="Majuscula",
  },
  ["Character"]={
   ["cs"]="Pismeno",
   ["de"]="Buchstabe",
   ["en"]="Character",
   ["fr"]="Caractere",
   ["it"]="Lettera",
   ["nl"]="Letter",
   ["pe"]="Character",
   ["ro"]="Caracter",
  },
  ["Characters"]={
   ["cs"]="Pismena",
   ["de"]="Buchstaben",
   ["en"]="Characters",
   ["fr"]="Caracteres",
   ["it"]="Lettere",
   ["nl"]="Letters",
   ["pe"]="Characters",
   ["ro"]="Caractere",
  },
  ["CloseDocument"]={
   ["cs"]="ZavritDokument",
   ["de"]="SchliesseDokument",
   ["en"]="CloseDocument",
   ["fr"]="FermerDocument",
   ["it"]="ChiudiDocumento",
   ["nl"]="SluitDocument",
   ["pe"]="بستن‌نوشتار",
   ["ro"]="InchideDocument",
  },
  ["ExitViewer"]={
   ["cs"]="UkoncitProhlizec",
   ["de"]="BeendeViewer",
   ["en"]="ExitViewer",
   ["fr"]="SortirVisualisateur",
   ["it"]="EsciVisualizzatore",
   ["nl"]="VerlaatViewer",
   ["pe"]="خروج‌نمایشگر",
   ["ro"]="IesireVizualizator",
  },
  ["FirstPage"]={
   ["cs"]="PrvniStrana",
   ["de"]="ErsteSeite",
   ["en"]="FirstPage",
   ["fr"]="PremierePage",
   ["it"]="PrimaPagina",
   ["nl"]="EerstePagina",
   ["pe"]="صفحه‌اول",
   ["ro"]="PrimaPagina",
  },
  ["FitHeight"]={
   ["cs"]="FitHeight",
   ["de"]="HoehePassend",
   ["en"]="FitHeight",
   ["fr"]="AdapterHauteur",
   ["it"]="FitHeight",
   ["nl"]="HoogtePassend",
   ["pe"]="پرکردن‌ارتفاع",
   ["ro"]="AjustatInaltime",
  },
  ["FitWidth"]={
   ["cs"]="FitWidth",
   ["de"]="BreitePassend",
   ["en"]="FitWidth",
   ["fr"]="AdapterLargeur",
   ["it"]="FitWidth",
   ["nl"]="BreedtePassend",
   ["pe"]="پرکردن‌عرض",
   ["ro"]="AjustatLatime",
  },
  ["GotoPage"]={
   ["cs"]="JdiNaStranku",
   ["de"]="GotoPage",
   ["en"]="GotoPage",
   ["fr"]="AlleraPage",
   ["it"]="VaiAPagina",
   ["nl"]="GaNaarPagina",
   ["pe"]="برو‌به‌صفحه",
   ["ro"]="LaPagina",
  },
  ["Greek"]={
   ["cs"]="Recky",
   ["de"]="Griechisch",
   ["en"]="Greek",
   ["fr"]="Grec",
   ["it"]="Greco",
   ["nl"]="Grieks",
   ["pe"]="یونانی",
   ["ro"]="Grec",
  },
  ["Help"]={
   ["cs"]="Napoveda",
   ["de"]="Hilfe",
   ["en"]="Help",
   ["fr"]="Aide",
   ["it"]="Aiuto",
   ["nl"]="Help",
   ["pe"]="کمک",
   ["ro"]="Ajutor",
  },
  ["HideField"]={
   ["cs"]="SkryjPole",
   ["de"]="VerbergeFeld",
   ["en"]="HideField",
   ["fr"]="CacherChamps",
   ["it"]="NascondiCampo",
   ["nl"]="VerbergVeld",
   ["pe"]="پنهان‌میدان",
   ["ro"]="AscundeCamp",
  },
  ["HideLayer"]={
   ["cs"]="HideLayer",
   ["de"]="HideLayer",
   ["en"]="HideLayer",
   ["fr"]="CacherCalque",
   ["it"]="HideLayer",
   ["nl"]="VerbergLaag",
   ["pe"]="پنهان‌لایه",
   ["ro"]="HideLayer",
  },
  ["Key"]={
   ["cs"]="Klavesa",
   ["de"]="Schluessel",
   ["en"]="Key",
   ["fr"]="Cle",
   ["it"]="Tasto",
   ["nl"]="Sleutel",
   ["pe"]="کلید",
   ["ro"]="Cheie",
  },
  ["LastPage"]={
   ["cs"]="PosledniStrana",
   ["de"]="LetzteSeite",
   ["en"]="LastPage",
   ["fr"]="DernierePage",
   ["it"]="UltimaPagina",
   ["nl"]="LaatstePagina",
   ["pe"]="صفحه‌آخر",
   ["ro"]="UltimaPagina",
  },
  ["LoadForm"]={
   ["cs"]="NacistFormular",
   ["de"]="LadeFormular",
   ["en"]="LoadForm",
   ["fr"]="ChargerFormulaire",
   ["it"]="CaricaForm",
   ["nl"]="LaadFormulier",
   ["pe"]="بارگذاشتن‌فرم",
   ["ro"]="IncarcaFormular",
  },
  ["MONTH"]={
   ["cs"]="MESIC",
   ["de"]="MONAT",
   ["en"]="MONTH",
   ["fr"]="MOIS",
   ["it"]="MESE",
   ["nl"]="MAAND",
   ["pe"]="ماه",
   ["ro"]="LUNA",
  },
  ["New"]={
   ["cs"]="Novy",
   ["de"]="Neu",
   ["en"]="New",
   ["fr"]="Nouveau",
   ["it"]="Nuovo",
   ["nl"]="Nieuw",
   ["pe"]="جدید",
   ["ro"]="Nou",
  },
  ["NextJump"]={
   ["cs"]="NasledujiciSkok",
   ["de"]="NaechsterSprung",
   ["en"]="NextJump",
   ["fr"]="SautSuivant",
   ["it"]="SaltoSuccessivo",
   ["nl"]="VolgendeSprong",
   ["pe"]="پرش‌بعدی",
   ["ro"]="SaltUrmator",
  },
  ["NextPage"]={
   ["cs"]="DalsiStrana",
   ["de"]="FolgendeSeite",
   ["en"]="NextPage",
   ["fr"]="PageSuivante",
   ["it"]="PaginaSuccessiva",
   ["nl"]="VolgendePagina",
   ["pe"]="صفحه‌بعدی",
   ["ro"]="PaginaUrmatoare",
  },
  ["Numbers"]={
   ["cs"]="Cisla",
   ["de"]="Ziffern",
   ["en"]="Numbers",
   ["fr"]="Numeros",
   ["it"]="Numeri",
   ["nl"]="Cijfers",
   ["pe"]="شماره‌ها",
   ["ro"]="Numere",
  },
  ["OpenNamedDocument"]={
   ["cs"]="OpenNamedDocument",
   ["de"]="OpenNamedDocument",
   ["en"]="OpenNamedDocument",
   ["fr"]="OuvrirDocumentNomme",
   ["it"]="OpenNamedDocument",
   ["nl"]="OpenDocumentOpNaam",
   ["pe"]="بازکردن‌نوشتار‌ذخیره‌شده",
   ["ro"]="OpenNamedDocument",
  },
  ["Ordinal"]={
   ["en"]="Ordinal",
   ["fr"]="Ordinal",
  },
  ["Paragraph"]={
   ["cs"]="Odstavec",
   ["de"]="Absatz",
   ["en"]="Paragraph",
   ["fr"]="Paragraphe",
   ["it"]="Capoverso",
   ["nl"]="Paragraaf",
   ["pe"]="پاراگراف",
   ["ro"]="Paragraf",
  },
  ["PauseMovie"]={
   ["cs"]="PozastavitFilm",
   ["de"]="PauseFilm",
   ["en"]="PauseMovie",
   ["fr"]="PauseFilm",
   ["it"]="PausaFilm",
   ["nl"]="PauzeerFilm",
   ["pe"]="ایست‌فیلم",
   ["ro"]="PauzaFilm",
  },
  ["PauseRendering"]={
   ["cs"]="PauseRendering",
   ["de"]="PauseRendering",
   ["en"]="PauseRendering",
   ["fr"]="PauseRendu",
   ["it"]="PauseRendering",
   ["nl"]="PauzeerRendering",
   ["pe"]="توقف‌تعبیر",
   ["ro"]="PauseRendering",
  },
  ["PauseSound"]={
   ["cs"]="PozastavitZvuk",
   ["de"]="PauseSound",
   ["en"]="PauseSound",
   ["fr"]="PauseSon",
   ["it"]="PausaSuono",
   ["nl"]="PauzeerGeluid",
   ["pe"]="ایست‌صدا",
   ["ro"]="PauzaSunet",
  },
  ["PreviousJump"]={
   ["cs"]="PredchoziSkok",
   ["de"]="VorigerSprung",
   ["en"]="PreviousJump",
   ["fr"]="SautPrecedent",
   ["it"]="SaltoPrecedente",
   ["nl"]="VorigeSprong",
   ["pe"]="پرش‌قبلی",
   ["ro"]="SaltPrecedent",
  },
  ["PreviousPage"]={
   ["cs"]="PredchoziStrana",
   ["de"]="VorigeSeite",
   ["en"]="PreviousPage",
   ["fr"]="PagePrecedente",
   ["it"]="PaginaPrecedente",
   ["nl"]="VorigePagina",
   ["pe"]="صفحه‌قبلی",
   ["ro"]="PaginaPrecedenta",
  },
  ["PrintDocument"]={
   ["cs"]="VytisknoutDokument",
   ["de"]="DruckeDokument",
   ["en"]="PrintDocument",
   ["fr"]="ImprimerDocument",
   ["it"]="StampaDocumento",
   ["nl"]="PrintDocument",
   ["pe"]="چاپ‌نوشتار",
   ["ro"]="TiparesteDocument",
  },
  ["Query"]={
   ["cs"]="Query",
   ["de"]="Query",
   ["en"]="Query",
   ["fr"]="Interroger",
   ["it"]="Richiesta",
   ["nl"]="RaadpleegDocument",
   ["pe"]="پرسش",
   ["ro"]="Query",
  },
  ["QueryAgain"]={
   ["cs"]="QueryAgain",
   ["de"]="QueryAgain",
   ["en"]="QueryAgain",
   ["fr"]="InterrogerSuivant",
   ["it"]="RinnovaRichiesta",
   ["nl"]="HerhaalRaadplegen",
   ["pe"]="پرسش‌دوباره",
   ["ro"]="QueryAgain",
  },
  ["ResetForm"]={
   ["cs"]="ResetFormulare",
   ["de"]="ZuruecksetzenFormular",
   ["en"]="ResetForm",
   ["fr"]="RazFormulaire",
   ["it"]="ReimpostaForm",
   ["nl"]="ResetFormulier",
   ["pe"]="دوباره‌نشانی‌فرم",
   ["ro"]="ResetareFormular",
  },
  ["ResumeMovie"]={
   ["cs"]="PokracovatFilm",
   ["de"]="FortsetzenFilm",
   ["en"]="ResumeMovie",
   ["fr"]="RepriseFilm",
   ["it"]="RiprendiFilm",
   ["nl"]="HervatFilm",
   ["pe"]="ادامه‌فیلم",
   ["ro"]="RepornesteFilm",
  },
  ["ResumeRendering"]={
   ["cs"]="ResumeRendering",
   ["de"]="ResumeRendering",
   ["en"]="ResumeRendering",
   ["fr"]="RepriseRendu",
   ["it"]="ResumeRendering",
   ["nl"]="HervatRendering",
   ["pe"]="ادامه‌تعبیر",
   ["ro"]="ResumeRendering",
  },
  ["ResumeSound"]={
   ["cs"]="PokracovatZvuk",
   ["de"]="ResumeSound",
   ["en"]="ResumeSound",
   ["fr"]="RepriseSon",
   ["it"]="RiprendiSuono",
   ["nl"]="HervatGeluid",
   ["pe"]="ادامه‌صدا",
   ["ro"]="RepornesteSunet",
  },
  ["Romannumerals"]={
   ["cs"]="Rimskecislice",
   ["de"]="Roemischezahlen",
   ["en"]="Romannumerals",
   ["fr"]="Chiffresromains",
   ["it"]="Numeriromani",
   ["nl"]="Romeins",
   ["pe"]="اعداد‌بزرگ‌لاتین",
   ["ro"]="Numereromane",
  },
  ["SaveDocument"]={
   ["cs"]="UlozitDokument",
   ["de"]="SpeichereDokument",
   ["en"]="SaveDocument",
   ["fr"]="EnregistrerDocument",
   ["it"]="SalvaDocumento",
   ["nl"]="BewaarDocument",
   ["pe"]="ذخیره‌نوشتار",
   ["ro"]="SalvareDocument",
  },
  ["SaveForm"]={
   ["cs"]="UlozitFormular",
   ["de"]="SpeicheFormular",
   ["en"]="SaveForm",
   ["fr"]="EnregistrerFormulaire",
   ["it"]="SalvaForm",
   ["nl"]="BewaarFormulier",
   ["pe"]="ذخیره‌فرم",
   ["ro"]="SalveazaFormular",
  },
  ["SaveNamedDocument"]={
   ["cs"]="SaveNamedDocument",
   ["de"]="SaveNamedDocument",
   ["en"]="SaveNamedDocument",
   ["fr"]="EnregistrerDocumentNomme",
   ["it"]="SaveNamedDocument",
   ["nl"]="BewaarDocumentOpNaam",
   ["pe"]="ذخیره‌نوشتار‌بانام",
   ["ro"]="SaveNamedDocument",
  },
  ["SearchAgain"]={
   ["cs"]="HledatZnovu",
   ["de"]="WeiterSuchen",
   ["en"]="SearchAgain",
   ["fr"]="ChercherSuivant",
   ["it"]="CercaAncora",
   ["nl"]="HerhaalZoeken",
   ["pe"]="جستجوی‌دوباره",
   ["ro"]="CautaDinNou",
  },
  ["SearchDocument"]={
   ["cs"]="ProhledatDokument",
   ["de"]="DokumentDurchsuchen",
   ["en"]="SearchDocument",
   ["fr"]="ChercherDocument",
   ["it"]="CercaDocumento",
   ["nl"]="DoorzoekDocument",
   ["pe"]="جستجوی‌نوشتار",
   ["ro"]="CautaDocument",
  },
  ["ShowField"]={
   ["cs"]="UkazPole",
   ["de"]="ZeigeFeld",
   ["en"]="ShowField",
   ["fr"]="MontrerChamp",
   ["it"]="MostraCampo",
   ["nl"]="ToonVeld",
   ["pe"]="نمایش‌میدان",
   ["ro"]="AfiseazaCamp",
  },
  ["StartMovie"]={
   ["cs"]="SpustitFilm",
   ["de"]="StartFilm",
   ["en"]="StartMovie",
   ["fr"]="débutFilm",
   ["it"]="IniziaFilm",
   ["nl"]="StartFilm",
   ["pe"]="شروع‌فیلم",
   ["ro"]="StartFilm",
  },
  ["StartRendering"]={
   ["cs"]="StartRendering",
   ["de"]="StartRendering",
   ["en"]="StartRendering",
   ["fr"]="débutRendu",
   ["it"]="StartRendering",
   ["nl"]="StartRendering",
   ["pe"]="شروع‌تعبیر",
   ["ro"]="StartRendering",
  },
  ["StartSound"]={
   ["cs"]="StartZvuk",
   ["de"]="StartSound",
   ["en"]="StartSound",
   ["fr"]="débutSon",
   ["it"]="IniziaSuono",
   ["nl"]="StartGeluid",
   ["pe"]="شروع‌صدا",
   ["ro"]="StartSunet",
  },
  ["StopMovie"]={
   ["cs"]="ZastavitFilm",
   ["de"]="StopFilm",
   ["en"]="StopMovie",
   ["fr"]="FinFilm",
   ["it"]="FermaFilm",
   ["nl"]="StopFilm",
   ["pe"]="پایان‌فیلم",
   ["ro"]="StopFilm",
  },
  ["StopRendering"]={
   ["cs"]="StopRendering",
   ["de"]="StopRendering",
   ["en"]="StopRendering",
   ["fr"]="FinRendu",
   ["it"]="StopRendering",
   ["nl"]="StopRendering",
   ["pe"]="پایان‌تعبیر",
   ["ro"]="StopRendering",
  },
  ["StopSound"]={
   ["cs"]="StopZvuk",
   ["de"]="StopSound",
   ["en"]="StopSound",
   ["fr"]="FinSon",
   ["it"]="FermaSuono",
   ["nl"]="StopGeluid",
   ["pe"]="پایان‌صدا",
   ["ro"]="StopSunet",
  },
  ["SubmitForm"]={
   ["cs"]="PoslatFormular",
   ["de"]="VersendeFormular",
   ["en"]="SubmitForm",
   ["fr"]="SoumettreFormulaire",
   ["it"]="MandaForm",
   ["nl"]="VerzendFormulier",
   ["pe"]="تسلیم‌فرم",
   ["ro"]="TrimiteFormular",
  },
  ["ToggleLayer"]={
   ["cs"]="ToggleLayer",
   ["de"]="ToggleLayer",
   ["en"]="ToggleLayer",
   ["fr"]="BasculerCalque",
   ["it"]="ToggleLayer",
   ["nl"]="WisselLaag",
   ["pe"]="تغییرلایه",
   ["ro"]="ToggleLayer",
  },
  ["ToggleViewer"]={
   ["cs"]="ZmenitProhlizec",
   ["de"]="WechseleViewer",
   ["en"]="ToggleViewer",
   ["fr"]="BasculerVisualisateur",
   ["it"]="ScambiaVisualizzatore",
   ["nl"]="WisselViewer",
   ["pe"]="تغییرنمایشگر",
   ["ro"]="SchimbaVizualizator",
  },
  ["URL"]={
   ["cs"]="URL",
   ["de"]="URL",
   ["en"]="URL",
   ["fr"]="URL",
   ["it"]="URL",
   ["nl"]="URL",
   ["pe"]="URL",
   ["ro"]="URL",
  },
  ["VideLayer"]={
   ["cs"]="VideLayer",
   ["de"]="VideLayer",
   ["en"]="VideLayer",
   ["fr"]="VideCalque",
   ["it"]="VideLayer",
   ["nl"]="ToonLaag",
   ["pe"]="VideLayer",
   ["ro"]="VideLayer",
  },
  ["ViewerHelp"]={
   ["cs"]="NapovedaProhlizece",
   ["de"]="ViewerHilfe",
   ["en"]="ViewerHelp",
   ["fr"]="AideVisualisateur",
   ["it"]="AiutoVisualizzatore",
   ["nl"]="ViewerHelp",
   ["pe"]="کمک‌نمایشگر",
   ["ro"]="AjutorVizualizator",
  },
  ["WEEKDAY"]={
   ["cs"]="VSEDNIDEN",
   ["de"]="WOCHENTAG",
   ["en"]="WEEKDAY",
   ["fr"]="JOURSEMAINE",
   ["it"]="GIORNOSETTIMANA",
   ["nl"]="WEEKDAG",
   ["pe"]="روزهفته",
   ["ro"]="ZISAPTAMANA",
  },
  ["WORD"]={
   ["cs"]="SLOVO",
   ["de"]="WORT",
   ["en"]="WORD",
   ["fr"]="MOT",
   ["it"]="PAROLA",
   ["nl"]="WOORD",
   ["pe"]="کلمه",
   ["ro"]="CUVANT",
  },
  ["WORDS"]={
   ["cs"]="SLOVA",
   ["de"]="WOERTER",
   ["en"]="WORDS",
   ["fr"]="MOTS",
   ["it"]="PAROLE",
   ["nl"]="WOORDEN",
   ["pe"]="WORDS",
   ["ro"]="CUVINTE",
  },
  ["Word"]={
   ["cs"]="Slovo",
   ["de"]="Wort",
   ["en"]="Word",
   ["fr"]="Mot",
   ["it"]="Parola",
   ["nl"]="Woord",
   ["pe"]="Word",
   ["ro"]="Cuvant",
  },
  ["Words"]={
   ["cs"]="Slova",
   ["de"]="Woerter",
   ["en"]="Words",
   ["fr"]="Mots",
   ["it"]="Parole",
   ["nl"]="Woorden",
   ["pe"]="Words",
   ["ro"]="Cuvinte",
  },
  ["abbreviation"]={
   ["cs"]="zkratka",
   ["de"]="abkuerzung",
   ["en"]="abbreviation",
   ["fr"]="abreviation",
   ["it"]="abbreviazione",
   ["nl"]="afkorting",
   ["pe"]="اختصار",
   ["ro"]="abreviere",
  },
  ["abbreviations"]={
   ["cs"]="zkratky",
   ["de"]="abkuerzungen",
   ["en"]="abbreviations",
   ["fr"]="abreviations",
   ["it"]="abbreviazioni",
   ["nl"]="afkortingen",
   ["pe"]="اختصارات",
   ["ro"]="abrevieri",
  },
  ["absolute"]={
   ["cs"]="absolutni",
   ["de"]="absolut",
   ["en"]="absolute",
   ["fr"]="absolu",
   ["it"]="assoluto",
   ["nl"]="absoluut",
   ["pe"]="قطعی",
   ["ro"]="absolut",
  },
  ["action"]={
   ["cs"]="akce",
   ["de"]="aktion",
   ["en"]="action",
   ["fr"]="action",
   ["it"]="azione",
   ["nl"]="actie",
   ["pe"]="کنش",
   ["ro"]="actiune",
  },
  ["after"]={
   ["cs"]="po",
   ["de"]="nach",
   ["en"]="after",
   ["fr"]="après",
   ["it"]="dopo",
   ["nl"]="na",
   ["pe"]="بعداز",
   ["ro"]="dupa",
  },
  ["all"]={
   ["cs"]="vse",
   ["de"]="alles",
   ["en"]="all",
   ["fr"]="tout",
   ["it"]="tutti",
   ["nl"]="alles",
   ["pe"]="همه",
   ["ro"]="tot",
  },
  ["alphabetic"]={
   ["en"]="alphabetic",
   ["fr"]="Alphabetique",
  },
  ["always"]={
   ["cs"]="vzdy",
   ["de"]="immer",
   ["en"]="always",
   ["fr"]="toujours",
   ["it"]="sempre",
   ["nl"]="altijd",
   ["pe"]="همواره",
   ["ro"]="totdeauna",
  },
  ["anchor"]={
   ["en"]="anchor",
   ["fr"]="ancre",
   ["nl"]="anker",
  },
  ["and"]={
   ["de"]="und",
   ["en"]="and",
   ["fr"]="et",
   ["nl"]="en",
  },
  ["answerarea"]={
   ["cs"]="answerarea",
   ["de"]="answerarea",
   ["en"]="answerarea",
   ["fr"]="zonereponse",
   ["it"]="answerarea",
   ["nl"]="antwoordgebied",
   ["pe"]="answerarea",
   ["ro"]="answerarea",
  },
  ["append"]={
   ["en"]="append",
  },
  ["prepend"]={
   ["en"]="prepend",
  },
  ["appendices"]={
   ["cs"]="dodatky",
   ["de"]="anhaenge",
   ["en"]="appendices",
   ["fr"]="annexes",
   ["it"]="appendici",
   ["nl"]="bijlagen",
   ["pe"]="پیوستها",
   ["ro"]="apendixuri",
  },
  ["appendix"]={
   ["cs"]="dodatek",
   ["de"]="anhang",
   ["en"]="appendix",
   ["fr"]="annexe",
   ["it"]="appendice",
   ["nl"]="bijlage",
   ["pe"]="پیوست",
   ["ro"]="apendix",
  },
  ["april"]={
   ["cs"]="duben",
   ["de"]="april",
   ["en"]="april",
   ["fr"]="avril",
   ["it"]="aprile",
   ["nl"]="april",
   ["pe"]="آوریل",
   ["ro"]="aprilie",
  },
  ["aside"]={
   ["en"]="aside",
   ["fr"]="apart",
  },
  ["atleftmargin"]={
   ["cs"]="atleftmargin",
   ["de"]="atleftmargin",
   ["en"]="atleftmargin",
   ["fr"]="enmargegauche",
   ["it"]="atleftmargin",
   ["nl"]="oplinkermarge",
 --["pe"]="درحاشیه‌چپ",  -- in/at clash
   ["ro"]="atleftmargin",
  },
  ["atmargin"]={
   ["cs"]="naokraji",
   ["de"]="amrand",
   ["en"]="atmargin",
   ["fr"]="enmarge",
   ["it"]="almargine",
   ["nl"]="opmarge",
   ["pe"]="درحاشیه",
   ["ro"]="lamargine",
  },
  ["atpage"]={
   ["cs"]="nastrance",
   ["de"]="aufseite",
   ["en"]="atpage",
   ["fr"]="enpage",
   ["it"]="apagina",
   ["nl"]="oppagina",
   ["pe"]="درصفحه",
   ["ro"]="lapagina",
  },
  ["atrightmargin"]={
   ["cs"]="atrightmargin",
   ["de"]="atrightmargin",
   ["en"]="atrightmargin",
   ["fr"]="enmargedroite",
   ["it"]="atrightmargin",
   ["nl"]="oprechtermarge",
 --["pe"]="درحاشیه‌راست",  -- in/at clash
   ["ro"]="atrightmargin",
  },
  ["attachment"]={
   ["en"]="attachment",
   ["fr"]="piecejointe",
   ["nl"]="aanhangsel",
  },
  ["august"]={
   ["cs"]="srpen",
   ["de"]="august",
   ["en"]="august",
   ["fr"]="aout",
   ["it"]="agosto",
   ["nl"]="augustus",
   ["pe"]="آگوست",
   ["ro"]="august",
  },
  ["author"]={
   ["cs"]="autor",
   ["de"]="autor",
   ["en"]="author",
   ["fr"]="auteur",
   ["it"]="autore",
   ["nl"]="auteur",
   ["ro"]="autor",
  },
  ["auto"]={
   ["cs"]="auto",
   ["de"]="auto",
   ["en"]="auto",
   ["fr"]="auto",
   ["it"]="auto",
   ["nl"]="auto",
   ["pe"]="خودکار",
   ["ro"]="auto",
  },
  ["autointro"]={
   ["cs"]="autouvod",
   ["de"]="autointro",
   ["en"]="autointro",
   ["fr"]="autointro",
   ["it"]="autointro",
   ["nl"]="autointro",
   ["pe"]="پیشگفتارخودکار",
   ["ro"]="autointro",
  },
  ["axis"]={
   ["en"]="axis",
  },
  ["back"]={
   ["cs"]="zpet",
   ["de"]="zurueck",
   ["en"]="back",
   ["fr"]="retour",
   ["it"]="dietro",
   ["nl"]="terug",
   ["pe"]="پشت",
   ["ro"]="inapot",
  },
  ["backgroundbar"]={
   ["en"]="backgroundbar",
  },
  ["background"]={
   ["cs"]="pozadi",
   ["de"]="hintergrund",
   ["en"]="background",
   ["fr"]="arriereplan",
   ["it"]="sfondo",
   ["nl"]="achtergrond",
   ["pe"]="پس‌زمینه",
   ["ro"]="fundal",
  },
  ["backmatter"]={
   ["cs"]="epilogy",
   ["de"]="epiloge",
   ["en"]="backmatter",
   ["fr"]="pagesdefin",
   ["it"]="postambolo",
   ["nl"]="uitleidingen",
   ["pe"]="پس‌مطلب",
   ["ro"]="epilogul",
  },
  ["backpart"]={
   ["cs"]="epilog",
   ["de"]="epilog",
   ["en"]="backpart",
   ["fr"]="postface",
   ["it"]="postfazione",
   ["nl"]="uitleiding",
   ["pe"]="پس‌قسمت",
   ["ro"]="epilog",
  },
  ["backspace"]={
   ["cs"]="zpetnamezera",
   ["de"]="rumpfspatium",
   ["en"]="backspace",
   ["fr"]="retourarrière",
   ["it"]="backspace",
   ["nl"]="rugwit",
   ["pe"]="فضای‌پشت",
   ["ro"]="spatiuspate",
  },
  ["backward"]={
   ["cs"]="zpet",
   ["de"]="rueckwaerts",
   ["en"]="backward",
   ["fr"]="enarriere",
   ["it"]="indietro",
   ["nl"]="achteruit",
   ["pe"]="عقب‌گرد",
   ["ro"]="inapoi",
  },
  ["bbl"]={
   ["en"]="bbl",
   ["fr"]="bbl",
  },
  ["before"]={
   ["cs"]="pred",
   ["de"]="vor",
   ["en"]="before",
   ["fr"]="avant",
   ["it"]="prima",
   ["nl"]="voor",
   ["pe"]="قبل‌از",
   ["ro"]="inainte",
  },
  ["begin"]={
   ["en"]="begin",
   ["fr"]="commencer",
   ["nl"]="begin",
  },
  ["bib"]={
   ["en"]="bib",
   ["fr"]="bib",
  },
  ["big"]={
   ["cs"]="velke",
   ["de"]="gross",
   ["en"]="big",
   ["fr"]="important",
   ["it"]="grande",
   ["nl"]="groot",
   ["pe"]="بزرگ",
   ["ro"]="mare",
  },
  ["bigbodyfont"]={
   ["cs"]="bigbodyfont",
   ["de"]="bigbodyfont",
   ["en"]="bigbodyfont",
   ["fr"]="policedecorpsimportante",
   ["it"]="grossofontdeltesto",
   ["nl"]="grootkorps",
   ["pe"]="قلم‌بدنه‌بزرگ",
   ["ro"]="bigbodyfont",
  },
  ["bigger"]={
   ["en"]="bigger",
   ["fr"]="plusimportant",
   ["nl"]="groter",
  },
  ["bigpreference"]={
   ["cs"]="vysokapriorita",
   ["de"]="grosszuegig",
   ["en"]="bigpreference",
   ["fr"]="preferenceimportante",
   ["it"]="grandepreferenza",
   ["nl"]="grotevoorkeur",
   ["pe"]="اولویت‌بزرگ",
   ["ro"]="preferintamare",
  },
  ["blank"]={
   ["cs"]="prazdny",
   ["de"]="blanko",
   ["en"]="blank",
   ["fr"]="vierge",
   ["it"]="rigovuoto",
   ["nl"]="blanko",
   ["pe"]="خالی",
   ["ro"]="blank",
  },
  ["block"]={
   ["en"]="block",
   ["fr"]="bloc",
  },
  ["blockquote"]={
   ["cs"]="blockquote",
   ["de"]="blockquote",
   ["en"]="blockquote",
   ["fr"]="citerbloc",
   ["it"]="blockquote",
   ["nl"]="blokcitaat",
   ["pe"]="نقل‌بلوک",
   ["ro"]="blockquote",
  },
  ["bodymatter"]={
   ["cs"]="hlavnicasti",
   ["de"]="haupttexte",
   ["en"]="bodymatter",
   ["fr"]="pagescorpsdetexte",
   ["it"]="testiprincipali",
   ["nl"]="hoofdteksten",
   ["pe"]="مطلب‌بدنه",
   ["ro"]="principalul",
  },
  ["bodypart"]={
   ["cs"]="hlavnicast",
   ["de"]="haupttext",
   ["en"]="bodypart",
   ["fr"]="corpsdetexte",
   ["it"]="testoprincipale",
   ["nl"]="hoofdtekst",
   ["pe"]="قسمت‌بدنه",
   ["ro"]="principal",
  },
  ["bold"]={
   ["cs"]="tucne",
   ["de"]="fett",
   ["en"]="bold",
   ["fr"]="gras",
   ["it"]="grassetto",
   ["nl"]="vet",
   ["pe"]="مشکی",
   ["ro"]="aldin",
  },
  ["bolditalic"]={
   ["cs"]="tucnekurzivni",
   ["de"]="fettitalic",
   ["en"]="bolditalic",
   ["fr"]="grasitalique",
   ["it"]="grassettocorsivo",
   ["nl"]="vetitalic",
   ["pe"]="ایتالیک‌مشکی",
   ["ro"]="aldinitalic",
  },
  ["boldslanted"]={
   ["cs"]="tucnesklonene",
   ["de"]="fettgeneigt",
   ["en"]="boldslanted",
   ["fr"]="grasincline",
   ["it"]="grassettoinclinato",
   ["nl"]="vetschuin",
   ["pe"]="خوابیده‌مشکی",
   ["ro"]="aldininclinat",
  },
  ["bookmark"]={
   ["cs"]="zalozka",
   ["de"]="bookmark",
   ["en"]="bookmark",
   ["fr"]="marquepage",
   ["it"]="segnalibro",
   ["nl"]="bookmark",
   ["pe"]="چوبخط",
   ["ro"]="semncarte",
  },
  ["both"]={
   ["cs"]="obe",
   ["de"]="beide",
   ["en"]="both",
   ["fr"]="chacun",
   ["it"]="entrambi",
   ["nl"]="beide",
   ["pe"]="هردو",
   ["ro"]="ambele",
  },
  ["bothtext"]={
   ["en"]="bothtext",
   ["fr"]="textechacun",
  },
  ["bottom"]={
   ["cs"]="spodek",
   ["de"]="unten",
   ["en"]="bottom",
   ["fr"]="bas",
   ["it"]="fondo",
   ["nl"]="onder",
   ["pe"]="پایین",
   ["ro"]="subsol",
  },
  ["box"]={
   ["en"]="box",
   ["fr"]="boite",
  },
  ["brief"]={
   ["cs"]="brief",
   ["de"]="brief",
   ["en"]="brief",
   ["fr"]="brief",
   ["it"]="brief",
   ["nl"]="brief",
   ["pe"]="مختصر",
   ["ro"]="brief",
  },
  ["broad"]={
   ["cs"]="siroky",
   ["de"]="breit",
   ["en"]="broad",
   ["fr"]="large",
   ["it"]="ampio",
   ["nl"]="ruim",
   ["pe"]="گسترده",
   ["ro"]="broad",
  },
  ["buffer"]={
   ["cs"]="buffer",
   ["de"]="puffer",
   ["en"]="buffer",
   ["fr"]="tampon",
   ["it"]="buffer",
   ["nl"]="buffer",
   ["pe"]="بافر",
   ["ro"]="buffer",
  },
  ["by"]={
   ["cs"]="skrz",
   ["de"]="pro",
   ["en"]="by",
   ["fr"]="par",
   ["it"]="da",
   ["nl"]="per",
   ["pe"]="بوسیله",
   ["ro"]="de",
  },
  ["calligraphic"]={
   ["cs"]="kaligraficke",
   ["de"]="kalligraphie",
   ["en"]="calligraphic",
   ["fr"]="calligraphique",
   ["it"]="calligrafico",
   ["nl"]="calligrafie",
   ["pe"]="خوشنویسی",
   ["ro"]="caligrafic",
  },
  ["camel"]={
   ["en"]="camel",
   ["fr"]="camel",
  },
  ["Camel"]={
   ["en"]="Camel",
   ["fr"]="Camel",
  },
  ["cap"]={
   ["cs"]="kap",
   ["de"]="kap",
   ["en"]="cap",
   ["fr"]="cap",
   ["it"]="cap",
   ["nl"]="kap",
   ["pe"]="cap",
   ["ro"]="cap",
  },
  ["capital"]={
   ["cs"]="kapitalky",
   ["de"]="versalien",
   ["en"]="capital",
   ["fr"]="capitales",
   ["it"]="maiuscolo",
   ["nl"]="kapitaal",
   ["pe"]="capital",
   ["ro"]="majuscula",
  },
  ["category"]={
   ["en"]="category",
   ["fr"]="categorie",
  },
  ["cd:character"]={
   ["en"]="cd:character",
   ["fr"]="caracteres",
  },
  ["cd:font"]={
   ["en"]="cd:font",
   ["fr"]="police",
  },
  ["center"]={
   ["cs"]="center",
   ["de"]="center",
   ["en"]="center",
   ["fr"]="centrer",
   ["it"]="center",
   ["nl"]="centreer",
   ["pe"]="مرکز",
   ["ro"]="center",
  },
  ["centerlast"]={
   ["en"]="centerlast",
   ["fr"]="centrerdernier",
  },
  ["chapter"]={
   ["cs"]="kapitola",
   ["de"]="kapitel",
   ["en"]="chapter",
   ["fr"]="chapitre",
   ["it"]="capitolo",
   ["nl"]="hoofdstuk",
   ["pe"]="فصل",
   ["ro"]="capitol",
  },
  ["character"]={
   ["cs"]="pismeno",
   ["de"]="buchstabe",
   ["en"]="character",
   ["fr"]="caractere",
   ["it"]="lettera",
   ["nl"]="letter",
   ["pe"]="حرف",
   ["ro"]="caracter",
  },
  ["characters"]={
   ["cs"]="pismena",
   ["de"]="buchstaben",
   ["en"]="characters",
   ["fr"]="caracteres",
   ["it"]="lettere",
   ["nl"]="letters",
   ["pe"]="حرفها",
   ["ro"]="caractere",
  },
  ["chemical"]={
   ["en"]="chemical",
   ["fr"]="chimique",
  },
  ["chemicals"]={
   ["en"]="chemicals",
   ["fr"]="chimiques",
  },
  ["chemistry"]={
   ["en"]="chemistry",
   ["fr"]="chimie",
   ["nl"]="chemie",
  },
  ["cite"]={
   ["en"]="cite",
   ["fr"]="cite",
  },
  ["clip"]={
   ["cs"]="orez",
   ["de"]="ausschnitt",
   ["en"]="clip",
   ["fr"]="decouper",
   ["it"]="clip",
   ["nl"]="clip",
   ["pe"]="گیره",
   ["ro"]="clip",
  },
  ["closed"]={
   ["en"]="closed",
   ["fr"]="ferme",
   ["nl"]="gesloten",
  },
  ["color"]={
   ["cs"]="barevne",
   ["de"]="farbe",
   ["en"]="color",
   ["fr"]="couleur",
   ["it"]="colore",
   ["nl"]="kleur",
   ["pe"]="رنگ",
   ["ro"]="culoare",
  },
  ["column"]={
   ["cs"]="column",
   ["de"]="column",
   ["en"]="column",
   ["fr"]="colonne",
   ["it"]="colonna",
   ["nl"]="kolom",
   ["pe"]="ستون",
   ["ro"]="coloana",
  },
  ["columns"]={
   ["cs"]="sloupce",
   ["de"]="spalten",
   ["en"]="columns",
   ["fr"]="colonnes",
   ["it"]="colonne",
   ["nl"]="kolommen",
   ["pe"]="ستونها",
   ["ro"]="coloane",
  },
  ["combination"]={
   ["en"]="combination",
   ["fr"]="combinaison",
   ["nl"]="combinatie",
  },
  ["command"]={
   ["cs"]="prikaz",
   ["de"]="befehl",
   ["en"]="command",
   ["fr"]="commande",
   ["it"]="comando",
   ["nl"]="commando",
   ["pe"]="فرمان",
   ["ro"]="comanda",
  },
  ["commands"]={
   ["cs"]="prikazy",
   ["de"]="befehle",
   ["en"]="commands",
   ["fr"]="commandes",
   ["it"]="comandi",
   ["nl"]="commandos",
   ["pe"]="فرمانها",
   ["ro"]="comenzi",
  },
  ["comment"]={
   ["cs"]="komentar",
   ["de"]="kommentar",
   ["en"]="comment",
   ["fr"]="commentaire",
   ["it"]="commento",
   ["nl"]="commentaar",
   ["pe"]="توضیح",
   ["ro"]="comentariu",
  },
  ["component"]={
   ["cs"]="komponenta",
   ["de"]="komponente",
   ["en"]="component",
   ["fr"]="composant",
   ["it"]="componente",
   ["nl"]="onderdeel",
   ["pe"]="مولفه",
   ["ro"]="componenta",
  },
  ["compressseparator"]={
   ["en"]="compressseparator",
   ["fr"]="separateurreduction",
  },
  ["compressstopper"]={
   ["en"]="compressstopper",
   ["fr"]="stoppeurreduction",
  },
  ["concept"]={
   ["cs"]="koncept",
   ["de"]="konzept",
   ["en"]="concept",
   ["fr"]="concept",
   ["it"]="concetto",
   ["nl"]="concept",
   ["pe"]="مفهوم",
   ["ro"]="concept",
  },
  ["construction"]={
   ["en"]="construction",
   ["fr"]="construction",
  },
  ["content"]={
   ["cs"]="obsah",
   ["de"]="inhalt",
   ["en"]="content",
   ["fr"]="contenu",
   ["it"]="indice",
   ["nl"]="inhoud",
   ["pe"]="محتوا",
   ["ro"]="cuprins",
  },
  ["contents"]={
   ["cs"]="obsah",
   ["de"]="inhalte",
   ["en"]="contents",
   ["fr"]="contenus",
   ["it"]="indici",
   ["nl"]="inhouden",
   ["pe"]="محتویات",
   ["ro"]="cuprinsuri",
  },
  ["continue"]={
   ["cs"]="pokracovat",
   ["de"]="fortsetzten",
   ["en"]="continue",
   ["fr"]="continuer",
   ["it"]="continua",
   ["nl"]="verder",
   ["pe"]="ادامه",
   ["ro"]="continuu",
  },
  ["continued"]={
   ["en"]="continued",
   ["fr"]="continue",
   ["nl"]="vervolgd",
  },
  ["controls"]={
   ["cs"]="controls",
   ["de"]="controls",
   ["en"]="controls",
   ["fr"]="controles",
   ["it"]="controlli",
   ["nl"]="sturing",
   ["pe"]="کنترلها",
   ["ro"]="controale",
  },
  ["conversion"]={
   ["cs"]="konverze",
   ["de"]="konversion",
   ["en"]="conversion",
   ["fr"]="conversion",
   ["it"]="coversione",
   ["nl"]="conversie",
   ["pe"]="تبدیل",
   ["ro"]="conversie",
  },
  ["current"]={
   ["cs"]="aktualni",
   ["de"]="aktuell",
   ["en"]="current",
   ["fr"]="courant",
   ["it"]="corrente",
   ["nl"]="huidige",
   ["pe"]="جاری",
   ["ro"]="curent",
  },
  ["cutspace"]={
   ["cs"]="cutspace",
   ["de"]="cutspace",
   ["en"]="cutspace",
   ["fr"]="espacecoupe",
   ["it"]="cutspace",
   ["nl"]="snijwit",
   ["pe"]="فضای‌برش",
   ["ro"]="cutspace",
  },
  ["dash"]={
   ["cs"]="pomlcka",
   ["de"]="strich",
   ["en"]="dash",
   ["fr"]="pointille",
   ["it"]="dash",
   ["nl"]="streep",
   ["pe"]="دش",
   ["ro"]="dash",
  },
  ["dataset"]={
   ["en"]="dataset",
   ["fr"]="jeudedonnees",
  },
  ["date"]={
   ["cs"]="datum",
   ["de"]="datum",
   ["en"]="date",
   ["fr"]="date",
   ["it"]="data",
   ["nl"]="datum",
   ["pe"]="تاریخ",
   ["ro"]="data",
  },
  ["day"]={
   ["cs"]="den",
   ["de"]="tag",
   ["en"]="day",
   ["fr"]="jour",
   ["it"]="giorno",
   ["nl"]="dag",
   ["pe"]="روز",
   ["ro"]="zi",
  },
  ["december"]={
   ["cs"]="prosinec",
   ["de"]="dezember",
   ["en"]="december",
   ["fr"]="decembre",
   ["it"]="dicembre",
   ["nl"]="december",
   ["pe"]="دسامبر",
   ["ro"]="decembrie",
  },
  ["default"]={
   ["cs"]="implicitni",
   ["de"]="default",
   ["en"]="default",
   ["fr"]="defaut",
   ["it"]="implicito",
   ["nl"]="default",
   ["pe"]="پیش‌فرض",
   ["ro"]="implicit",
  },
  ["depth"]={
   ["cs"]="podlehloubky",
   ["de"]="tiefe",
   ["en"]="depth",
   ["fr"]="profondeur",
   ["it"]="profondita",
   ["nl"]="diepte",
   ["pe"]="عمق",
   ["ro"]="adancime",
  },
  ["description"]={
   ["cs"]="popis",
   ["de"]="beschreibung",
   ["en"]="description",
   ["fr"]="description",
   ["it"]="descrizione",
   ["nl"]="doordefinitie",
   ["pe"]="شرح",
   ["ro"]="descriere",
  },
  ["disable"]={
   ["cs"]="zablokovat",
   ["de"]="sperren",
   ["en"]="disable",
   ["fr"]="desactiver",
   ["it"]="disattiva",
   ["nl"]="blokkeer",
   ["pe"]="ناتوان",
   ["ro"]="dezactivat",
  },
  ["display"]={
   ["cs"]="obrazovka",
   ["de"]="bildschirm",
   ["en"]="display",
   ["fr"]="affichage",
   ["it"]="schermo",
   ["nl"]="scherm",
   ["pe"]="نمایش",
   ["ro"]="display",
  },
  ["document"]={
   ["en"]="document",
  },
  ["dot"]={
   ["cs"]="tecka",
   ["de"]="punkt",
   ["en"]="dot",
   ["fr"]="point",
   ["it"]="punto",
   ["nl"]="punt",
   ["pe"]="نقطه",
   ["ro"]="punct",
  },
  ["doublesided"]={
   ["cs"]="dvoustranny",
   ["de"]="doppelseitig",
   ["en"]="doublesided",
   ["fr"]="rectoverso",
   ["it"]="doppiafaccia",
   ["nl"]="dubbelzijdig",
   ["pe"]="دورو",
   ["ro"]="douafete",
  },
  ["down"]={
   ["en"]="down",
   ["fr"]="enbas",
   ["nl"]="omlaag",
  },
  ["each"]={
   ["cs"]="kazdy",
   ["de"]="jede",
   ["en"]="each",
   ["fr"]="chaque",
   ["it"]="ogni",
   ["nl"]="elk",
   ["pe"]="هر",
   ["ro"]="fiecare",
  },
  ["edge"]={
   ["cs"]="hrana",
   ["de"]="kante",
   ["en"]="edge",
   ["fr"]="bord",
   ["it"]="bordo",
   ["nl"]="rand",
   ["pe"]="لبه",
   ["ro"]="bordura",
  },
  ["effective"]={
   ["en"]="effective",
   ["fr"]="effectif",
   ["nl"]="effectief",
  },
  ["eight"]={
   ["cs"]="eight",
   ["de"]="acht",
   ["en"]="eight",
   ["fr"]="huit",
   ["it"]="eight",
   ["nl"]="acht",
   ["pe"]="eight",
   ["ro"]="eight",
  },
  ["compress"]={
   ["en"]="compress",
   ["nl"]="comprimeer",
  },
  ["embed"]={
   ["en"]="embed",
   ["fr"]="integrer",
   ["nl"]="sluitin",
  },
  ["empty"]={
   ["cs"]="prazdne",
   ["de"]="leer",
   ["en"]="empty",
   ["fr"]="vide",
   ["it"]="vuoto",
   ["nl"]="leeg",
   ["pe"]="تهی",
   ["ro"]="gol",
  },
  ["enable"]={
   ["en"]="enable",
   ["fr"]="activer",
  },
  ["end"]={
   ["en"]="end",
   ["fr"]="terminer",
   ["nl"]="eind",
  },
  ["endnote"]={
   ["cs"]="endnote",
   ["de"]="endnote",
   ["en"]="endnote",
   ["fr"]="noteseule",
   ["it"]="endnote",
   ["nl"]="eindnoot",
   ["pe"]="ته‌نوشت",
   ["ro"]="endnote",
  },
  ["mathnote"]={
   ["en"]="mathnote",
  },
  ["enumeration"]={
   ["cs"]="vycet",
   ["de"]="nummerierung",
   ["en"]="enumeration",
   ["fr"]="énumération",
   ["it"]="enumerazione",
   ["nl"]="doornummering",
   ["pe"]="شماره‌بندی",
   ["ro"]="enumerare",
  },
  ["environment"]={
   ["cs"]="prostredi",
   ["de"]="umgebung",
   ["en"]="environment",
   ["fr"]="environnement",
   ["it"]="ambiente",
   ["nl"]="omgeving",
   ["pe"]="محیط",
   ["ro"]="mediu",
  },
  ["even"]={
   ["cs"]="sude",
   ["de"]="gerade",
   ["en"]="even",
   ["fr"]="paire",
   ["it"]="pari",
   ["nl"]="even",
   ["pe"]="زوج",
   ["ro"]="par",
  },
  ["export"]={
   ["en"]="export",
   ["fr"]="exporter",
  },
  ["external"]={
   ["cs"]="externi",
   ["de"]="extern",
   ["en"]="external",
   ["fr"]="externe",
   ["it"]="esterno",
   ["nl"]="extern",
   ["pe"]="خارجی",
   ["ro"]="extern",
  },
  ["extremestretch"]={
   ["en"]="extremestretch",
   ["fr"]="etirementextreme",
  },
  ["fact"]={
   ["cs"]="fakt",
   ["de"]="gegeben",
   ["en"]="fact",
   ["fr"]="fait",
   ["it"]="fatto",
   ["nl"]="gegeven",
   ["pe"]="fact",
   ["ro"]="fapt",
  },
  ["fallback"]={
   ["en"]="fallback",
   ["fr"]="repli",
  },
  ["february"]={
   ["cs"]="unor",
   ["de"]="februar",
   ["en"]="february",
   ["fr"]="fevrier",
   ["it"]="febbraio",
   ["nl"]="februari",
   ["pe"]="فوریه",
   ["ro"]="februarie",
  },
  ["field"]={
   ["en"]="field",
   ["fr"]="champ",
   ["nl"]="veld",
  },
  ["figure"]={
   ["cs"]="obrazek",
   ["de"]="abbildung",
   ["en"]="figure",
   ["fr"]="figure",
   ["it"]="figura",
   ["nl"]="figuur",
   ["pe"]="شکل",
   ["ro"]="figura",
  },
  ["figures"]={
   ["cs"]="obrazky",
   ["de"]="abbildungen",
   ["en"]="figures",
   ["fr"]="figures",
   ["it"]="figure",
   ["nl"]="figuren",
   ["pe"]="شکلها",
   ["ro"]="figure",
  },
  ["file"]={
   ["cs"]="soubor",
   ["de"]="datei",
   ["en"]="file",
   ["fr"]="fichier",
   ["it"]="file",
   ["nl"]="file",
   ["pe"]="پرونده",
   ["ro"]="fisier",
  },
  ["final"]={
   ["cs"]="finalni",
   ["de"]="endfassung",
   ["en"]="final",
   ["fr"]="final",
   ["it"]="finale",
   ["nl"]="definitief",
   ["pe"]="نهایی",
   ["ro"]="final",
  },
  ["first"]={
   ["cs"]="prvni",
   ["de"]="erste",
   ["en"]="first",
   ["fr"]="premier",
   ["it"]="primo",
   ["nl"]="eerste",
   ["pe"]="اولی",
   ["ro"]="primul",
  },
  ["firstcolumn"]={
   ["cs"]="firstcolumn",
   ["de"]="firstcolumn",
   ["en"]="firstcolumn",
   ["fr"]="premierecolonne",
   ["it"]="firstcolumn",
   ["nl"]="eerstekolom",
   ["pe"]="ستون‌اول",
   ["ro"]="firstcolumn",
  },
  ["firstpage"]={
   ["cs"]="prvnistranka",
   ["de"]="ersteseite",
   ["en"]="firstpage",
   ["fr"]="premierepage",
   ["it"]="primapagina",
   ["nl"]="eerstepagina",
   ["pe"]="صفحه‌اول",
   ["ro"]="primapagina",
  },
  ["firstsubpage"]={
   ["cs"]="prvnipodstranka",
   ["de"]="ersteunterseite",
   ["en"]="firstsubpage",
   ["fr"]="premieresouspage",
   ["it"]="primasottopagina",
   ["nl"]="eerstesubpagina",
   ["pe"]="زیرصفحه‌اول",
   ["ro"]="primasubpagina",
  },
  ["fit"]={
   ["cs"]="prizpusobive",
   ["de"]="passend",
   ["en"]="fit",
   ["fr"]="adapter",
   ["it"]="adatta",
   ["nl"]="passend",
   ["pe"]="پرکردن",
   ["ro"]="ajustat",
  },
  ["five"]={
   ["cs"]="pet",
   ["de"]="fuenf",
   ["en"]="five",
   ["fr"]="cinq",
   ["it"]="cinque",
   ["nl"]="vijf",
   ["pe"]="پنج",
   ["ro"]="cinci",
  },
  ["fix"]={
   ["cs"]="fixuj",
   ["de"]="stellewiederher",
   ["en"]="fix",
   ["fr"]="fix",
   ["it"]="fisso",
   ["nl"]="herstel",
   ["pe"]="ثابت‌کن",
   ["ro"]="fix",
  },
  ["fixed"]={
   ["cs"]="fixne",
   ["de"]="fest",
   ["en"]="fixed",
   ["fr"]="fixe",
   ["it"]="fisso",
   ["nl"]="vast",
   ["pe"]="ثابت",
   ["ro"]="fixat",
  },
  ["spread"]={
    ["en"]="spread",
    ["nl"]="gespreid",
  },
  ["flexible"]={
   ["cs"]="prizpusobive",
   ["de"]="flexibel",
   ["en"]="flexible",
   ["fr"]="flexible",
   ["it"]="flessibile",
   ["nl"]="flexibel",
   ["pe"]="انعطافپذیر",
   ["ro"]="flexibil",
  },
  ["float"]={
   ["cs"]="plvouciobjekt",
   ["de"]="gleitobjekt",
   ["en"]="float",
   ["fr"]="flottant",
   ["it"]="oggettomobile",
   ["nl"]="blok",
   ["pe"]="شناور",
   ["ro"]="obiectmobil",
  },
  ["flushforward"]={
   ["en"]="flushforward",
  },
  ["flushbackward"]={
   ["en"]="flushbackward",
  },
  ["flushinner"]={
   ["cs"]="flushinner",
   ["de"]="flushinner",
   ["en"]="flushinner",
   ["fr"]="alignerinterieur",
   ["it"]="flushinner",
   ["nl"]="lijnbinnen",
   ["pe"]="پمپ‌داخلی",
   ["ro"]="flushinner",
  },
  ["flushleft"]={
   ["cs"]="flushleft",
   ["de"]="flushleft",
   ["en"]="flushleft",
   ["fr"]="alignergauche",
   ["it"]="flushleft",
   ["nl"]="lijnlinks",
   ["pe"]="پمپ‌چپ",
   ["ro"]="flushleft",
  },
  ["flushouter"]={
   ["cs"]="flushouter",
   ["de"]="flushouter",
   ["en"]="flushouter",
   ["fr"]="alignerexterieur",
   ["it"]="flushouter",
   ["nl"]="lijnbuiten",
   ["pe"]="پمپ‌خارجی",
   ["ro"]="flushouter",
  },
  ["flushright"]={
   ["cs"]="flushright",
   ["de"]="flushright",
   ["en"]="flushright",
   ["fr"]="alignerdroite",
   ["it"]="flushright",
   ["nl"]="lijnrechts",
   ["pe"]="پمپ‌راست",
   ["ro"]="flushright",
  },
  ["followingpage"]={
   ["en"]="followingpage",
   ["fr"]="pagesuccesseur",
   ["nl"]="opvolgendepagina",
  },
  ["font"]={
   ["en"]="font",
   ["nl"]="font",
  },
  ["footer"]={
   ["cs"]="upati",
   ["de"]="fusszeile",
   ["en"]="footer",
   ["fr"]="pdp",
   ["it"]="piedipagina",
   ["nl"]="voet",
   ["pe"]="ته‌برگ",
   ["ro"]="subsol",
  },
  ["footnote"]={
   ["cs"]="poznamkapodcarou",
   ["de"]="fussnote",
   ["en"]="footnote",
   ["fr"]="notebdp",
   ["it"]="notapdp",
   ["nl"]="voetnoot",
   ["pe"]="پانوشت",
   ["ro"]="notasubsol",
  },
  ["force"]={
   ["cs"]="sila",
   ["de"]="zwinge",
   ["en"]="force",
   ["fr"]="forcer",
   ["it"]="forza",
   ["nl"]="forceer",
   ["pe"]="اجبار",
   ["ro"]="fortat",
  },
  ["foreground"]={
   ["cs"]="popredi",
   ["de"]="vordergrund",
   ["en"]="foreground",
   ["fr"]="premierplan",
   ["it"]="foreground",
   ["nl"]="voorgrond",
   ["pe"]="پیش‌زمینه",
   ["ro"]="primplan",
  },
  ["formula"]={
   ["cs"]="rovnice",
   ["de"]="formel",
   ["en"]="formula",
   ["fr"]="formule",
   ["it"]="formula",
   ["nl"]="formule",
   ["pe"]="فرمول",
   ["ro"]="formula",
  },
  ["formulas"]={
   ["cs"]="rovnice",
   ["de"]="formeln",
   ["en"]="formulas",
   ["fr"]="formules",
   ["it"]="formule",
   ["nl"]="formules",
   ["pe"]="فرمولها",
   ["ro"]="formule",
  },
  ["forward"]={
   ["cs"]="vpred",
   ["de"]="vorwaerts",
   ["en"]="forward",
   ["fr"]="enavant",
   ["it"]="avanti",
   ["nl"]="vooruit",
   ["pe"]="به‌جلو",
   ["ro"]="avans",
  },
  ["four"]={
   ["cs"]="ctyri",
   ["de"]="vier",
   ["en"]="four",
   ["fr"]="quatre",
   ["it"]="quattro",
   ["nl"]="vier",
   ["pe"]="چهار",
   ["ro"]="patru",
  },
  ["fractions"]={
   ["en"]="fractions",
   ["fr"]="fractions",
  },
  ["frame"]={
   ["cs"]="ramecek",
   ["de"]="rahmen",
   ["en"]="frame",
   ["fr"]="cadre",
   ["it"]="cornice",
   ["nl"]="kader",
   ["pe"]="قالب",
   ["ro"]="incadrat",
  },
  ["framedtext"]={
   ["cs"]="oramovanytext",
   ["de"]="umrahmtertext",
   ["en"]="framedtext",
   ["fr"]="texteencadre",
   ["it"]="testoincorniciato",
   ["nl"]="kadertekst",
   ["pe"]="متن‌قالبی",
   ["ro"]="textinconjurat",
  },
  ["friday"]={
   ["cs"]="patek",
   ["de"]="freitag",
   ["en"]="friday",
   ["fr"]="vendredi",
   ["it"]="venerdi",
   ["nl"]="vrijdag",
   ["pe"]="جمعه",
   ["ro"]="vineri",
  },
  ["frontmatter"]={
   ["cs"]="prednicasti",
   ["de"]="einleitungen",
   ["en"]="frontmatter",
   ["fr"]="pagesliminaires",
   ["it"]="preambolo",
   ["nl"]="inleidingen",
   ["pe"]="پیش‌مطلب",
   ["ro"]="prologul",
  },
  ["frontpart"]={
   ["cs"]="prednicast",
   ["de"]="einleitung",
   ["en"]="frontpart",
   ["fr"]="preface",
   ["it"]="prefazione",
   ["nl"]="inleiding",
   ["pe"]="پیش‌قسمت",
   ["ro"]="prolog",
  },
  ["fullhz"]={
   ["cs"]="fullhz",
   ["de"]="fullhz",
   ["en"]="fullhz",
   ["fr"]="hzcomplet",
   ["it"]="fullhz",
   ["nl"]="fullenhz",
   ["pe"]="fullhz",
   ["ro"]="fullhz",
  },
  ["global"]={
   ["cs"]="globalne",
   ["de"]="global",
   ["en"]="global",
   ["fr"]="global",
   ["it"]="globale",
   ["nl"]="globaal",
   ["pe"]="سراسری",
   ["ro"]="global",
  },
  ["graphic"]={
   ["cs"]="graf",
   ["de"]="grafik",
   ["en"]="graphic",
   ["fr"]="graphique",
   ["it"]="grafico",
   ["nl"]="grafiek",
   ["pe"]="گرافیک",
   ["ro"]="graficul",
  },
  ["graphics"]={
   ["cs"]="grafy",
   ["de"]="grafiken",
   ["en"]="graphics",
   ["fr"]="graphiques",
   ["it"]="grafici",
   ["nl"]="grafieken",
   ["pe"]="گرافیکها",
   ["ro"]="graficele",
  },
  ["gray"]={
   ["cs"]="seda",
   ["de"]="grau",
   ["en"]="gray",
   ["fr"]="gris",
   ["it"]="grigio",
   ["nl"]="grijs",
   ["pe"]="خاکستری",
   ["ro"]="gri",
  },
  ["greek"]={
   ["cs"]="recky",
   ["de"]="griechisch",
   ["en"]="greek",
   ["fr"]="grec",
   ["it"]="greco",
   ["nl"]="grieks",
   ["pe"]="یونانی",
   ["ro"]="grec",
  },
  ["grid"]={
   ["cs"]="mrizka",
   ["de"]="gritter",
   ["en"]="grid",
   ["fr"]="grille",
   ["it"]="griglia",
   ["nl"]="grid",
   ["pe"]="توری",
   ["ro"]="grila",
  },
  ["halfline"]={
   ["cs"]="pulradku",
   ["de"]="halbezeile",
   ["en"]="halfline",
   ["fr"]="demiligne",
   ["it"]="mezzariga",
   ["nl"]="halveregel",
   ["pe"]="نیم‌خط",
   ["ro"]="jumatatelinie",
  },
  ["handwritten"]={
   ["cs"]="rukopisne",
   ["de"]="handschrift",
   ["en"]="handwritten",
   ["fr"]="manuscript",
   ["it"]="grafiamanuale",
   ["nl"]="handschrift",
   ["pe"]="دست‌نوشته",
   ["ro"]="scrismanual",
  },
  ["hang"]={
   ["cs"]="zaveseni",
   ["de"]="haengend",
   ["en"]="hang",
   ["fr"]="suspendre",
   ["it"]="sospendi",
   ["nl"]="hang",
   ["pe"]="بیاویز",
   ["ro"]="suspenda",
  },
  ["hangleft"]={
   ["en"]="hangleft",
   ["nl"]="hanglinks",
  },
  ["hangright"]={
   ["en"]="hangright",
   ["nl"]="hangrechts",
  },
  ["hanging"]={
   ["cs"]="visici",
   ["de"]="haengend",
   ["en"]="hanging",
   ["fr"]="suspension",
   ["it"]="sospeso",
   ["nl"]="hangend",
   ["pe"]="آویزان",
   ["ro"]="suspendat",
  },
  ["hangingboth"]={
   ["en"]="hangingboth",
   ["fr"]="suspensionchacun",
   ["nl"]="beidehangend",
  },
  ["head"]={
   ["cs"]="hlavicka",
   ["de"]="kopf",
   ["en"]="head",
   ["fr"]="tete",
   ["it"]="testa",
   ["nl"]="kop",
   ["pe"]="سر",
   ["ro"]="antet",
  },
  ["header"]={
   ["cs"]="zahlavi",
   ["de"]="kopfzeile",
   ["en"]="header",
   ["fr"]="entête",
   ["it"]="intestazione",
   ["nl"]="hoofd",
   ["pe"]="سربرگ",
   ["ro"]="antet",
  },
  ["headintext"]={
   ["en"]="headintext",
   ["fr"]="tetedanstexte",
   ["nl"]="kopintekst",
  },
  ["height"]={
   ["cs"]="vyska",
   ["de"]="hoehe",
   ["en"]="height",
   ["fr"]="hauteur",
   ["it"]="altezza",
   ["nl"]="hoogte",
   ["pe"]="ارتفاع",
   ["ro"]="inaltime",
  },
  ["helptext"]={
   ["cs"]="textnapovedy",
   ["de"]="hilfetext",
   ["en"]="helptext",
   ["fr"]="texteaide",
   ["it"]="testoaiuto",
   ["nl"]="helptekst",
   ["pe"]="متن‌کمکی",
   ["ro"]="textajutator",
  },
  ["hencefore"]={
   ["cs"]="vyse",
   ["de"]="vorher",
   ["en"]="hencefore",
   ["fr"]="precedemment",
   ["it"]="precedente",
   ["nl"]="hierboven",
   ["pe"]="hencefore",
   ["ro"]="precedent",
  },
  ["here"]={
   ["cs"]="zde",
   ["de"]="hier",
   ["en"]="here",
   ["fr"]="ici",
   ["it"]="qui",
   ["nl"]="hier",
   ["pe"]="اینجا",
   ["ro"]="aici",
  },
  ["hereafter"]={
   ["cs"]="nize",
   ["de"]="nachher",
   ["en"]="hereafter",
   ["fr"]="ciapres",
   ["it"]="seguente",
   ["nl"]="hieronder",
   ["pe"]="ازاین‌به‌بعد",
   ["ro"]="urmator",
  },
  ["hidden"]={
   ["cs"]="skryte",
   ["de"]="versteckt",
   ["en"]="hidden",
   ["fr"]="cache",
   ["it"]="nascosto",
   ["nl"]="verborgen",
   ["pe"]="پنهانی",
   ["ro"]="ascuns",
  },
  ["hiddenbar"]={
   ["en"]="hiddenbar",
   ["fr"]="barrecachee",
  },
  ["hiding"]={
   ["cs"]="skryt",
   ["de"]="verbergen",
   ["en"]="hiding",
   ["fr"]="encachant",
   ["it"]="nascondere",
   ["nl"]="verbergen",
   ["pe"]="پنهان‌کردن",
   ["ro"]="ascundere",
  },
  ["high"]={
   ["cs"]="vysoko",
   ["de"]="hoch",
   ["en"]="high",
   ["fr"]="exp",
   ["it"]="alto",
   ["nl"]="hoog",
   ["pe"]="بلند",
   ["ro"]="inalt",
  },
  ["horizontal"]={
   ["cs"]="horizontalne",
   ["de"]="horizontal",
   ["en"]="horizontal",
   ["fr"]="horizontal",
   ["it"]="orizzontale",
   ["nl"]="horizontaal",
   ["pe"]="افقی",
   ["ro"]="orizontal",
  },
  ["collapsed"]={
   ["en"]="collapsed",
  },
  ["hyphenated"]={
   ["cs"]="hyphenated",
   ["de"]="hyphenate",
   ["en"]="hyphenated",
   ["fr"]="aveccesure",
   ["it"]="sillabato",
   ["nl"]="afgebroken",
   ["pe"]="شکسته",
   ["ro"]="despsilabe",
  },
  ["hz"]={
   ["en"]="hz",
  },
  ["vz"]={
   ["en"]="vz",
  },
  ["inbetween"]={
   ["cs"]="mezi",
   ["de"]="zwischen",
   ["en"]="inbetween",
   ["fr"]="entre",
   ["it"]="tra",
   ["nl"]="tussen",
   ["pe"]="دربین",
   ["ro"]="intre",
  },
  ["index"]={
   ["cs"]="rejstrik",
   ["de"]="index",
   ["en"]="index",
   ["fr"]="index",
   ["it"]="indiceanalitico",
   ["nl"]="index",
   ["pe"]="نمایه",
   ["ro"]="index",
  },
  ["indices"]={
   ["cs"]="rejstriky",
   ["de"]="indizies",
   ["en"]="indices",
   ["fr"]="indices",
   ["it"]="indicianalitici",
   ["nl"]="indices",
   ["pe"]="نمایه‌ها",
   ["ro"]="indexuri",
  },
  ["inherit"]={
   ["cs"]="inherit",
   ["de"]="inherit",
   ["en"]="inherit",
   ["fr"]="heriter",
   ["it"]="inherit",
   ["nl"]="erf",
   ["pe"]="ارث‌بردن",
   ["ro"]="inherit",
  },
  ["ininner"]={
   ["cs"]="ininner",
   ["de"]="ininner",
   ["en"]="ininner",
   ["fr"]="dansinterieur",
   ["it"]="ininner",
   ["nl"]="inbinnen",
   ["pe"]="درداخلی",
   ["ro"]="ininner",
  },
  ["ininneredge"]={
   ["en"]="ininneredge",
   ["fr"]="dansbordinterieur",
  },
  ["ininnermargin"]={
   ["en"]="ininnermargin",
   ["fr"]="dansmargeinterieure",
  },
  ["inleft"]={
   ["cs"]="ivlevo",
   ["de"]="imlinken",
   ["en"]="inleft",
   ["fr"]="dansgauche",
   ["it"]="insinistra",
   ["nl"]="inlinker",
   ["pe"]="درون‌چپ",
   ["ro"]="instanga",
  },
  ["inleftedge"]={
   ["cs"]="nalevo",
   ["de"]="imlinkenrand",
   ["en"]="inleftedge",
   ["fr"]="dansbordgauche",
   ["it"]="inlatosinistro",
   ["nl"]="inlinkerrand",
   ["pe"]="درلبه‌چپ",
   ["ro"]="inparteastanga",
  },
  ["inleftmargin"]={
   ["cs"]="nalevyokraj",
   ["de"]="inlinkermarginale",
   ["en"]="inleftmargin",
   ["fr"]="dansmargegauche",
   ["it"]="inmarginesinistro",
   ["nl"]="inlinkermarge",
   ["pe"]="درحاشیه‌چپ",
   ["ro"]="inmargineastanga",
  },
  ["inline"]={
   ["en"]="inline",
   ["fr"]="enligne",
  },
  ["inmargin"]={
   ["cs"]="naokraji",
   ["de"]="imrand",
   ["en"]="inmargin",
   ["fr"]="dansmarge",
   ["it"]="inmargine",
   ["nl"]="inmarge",
   ["pe"]="درون‌حاشیه",
   ["ro"]="inmargine",
  },
  ["inner"]={
   ["cs"]="uvnitr",
   ["de"]="innen",
   ["en"]="inner",
   ["fr"]="interieur",
   ["it"]="interno",
   ["nl"]="binnen",
   ["pe"]="داخلی",
   ["ro"]="intern",
  },
  ["inneredge"]={
   ["cs"]="inneredge",
   ["de"]="inneredge",
   ["en"]="inneredge",
   ["fr"]="bordinterieur",
   ["it"]="bordointerno",
   ["nl"]="binnenrand",
   ["pe"]="لبه‌داخلی",
   ["ro"]="inneredge",
  },
  ["innermargin"]={
   ["cs"]="innermargin",
   ["de"]="innermargin",
   ["en"]="innermargin",
   ["fr"]="margeinterieure",
   ["it"]="margineinterno",
   ["nl"]="binnenmarge",
   ["pe"]="حاشیه‌داخلی",
   ["ro"]="innermargin",
  },
  ["inother"]={
   ["cs"]="inother",
   ["de"]="inother",
   ["en"]="inother",
   ["fr"]="dansautre",
   ["it"]="inother",
   ["nl"]="inandere",
   ["pe"]="inother",
   ["ro"]="inother",
  },
  ["inouter"]={
   ["cs"]="inouter",
   ["de"]="inouter",
   ["en"]="inouter",
   ["fr"]="dansexterieur",
   ["it"]="inouter",
   ["nl"]="inbuiten",
   ["pe"]="درخارجی",
   ["ro"]="inouter",
  },
  ["inouteredge"]={
   ["en"]="inouteredge",
   ["fr"]="dansbordexterieur",
  },
  ["inoutermargin"]={
   ["en"]="inoutermargin",
   ["fr"]="dansmargeexterieure",
  },
  ["inright"]={
   ["cs"]="ivpravo",
   ["de"]="imrechten",
   ["en"]="inright",
   ["fr"]="dansdroite",
   ["it"]="indestra",
   ["nl"]="inrechter",
   ["pe"]="درون‌راست",
   ["ro"]="indreapta",
  },
  ["inrightedge"]={
   ["cs"]="napravo",
   ["de"]="imrechtenrand",
   ["en"]="inrightedge",
   ["fr"]="dansborddroit",
   ["it"]="inlatodestro",
   ["nl"]="inrechterrand",
   ["pe"]="درلبه‌راست",
   ["ro"]="inparteadreapta",
  },
  ["inrightmargin"]={
   ["cs"]="napravyokraj",
   ["de"]="inrechtermarginale",
   ["en"]="inrightmargin",
   ["fr"]="dansmargedroite",
   ["it"]="inmarginedestro",
   ["nl"]="inrechtermarge",
   ["pe"]="درحاشیه‌راست",
   ["ro"]="inmargineadreapta",
  },
  ["interaction"]={
   ["cs"]="interakce",
   ["de"]="interaktion",
   ["en"]="interaction",
   ["fr"]="interaction",
   ["it"]="interazione",
   ["nl"]="interactie",
   ["pe"]="پانل",
   ["ro"]="interactiune",
  },
  ["interactionmenu"]={
   ["cs"]="interaktivnimenu",
   ["de"]="interaktionsmenue",
   ["en"]="interactionmenu",
   ["fr"]="menuinteraction",
   ["it"]="menuinterattivo",
   ["nl"]="interactiemenu",
   ["pe"]="منوی‌پانل",
   ["ro"]="meniuinteractiune",
  },
  ["interactive"]={
   ["en"]="interactive",
   ["fr"]="interactif",
  },
  ["intermezzi"]={
   ["cs"]="intermezzi",
   ["de"]="intermezzi",
   ["en"]="intermezzi",
   ["fr"]="interludes",
   ["it"]="intermezzi",
   ["nl"]="intermezzos",
   ["pe"]="میان‌پرده‌ها",
   ["ro"]="intermezzi",
  },
  ["intermezzo"]={
   ["cs"]="intermezzo",
   ["de"]="intermezzo",
   ["en"]="intermezzo",
   ["fr"]="interlude",
   ["it"]="intermezzo",
   ["nl"]="intermezzo",
   ["pe"]="میان‌پرده",
   ["ro"]="intermezzo",
  },
  ["intext"]={
   ["cs"]="dotextu",
   ["de"]="imtext",
   ["en"]="intext",
   ["fr"]="danstexte",
   ["it"]="intesto",
   ["nl"]="intekst",
   ["pe"]="درون‌متن",
   ["ro"]="intext",
  },
  ["intro"]={
   ["cs"]="uvod",
   ["de"]="intro",
   ["en"]="intro",
   ["fr"]="intro",
   ["it"]="intro",
   ["nl"]="intro",
   ["pe"]="پیشگفتار",
   ["ro"]="intro",
  },
  ["invertedshort"]={
   ["en"]="invertedshort",
  },
  ["italic"]={
   ["cs"]="kurziva",
   ["de"]="italic",
   ["en"]="italic",
   ["fr"]="italique",
   ["it"]="corsivo",
   ["nl"]="italic",
   ["pe"]="ایتالیک",
   ["ro"]="italic",
  },
  ["italicbold"]={
   ["cs"]="kurzivnitucne",
   ["de"]="italicfett",
   ["en"]="italicbold",
   ["fr"]="italiquegras",
   ["it"]="corsivograssetto",
   ["nl"]="italicvet",
   ["pe"]="مشکی‌ایتالیک",
   ["ro"]="italicaldin",
  },
  ["item"]={
   ["cs"]="polozka",
   ["de"]="pos",
   ["en"]="item",
   ["fr"]="élément",
   ["it"]="elemento",
   ["nl"]="som",
   ["pe"]="آیتم",
   ["ro"]="element",
  },
  ["itemize"]={
   ["cs"]="vycet",
   ["de"]="aufzaehlung",
   ["en"]="itemize",
   ["fr"]="détailler",
   ["it"]="elenco",
   ["nl"]="opsomming",
   ["pe"]="آیتم‌بندی",
   ["ro"]="enumerare",
  },
  ["its"]={
   ["cs"]="pol",
   ["de"]="its",
   ["en"]="its",
   ["fr"]="els",
   ["it"]="its",
   ["nl"]="its",
   ["pe"]="آیم",
   ["ro"]="propriu",
  },
  ["january"]={
   ["cs"]="leden",
   ["de"]="januar",
   ["en"]="january",
   ["fr"]="janvier",
   ["it"]="gennaio",
   ["nl"]="januari",
   ["pe"]="ژانویه",
   ["ro"]="ianuarie",
  },
  ["joinedup"]={
   ["cs"]="spojeno",
   ["de"]="keinabstand",
   ["en"]="joinedup",
   ["fr"]="sansespaceentre",
   ["it"]="unito",
   ["nl"]="aansluitend",
   ["pe"]="متصل‌بالا",
   ["ro"]="unit",
  },
  ["july"]={
   ["cs"]="cervenec",
   ["de"]="juli",
   ["en"]="july",
   ["fr"]="juillet",
   ["it"]="luglio",
   ["nl"]="juli",
   ["pe"]="ژولای",
   ["ro"]="iulie",
  },
  ["june"]={
   ["cs"]="cerven",
   ["de"]="juni",
   ["en"]="june",
   ["fr"]="juin",
   ["it"]="giugno",
   ["nl"]="juni",
   ["pe"]="ژوئن",
   ["ro"]="iunie",
  },
  ["keep"]={
   ["cs"]="drzet",
   ["de"]="behalte",
   ["en"]="keep",
   ["fr"]="conserver",
   ["it"]="mantieni",
   ["nl"]="handhaaf",
   ["pe"]="نگهدار",
   ["ro"]="mentine",
  },
  ["keeptogether"]={
   ["en"]="keeptogether",
   ["fr"]="conserverensemble",
  },
  ["kerncharacters"]={
   ["en"]="kerncharacters",
   ["fr"]="approchercaracteres",
  },
  ["knockout"]={
   ["cs"]="knockout",
   ["de"]="knockout",
   ["en"]="knockout",
   ["fr"]="knockout",
   ["it"]="knockout",
   ["nl"]="knockout",
   ["pe"]="knockout",
   ["ro"]="knockout",
  },
  ["label"]={
   ["cs"]="popisek",
   ["de"]="label",
   ["en"]="label",
   ["fr"]="etiquette",
   ["it"]="etichetta",
   ["nl"]="label",
   ["pe"]="برچسب",
   ["ro"]="eticheta",
  },
  ["landscape"]={
   ["cs"]="nasirku",
   ["de"]="querformat",
   ["en"]="landscape",
   ["fr"]="paysage",
   ["it"]="orizzontale",
   ["nl"]="liggend",
   ["pe"]="صفحه‌گسترده",
   ["ro"]="vedere",
  },
  ["last"]={
   ["cs"]="posledni",
   ["de"]="letzte",
   ["en"]="last",
   ["fr"]="dernier",
   ["it"]="ultimo",
   ["nl"]="laatste",
   ["pe"]="آخرین",
   ["ro"]="ultim",
  },
  ["lastcolumn"]={
   ["cs"]="lastcolumn",
   ["de"]="lastcolumn",
   ["en"]="lastcolumn",
   ["fr"]="dernierecolonne",
   ["it"]="lastcolumn",
   ["nl"]="laatstekolom",
   ["pe"]="آخرین‌ستون",
   ["ro"]="lastcolumn",
  },
  ["lastpage"]={
   ["cs"]="poslednistrana",
   ["de"]="letzteseite",
   ["en"]="lastpage",
   ["fr"]="dernierepage",
   ["it"]="ultimapagina",
   ["nl"]="laatstepagina",
   ["pe"]="صفحه‌آخر",
   ["ro"]="ultimapagina",
  },
  ["lastpagenumber"]={
   ["cs"]="lastpagenumber",
   ["de"]="lastpagenumber",
   ["en"]="lastpagenumber",
   ["fr"]="numerodenierepage",
   ["it"]="lastpagenumber",
   ["nl"]="laatstepaginanummer",
   ["pe"]="شماه‌صفحه‌آخر",
   ["ro"]="lastpagenumber",
  },
  ["lastsubpage"]={
   ["cs"]="poslaednipodstranka",
   ["de"]="letzteunterseite",
   ["en"]="lastsubpage",
   ["fr"]="dernieresouspage",
   ["it"]="ultimasottopagina",
   ["nl"]="laatstesubpagina",
   ["pe"]="زیرصفحه‌آخر",
   ["ro"]="ultimasubpagina",
  },
  ["layer"]={
   ["cs"]="layer",
   ["de"]="layer",
   ["en"]="layer",
   ["fr"]="calque",
   ["it"]="layer",
   ["nl"]="layer",
   ["pe"]="layer",
   ["ro"]="layer",
  },
  ["left"]={
   ["cs"]="vlevo",
   ["de"]="links",
   ["en"]="left",
   ["fr"]="gauche",
   ["it"]="sinistra",
   ["nl"]="links",
   ["pe"]="چپ",
   ["ro"]="stanga",
  },
  ["leftedge"]={
   ["cs"]="levahrana",
   ["de"]="linkekante",
   ["en"]="leftedge",
   ["fr"]="bordgauche",
   ["it"]="bordosinistro",
   ["nl"]="linkerrand",
   ["pe"]="لبه‌چپ",
   ["ro"]="bordurastanga",
  },
  ["lefthanging"]={
   ["cs"]="lefthanging",
   ["de"]="lefthanging",
   ["en"]="lefthanging",
   ["fr"]="suspensiongauche",
   ["it"]="lefthanging",
   ["nl"]="linkshangend",
   ["pe"]="آویزان‌چپ",
   ["ro"]="lefthanging",
  },
  ["leftmargin"]={
   ["cs"]="levyokraj",
   ["de"]="linkerrand",
   ["en"]="leftmargin",
   ["fr"]="margegauche",
   ["it"]="marginesinistro",
   ["nl"]="linkermarge",
   ["pe"]="حاشیه‌چپ",
   ["ro"]="marginestanga",
  },
  ["lefttext"]={
   ["cs"]="textvlevo",
   ["de"]="linkertext",
   ["en"]="lefttext",
   ["fr"]="textegauche",
   ["it"]="testosinistro",
   ["nl"]="linkertekst",
   ["pe"]="متن‌چپ",
   ["ro"]="textstanga",
  },
  ["leftpage"]={
   ["cs"]="levastranka",
   ["de"]="linkerseite",
   ["en"]="leftpage",
   ["fr"]="surpagegauche",
   ["it"]="paginasinistra",
   ["nl"]="linkerpagina",
   ["pe"]="صفحه‌چپ",
   ["ro"]="paginastanga",
  },
  ["lefttoright"]={
   ["cs"]="lefttoright",
   ["de"]="lefttoright",
   ["en"]="lefttoright",
   ["fr"]="gaucheadroite",
   ["it"]="lefttoright",
   ["nl"]="lefttoright",
   ["pe"]="lefttoright",
   ["ro"]="lefttoright",
  },
  ["legend"]={
   ["cs"]="legenda",
   ["de"]="legende",
   ["en"]="legend",
   ["fr"]="legende",
   ["it"]="legenda",
   ["nl"]="legenda",
   ["pe"]="راهنما",
   ["ro"]="legenda",
  },
  ["less"]={
   ["en"]="less",
   ["fr"]="moins",
   ["nl"]="minder",
  },
  ["lesshyphenation"]={
   ["cs"]="lesshyphenation",
   ["de"]="lesshyphenation",
   ["en"]="lesshyphenation",
   ["fr"]="moinsdecesure",
   ["it"]="lesshyphenation",
   ["nl"]="lesshyphenation",
   ["pe"]="شکست‌کلمات‌کمتر",
   ["ro"]="lesshyphenation",
  },
  ["letterspacing"]={
   ["en"]="letterspacing",
   ["fr"]="espacementlettre",
  },
  ["limit"]={
   ["en"]="limit",
   ["nl"]="beperk",
  },
  ["line"]={
   ["cs"]="radek",
   ["de"]="zeile",
   ["en"]="line",
   ["fr"]="ligne",
   ["it"]="riga",
   ["nl"]="regel",
   ["pe"]="خط",
   ["ro"]="linie",
  },
  ["linenote"]={
   ["cs"]="linenote",
   ["de"]="linenote",
   ["en"]="linenote",
   ["fr"]="noteligne",
   ["it"]="linenote",
   ["nl"]="regelnoot",
   ["pe"]="خط‌نوشت",
   ["ro"]="linenote",
  },
  ["lines"]={
   ["cs"]="radky",
   ["de"]="zeilen",
   ["en"]="lines",
   ["fr"]="lignes",
   ["it"]="righe",
   ["nl"]="regels",
   ["pe"]="خطها",
   ["ro"]="linii",
  },
  ["linked"]={
   ["en"]="linked",
   ["fr"]="lie",
  },
  ["list"]={
   ["cs"]="seznam",
   ["de"]="liste",
   ["en"]="list",
   ["fr"]="liste",
   ["it"]="elenco",
   ["nl"]="lijst",
   ["pe"]="لیست",
   ["ro"]="lista",
  },
  ["local"]={
   ["cs"]="lokalne",
   ["de"]="lokal",
   ["en"]="local",
   ["fr"]="local",
   ["it"]="locale",
   ["nl"]="lokaal",
   ["pe"]="موضعی",
   ["ro"]="local",
  },
  ["localenvironment"]={
   ["cs"]="lokalnihoprostredi",
   ["de"]="lokaleumgebung",
   ["en"]="localenvironment",
   ["fr"]="environnementlocal",
   ["it"]="ambientelocale",
   ["nl"]="deelomgeving",
   ["pe"]="محیط‌موضعی",
   ["ro"]="mediulocal",
  },
  ["logo"]={
   ["cs"]="logo",
   ["de"]="logo",
   ["en"]="logo",
   ["fr"]="logo",
   ["it"]="logo",
   ["nl"]="logo",
   ["pe"]="آرم",
   ["ro"]="logo",
  },
  ["logos"]={
   ["cs"]="loga",
   ["de"]="logos",
   ["en"]="logos",
   ["fr"]="logos",
   ["it"]="loghi",
   ["nl"]="logos",
   ["pe"]="آرمها",
   ["ro"]="logos",
  },
  ["lohi"]={
   ["cs"]="nivy",
   ["de"]="hoti",
   ["en"]="lohi",
   ["fr"]="inex",
   ["it"]="pedap",
   ["nl"]="laho",
   ["pe"]="پابا",
   ["ro"]="lohi",
  },
  ["long"]={
   ["en"]="long",
   ["fr"]="long",
  },
  ["loose"]={
   ["cs"]="uvolnene",
   ["de"]="lose",
   ["en"]="loose",
   ["fr"]="envrac",
   ["it"]="lento",
   ["nl"]="los",
   ["pe"]="شل",
   ["ro"]="larg",
  },
  ["low"]={
   ["cs"]="nizko",
   ["de"]="tief",
   ["en"]="low",
   ["fr"]="ind",
   ["it"]="basso",
   ["nl"]="laag",
   ["pe"]="پایین",
   ["ro"]="jos",
  },
  ["ls"]={
   ["cs"]="ls",
   ["de"]="ls",
   ["en"]="ls",
   ["fr"]="ls",
   ["it"]="ls",
   ["nl"]="ls",
   ["pe"]="ls",
   ["ro"]="ls",
  },
  ["makeup"]={
   ["cs"]="zlom",
   ["de"]="umbruch",
   ["en"]="makeup",
   ["fr"]="composition",
   ["it"]="makeup",
   ["nl"]="opmaak",
   ["pe"]="آرایش",
   ["ro"]="marcaj",
  },
  ["mar"]={
   ["cs"]="mar",
   ["de"]="mar",
   ["en"]="mar",
   ["fr"]="rem",
   ["it"]="mar",
   ["nl"]="mar",
   ["pe"]="حاش",
   ["ro"]="mar",
  },
  ["march"]={
   ["cs"]="brezen",
   ["de"]="maerz",
   ["en"]="march",
   ["fr"]="mars",
   ["it"]="marzo",
   ["nl"]="maart",
   ["pe"]="مارس",
   ["ro"]="martie",
  },
  ["margin"]={
   ["cs"]="marginalie",
   ["de"]="marginalie",
   ["en"]="margin",
   ["fr"]="marge",
   ["it"]="margine",
   ["nl"]="marge",
   ["pe"]="حاشیه",
   ["ro"]="margine",
  },
  ["marginedge"]={
   ["cs"]="textovahrana",
   ["de"]="marginalkante",
   ["en"]="marginedge",
   ["fr"]="margebord",
   ["it"]="bordomargine",
   ["nl"]="kantlijn",
   ["pe"]="لبه‌حاشیه",
   ["ro"]="marginebordura",
  },
  ["margintext"]={
   ["cs"]="marginalnitext",
   ["de"]="marginaltext",
   ["en"]="margintext",
   ["fr"]="textemarge",
   ["it"]="testoinmargine",
   ["nl"]="margetekst",
   ["pe"]="متن‌حاشیه",
   ["ro"]="textmarginal",
  },
  ["margintitle"]={
   ["cs"]="titulmarginalie",
   ["de"]="marginaltitel",
   ["en"]="margintitle",
   ["fr"]="titremarge",
   ["it"]="titoloinmargine",
   ["nl"]="margetitel",
   ["pe"]="عنوان‌حاشیه",
   ["ro"]="titlumarginal",
  },
  ["marking"]={
   ["cs"]="znaceni",
   ["de"]="beschriftung",
   ["en"]="marking",
   ["fr"]="marquage",
   ["it"]="marcatura",
   ["nl"]="markering",
   ["pe"]="نشانه‌گذاری",
   ["ro"]="marcaje",
  },
  ["math"]={
   ["en"]="math",
   ["fr"]="math",
  },
  ["mathalignment"]={
   ["cs"]="mathalignment",
   ["de"]="mathalignment",
   ["en"]="mathalignment",
   ["fr"]="alignementmath",
   ["it"]="mathalignment",
   ["nl"]="wiskundeuitlijnen",
   ["pe"]="تنظیم‌ریاضی",
   ["ro"]="mathalignment",
  },
  ["mathcases"]={
   ["cs"]="mathcases",
   ["de"]="mathcases",
   ["en"]="mathcases",
   ["fr"]="casmath",
   ["it"]="mathcases",
   ["nl"]="mathcases",
   ["pe"]="حالتهای‌ریاضی",
   ["ro"]="mathcases",
  },
  ["mathematics"]={
   ["cs"]="mathematika",
   ["de"]="mathematik",
   ["en"]="mathematics",
   ["fr"]="mathematics",
   ["it"]="matematica",
   ["nl"]="wiskunde",
   ["pe"]="ریاضی",
   ["ro"]="matematica",
  },
  ["mathmatrix"]={
   ["cs"]="mathmatrix",
   ["de"]="mathmatrix",
   ["en"]="mathmatrix",
   ["fr"]="matricemath",
   ["it"]="mathmatrix",
   ["nl"]="wiskundematrix",
   ["pe"]="ماتریش‌ریاضی",
   ["ro"]="mathmatrix",
  },
  ["max"]={
   ["cs"]="max",
   ["de"]="max",
   ["en"]="max",
   ["fr"]="max",
   ["it"]="max",
   ["nl"]="max",
   ["pe"]="بیشترین",
   ["ro"]="max",
  },
  ["maxdepth"]={
   ["en"]="maxdepth",
   ["fr"]="profondeurmax",
   ["nl"]="maxdiepte",
  },
  ["maxheight"]={
   ["en"]="maxheight",
   ["fr"]="hauteurmax",
   ["nl"]="maxhoogte",
  },
  ["maxwidth"]={
   ["en"]="maxwidth",
   ["fr"]="largeurmax",
   ["nl"]="maxbreedte",
  },
  ["may"]={
   ["cs"]="kveten",
   ["de"]="mai",
   ["en"]="may",
   ["fr"]="mai",
   ["it"]="maggio",
   ["nl"]="mei",
   ["pe"]="مه",
   ["ro"]="mai",
  },
  ["mediaeval"]={
   ["cs"]="stredoveky",
   ["de"]="mittelalterlich",
   ["en"]="mediaeval",
   ["fr"]="medieval",
   ["it"]="medievale",
   ["nl"]="mediaeval",
   ["pe"]="mediaeval",
   ["ro"]="medieval",
  },
  ["medium"]={
   ["cs"]="stredni",
   ["de"]="mittel",
   ["en"]="medium",
   ["fr"]="moyen",
   ["it"]="medio",
   ["nl"]="middel",
   ["pe"]="متوسط",
   ["ro"]="mediu",
  },
  ["middle"]={
   ["cs"]="nastred",
   ["de"]="mittig",
   ["en"]="middle",
   ["fr"]="milieu",
   ["it"]="centro",
   ["nl"]="midden",
   ["pe"]="میان",
   ["ro"]="centru",
  },
  ["min"]={
   ["cs"]="min",
   ["de"]="min",
   ["en"]="min",
   ["fr"]="min",
   ["it"]="min",
   ["nl"]="min",
   ["pe"]="کمترین",
   ["ro"]="min",
  },
  ["mindepth"]={
   ["en"]="mindepth",
   ["fr"]="profondeurmin",
   ["nl"]="mindiepte",
  },
  ["minheight"]={
   ["cs"]="minvyska",
   ["de"]="minhoehe",
   ["en"]="minheight",
   ["fr"]="hauteurmin",
   ["it"]="altezzamin",
   ["nl"]="minhoogte",
   ["pe"]="کمترین‌ارتفاع",
   ["ro"]="inaltimeminima",
  },
  ["minwidth"]={
   ["cs"]="minsirka",
   ["de"]="minbreite",
   ["en"]="minwidth",
   ["fr"]="largeurmin",
   ["it"]="ampiezzamin",
   ["nl"]="minbreedte",
   ["pe"]="کمترین‌عرض",
   ["ro"]="latimeminima",
  },
  ["mirrored"]={
   ["cs"]="zrcadleno",
   ["de"]="gespiegelt",
   ["en"]="mirrored",
   ["fr"]="reflete",
   ["it"]="riflesso",
   ["nl"]="gespiegeld",
   ["pe"]="منعکس",
   ["ro"]="oglindit",
  },
  ["mixed"]={
   ["en"]="mixed",
   ["fr"]="mixte",
  },
  ["monday"]={
   ["cs"]="pondeli",
   ["de"]="montag",
   ["en"]="monday",
   ["fr"]="lundi",
   ["it"]="lunedi",
   ["nl"]="maandag",
   ["pe"]="دوشنبه",
   ["ro"]="luni",
  },
  ["mono"]={
   ["cs"]="mono",
   ["de"]="mono",
   ["en"]="mono",
   ["fr"]="mono",
   ["it"]="mono",
   ["nl"]="mono",
   ["pe"]="مونو",
   ["ro"]="mono",
  },
  ["monobold"]={
   ["cs"]="monotucne",
   ["de"]="monofett",
   ["en"]="monobold",
   ["fr"]="monogras",
   ["it"]="monograssetto",
   ["nl"]="monovet",
   ["pe"]="monobold",
   ["ro"]="monoaldin",
  },
  ["mononormal"]={
   ["en"]="mononormal",
   ["fr"]="normalmono",
  },
  ["month"]={
   ["cs"]="mesic",
   ["de"]="monat",
   ["en"]="month",
   ["fr"]="mois",
   ["it"]="mese",
   ["nl"]="maand",
   ["pe"]="ماه",
   ["ro"]="luna",
  },
  ["more"]={
   ["en"]="more",
   ["fr"]="plus",
   ["nl"]="meer",
  },
  ["morehyphenation"]={
   ["cs"]="morehyphenation",
   ["de"]="morehyphenation",
   ["en"]="morehyphenation",
   ["fr"]="plusdecesure",
   ["it"]="morehyphenation",
   ["nl"]="morehyphenation",
   ["pe"]="شکست‌کلمات‌بیشتر",
   ["ro"]="morehyphenation",
  },
  ["name"]={
   ["cs"]="jmeno",
   ["de"]="name",
   ["en"]="name",
   ["fr"]="nom",
   ["it"]="nome",
   ["nl"]="naam",
   ["pe"]="نام",
   ["ro"]="nume",
  },
  ["narrow"]={
   ["cs"]="uzky",
   ["de"]="schmall",
   ["en"]="narrow",
   ["fr"]="etroit",
   ["it"]="stretto",
   ["nl"]="smal",
   ["pe"]="نازک",
   ["ro"]="ingust",
  },
  ["negative"]={
   ["cs"]="negativ",
   ["de"]="negativ",
   ["en"]="negative",
   ["fr"]="negatif",
   ["it"]="negativo",
   ["nl"]="negatief",
   ["pe"]="منفی",
   ["ro"]="negativ",
  },
  ["never"]={
   ["cs"]="nikdy",
   ["de"]="nie",
   ["en"]="never",
   ["fr"]="jamais",
   ["it"]="mai",
   ["nl"]="nooit",
   ["pe"]="هیچگاه",
   ["ro"]="niciodata",
  },
  ["new"]={
   ["cs"]="novy",
   ["de"]="neu",
   ["en"]="new",
   ["fr"]="nouveau",
   ["it"]="nuovo",
   ["nl"]="nieuw",
   ["pe"]="جدید",
   ["ro"]="nou",
  },
  ["next"]={
   ["cs"]="dalsi",
   ["de"]="folgende",
   ["en"]="next",
   ["fr"]="suivant",
   ["it"]="successivo",
   ["nl"]="volgende",
   ["pe"]="بعدی",
   ["ro"]="urmatorul",
  },
  ["nextevenpage"]={
   ["cs"]="dalsisudastranka",
   ["de"]="naechstegeradeseite",
   ["en"]="nextevenpage",
   ["fr"]="pagepairesuivante",
   ["it"]="paginaparisuccessiva",
   ["nl"]="volgendeevenpagina",
   ["pe"]="صفحه‌زوج‌بعدی",
   ["ro"]="paginaparaurmatoarea",
  },
  ["nextoddpage"]={
   ["cs"]="dalsilichastranka",
   ["de"]="naechsteungeradeseite",
   ["en"]="nextoddpage",
   ["fr"]="pageimpairesuivante",
   ["it"]="paginadisparisuccessiva",
   ["nl"]="volgendeonevenpagina",
   ["pe"]="صفحه‌فردبعدی",
   ["ro"]="paginaimparaurmatoarea",
  },
  ["nextpage"]={
   ["cs"]="dalsistranka",
   ["de"]="folgendeseite",
   ["en"]="nextpage",
   ["fr"]="pagesuivante",
   ["it"]="paginasuccessiva",
   ["nl"]="volgendepagina",
   ["pe"]="صفحه‌بعدی",
   ["ro"]="paginaurmatoare",
  },
  ["nextsubpage"]={
   ["cs"]="dalsipodstranka",
   ["de"]="folgendeunterseite",
   ["en"]="nextsubpage",
   ["fr"]="souspagesuivante",
   ["it"]="sottopaginasuccessiva",
   ["nl"]="volgendesubpagina",
   ["pe"]="زیرصفحه‌بعدی",
   ["ro"]="urmatoareasubpagina",
  },
  ["nine"]={
   ["cs"]="nine",
   ["de"]="neun",
   ["en"]="nine",
   ["fr"]="neuf",
   ["it"]="nine",
   ["nl"]="negen",
   ["pe"]="nine",
   ["ro"]="nine",
  },
  ["no"]={
   ["cs"]="ne",
   ["de"]="nein",
   ["en"]="no",
   ["fr"]="non",
   ["it"]="no",
   ["nl"]="nee",
   ["pe"]="نه",
   ["ro"]="nu",
  },
  ["nocheck"]={
   ["cs"]="nocheck",
   ["de"]="nocheck",
   ["en"]="nocheck",
   ["fr"]="sansverifier",
   ["it"]="nocheck",
   ["nl"]="geencontrole",
   ["pe"]="بدون‌بررسی",
   ["ro"]="nocheck",
  },
  ["nodepth"]={
   ["en"]="nodepth",
   ["fr"]="sansprodondeur",
  },
  ["nofit"]={
   ["cs"]="nofit",
   ["de"]="nofit",
   ["en"]="nofit",
   ["fr"]="sansadapter",
   ["it"]="nofit",
   ["nl"]="nietpassend",
   ["pe"]="بدون‌پرکردن",
   ["ro"]="nofit",
  },
  ["nogrid"]={
   ["cs"]="nogrid",
   ["de"]="nogrid",
   ["en"]="nogrid",
   ["fr"]="sansgrille",
   ["it"]="nogrid",
   ["nl"]="geengrid",
   ["pe"]="بدون‌توری",
   ["ro"]="nogrid",
  },
  ["noheight"]={
   ["en"]="noheight",
   ["fr"]="sanshauteur",
  },
  ["nohz"]={
   ["cs"]="nohz",
   ["de"]="nohz",
   ["en"]="nohz",
   ["fr"]="sanshz",
   ["it"]="nohz",
   ["nl"]="geenhz",
   ["pe"]="nohz",
   ["ro"]="nohz",
  },
  ["nointro"]={
   ["en"]="nointro",
   ["nl"]="nointro",
  },
  ["noline"]={
   ["en"]="noline",
   ["fr"]="sansligne",
  },
  ["nomarking"]={
   ["cs"]="zadneznaceni",
   ["de"]="keinebeschriftung",
   ["en"]="nomarking",
   ["fr"]="sansmarquage",
   ["it"]="nomarcature",
   ["nl"]="geenmarkering",
   ["pe"]="بدون‌نشانه‌گذاری",
   ["ro"]="faramarcare",
  },
  ["nomenubar"]={
   ["en"]="nomenubar",
   ["fr"]="sansbarremenu",
   ["nl"]="geenmenubalk",
  },
  ["none"]={
   ["cs"]="zadny",
   ["de"]="kein",
   ["en"]="none",
   ["fr"]="aucun",
   ["it"]="nessuno",
   ["nl"]="geen",
   ["pe"]="هیچکدام",
   ["ro"]="niciunul",
  },
  ["nonumber"]={
   ["cs"]="nonumber",
   ["de"]="nonumber",
   ["en"]="nonumber",
   ["fr"]="sansnumero",
   ["it"]="nonumber",
   ["nl"]="geennummer",
   ["pe"]="بدون‌شماره",
   ["ro"]="nonumber",
  },
  ["norepeat"]={
   ["en"]="norepeat",
   ["fr"]="sansrepetition",
  },
  ["normal"]={
   ["cs"]="normalni",
   ["de"]="normal",
   ["en"]="normal",
   ["fr"]="normal",
   ["it"]="normale",
   ["nl"]="normaal",
   ["pe"]="نرمال",
   ["ro"]="normal",
  },
  ["normalshort"]={
   ["en"]="normalshort",
  },
  ["nospacing"]={
   ["cs"]="nospacing",
   ["de"]="nospacing",
   ["en"]="nospacing",
   ["fr"]="sansespacement",
   ["it"]="nospacing",
   ["nl"]="geenspatiering",
   ["pe"]="بدون‌فضاگذاری",
   ["ro"]="nospacing",
  },
  ["nosnapping"]={
    ["en"]="nosnapping",
  },
  ["nostopper"]={
   ["en"]="nostopper",
   ["fr"]="sansstoppeur",
   ["nl"]="geenafsluiter",
  },
  ["not"]={
   ["cs"]="ne",
   ["de"]="nicht",
   ["en"]="not",
   ["fr"]="pas",
   ["it"]="non",
   ["nl"]="niet",
   ["pe"]="بدون",
   ["ro"]="nu",
  },
  ["notation"]={
   ["en"]="notation",
   ["fr"]="notation",
  },
  ["note"]={
   ["en"]="note",
   ["fr"]="note",
  },
  ["nothanging"]={
   ["cs"]="nothanging",
   ["de"]="nothanging",
   ["en"]="nothanging",
   ["fr"]="sanssuspension",
   ["it"]="nonsospeso",
   ["nl"]="niethangend",
   ["pe"]="بدون‌آویزان‌کردن",
   ["ro"]="nothanging",
  },
  ["explicit"]={
   ["en"]="explicit",
  },
  ["notcollapsed"]={
   ["en"]="notcollapsed",
  },
  ["nothyphenated"]={
   ["cs"]="nothyphenated",
   ["de"]="nothyphenated",
   ["en"]="nothyphenated",
   ["fr"]="sanscesure",
   ["it"]="nonsillabato",
   ["nl"]="nietafgebroken",
   ["pe"]="بدون‌شکست",
   ["ro"]="nedespsilabe",
  },
  ["notjoinedup"]={
   ["en"]="notjoinedup",
   ["fr"]="passansespaceentre",
   ["nl"]="nietaansluitend",
  },
  ["november"]={
   ["cs"]="listopad",
   ["de"]="november",
   ["en"]="november",
   ["fr"]="novembre",
   ["it"]="novembre",
   ["nl"]="november",
   ["pe"]="نوامبر",
   ["ro"]="noiembrie",
  },
  ["nowhere"]={
   ["cs"]="nikde",
   ["de"]="nirgens",
   ["en"]="nowhere",
   ["fr"]="nullepart",
   ["it"]="danessunaparte",
   ["nl"]="nergens",
   ["pe"]="هیچ‌حا",
   ["ro"]="niciunde",
  },
  ["nowhite"]={
   ["cs"]="zadnabila",
   ["de"]="keinweiss",
   ["en"]="nowhite",
   ["fr"]="sansblanc",
   ["it"]="nobianco",
   ["nl"]="geenwit",
   ["pe"]="سفید‌نه",
   ["ro"]="faraalb",
  },
  ["number"]={
   ["cs"]="cislo",
   ["de"]="nummer",
   ["en"]="number",
   ["fr"]="numero",
   ["it"]="numero",
   ["nl"]="nummer",
   ["pe"]="شماره",
   ["ro"]="numar",
  },
  ["numbers"]={
   ["cs"]="cisla",
   ["de"]="ziffern",
   ["en"]="numbers",
   ["fr"]="numeros",
   ["it"]="numeri",
   ["nl"]="cijfers",
   ["pe"]="شماره‌ها",
   ["ro"]="numere",
  },
  ["october"]={
   ["cs"]="rijen",
   ["de"]="oktober",
   ["en"]="october",
   ["fr"]="octobre",
   ["it"]="ottobre",
   ["nl"]="oktober",
   ["pe"]="اکتبر",
   ["ro"]="octombrie",
  },
  ["odd"]={
   ["cs"]="liche",
   ["de"]="ungerade",
   ["en"]="odd",
   ["fr"]="impair",
   ["it"]="dispari",
   ["nl"]="oneven",
   ["pe"]="فرد",
   ["ro"]="impar",
  },
  ["off"]={
   ["cs"]="vyp",
   ["de"]="aus",
   ["en"]="off",
   ["fr"]="arret",
   ["it"]="disattivo",
   ["nl"]="uit",
   ["pe"]="خاموش",
   ["ro"]="dezactivat",
  },
  ["offset"]={
   ["cs"]="offset",
   ["de"]="offset",
   ["en"]="offset",
   ["fr"]="decalage",
   ["it"]="offset",
   ["nl"]="offset",
   ["pe"]="آفست",
   ["ro"]="offset",
  },
  ["old"]={
   ["cs"]="old",
   ["de"]="old",
   ["en"]="old",
   ["fr"]="ancien",
   ["it"]="old",
   ["nl"]="oud",
   ["pe"]="قدیمی",
   ["ro"]="old",
  },
  ["oldstyle"]={
   ["en"]="oldstyle",
   ["fr"]="styleancien",
  },
  ["on"]={
   ["cs"]="zap",
   ["de"]="an",
   ["en"]="on",
   ["fr"]="marche",
   ["it"]="attivo",
   ["nl"]="aan",
   ["pe"]="روی",
   ["ro"]="on",
  },
  ["one"]={
   ["cs"]="jedna",
   ["de"]="eins",
   ["en"]="one",
   ["fr"]="un",
   ["it"]="uno",
   ["nl"]="een",
   ["pe"]="یک",
   ["ro"]="unu",
  },
  ["opposite"]={
   ["cs"]="naproti",
   ["de"]="gegenueber",
   ["en"]="opposite",
   ["fr"]="enface",
   ["it"]="opposto",
   ["nl"]="naast",
   ["pe"]="مخالف",
   ["ro"]="opus",
  },
  ["ordinal"]={
   ["en"]="ordinal",
   ["fr"]="ordinal",
  },
  ["outer"]={
   ["cs"]="vnejsi",
   ["de"]="aussen",
   ["en"]="outer",
   ["fr"]="exterieur",
   ["it"]="esterno",
   ["nl"]="buiten",
   ["pe"]="خارجی",
   ["ro"]="extern",
  },
  ["outeredge"]={
   ["cs"]="outeredge",
   ["de"]="outeredge",
   ["en"]="outeredge",
   ["fr"]="bordexterieur",
   ["it"]="bordoesterno",
   ["nl"]="buitenrand",
   ["pe"]="لبه‌خارجی",
   ["ro"]="outeredge",
  },
  ["outlined"]={
   ["en"]="outlined",
  },
  ["outline"]={
   ["en"]="outline",
  },
  ["outermargin"]={
   ["cs"]="outermargin",
   ["de"]="outermargin",
   ["en"]="outermargin",
   ["fr"]="margeexterieure",
   ["it"]="margineesterno",
   ["nl"]="buitenmarge",
   ["pe"]="حاشیه‌خارجی",
   ["ro"]="outermargin",
  },
  ["overbar"]={
   ["cs"]="nadtrzeno",
   ["de"]="ueberstrichen",
   ["en"]="overbar",
   ["fr"]="surligner",
   ["it"]="sopralinea",
   ["nl"]="overstreep",
   ["pe"]="میله‌رو",
   ["ro"]="barasus",
  },
  ["overbars"]={
   ["cs"]="nadtrzeni",
   ["de"]="ueberstreichen",
   ["en"]="overbars",
   ["fr"]="surlignermots",
   ["it"]="sopralinee",
   ["nl"]="overstrepen",
   ["pe"]="میله‌ها‌رو",
   ["ro"]="supralinie",
  },
  ["overlay"]={
   ["cs"]="prekryv",
   ["de"]="overlay",
   ["en"]="overlay",
   ["fr"]="superposition",
   ["it"]="sovrapponi",
   ["nl"]="overlay",
   ["pe"]="پوشش",
   ["ro"]="overlay",
  },
  ["overprint"]={
   ["cs"]="overprint",
   ["de"]="overprint",
   ["en"]="overprint",
   ["fr"]="surimpression",
   ["it"]="overprint",
   ["nl"]="overprint",
   ["pe"]="overprint",
   ["ro"]="overprint",
  },
  ["overstrike"]={
   ["cs"]="preskrtnuto",
   ["de"]="durchgestrichen",
   ["en"]="overstrike",
   ["fr"]="barrer",
   ["it"]="sbarrato",
   ["nl"]="doorstreep",
   ["pe"]="خط‌زدن",
   ["ro"]="liniepeste",
  },
  ["overstrikes"]={
   ["cs"]="preskrtnuti",
   ["de"]="durchstreichen",
   ["en"]="overstrikes",
   ["fr"]="barrermots",
   ["it"]="sbarrati",
   ["nl"]="doorstrepen",
   ["pe"]="خط‌زدنها",
   ["ro"]="liniipeste",
  },
  ["packed"]={
   ["cs"]="zhustene",
   ["de"]="gepackt",
   ["en"]="packed",
   ["fr"]="sansespaceentreitem",
   ["it"]="impaccato",
   ["nl"]="opelkaar",
   ["pe"]="فشرده",
   ["ro"]="impachetat",
  },
  ["page"]={
   ["cs"]="stranka",
   ["de"]="seite",
   ["en"]="page",
   ["fr"]="page",
   ["it"]="pagina",
   ["nl"]="pagina",
   ["pe"]="صفحه",
   ["ro"]="pagina",
  },
  ["pagecomment"]={
   ["cs"]="komentarstranky",
   ["de"]="pagecomment",
   ["en"]="pagecomment",
   ["fr"]="commentairepage",
   ["it"]="commentopagina",
   ["nl"]="paginacommentaar",
   ["pe"]="توضیح‌صفحه",
   ["ro"]="comentariupagina",
  },
  ["pagenumber"]={
   ["cs"]="cislostranky",
   ["de"]="seitennummer",
   ["en"]="pagenumber",
   ["fr"]="numeropage",
   ["it"]="numeropagina",
   ["nl"]="paginanummer",
   ["pe"]="شماره‌صفحه",
   ["ro"]="numarpagina",
  },
  ["paper"]={
   ["cs"]="papir",
   ["de"]="papier",
   ["en"]="paper",
   ["fr"]="papier",
   ["it"]="carta",
   ["nl"]="papier",
   ["pe"]="برگ",
   ["ro"]="foaie",
  },
  ["paragraph"]={
   ["cs"]="odstavec",
   ["de"]="absatz",
   ["en"]="paragraph",
   ["fr"]="paragraphe",
   ["it"]="capoverso",
   ["nl"]="alinea",
   ["pe"]="پاراگراف",
   ["ro"]="paragraf",
  },
  ["part"]={
   ["cs"]="cast",
   ["de"]="teil",
   ["en"]="part",
   ["fr"]="partie",
   ["it"]="parte",
   ["nl"]="deel",
   ["pe"]="قسمت",
   ["ro"]="parte",
  },
  ["positive"]={
   ["cs"]="positiv",
   ["de"]="positiv",
   ["en"]="positive",
   ["fr"]="positif",
   ["it"]="positivo",
   ["nl"]="positief",
   ["pe"]="مثبت",
   ["ro"]="positiv",
  },
  ["postponing"]={
   ["cs"]="odlozit",
   ["de"]="verschieben",
   ["en"]="postponing",
   ["fr"]="report",
   ["it"]="posporre",
   ["nl"]="uitstellen",
   ["pe"]="تاخیر",
   ["ro"]="postponing",
  },
  ["postscript"]={
   ["cs"]="postscript",
   ["de"]="postscript",
   ["en"]="postscript",
   ["fr"]="postscript",
   ["it"]="postscript",
   ["nl"]="postscript",
   ["pe"]="پست‌اسکریپت",
   ["ro"]="postscript",
  },
  ["precedingpage"]={
   ["en"]="precedingpage",
   ["fr"]="pageantecedent",
   ["nl"]="voorafgaandepagina",
  },
  ["preference"]={
   ["cs"]="nastaveni",
   ["de"]="einstellung",
   ["en"]="preference",
   ["fr"]="preference",
   ["it"]="preferenza",
   ["nl"]="voorkeur",
   ["pe"]="ترجیح",
   ["ro"]="preferinta",
  },
  ["preview"]={
   ["cs"]="nahled",
   ["de"]="vorschau",
   ["en"]="preview",
   ["fr"]="apercu",
   ["it"]="anteprima",
   ["nl"]="preview",
   ["pe"]="پیش‌دید",
   ["ro"]="previzualizare",
  },
  ["previous"]={
   ["cs"]="predchozi",
   ["de"]="vorig",
   ["en"]="previous",
   ["fr"]="precedent",
   ["it"]="precedente",
   ["nl"]="vorige",
   ["pe"]="قبلی",
   ["ro"]="precedent",
  },
  ["previousevenpage"]={
   ["cs"]="predchozisudastranka",
   ["de"]="vorigegeradeseite",
   ["en"]="previousevenpage",
   ["fr"]="pagepaireprecedente",
   ["it"]="paginapariprecedente",
   ["nl"]="vorigeevenpagina",
   ["pe"]="صفحه‌زوج‌قبلی",
   ["ro"]="paginaparaprecedenta",
  },
  ["previousoddpage"]={
   ["cs"]="predchozilichastranka",
   ["de"]="vorigeungeradeseite",
   ["en"]="previousoddpage",
   ["fr"]="pageimpaireprecedente",
   ["it"]="paginadispariprecedente",
   ["nl"]="vorigeonevenpagina",
   ["pe"]="صفحه‌فردقبلی",
   ["ro"]="paginaimparaprecedenta",
  },
  ["previouspage"]={
   ["cs"]="predchozistranka",
   ["de"]="vorigeseite",
   ["en"]="previouspage",
   ["fr"]="pageprecedente",
   ["it"]="paginaprecedente",
   ["nl"]="vorigepagina",
   ["pe"]="صفحه‌قبلی",
   ["ro"]="paginaanterioara",
  },
  ["previoussubpage"]={
   ["cs"]="predchozipodstranka",
   ["de"]="vorigeunterseite",
   ["en"]="previoussubpage",
   ["fr"]="souspageprecedente",
   ["it"]="sottopaginaprecedente",
   ["nl"]="vorigesubpagina",
   ["pe"]="زیرصفحه‌قبلی",
   ["ro"]="subpaginaanterioara",
  },
  ["print"]={
   ["en"]="print",
   ["fr"]="imprimer",
   ["nl"]="print",
  },
  ["printable"]={
   ["cs"]="tisknutelne",
   ["de"]="druckbar",
   ["en"]="printable",
   ["fr"]="imprimable",
   ["it"]="stampabile",
   ["nl"]="printbaar",
   ["pe"]="قابل‌چاپ",
   ["ro"]="tiparibil",
  },
  ["process"]={
   ["cs"]="process",
   ["de"]="process",
   ["en"]="process",
   ["fr"]="traiter",
   ["it"]="process",
   ["nl"]="proces",
   ["pe"]="پردازش",
   ["ro"]="process",
  },
  ["product"]={
   ["cs"]="produkt",
   ["de"]="produkt",
   ["en"]="product",
   ["fr"]="produit",
   ["it"]="prodotto",
   ["nl"]="produkt",
   ["pe"]="محصول",
   ["ro"]="produs",
  },
  ["profile"]={
   ["en"]="profile",
   ["fr"]="profil",
  },
  ["program"]={
   ["cs"]="program",
   ["de"]="programm",
   ["en"]="program",
   ["fr"]="programme",
   ["it"]="programma",
   ["nl"]="programma",
   ["pe"]="برنامه",
   ["ro"]="program",
  },
  ["project"]={
   ["cs"]="projekt",
   ["de"]="projekt",
   ["en"]="project",
   ["fr"]="projet",
   ["it"]="progetto",
   ["nl"]="project",
   ["pe"]="پروژه",
   ["ro"]="proiect",
  },
  ["protected"]={
   ["cs"]="chranene",
   ["de"]="geschuetzt",
   ["en"]="protected",
   ["fr"]="protege",
   ["it"]="protetto",
   ["nl"]="beschermd",
   ["pe"]="حفاظت‌شده",
   ["ro"]="protejat",
  },
  ["quadruple"]={
   ["cs"]="ctyrnasobny",
   ["de"]="viertel",
   ["en"]="quadruple",
   ["fr"]="quadruple",
   ["it"]="quadruplo",
   ["nl"]="viertal",
   ["pe"]="quadruple",
   ["ro"]="cvadrupul",
  },
  ["quarterline"]={
   ["en"]="quarterline",
   ["fr"]="quartdeligne",
   ["nl"]="kwartregel",
  },
  ["quotation"]={
   ["cs"]="citace",
   ["de"]="zitat",
   ["en"]="quotation",
   ["fr"]="citation",
   ["it"]="citazione",
   ["nl"]="citaat",
   ["pe"]="نقل‌قول",
   ["ro"]="citat",
  },
  ["quote"]={
   ["cs"]="citovat",
   ["de"]="zitieren",
   ["en"]="quote",
   ["fr"]="citer",
   ["it"]="menzione",
   ["nl"]="citeer",
   ["pe"]="نقل",
   ["ro"]="minicitat",
  },
  ["ran"]={
   ["cs"]="rozsah",
   ["de"]="ran",
   ["en"]="ran",
   ["fr"]="gam",
   ["it"]="corse",
   ["nl"]="ran",
   ["pe"]="برد",
   ["ro"]="rulat",
  },
  ["random"]={
   ["cs"]="nahodny",
   ["de"]="zufaellig",
   ["en"]="random",
   ["fr"]="aleatoire",
   ["it"]="casuale",
   ["nl"]="willekeurig",
   ["pe"]="تصادفی",
   ["ro"]="aleator",
  },
  ["readonly"]={
   ["cs"]="pouzeprocteni",
   ["de"]="nurlesbar",
   ["en"]="readonly",
   ["fr"]="lectureseule",
   ["it"]="solalettura",
   ["nl"]="alleenleesbaar",
   ["pe"]="تنهاخواندنی",
   ["ro"]="readonly",
  },
  ["rectangular"]={
   ["cs"]="pravouhly",
   ["de"]="rechteckig",
   ["en"]="rectangular",
   ["fr"]="rectangulaire",
   ["it"]="rettangolare",
   ["nl"]="recht",
   ["pe"]="چهارگوشه",
   ["ro"]="rectangular",
  },
  ["reference"]={
   ["cs"]="odkaz",
   ["de"]="referenz",
   ["en"]="reference",
   ["fr"]="reference",
   ["it"]="riferimento",
   ["nl"]="referentie",
   ["pe"]="مرجع",
   ["ro"]="referinta",
  },
  ["referral"]={
   ["cs"]="znacka",
   ["de"]="merkmal",
   ["en"]="referral",
   ["fr"]="redirection",
   ["it"]="referral",
   ["nl"]="kenmerk",
   ["pe"]="مراجعه",
   ["ro"]="referinta",
  },
  ["region"]={
   ["en"]="region",
   ["fr"]="region",
   ["nl"]="gebied",
  },
  ["register"]={
   ["cs"]="rejstrik",
   ["de"]="register",
   ["en"]="register",
   ["fr"]="registre",
   ["it"]="registro",
   ["nl"]="register",
   ["pe"]="ثبت",
   ["ro"]="registru",
  },
  ["regular"]={
   ["cs"]="pravidelne",
   ["de"]="regular",
   ["en"]="regular",
   ["fr"]="ordinaire",
   ["it"]="regolare",
   ["nl"]="regular",
   ["pe"]="منظم",
   ["ro"]="regular",
  },
  ["relative"]={
   ["cs"]="relativni",
   ["de"]="relativ",
   ["en"]="relative",
   ["fr"]="relatif",
   ["it"]="relativo",
   ["nl"]="relatief",
   ["pe"]="نسبی",
   ["ro"]="relativ",
  },
  ["repeat"]={
   ["cs"]="opakovat",
   ["de"]="wiederholen",
   ["en"]="repeat",
   ["fr"]="repeter",
   ["it"]="ripeti",
   ["nl"]="herhaal",
   ["pe"]="تکرار",
   ["ro"]="repetat",
  },
  ["required"]={
   ["cs"]="pozadovane",
   ["de"]="obligatorisch",
   ["en"]="required",
   ["fr"]="requis",
   ["it"]="richiestp",
   ["nl"]="verplicht",
   ["pe"]="موردنیاز",
   ["ro"]="obligatoriu",
  },
  ["reset"]={
   ["cs"]="reset",
   ["de"]="zuruecksetzten",
   ["en"]="reset",
   ["fr"]="raz",
   ["it"]="reimposta",
   ["nl"]="reset",
   ["pe"]="بازنشانی",
   ["ro"]="reset",
  },
  ["reverse"]={
   ["cs"]="reverse",
   ["de"]="reverse",
   ["en"]="reverse",
   ["fr"]="inverser",
   ["it"]="invertito",
   ["nl"]="omgekeerd",
   ["pe"]="برعکس",
   ["ro"]="reverse",
  },
  ["right"]={
   ["cs"]="vpravo",
   ["de"]="rechts",
   ["en"]="right",
   ["fr"]="droite",
   ["it"]="destra",
   ["nl"]="rechts",
   ["pe"]="راست",
   ["ro"]="dreapta",
  },
  ["rightedge"]={
   ["cs"]="pravahrana",
   ["de"]="rechterkante",
   ["en"]="rightedge",
   ["fr"]="borddroit",
   ["it"]="bordodestro",
   ["nl"]="rechterrand",
   ["pe"]="لبه‌راست",
   ["ro"]="borduradreapta",
  },
  ["righthanging"]={
   ["cs"]="righthanging",
   ["de"]="righthanging",
   ["en"]="righthanging",
   ["fr"]="suspensiondroite",
   ["it"]="righthanging",
   ["nl"]="rechtshangend",
   ["pe"]="آویزان‌کردن‌راست",
   ["ro"]="righthanging",
  },
  ["rightmargin"]={
   ["cs"]="pravyokraj",
   ["de"]="rechterrand",
   ["en"]="rightmargin",
   ["fr"]="margedroite",
   ["it"]="marginedestro",
   ["nl"]="rechtermarge",
   ["pe"]="حاشیه‌راست",
   ["ro"]="marginedreapta",
  },
  ["rightpage"]={
   ["cs"]="pravastranka",
   ["de"]="rechterseite",
   ["en"]="rightpage",
   ["fr"]="surpagedroite",
   ["it"]="paginadestra",
   ["nl"]="rechterpagina",
   ["pe"]="صفحه‌راست",
   ["ro"]="paginadreapta",
  },
  ["righttext"]={
   ["cs"]="textvpravo",
   ["de"]="rechtertext",
   ["en"]="righttext",
   ["fr"]="textedroite",
   ["it"]="testodestro",
   ["nl"]="rechtertekst",
   ["pe"]="متن‌راست",
   ["ro"]="textdreapta",
  },
  ["righttoleft"]={
   ["cs"]="righttoleft",
   ["de"]="righttoleft",
   ["en"]="righttoleft",
   ["fr"]="droiteagauche",
   ["it"]="righttoleft",
   ["nl"]="righttoleft",
   ["pe"]="righttoleft",
   ["ro"]="righttoleft",
  },
  ["roman"]={
   ["cs"]="antikva",
   ["de"]="antiqua",
   ["en"]="roman",
   ["fr"]="romain",
   ["it"]="roman",
   ["nl"]="romaan",
   ["pe"]="رومن",
   ["ro"]="roman",
  },
  ["romannumerals"]={
   ["cs"]="rimskecislice",
   ["de"]="roemischezahlen",
   ["en"]="romannumerals",
   ["fr"]="chiffresromains",
   ["it"]="numeriromani",
   ["nl"]="romeins",
   ["pe"]="شماره‌لاتین",
   ["ro"]="numereromane",
  },
  ["rotate"]={
   ["cs"]="otoc",
   ["de"]="drehe",
   ["en"]="rotate",
   ["fr"]="tourner",
   ["it"]="ruota",
   ["nl"]="roteer",
   ["pe"]="دوران",
   ["ro"]="rotit",
  },
  ["rotated"]={
   ["cs"]="otoceno",
   ["de"]="gedreht",
   ["en"]="rotated",
   ["fr"]="tourne",
   ["it"]="ruotato",
   ["nl"]="geroteerd",
   ["pe"]="دوران‌یافته",
   ["ro"]="rotat",
  },
  ["round"]={
   ["cs"]="zaobleny",
   ["de"]="rund",
   ["en"]="round",
   ["fr"]="rond",
   ["it"]="rotondo",
   ["nl"]="rond",
   ["pe"]="گرد",
   ["ro"]="rotund",
  },
  ["row"]={
   ["cs"]="row",
   ["de"]="row",
   ["en"]="row",
   ["fr"]="rang",
   ["it"]="row",
   ["nl"]="rij",
   ["pe"]="سطر",
   ["ro"]="rand",
  },
  ["rule"]={
   ["cs"]="linka",
   ["de"]="linie",
   ["en"]="rule",
   ["fr"]="trait",
   ["it"]="linea",
   ["nl"]="lijn",
   ["pe"]="خط",
   ["ro"]="rigla",
  },
  ["samepage"]={
   ["cs"]="stejnastranka",
   ["de"]="selbeseite",
   ["en"]="samepage",
   ["fr"]="memepage",
   ["it"]="stessapagina",
   ["nl"]="zelfdepagina",
   ["pe"]="همان‌صفحه",
   ["ro"]="aceeasipagina",
  },
  ["sans"]={
   ["cs"]="sans",
   ["de"]="sans",
   ["en"]="sans",
   ["fr"]="sans",
   ["it"]="sans",
   ["nl"]="sans",
   ["pe"]="سانز",
   ["ro"]="sans",
  },
  ["sansbold"]={
   ["cs"]="sanstucne",
   ["de"]="sansfett",
   ["en"]="sansbold",
   ["fr"]="sansserifbold",
   ["it"]="sansgrassetto",
   ["nl"]="sansvet",
   ["pe"]="مشکی‌سانز",
   ["ro"]="sansaldin",
  },
  ["sansnormal"]={
   ["en"]="sansnormal",
   ["fr"]="normalsans",
  },
  ["sansserif"]={
   ["cs"]="bezserifu",
   ["de"]="grotesk",
   ["en"]="sansserif",
   ["fr"]="sansserif",
   ["it"]="sansserif",
   ["nl"]="schreefloos",
   ["pe"]="سانزسریف",
   ["ro"]="sansserif",
  },
  ["saturday"]={
   ["cs"]="sobota",
   ["de"]="samstag",
   ["en"]="saturday",
   ["fr"]="samedi",
   ["it"]="sabato",
   ["nl"]="zaterdag",
   ["pe"]="شنبه",
   ["ro"]="sambata",
  },
  ["screen"]={
   ["cs"]="rastr",
   ["de"]="raster",
   ["en"]="screen",
   ["fr"]="ecran",
   ["it"]="schermo",
   ["nl"]="raster",
   ["pe"]="پرده",
   ["ro"]="ecran",
  },
  ["section"]={
   ["cs"]="sekce",
   ["de"]="absatz",
   ["en"]="section",
   ["fr"]="section",
   ["it"]="capoverso",
   ["nl"]="paragraaf",
   ["pe"]="بخش",
   ["ro"]="sectiune",
  },
  ["sectionblockenvironment"]={
   ["cs"]="sectionblockenvironment",
   ["de"]="sectionblockenvironment",
   ["en"]="sectionblockenvironment",
   ["fr"]="environnementblocsection",
   ["it"]="ambientebloccosezione",
   ["nl"]="sectieblokomgeving",
   ["pe"]="محیط‌بلوک‌بخش",
   ["ro"]="blocsectiuneambient",
  },
  ["sectionnumber"]={
   ["cs"]="cislooddilu",
   ["de"]="abschnittsnummer",
   ["en"]="sectionnumber",
   ["fr"]="numerosection",
   ["it"]="numerosezione",
   ["nl"]="sectienummer",
   ["pe"]="شماره‌بخش",
   ["ro"]="numarsetiune",
  },
  ["see"]={
   ["cs"]="viz",
   ["de"]="sieh",
   ["en"]="see",
   ["fr"]="voir",
   ["it"]="vedi",
   ["nl"]="zie",
   ["pe"]="ببینید",
   ["ro"]="vezi",
  },
  ["selectfont"]={
   ["en"]="selectfont",
   ["fr"]="selectionnerpolice",
  },
  ["separator"]={
   ["en"]="separator",
   ["fr"]="separateur",
   ["nl"]="scheider",
  },
  ["september"]={
   ["cs"]="zari",
   ["de"]="september",
   ["en"]="september",
   ["fr"]="septembre",
   ["it"]="settembre",
   ["nl"]="september",
   ["pe"]="سپتامبر",
   ["ro"]="septembrie",
  },
  ["serif"]={
   ["cs"]="serif",
   ["de"]="serif",
   ["en"]="serif",
   ["fr"]="serif",
   ["it"]="serif",
   ["nl"]="serif",
   ["pe"]="سریف",
   ["ro"]="serif",
  },
  ["serifbold"]={
   ["en"]="serifbold",
   ["fr"]="serifgras",
  },
  ["serifnormal"]={
   ["en"]="serifnormal",
   ["fr"]="serifnormal",
  },
  ["serried"]={
   ["cs"]="semknuto",
   ["de"]="kleinerabstand",
   ["en"]="serried",
   ["fr"]="serre",
   ["it"]="vicino",
   ["nl"]="aanelkaar",
   ["pe"]="تنگ‌هم",
   ["ro"]="serried",
  },
  ["setups"]={
   ["en"]="setups",
  },
  ["seven"]={
   ["cs"]="seven",
   ["de"]="sieben",
   ["en"]="seven",
   ["fr"]="sept",
   ["it"]="seven",
   ["nl"]="zeven",
   ["pe"]="seven",
   ["ro"]="seven",
  },
  ["sheet"]={
   ["cs"]="sheet",
   ["de"]="sheet",
   ["en"]="sheet",
   ["fr"]="feuille",
   ["it"]="sheet",
   ["nl"]="sheet",
   ["pe"]="ورقه",
   ["ro"]="sheet",
  },
  ["shiftdown"]={
   ["en"]="shiftdown",
   ["fr"]="decalerenbas",
  },
  ["shiftup"]={
   ["en"]="shiftup",
   ["fr"]="decalerenhaut",
  },
  ["short"]={
   ["cs"]="short",
   ["de"]="kurz",
   ["en"]="short",
   ["fr"]="court",
   ["it"]="short",
   ["nl"]="kort",
   ["ro"]="short",
  },
  ["simplefonts"]={
   ["en"]="simplefonts",
   ["fr"]="policessimple",
  },
  ["simplelist"]={
   ["en"]="simplelist",
   ["fr"]="listesimple",
  },
  ["singlesided"]={
   ["cs"]="jednostranne",
   ["de"]="einzelseitig",
   ["en"]="singlesided",
   ["fr"]="recto",
   ["it"]="facciasingola",
   ["nl"]="enkelzijdig",
   ["pe"]="یک‌رو",
   ["ro"]="ofata",
  },
  ["six"]={
   ["cs"]="six",
   ["de"]="sechs",
   ["en"]="six",
   ["fr"]="six",
   ["it"]="six",
   ["nl"]="zes",
   ["pe"]="six",
   ["ro"]="six",
  },
  ["size"]={
   ["cs"]="velikost",
   ["de"]="groesse",
   ["en"]="size",
   ["fr"]="taille",
   ["it"]="dimensione",
   ["nl"]="formaat",
   ["pe"]="اندازه",
   ["ro"]="dimensiune",
  },
  ["slanted"]={
   ["cs"]="sklonene",
   ["de"]="geneigt",
   ["en"]="slanted",
   ["fr"]="incline",
   ["it"]="inclinato",
   ["nl"]="schuin",
   ["pe"]="خوابیده",
   ["ro"]="inclinat",
  },
  ["slantedbold"]={
   ["cs"]="sklonenetucne",
   ["de"]="geneigtfett",
   ["en"]="slantedbold",
   ["fr"]="inclinegras",
   ["it"]="inclinatograssetto",
   ["nl"]="schuinvet",
   ["pe"]="مشکی‌خوابیده",
   ["ro"]="inclinataldin",
  },
  ["small"]={
   ["cs"]="male",
   ["de"]="klein",
   ["en"]="small",
   ["fr"]="petit",
   ["it"]="piccolo",
   ["nl"]="klein",
   ["pe"]="کوچک",
   ["ro"]="mic",
  },
  ["smallbodyfont"]={
   ["cs"]="smallbodyfont",
   ["de"]="smallbodyfont",
   ["en"]="smallbodyfont",
   ["fr"]="policedecorpspetite",
   ["it"]="piccolofontdelcorpo",
   ["nl"]="kleinkorps",
   ["pe"]="قلم‌بدنه‌کوچک",
   ["ro"]="smallbodyfont",
  },
  ["smallbold"]={
   ["cs"]="maletucne",
   ["de"]="kleinfett",
   ["en"]="smallbold",
   ["fr"]="petitgras",
   ["it"]="piccolograssetto",
   ["nl"]="kleinvet",
   ["pe"]="مشکی‌کوچک",
   ["ro"]="micaldin",
  },
  ["smallbolditalic"]={
   ["cs"]="maletucnekurzivni",
   ["de"]="kleinfettitalic",
   ["en"]="smallbolditalic",
   ["fr"]="petitgrasitalique",
   ["it"]="piccolograssettocorsivo",
   ["nl"]="kleinvetitalic",
   ["pe"]="ایتالیک‌مشکی‌کوچک",
   ["ro"]="micaldininclinat",
  },
  ["smallboldslanted"]={
   ["cs"]="maletucnesklonene",
   ["de"]="kleinfettgeneigt",
   ["en"]="smallboldslanted",
   ["fr"]="petitgrasincline",
   ["it"]="piccolograssettoinclinato",
   ["nl"]="kleinvetschuin",
   ["pe"]="خوابیده‌مشکی‌کوچک",
   ["ro"]="micaldininclinat",
  },
  ["smallcaps"]={
   ["cs"]="kapitalky",
   ["de"]="smallcaps",
   ["en"]="smallcaps",
   ["fr"]="petitescaps",
   ["it"]="maiuscoletto",
   ["nl"]="smallcaps",
   ["pe"]="smallcaps",
   ["ro"]="majusculemici",
  },
  ["fakecaps"]={
   ["en"]="fakecaps",
   ["nl"]="fakecaps",
  },
  ["smaller"]={
   ["en"]="smaller",
   ["fr"]="pluspetit",
   ["nl"]="kleiner",
  },
  ["smallitalic"]={
   ["cs"]="malekurzivni",
   ["de"]="kleinitalic",
   ["en"]="smallitalic",
   ["fr"]="petititalique",
   ["it"]="piccolocorsivo",
   ["nl"]="kleinitalic",
   ["pe"]="ایتالیک‌کوچک",
   ["ro"]="micitalic",
  },
  ["smallitalicbold"]={
   ["cs"]="malekurzivnitucne",
   ["de"]="kleinitalicfett",
   ["en"]="smallitalicbold",
   ["fr"]="petititaliquegras",
   ["it"]="piccolocorsivograssetto",
   ["nl"]="kleinitalicvet",
   ["pe"]="مشکی‌ایتالیک‌کوچک",
   ["ro"]="micitalicaldin",
  },
  ["smallnormal"]={
   ["cs"]="malenormalni",
   ["de"]="kleinnormal",
   ["en"]="smallnormal",
   ["fr"]="petitnormal",
   ["it"]="piccolonormale",
   ["nl"]="kleinnormaal",
   ["pe"]="نرمال‌کوچک",
   ["ro"]="micnormal",
  },
  ["smallslanted"]={
   ["cs"]="malesklonene",
   ["de"]="kleingeneigt",
   ["en"]="smallslanted",
   ["fr"]="petitincline",
   ["it"]="piccoloinclinato",
   ["nl"]="kleinschuin",
   ["pe"]="خوابیده‌کوچک",
   ["ro"]="micinclinat",
  },
  ["smallslantedbold"]={
   ["cs"]="malesklonenetucne",
   ["de"]="kleingeneigtfett",
   ["en"]="smallslantedbold",
   ["fr"]="petitinclinegras",
   ["it"]="piccoloinclinatograssetto",
   ["nl"]="kleinschuinvet",
   ["pe"]="مشکی‌خوابیده‌کوچک",
   ["ro"]="micinclinataldin",
  },
  ["smalltype"]={
   ["cs"]="maletype",
   ["de"]="kleintippen",
   ["en"]="smalltype",
   ["fr"]="petitcaractere",
   ["it"]="piccolotype",
   ["nl"]="kleintype",
   ["pe"]="تایپ‌کوچک",
   ["ro"]="mictype",
  },
  ["snapping"]={
    ["en"]="snapping",
  },
  ["somewhere"]={
   ["cs"]="nekde",
   ["de"]="irgendwo",
   ["en"]="somewhere",
   ["fr"]="qqpart",
   ["it"]="daqualcheparte",
   ["nl"]="ergens",
   ["pe"]="جایی",
   ["ro"]="undeva",
  },
  ["sorted"]={
   ["cs"]="tridene",
   ["de"]="sortiert",
   ["en"]="sorted",
   ["fr"]="trie",
   ["it"]="ordinato",
   ["nl"]="gesorteerd",
   ["pe"]="مرتب",
   ["ro"]="sortat",
  },
  ["sorting"]={
   ["en"]="sorting",
   ["fr"]="tri",
  },
  ["space"]={
   ["cs"]="mezera",
   ["de"]="spatium",
   ["en"]="space",
   ["fr"]="espace",
   ["it"]="spazio",
   ["nl"]="spatie",
   ["pe"]="فضا",
   ["ro"]="spatiu",
  },
  ["spacing"]={
   ["cs"]="mezerovani",
   ["de"]="spatiumausgleich",
   ["en"]="spacing",
   ["fr"]="espacement",
   ["it"]="spaziatura",
   ["nl"]="spatiering",
   ["pe"]="فضاگذاری",
   ["ro"]="spatiere",
  },
  ["speech"]={
   ["cs"]="speech",
   ["de"]="speech",
   ["en"]="speech",
   ["fr"]="discours",
   ["it"]="speech",
   ["nl"]="spraak",
   ["pe"]="سخنرانی",
   ["ro"]="speech",
  },
  ["split"]={
   ["cs"]="split",
   ["de"]="split",
   ["en"]="split",
   ["fr"]="partager",
   ["it"]="separa",
   ["nl"]="splitsen",
   ["pe"]="شکافتن",
   ["ro"]="split",
  },
  ["spot"]={
   ["cs"]="spot",
   ["de"]="spot",
   ["en"]="spot",
   ["fr"]="tondirect",
   ["it"]="spot",
   ["nl"]="spot",
   ["pe"]="لکه",
   ["ro"]="spot",
  },
  ["standard"]={
   ["cs"]="standardni",
   ["de"]="standard",
   ["en"]="standard",
   ["fr"]="standard",
   ["it"]="standard",
   ["nl"]="standaard",
   ["pe"]="استاندارد",
   ["ro"]="standard",
  },
  ["start"]={
   ["cs"]="start",
   ["de"]="start",
   ["en"]="start",
   ["fr"]="activé",
   ["it"]="inizia",
   ["nl"]="start",
   ["pe"]="شروع",
   ["ro"]="start",
  },
  ["starter"]={
   ["en"]="starter",
   ["fr"]="demarreur",
   ["nl"]="opener",
  },
  ["sticker"]={
   ["cs"]="sticker",
   ["de"]="sticker",
   ["en"]="sticker",
   ["fr"]="autocollant",
   ["it"]="sticker",
   ["nl"]="sticker",
   ["pe"]="دشنه",
   ["ro"]="sticker",
  },
  ["stop"]={
   ["cs"]="stop",
   ["de"]="stop",
   ["en"]="stop",
   ["fr"]="fin",
   ["it"]="termina",
   ["nl"]="stop",
   ["pe"]="پایان",
   ["ro"]="stop",
  },
  ["stopper"]={
   ["cs"]="predel",
   ["de"]="trenner",
   ["en"]="stopper",
   ["fr"]="stoppeur",
   ["it"]="stopper",
   ["nl"]="afsluiter",
   ["pe"]="ایست",
   ["ro"]="stopper",
  },
  ["stretch"]={
   ["cs"]="natahnout",
   ["de"]="strecken",
   ["en"]="stretch",
   ["fr"]="etirer",
   ["it"]="dilata",
   ["nl"]="rek",
   ["pe"]="بکش",
   ["ro"]="dilatat",
  },
  ["stretched"]={
   ["en"]="stretched",
   ["fr"]="etire",
  },
  ["strict"]={
   ["cs"]="striktni",
   ["de"]="streng",
   ["en"]="strict",
   ["fr"]="strict",
   ["it"]="rigido",
   ["nl"]="streng",
   ["pe"]="اکید",
   ["ro"]="strict",
  },
  ["strong"]={
   ["cs"]="strong",
   ["de"]="strong",
   ["en"]="strong",
   ["fr"]="fort",
   ["it"]="strong",
   ["nl"]="sterk",
   ["pe"]="محکم",
   ["ro"]="strong",
  },
  ["strut"]={
   ["cs"]="strut",
   ["de"]="strut",
   ["en"]="strut",
   ["fr"]="entretoise",
   ["it"]="strut",
   ["nl"]="strut",
   ["pe"]="بست",
   ["ro"]="strut",
  },
  ["sub"]={
   ["cs"]="pod",
   ["de"]="unter",
   ["en"]="sub",
   ["fr"]="sous",
   ["it"]="sotto",
   ["nl"]="sub",
   ["pe"]="زیر",
   ["ro"]="sub",
  },
  ["subbackward"]={
   ["cs"]="podzpet",
   ["de"]="unterrueckwaerts",
   ["en"]="subbackward",
   ["fr"]="petitenarriere",
   ["it"]="sottoindietro",
   ["nl"]="subachteruit",
   ["pe"]="زیرعقب‌گرد",
   ["ro"]="subinapoi",
  },
  ["subformula"]={
   ["cs"]="subrovnice",
   ["de"]="subformel",
   ["en"]="subformula",
   ["fr"]="sousformule",
   ["it"]="subformula",
   ["nl"]="subformule",
   ["pe"]="زیرفرمول",
   ["ro"]="subformula",
  },
  ["subforward"]={
   ["cs"]="podvpred",
   ["de"]="untervorwaerts",
   ["en"]="subforward",
   ["fr"]="petitenavant",
   ["it"]="sottoavanti",
   ["nl"]="subvooruit",
   ["pe"]="زیرجلوگرد",
   ["ro"]="subavans",
  },
  ["subject"]={
   ["cs"]="tema",
   ["de"]="thema",
   ["en"]="subject",
   ["fr"]="sujet",
   ["it"]="argomento",
   ["nl"]="onderwerp",
   ["pe"]="موضوع",
   ["ro"]="subiect",
  },
  ["subpage"]={
   ["cs"]="podstranka",
   ["de"]="unterseite",
   ["en"]="subpage",
   ["fr"]="souspage",
   ["it"]="sottopagina",
   ["nl"]="subpagina",
   ["pe"]="زیرصفحه",
   ["ro"]="subpagina",
  },
  ["subs"]={
   ["en"]="subs",
   ["fr"]="subs",
  },
  ["subsection"]={
   ["cs"]="podsekce",
   ["de"]="unterabsatz",
   ["en"]="subsection",
   ["fr"]="soussection",
   ["it"]="sottocapoverso",
   ["nl"]="subparagraaf",
   ["pe"]="زیربخش",
   ["ro"]="subsectiune",
  },
  ["subsubject"]={
   ["cs"]="podtema",
   ["de"]="unterthema",
   ["en"]="subsubject",
   ["fr"]="soussujet",
   ["it"]="sottoargomento",
   ["nl"]="subonderwerp",
   ["pe"]="زیرموضوع",
   ["ro"]="subsubiect",
  },
  ["subsubsection"]={
   ["cs"]="podpodsekce",
   ["de"]="unterunterabsatz",
   ["en"]="subsubsection",
   ["fr"]="soussoussection",
   ["it"]="sottosottocapoverso",
   ["nl"]="subsubparagraaf",
   ["pe"]="زیرزیربخش",
   ["ro"]="subsubsectiune",
  },
  ["subsubsubject"]={
   ["cs"]="podpodtema",
   ["de"]="unterunterthema",
   ["en"]="subsubsubject",
   ["fr"]="soussoussujet",
   ["it"]="sottosottoargomento",
   ["nl"]="subsubonderwerp",
   ["pe"]="زیرزیرموضوع",
   ["ro"]="subsubsubiect",
  },
  ["subsubsubsection"]={
   ["cs"]="podpodpodsekce",
   ["de"]="unterunterunterabsatz",
   ["en"]="subsubsubsection",
   ["fr"]="soussoussoussection",
   ["it"]="sottosottosottocapoverso",
   ["nl"]="subsubsubparagraaf",
   ["pe"]="زیرزیرزیربخش",
   ["ro"]="subsubsubsectiune",
  },
  ["subsubsubsubject"]={
   ["cs"]="podpodpodtema",
   ["de"]="unterunterunterthema",
   ["en"]="subsubsubsubject",
   ["fr"]="soussoussoussujet",
   ["it"]="sottosottosottoargomento",
   ["nl"]="subsubsubonderwerp",
   ["pe"]="زیرزیرزیرموضوع",
   ["ro"]="subsubsubsubiect",
  },
  ["subsubsubsubsection"]={
   ["cs"]="podpodpodpodsekce",
   ["de"]="unterunterunterunterabsatz",
   ["en"]="subsubsubsubsection",
   ["fr"]="soussoussoussoussection",
   ["it"]="sottosottosottosottocapoverso",
   ["nl"]="subsubsubsubparagraaf",
   ["pe"]="زیرزیرزیرزیربخش",
   ["ro"]="subsubsubsubsectiune",
  },
  ["subsubsubsubsubject"]={
   ["cs"]="podpodpodpodtema",
   ["de"]="unterunterunterunterthema",
   ["en"]="subsubsubsubsubject",
   ["fr"]="soussoussoussoussujet",
   ["it"]="sottosottosottosottoargomento",
   ["nl"]="subsubsubsubonderwerp",
   ["pe"]="زیرزیرزیرزیرموضوع",
   ["ro"]="subsubsubsubsubiect",
  },
  ["subsubsubsubsubsection"]={
   ["cs"]="podpodpodpodpodsekce",
   ["de"]="unterunterunterunterunterabsatz",
   ["en"]="subsubsubsubsubsection",
   ["fr"]="soussoussoussoussoussection",
   ["it"]="sottosottosottosottosottocapoverso",
   ["nl"]="subsubsubsubsubparagraaf",
   ["pe"]="زیرزیرزیرزیرزیربخش",
   ["ro"]="subsubsubsubsubsectiune",
  },
  ["subsubsubsubsubsubject"]={
   ["cs"]="podpodpodpodpodtema",
   ["de"]="unterunterunterunterunterthema",
   ["en"]="subsubsubsubsubsubject",
   ["fr"]="soussoussoussoussoussujet",
   ["it"]="sottosottosottosottosottoargomento",
   ["nl"]="subsubsubsubsubonderwerp",
   ["pe"]="زیرزیرزیرزیرزیرموضوع",
   ["ro"]="subsubsubsubsubsubiect",
  },
  ["subsubsubsubsubsubsection"]={
   ["cs"]="podpodpodpodpodpodsekce",
   ["de"]="unterunterunterunterunterunterabsatz",
   ["en"]="subsubsubsubsubsubsection",
   ["fr"]="soussoussoussoussoussoussection",
   ["it"]="sottosottosottosottosottosottocapoverso",
   ["nl"]="subsubsubsubsubsubparagraaf",
   ["pe"]="زیرزیرزیرزیرزیرزیربخش",
   ["ro"]="subsubsubsubsubsubsectiune",
  },
  ["subsubsubsubsubsubsubject"]={
   ["cs"]="podpodpodpodpodpodtema",
   ["de"]="unterunterunterunterunterunterthema",
   ["en"]="subsubsubsubsubsubsubject",
   ["fr"]="soussoussoussoussoussoussujet",
   ["it"]="sottosottosottosottosottosottoargomento",
   ["nl"]="subsubsubsubsubsubonderwerp",
   ["pe"]="زیرزیرزیرزیرزیرزیرموضوع",
   ["ro"]="subsubsubsubsubsubsubiect",
  },
  ["subsubsubsubsubsubsubsection"]={
   ["cs"]="podpodpodpodpodpodpodsekce",
   ["de"]="unterunterunterunterunterunterunterabsatz",
   ["en"]="subsubsubsubsubsubsubsection",
   ["fr"]="soussoussoussoussoussoussoussection",
   ["it"]="sottosottosottosottosottosottosottocapoverso",
   ["nl"]="subsubsubsubsubsubsubparagraaf",
   ["pe"]="زیرزیرزیرزیرزیرزیرزیربخش",
   ["ro"]="subsubsubsubsubsubsubsectiune",
  },
  ["subsubsubsubsubsubsubsubject"]={
   ["cs"]="podpodpodpodpodpodpodtema",
   ["de"]="unterunterunterunterunterunterunterthema",
   ["en"]="subsubsubsubsubsubsubsubject",
   ["fr"]="soussoussoussoussoussoussoussujet",
   ["it"]="sottosottosottosottosottosottosottoargomento",
   ["nl"]="subsubsubsubsubsubsubonderwerp",
   ["pe"]="زیرزیرزیرزیرزیرزیرزیرموضوع",
   ["ro"]="subsubsubsubsubsubsubsubiect",
  },
  ["subsubsubsubsubsubsubsubsection"]={
   ["cs"]="podpodpodpodpodpodpodpodsekce",
   ["de"]="unterunterunterunterunterunterunterunterabsatz",
   ["en"]="subsubsubsubsubsubsubsubsection",
   ["fr"]="soussoussoussoussoussoussoussoussection",
   ["it"]="sottosottosottosottosottosottosottosottocapoverso",
   ["nl"]="subsubsubsubsubsubsubsubparagraaf",
   ["pe"]="زیرزیرزیرزیرزیرزیرزیرزیربخش",
   ["ro"]="subsubsubsubsubsubsubsubsectiune",
  },
  ["subsubsubsubsubsubsubsubsubject"]={
   ["cs"]="podpodpodpodpodpodpodpodtema",
   ["de"]="unterunterunterunterunterunterunterunterthema",
   ["en"]="subsubsubsubsubsubsubsubsubject",
   ["fr"]="soussoussoussoussoussoussoussoussujet",
   ["it"]="sottosottosottosottosottosottosottosottoargomento",
   ["nl"]="subsubsubsubsubsubsubsubonderwerp",
   ["pe"]="زیرزیرزیرزیرزیرزیرزیرزیرموضوع",
   ["ro"]="subsubsubsubsubsubsubsubsubiect",
  },
  ["subsubsubsubsubsubsubsubsubsection"]={
   ["cs"]="podpodpodpodpodpodpodpodpodsekce",
   ["de"]="unterunterunterunterunterunterunterunterunterabsatz",
   ["en"]="subsubsubsubsubsubsubsubsubsection",
   ["fr"]="soussoussoussoussoussoussoussoussoussection",
   ["it"]="sottosottosottosottosottosottosottosottosottocapoverso",
   ["nl"]="subsubsubsubsubsubsubsubsubparagraaf",
   ["pe"]="زیرزیرزیرزیرزیرزیرزیرزیرزیربخش",
   ["ro"]="subsubsubsubsubsubsubsubsubsectiune",
  },
  ["subsubsubsubsubsubsubsubsubsubject"]={
   ["cs"]="podpodpodpodpodpodpodpodpodtema",
   ["de"]="unterunterunterunterunterunterunterunterunterthema",
   ["en"]="subsubsubsubsubsubsubsubsubsubject",
   ["fr"]="soussoussoussoussoussoussoussoussoussujet",
   ["it"]="sottosottosottosottosottosottosottosottosottoargomento",
   ["nl"]="subsubsubsubsubsubsubsubsubonderwerp",
   ["pe"]="زیرزیرزیرزیرزیرزیرزیرزیرزیرموضوع",
   ["ro"]="subsubsubsubsubsubsubsubsubsubiect",
  },
  ["sunday"]={
   ["cs"]="nedele",
   ["de"]="sonntag",
   ["en"]="sunday",
   ["fr"]="dimanche",
   ["it"]="domenica",
   ["nl"]="zondag",
   ["pe"]="یک‌شنبه",
   ["ro"]="duminica",
  },
  ["support"]={
   ["cs"]="podpora",
   ["de"]="support",
   ["en"]="support",
   ["fr"]="support",
   ["it"]="supporto",
   ["nl"]="support",
   ["pe"]="حمایت",
   ["ro"]="suport",
  },
  ["sym"]={
   ["cs"]="sym",
   ["de"]="sym",
   ["en"]="sym",
   ["fr"]="sym",
   ["it"]="sim",
   ["nl"]="sym",
   ["pe"]="نم",
   ["ro"]="sym",
  },
  ["symbol"]={
   ["cs"]="symbol",
   ["de"]="symbol",
   ["en"]="symbol",
   ["fr"]="symbole",
   ["it"]="simbolo",
   ["nl"]="symbool",
   ["pe"]="نماد",
   ["ro"]="simbol",
  },
  ["synchronize"]={
   ["cs"]="synchronize",
   ["de"]="synchronize",
   ["en"]="synchronize",
   ["fr"]="synchroniser",
   ["it"]="synchronize",
   ["nl"]="synchroniseer",
   ["pe"]="تطابق",
   ["ro"]="synchronize",
  },
  ["synonym"]={
   ["en"]="synonym",
   ["fr"]="synonyme",
  },
  ["system"]={
   ["cs"]="system",
   ["de"]="systeme",
   ["en"]="system",
   ["fr"]="systeme",
   ["it"]="sistema",
   ["nl"]="systeem",
   ["pe"]="سیستم",
   ["ro"]="sistem",
  },
  ["table"]={
   ["cs"]="tabulka",
   ["de"]="tabelle",
   ["en"]="table",
   ["fr"]="table",
   ["it"]="tabella",
   ["nl"]="tabel",
   ["pe"]="جدول",
   ["ro"]="tabelul",
  },
  ["tablehead"]={
   ["cs"]="zahlavitabulky",
   ["de"]="tabellenueberschrift",
   ["en"]="tablehead",
   ["fr"]="tetetableau",
   ["it"]="testatabella",
   ["nl"]="tabelkop",
   ["pe"]="سرجدول",
   ["ro"]="antettabel",
  },
  ["tables"]={
   ["cs"]="tabulky",
   ["de"]="tabellen",
   ["en"]="tables",
   ["fr"]="tables",
   ["it"]="tabelle",
   ["nl"]="tabellen",
   ["pe"]="جدولها",
   ["ro"]="tabelele",
  },
  ["tabletail"]={
   ["cs"]="konectabulky",
   ["de"]="tabellenende",
   ["en"]="tabletail",
   ["fr"]="queuetable",
   ["it"]="codatabella",
   ["nl"]="tabelstaart",
   ["pe"]="دنباله‌جدول",
   ["ro"]="subsoltabel",
  },
  ["tabulate"]={
   ["cs"]="tabelator",
   ["de"]="tabulator",
   ["en"]="tabulate",
   ["fr"]="tabulation",
   ["it"]="tabulato",
   ["nl"]="tabulatie",
   ["pe"]="جدول‌بندی",
   ["ro"]="tabulate",
  },
  ["tabulatehead"]={
   ["cs"]="tabulatehead",
   ["de"]="tabulatehead",
   ["en"]="tabulatehead",
   ["fr"]="tetetabulation",
   ["it"]="testatabulato",
   ["nl"]="tabulatiekop",
   ["pe"]="سرجدول‌بندی",
   ["ro"]="tabulatehead",
  },
  ["tabulatetail"]={
   ["cs"]="tabulatetail",
   ["de"]="tabulatetail",
   ["en"]="tabulatetail",
   ["fr"]="queuetabulation",
   ["it"]="codatabulato",
   ["nl"]="tabulatiestaart",
   ["pe"]="دنباله‌جدول‌بندی",
   ["ro"]="tabulatetail",
  },
  ["tall"]={
   ["cs"]="vysoko",
   ["de"]="lang",
   ["en"]="tall",
   ["fr"]="grand",
   ["it"]="alto",
   ["nl"]="lang",
   ["pe"]="دراز",
   ["ro"]="inalt",
  },
  ["teletype"]={
   ["cs"]="strojopis",
   ["de"]="fernschreiber",
   ["en"]="teletype",
   ["fr"]="caractereterminal",
   ["it"]="teletype",
   ["nl"]="teletype",
   ["pe"]="دورنگاره",
   ["ro"]="teletype",
  },
  ["temporary"]={
   ["cs"]="docasne",
   ["de"]="temporaer",
   ["en"]="temporary",
   ["fr"]="temporaire",
   ["it"]="temporaneo",
   ["nl"]="voorlopig",
   ["pe"]="موقتی",
   ["ro"]="temporar",
  },
  ["test"]={
   ["cs"]="test",
   ["de"]="test",
   ["en"]="test",
   ["fr"]="test",
   ["it"]="test",
   ["nl"]="test",
   ["pe"]="تست",
   ["ro"]="test",
  },
  ["text"]={
   ["cs"]="text",
   ["de"]="text",
   ["en"]="text",
   ["fr"]="texte",
   ["it"]="testo",
   ["nl"]="tekst",
   ["pe"]="متن",
   ["ro"]="text",
  },
  ["textdisplay"]={
   ["en"]="textdisplay",
  },
  ["textnote"]={
   ["en"]="textnote",
   ["fr"]="notetexte",
   ["nl"]="tekstnoot",
  },
  ["three"]={
   ["cs"]="tri",
   ["de"]="drei",
   ["en"]="three",
   ["fr"]="trois",
   ["it"]="tre",
   ["nl"]="drie",
   ["pe"]="سه",
   ["ro"]="trei",
  },
  ["thursday"]={
   ["cs"]="ctvrtek",
   ["de"]="donnerstag",
   ["en"]="thursday",
   ["fr"]="jeudi",
   ["it"]="giovedi",
   ["nl"]="donderdag",
   ["pe"]="پنج‌شنبه",
   ["ro"]="joi",
  },
  ["tight"]={
   ["en"]="tight",
   ["fr"]="resserre",
   ["nl"]="krap",
  },
  ["verytight"]={
   ["en"]="verytight",
   ["nl"]="heelkrap",
  },
  ["title"]={
   ["cs"]="titul",
   ["de"]="titel",
   ["en"]="title",
   ["fr"]="titre",
   ["it"]="titolo",
   ["nl"]="titel",
   ["pe"]="عنوان",
   ["ro"]="titlu",
  },
  ["toggle"]={
   ["cs"]="toggle",
   ["de"]="toggle",
   ["en"]="wissel",
   ["fr"]="basculer",
   ["it"]="toggle",
   ["nl"]="toggle",
   ["pe"]="تغییر",
   ["ro"]="toggle",
  },
  ["granular"]={
   ["en"]="granular",
  },
  ["tolerant"]={
   ["cs"]="tolerantni",
   ["de"]="tolerant",
   ["en"]="tolerant",
   ["fr"]="tolerant",
   ["it"]="tollerante",
   ["nl"]="soepel",
   ["pe"]="بردبار",
   ["ro"]="tolerant",
  },
  ["top"]={
   ["cs"]="vrsek",
   ["de"]="oben",
   ["en"]="top",
   ["fr"]="haut",
   ["it"]="cima",
   ["nl"]="boven",
   ["pe"]="بالا",
   ["ro"]="sus",
  },
  ["tuesday"]={
   ["cs"]="utery",
   ["de"]="dienstag",
   ["en"]="tuesday",
   ["fr"]="mardi",
   ["it"]="martedi",
   ["nl"]="dinsdag",
   ["pe"]="سه‌شنبه",
   ["ro"]="marti",
  },
  ["two"]={
   ["cs"]="dve",
   ["de"]="zwei",
   ["en"]="two",
   ["fr"]="deux",
   ["it"]="due",
   ["nl"]="twee",
   ["pe"]="دو",
   ["ro"]="doi",
  },
  ["txt"]={
   ["cs"]="txt",
   ["de"]="txt",
   ["en"]="txt",
   ["fr"]="txt",
   ["it"]="txt",
   ["nl"]="txt",
   ["pe"]="txt",
   ["ro"]="txt",
  },
  ["type"]={
   ["cs"]="opis",
   ["de"]="tippen",
   ["en"]="type",
   ["fr"]="transcrire",
   ["it"]="type",
   ["nl"]="type",
   ["pe"]="تایپ",
   ["ro"]="type",
  },
  ["typing"]={
   ["cs"]="typing",
   ["de"]="tippen",
   ["en"]="typing",
   ["fr"]="transcription",
   ["it"]="typing",
   ["nl"]="typen",
   ["pe"]="تایپ‌کردن",
   ["ro"]="typing",
  },
  ["unavailable"]={
   ["cs"]="nedostupne",
   ["de"]="nichtverfuegbar",
   ["en"]="unavailable",
   ["fr"]="indisponible",
   ["it"]="nondisponibile",
   ["nl"]="nietbeschikbaar",
   ["pe"]="غیرموجود",
   ["ro"]="nedisponibil",
  },
  ["underbar"]={
   ["cs"]="podtrzeno",
   ["de"]="unterstrichen",
   ["en"]="underbar",
   ["fr"]="souligner",
   ["it"]="sottolinea",
   ["nl"]="onderstreep",
   ["pe"]="میله‌زیر",
   ["ro"]="sublinie",
  },
  ["underbars"]={
   ["cs"]="podtrzeni",
   ["de"]="unterstreichen",
   ["en"]="underbars",
   ["fr"]="soulignermots",
   ["it"]="sottolinee",
   ["nl"]="onderstrepen",
   ["pe"]="میله‌‌های‌زیر",
   ["ro"]="sublinii",
  },
  ["understrike"]={
   ["en"]="understrike",
   ["fr"]="soubarrer",
  },
  ["understrikes"]={
   ["en"]="understrikes",
   ["fr"]="soubarrermots",
  },
  ["unframed"]={
   ["en"]="unframed",
   ["fr"]="desencadre",
  },
  ["unicode"]={
   ["en"]="unicode",
   ["fr"]="unicode",
  },
  ["unit"]={
   ["cs"]="jednotka",
   ["de"]="einheit",
   ["en"]="unit",
   ["fr"]="unite",
   ["it"]="unita",
   ["nl"]="eenheid",
   ["pe"]="واحد",
   ["ro"]="unitate",
  },
  ["units"]={
   ["cs"]="jednotky",
   ["de"]="einheiten",
   ["en"]="units",
   ["fr"]="unites",
   ["it"]="unita",
   ["nl"]="eenheden",
   ["pe"]="واحدها",
   ["ro"]="unitati",
  },
  ["unknown"]={
   ["cs"]="neznamy",
   ["de"]="unbekannt",
   ["en"]="unknown",
   ["fr"]="inconnu",
   ["it"]="ignoto",
   ["nl"]="onbekend",
   ["pe"]="ناشناس",
   ["ro"]="necunoscut",
  },
  ["unpacked"]={
   ["cs"]="rozbalene",
   ["de"]="ungepackt",
   ["en"]="unpacked",
   ["fr"]="avecespaceentreitem",
   ["it"]="unpacked",
   ["nl"]="vanelkaar",
   ["pe"]="غیرفشرده",
   ["ro"]="despachetat",
  },
  ["up"]={
   ["en"]="up",
   ["fr"]="enhaut",
   ["nl"]="omhoog",
  },
  ["url"]={
   ["cs"]="url",
   ["de"]="url",
   ["en"]="url",
   ["fr"]="url",
   ["it"]="url",
   ["nl"]="url",
   ["pe"]="url",
   ["ro"]="url",
  },
  ["used"]={
   ["cs"]="uzito",
   ["de"]="verwende",
   ["en"]="used",
   ["fr"]="utilise",
   ["it"]="usato",
   ["nl"]="gebruikt",
   ["pe"]="استفاده‌شده",
   ["ro"]="folosit",
  },
  ["value"]={
   ["cs"]="hodnota",
   ["de"]="wert",
   ["en"]="value",
   ["fr"]="valeur",
   ["it"]="valore",
   ["nl"]="waarde",
   ["pe"]="مقدار",
   ["ro"]="valoare",
  },
  ["vertical"]={
   ["cs"]="vertikalne",
   ["de"]="vertikal",
   ["en"]="vertical",
   ["fr"]="vertical",
   ["it"]="verticale",
   ["nl"]="vertikaal",
   ["pe"]="عمودی",
   ["ro"]="vertical",
  },
  ["very"]={
   ["cs"]="velmi",
   ["de"]="sehr",
   ["en"]="very",
   ["fr"]="tres",
   ["it"]="molto",
   ["nl"]="zeer",
   ["pe"]="خیلی",
   ["ro"]="foarte",
  },
  ["verystrict"]={
   ["cs"]="velmistriktni",
   ["de"]="sehrstreng",
   ["en"]="verystrict",
   ["fr"]="tresstrict",
   ["it"]="moltorigido",
   ["nl"]="zeerstreng",
   ["pe"]="خیلی‌سختگیر",
   ["ro"]="foartestrict",
  },
  ["verytolerant"]={
   ["cs"]="velmitolerantni",
   ["de"]="sehrtolerant",
   ["en"]="verytolerant",
   ["fr"]="trestolerant",
   ["it"]="moltotollerante",
   ["nl"]="zeersoepel",
   ["pe"]="خیلی‌بردبار",
   ["ro"]="foartetolerant",
  },
  ["vfenced"]={
   ["en"]="vfenced",
   ["fr"]="cloturev",
  },
  ["vulgarfraction"]={
   ["en"]="vulgarfraction",
   ["fr"]="vulgarfraction",
  },
  ["weak"]={
   ["cs"]="weak",
   ["de"]="weak",
   ["en"]="weak",
   ["fr"]="faible",
   ["it"]="weak",
   ["nl"]="zwak",
   ["pe"]="هفته",
   ["ro"]="weak",
  },
  ["wednesday"]={
   ["cs"]="streda",
   ["de"]="mittwoch",
   ["en"]="wednesday",
   ["fr"]="mercredi",
   ["it"]="mercoledi",
   ["nl"]="woensdag",
   ["pe"]="چهارشنبه",
   ["ro"]="miercuri",
  },
  ["week"]={
   ["cs"]="tyden",
   ["de"]="woche",
   ["en"]="week",
   ["fr"]="semaine",
   ["it"]="settimana",
   ["nl"]="week",
   ["pe"]="هفته",
   ["ro"]="saptamana",
  },
  ["weekday"]={
   ["cs"]="vsedniden",
   ["de"]="wochentag",
   ["en"]="weekday",
   ["fr"]="joursemaine",
   ["it"]="giornosettimana",
   ["nl"]="weekdag",
   ["pe"]="روزهفته",
   ["ro"]="zisaptamana",
  },
  ["white"]={
   ["cs"]="bily",
   ["de"]="weiss",
   ["en"]="white",
   ["fr"]="blanc",
   ["it"]="bianco",
   ["nl"]="wit",
   ["pe"]="سفید",
   ["ro"]="alb",
  },
  ["wide"]={
   ["cs"]="siroce",
   ["de"]="breit",
   ["en"]="wide",
   ["fr"]="ample",
   ["it"]="ampio",
   ["nl"]="breed",
   ["pe"]="گسترده",
   ["ro"]="larg",
  },
  ["width"]={
   ["cs"]="sirka",
   ["de"]="breite",
   ["en"]="width",
   ["fr"]="largeur",
   ["it"]="ampiezza",
   ["nl"]="breedte",
   ["pe"]="عرض",
   ["ro"]="latime",
  },
  ["word"]={
   ["cs"]="slovo",
   ["de"]="wort",
   ["en"]="word",
   ["fr"]="mot",
   ["it"]="parola",
   ["nl"]="woord",
   ["pe"]="word",
   ["ro"]="cuvant",
  },
  ["words"]={
   ["en"]="words",
   ["fr"]="mots",
  },
  ["xml"]={
   ["cs"]="xml",
   ["de"]="xml",
   ["en"]="xml",
   ["fr"]="xml",
   ["it"]="xml",
   ["nl"]="xml",
   ["pe"]="xml",
   ["ro"]="xml",
  },
  ["year"]={
   ["cs"]="rok",
   ["de"]="jahr",
   ["en"]="year",
   ["fr"]="annee",
   ["it"]="anno",
   ["nl"]="jaar",
   ["pe"]="سال",
   ["ro"]="an",
  },
  ["yes"]={
   ["cs"]="ano",
   ["de"]="ja",
   ["en"]="yes",
   ["fr"]="oui",
   ["it"]="si",
   ["nl"]="ja",
   ["pe"]="بله",
   ["ro"]="da",
  },
 },
}
